/*------------------------------------------------------------------------
 * $Id: Device.c++,v 2.8 1993/11/16 17:56:46 carlson Exp $
 *
 * Methods for Device class.
 *
 * Revision History:
 *	$Log: Device.c++,v $
// Revision 2.8  1993/11/16  17:56:46  carlson
// Include systemfiles.h and we get to remove prototype for gettoken and
//   remove the static function 'Copy'.
//
// Revision 2.7  1993/11/16  02:20:19  carlson
// Added code to support modified flag.
// Removed code that checks for a bad baud rate.
//
// Revision 2.6  1993/11/05  00:48:12  carlson
// Had to change my COPY macro to call a function, since it didn't do what
//   I really wanted.
// Modified various areas to allow us to create an instance of this class
//   without any information.
//
// Revision 2.5  1993/10/22  00:25:55  carlson
// Added removeChangeCallback method.
//
// Revision 2.4  1993/10/18  18:40:22  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/10/06  17:12:41  chris
// Added method for generating a line to be put back into the Devices file.
//
// Revision 2.2  1993/08/10  22:21:41  chris
// Added ability to create callbacks for when entries are changed.
// Removed a couple of strdup's and changed to COPY macro.
// Changed names of all variables so methods of the same name could be
//   made (to handle callbacks).
// Added destructor.
//
// Revision 2.1  1993/07/28  16:14:03  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <iostream.h>

#include "Device.h"
#include "systemfiles.h"

#define COPY(from,to)	(to) = Copy (from)
#define DELETE(field)	if (field) delete [] field

/*========================================================================
 * Device class
 *========================================================================*/

/*------------------------------------------------------------------------
 * Device::Device \(const char *\)
 *------------------------------------------------------------------------*/

Device::Device (const char *line)
{
    char	*cptr, *c;
    char	*newline, *nexttoken;

    callbacks = NULL;
    modified = 0;

    /*----
     * Make a duplicate of the line so we can edit it without screwing
     * up the original.
     *----*/

    newline = new char [strlen (line) + 1];
    nexttoken = newline;
    strcpy (newline, line);

    /*----
     * The following variables may or may not be used so we need to
     * initialize them to 0.
     *----*/

    m = 0;
    p = NULL;

    /*----
     * First token is the device name.
     * The device name can be followed by ",<proto>".  If it is, put this
     * into the proto field.
     *----*/

    cptr = gettoken (&nexttoken, " \t\n");
    c = strchr (cptr, ',');
    if (c)
    {
	*c++ = '\0';
	if (*c)
	    COPY (c, p);
    }
    setKey (cptr);

    /*----
     * Second token is the data port.
     * The data port can be followed by a \',M\' which indicates that
     * modem control is to be used.  Set the flag if it is.
     *----*/

    cptr = gettoken (&nexttoken, " \t\n");
    if (cptr)
    {
	c = strchr (cptr, ',');
	if (c)
	{
	    *c++ = '\0';
	    if (*c == 'M')
		m = 1;
	}
    }
    COPY (cptr,dataP);

    /*----
     * Third token is the dialer port.
     *----*/

    cptr = gettoken (&nexttoken, " \t\n");
    COPY (cptr,dialerP);

    /*----
     * Fourth token is the baud rate.
     *----*/

    b = 0;
    cptr = gettoken (&nexttoken, " \t\n");
    if (cptr)
	b = (strcmp (cptr, "Any") == 0) ? ANYBAUD : atoi (cptr);

    /*----
     * Fifth and following tokens are dialer-token pairs \(only two
     * pairs are usually provided\).  Currently, just remember what
     * was provided.
     *----*/

#if 0
    c = buff1;
    *c = '\0';
    while (cptr = gettoken (&nexttoken, " \t\n"))
    {
	dialer = (Dialer *)d->find (cptr);
	tptr = gettoken (&nexttoken, " \t\n");
	if (dialer)
	{
	    char	*c1, *c2;

	    for (c1 = dialer->script (); (*c = *c1); c++, c1++)
	    {
		if ((*c1 == 0134) && (*(c1+1) == 'D') && tptr)
		    for (c2 = tptr; (*c = *c2); c++, c2++);
	    }
	}
    }

    COPY (buff1, chat);
#else
    COPY (nexttoken, chat);
#endif

    delete [] newline;
}

/*------------------------------------------------------------------------
 * Device::~Device \(\)
 *------------------------------------------------------------------------*/

Device::~Device ()
{
    DELETE (dataP);
    DELETE (dialerP);
    DELETE (chat);
    DELETE (p);
}

/*------------------------------------------------------------------------
 * Device::line \(void\)
 *------------------------------------------------------------------------*/

char *Device::line (void)
{
    static char		buffer[256];
    int			i;

    if (p)
	sprintf (buffer, "%s,%s ", theKey (), p);
    else
	sprintf (buffer, "%s ", theKey ());
    i = strlen (buffer);

    if (m)
	sprintf (&buffer[i], "%s,M %s ", dataP, dialerP);
    else
	sprintf (&buffer[i], "%s %s ", dataP, dialerP);
    i = strlen (buffer);

    if (b == ANYBAUD)
	sprintf (&buffer[i], "%s %s", "Any", chat);
    else
	sprintf (&buffer[i], "%d %s", b, chat);

    return buffer;
}

/*------------------------------------------------------------------------
 * Device::setDataPort \(char *\)
 *------------------------------------------------------------------------*/

void Device::setDataPort (char *ndp)
{
    callbackStruct	*cbs;

    if (dataP)
	DELETE (dataP);

    COPY (ndp, dataP);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, DATAPORT,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setDialerPort \(char *\)
 *------------------------------------------------------------------------*/

void Device::setDialerPort (char *ndp)
{
    callbackStruct	*cbs;

    if (dialerP)
	DELETE (dialerP);

    COPY (ndp, dialerP);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, DIALERPORT,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setBaud \(int\)
 *------------------------------------------------------------------------*/

void Device::setBaud (int nb)
{
    callbackStruct	*cbs;

    b = nb;
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, BAUD,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setBaud \(char *\)
 *------------------------------------------------------------------------*/

void Device::setBaud (char *nb)
{
    callbackStruct	*cbs;
    int			old_baud = b;

    b = (strcmp (nb, "Any") == 0) ? ANYBAUD : atoi (nb);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, BAUD,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setChatScript \(char *\)
 *------------------------------------------------------------------------*/

void Device::setChatScript (char *nc)
{
    callbackStruct	*cbs;

    if (chat)
	DELETE (chat);

    COPY (nc, chat);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, CHATSCRIPT,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setModemControl \(int\)
 *------------------------------------------------------------------------*/

void Device::setModemControl (int nm)
{
    callbackStruct	*cbs;

    m = nm;
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, MODEMCTRL,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::setProto \(char *\)
 *------------------------------------------------------------------------*/

void Device::setProto (char *np)
{
    callbackStruct	*cbs;

    if (p)
	DELETE (p);

    COPY (np, p);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, PROTO,
						cbs->clientData);
}

/*------------------------------------------------------------------------
 * Device::addChangeCallback \(callbackFunction *, void *\)
 *------------------------------------------------------------------------*/

void Device::addChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp;

    cbs = new callbackStruct;
    cbs->cbf = cbf;
    cbs->clientData = cd;
    cbs->next = NULL;

    if (callbacks)
    {
	for (cbstemp = callbacks; cbstemp->next; cbstemp = cbstemp->next)
	    ;
	cbstemp->next = cbs;
    }
    else
	callbacks = cbs;
}

/*------------------------------------------------------------------------
 * Device::removeChangeCallback \(callbackFunction, void *\)
 *------------------------------------------------------------------------*/

void Device::removeChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp;

    if (callbacks)
    {
	for (cbs = NULL, cbstemp = callbacks; cbstemp; cbstemp = cbstemp->next)
	{
	    if ((cbstemp->cbf == cbf) && (cbstemp->clientData == cd))
	    {
		if (cbs)
		    cbs->next = cbstemp->next;
		else
		    callbacks = cbstemp->next;
		delete cbstemp;
		break;
	    }
	    cbs = cbstemp;
	}
    }
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * Device::dump \(char *\)
 *------------------------------------------------------------------------*/

void Device::dump (char *indent)
{
    char		*nextIndent;
    int			i;
    callbackStruct	*cb;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "Device dump (" << this << "):" << endl;
    cout << indent << "  device  = " << theKey () << endl;
    cout << indent << "  dataP   = " << dataP << endl;
    cout << indent << "  dialerP = " << dialerP << endl;
    cout << indent << "  baud    = " << b << endl;
    cout << indent << "  chat    = " << chat << endl;
    cout << indent << "  m       = " << m << endl;
    cout << indent << "  p       = " << p << endl;
    cout << endl;
    cout << indent << "  Callbacks:" << endl;
    for (cb = callbacks; cb; cb = cb->next)
    {
	cout << indent << "  . function = " << cb->cbf << endl;
	cout << indent << "    data     = " << cb->clientData << endl;
    }
}
#endif
