/*---*- C++ -*------------------------------------------------------------
 * $Id: Device.h,v 2.6 1993/11/16 02:20:19 carlson Exp $
 *
 * Header file describing Device class.
 *
 * Revision History:
 *	$Log: Device.h,v $
 * Revision 2.6  1993/11/16  02:20:19  carlson
 * Added modified flag.
 * Added prototypes for methods to check and clear modified flag.
 * Cleaned up some.
 *
 * Revision 2.5  1993/10/22  00:25:55  carlson
 * Added prototype for removeChangeCallback method.
 *
 * Revision 2.4  1993/10/14  23:07:13  carlson
 * Added prototype for dump method.
 *
 * Revision 2.3  1993/10/06  17:41:50  chris
 * Added documentation.
 * Added declaration for method for outputing a line.
 * Changed all protected members to be private.
 *
 * Revision 2.2  1993/08/10  19:39:52  chris
 * Added stuff for handling callbacks.
 * Made destructor a non-inline method.
 * Added methods for getting and setting the variables.
 * Changed names of variables so methods of the same name could be made.
 *
 * Revision 2.1  1993/07/27  22:49:36  chris
 * Initial revision.
 * Broken out from Devices.h.
 *
 *------------------------------------------------------------------------*/

#ifndef _DEVICE_H_
#define _DEVICE_H_

#include "linkList.h"

#ifndef ANYBAUD
#define ANYBAUD		-1
#endif

/*------------------------------------------------------------------------
 * Device Class
 *	When instantiating, this class must be provided a line from
 *	the "Devices" file that describes a device.  The line will be
 *	parsed and placed in the various private locations to be
 *	referenced or set through the methods.
 *
 *	The line can be returned \(as it should appear in the "Devices"
 *	file\) by using the method \'line\'.
 *
 *	A callback can be set up using \'addChangeCallback\' which
 *	will run a function when any of the values is changed \(using
 *	one of the methods\).
 *------------------------------------------------------------------------*/

class Device;
class Device : public LinkData
{
  public:
    enum Field { DATAPORT, DIALERPORT, BAUD, CHATSCRIPT, MODEMCTRL, PROTO };
    typedef void (*callbackFunction)(Device*, Field, void*);

    Device (const char *line);
    ~Device ();

    const char*		data_port (void) { return dataP; }
    const char*		dialer_port (void) { return dialerP; }
    int			baud (void) { return b; }
    const char*		chat_script (void) { return chat; }
    int			modem_control (void) { return m; }
    const char*		proto (void) { return p; }
    char*		line (void);

    void		setDataPort (char *dp);
    void		setDialerPort (char *dp);
    void		setBaud (int nb);
    void		setBaud (char *nb);
    void		setChatScript (char *nc);
    void		setModemControl (int nm);
    void		setProto (char *np);

    int			wasModified (void) { return modified; }
    void		clearModified (void) { modified = 0; }

    void		addChangeCallback (callbackFunction ncbf,
					   void *clientData);
    void		removeChangeCallback (callbackFunction ncbf,
					      void *clientData);

#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    char		*dataP;		// Data port
    char		*dialerP;	// Dialer port
    int			b;		// Baud rate
    char		*chat;		// Chat script
    int			m;		// Modem control
    char		*p;		// Proto

    int			modified;

    struct callbackStruct
    {
	callbackFunction	cbf;
	void			*clientData;
	callbackStruct		*next;
    };
    callbackStruct	*callbacks;
};

#endif
