/*------------------------------------------------------------------------
 * $Id: DeviceWindow.c++,v 2.8 1994/01/20 03:38:18 carlson Exp $
 *
 * Methods for DeviceWindow class.
 *
 * Revision History:
 *	$Log: DeviceWindow.c++,v $
// Revision 2.8  1994/01/20  03:38:18  carlson
// Modified to support new kind of help facility.
//
// Revision 2.7  1993/11/05  19:55:12  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.6  1993/11/05  02:27:38  carlson
// Added help for buttons and labels.
//
// Revision 2.5  1993/10/22  00:33:21  carlson
// Added destructor that removes any callbacks set up.
//
// Revision 2.4  1993/10/14  23:45:24  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/09/28  12:48:56  chris
// Removed the forced labels to allow labels to be provided in the app-defaults
//   file.
//
// Revision 2.2  1993/08/10  22:15:43  chris
// Added callback for when the Device class is changed.
// Added callbacks for all fields displayed.
// No longer directly reference fields in Device class but call methods
//   to access the fields.
//
// Revision 2.1  1993/07/29  01:24:26  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "DeviceWindow.h"
#include "Dialer.h"
#include "HelpDialog.h"

#ifdef DEBUG
#include <iostream.h>
#endif

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/List.h>

/*------------------------------------------------------------------------
 * DeviceWindow::deviceChangedCallback \(Device *, Device::Field, void *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::deviceChangedCallback (Device *d, Device::Field field,
					  void *clientData)
{
    DeviceWindow	*dw;

    dw = (DeviceWindow *)clientData;
    dw->deviceChanged (field, d);
}

/*------------------------------------------------------------------------
 * DeviceWindow::deviceChanged \(Device::Field, Device *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::deviceChanged (Device::Field field, Device *)
{
    char		baud_rate[50];

    switch (field)
    {
      case Device::DATAPORT:
	XtVaSetValues (w_data_port, XmNvalue, (char *)device->data_port (),
		       NULL);
	break;
      case Device::DIALERPORT:
	XtVaSetValues (w_dialer_port, XmNvalue, (char *)device->dialer_port (),
		       NULL);
	break;
      case Device::BAUD:
	if (device->baud () != ANYBAUD)
	    sprintf (baud_rate, "%d", device->baud ());
	else
	    strcpy (baud_rate, "Any");
	XtVaSetValues (w_baud, XmNvalue, baud_rate, NULL);
	break;
      case Device::CHATSCRIPT:
	XtVaSetValues (w_chat_script, XmNvalue, (char *)device->chat_script (),
		       NULL);
	break;
      case Device::MODEMCTRL:
	XtVaSetValues (w_modem_control, XmNvalue,
		       ((device->modem_control ()) ? "TRUE" : "FALSE"), NULL);
	break;
      case Device::PROTO:
	XtVaSetValues (w_proto, XmNvalue, (char *)device->proto (), NULL);
	break;
    }
}

/*------------------------------------------------------------------------
 * DeviceWindow::dataPChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::dataPChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->dataPChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::dialerPChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::dialerPChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->dialerPChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::baudChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::baudChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->baudChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::chatChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::chatChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->chatChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::modemChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::modemChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->modemChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::protoChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DeviceWindow::protoChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DeviceWindow		*hw;

    hw = (DeviceWindow *)clientData;
    hw->protoChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DeviceWindow::dataPChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::dataPChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_data_port, XmNvalue, &str, NULL);
    device->setDataPort (str);
}

/*------------------------------------------------------------------------
 * DeviceWindow::dialerPChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::dialerPChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_dialer_port, XmNvalue, &str, NULL);
    device->setDialerPort (str);
}

/*------------------------------------------------------------------------
 * DeviceWindow::baudChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::baudChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_baud, XmNvalue, &str, NULL);
    device->setBaud (str);
}

/*------------------------------------------------------------------------
 * DeviceWindow::chatChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::chatChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_chat_script, XmNvalue, &str, NULL);
    device->setChatScript (str);
}

/*------------------------------------------------------------------------
 * DeviceWindow::modemChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::modemChanged (XmAnyCallbackStruct *)
{
    char		*str;
    int			val;

    XtVaGetValues (w_modem_control, XmNvalue, &str, NULL);
    val = (strcasecmp (str, "true") == 0);
    device->setModemControl (val);
}

/*------------------------------------------------------------------------
 * DeviceWindow::protoChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::protoChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_proto, XmNvalue, &str, NULL);
    device->setProto (str);
}

/*------------------------------------------------------------------------
 * DeviceWindow::DeviceWindow (const char *)
 *------------------------------------------------------------------------*/

DeviceWindow::DeviceWindow (Widget parent, Device *dev, SmartList *dlrs)
: VkComponent (dev->theKey ())
{
    Arg		wargs[10];
    int		n;
    XmString	xmstr;
    Widget	label;
    char	baud_rate[50];

    device = dev;
    dlrList = dlrs;

    /*----
     * Create a row/column widget to hold labels and text for device
     * data.  This way, everything should line up properly.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, 2); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    _baseWidget = XmCreateRowColumn (parent, "deviceWindow", wargs, n);
    installDestroyHandler ();

    /*----
     * Create information.
     * For the RowColumn widget, we have to give all the labels
     * first, then all the text portions.
     *----*/

    n = 0;
    label = XmCreateLabel (_baseWidget, "devNameLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_name_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "dataPortLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_data_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "dialerPortLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_dialer_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "baudLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_baud_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "chatLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_chat_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "mControlLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_mctrl_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "protoLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_proto_help");

/*---
 * Do the widget that actually show the data.
 *---*/

    n = 0;
    xmstr = XmStringCreateSimple ((char *)device->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    label = XmCreateLabel (_baseWidget, "devNameText", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "devw_name_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, (char *)device->data_port ()); n++;
    w_data_port = XmCreateTextField (_baseWidget, "dataPortText", wargs, n);
    XtManageChild (w_data_port);

    XtAddCallback (w_data_port, XmNactivateCallback, dataPChangedCallback,
		   this);
    XtAddCallback (w_data_port, XmNhelpCallback, aHelpCallback,
		   "devw_data_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, (char *)device->dialer_port ()); n++;
    w_dialer_port = XmCreateTextField (_baseWidget, "dialerPortText", wargs,
				       n);
    XtManageChild (w_dialer_port);

    XtAddCallback (w_dialer_port, XmNactivateCallback, dialerPChangedCallback,
		   this);
    XtAddCallback (w_dialer_port, XmNhelpCallback, aHelpCallback,
		   "devw_dialer_help");

    n = 0;
    if (device->baud () != ANYBAUD)
	sprintf (baud_rate, "%d", device->baud ());
    else
	strcpy (baud_rate, "Any");
    XtSetArg (wargs[n], XmNvalue, baud_rate); n++;
    w_baud = XmCreateTextField (_baseWidget, "baudText", wargs, n);
    XtManageChild (w_baud);

    XtAddCallback (w_baud, XmNactivateCallback, baudChangedCallback, this);
    XtAddCallback (w_baud, XmNhelpCallback, aHelpCallback, "devw_baud_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, (char *)device->chat_script ()); n++;
    w_chat_script = XmCreateTextField (_baseWidget, "chatText", wargs, n);
    XtManageChild (w_chat_script);

    XtAddCallback (w_chat_script, XmNactivateCallback, chatChangedCallback,
		   this);
    XtAddCallback (w_chat_script, XmNhelpCallback, aHelpCallback,
		   "devw_chat_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue,
	      ((device->modem_control ()) ? "TRUE" : "FALSE")); n++;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    w_modem_control = XmCreateTextField (_baseWidget, "mControlText", wargs,
					 n);
    XtManageChild (w_modem_control);

    XtAddCallback (w_modem_control, XmNhelpCallback, aHelpCallback,
		   "devw_mctrl_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, (char *)device->proto ()); n++;
    w_proto = XmCreateTextField (_baseWidget, "protoText", wargs, n);
    XtManageChild (w_proto);

    XtAddCallback (w_proto, XmNactivateCallback, protoChangedCallback, this);
    XtAddCallback (w_proto, XmNhelpCallback, aHelpCallback, "devw_proto_help");

    device->addChangeCallback (DeviceWindow::deviceChangedCallback,
			       (void *)this);
}

/*------------------------------------------------------------------------
 * DeviceWindow::~DeviceWindow \(\)
 *------------------------------------------------------------------------*/

DeviceWindow::~DeviceWindow ()
{
    device->removeChangeCallback (DeviceWindow::deviceChangedCallback,
				  (void *)this);
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * DeviceWindow::dump \(char *\)
 *------------------------------------------------------------------------*/

void DeviceWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "DeviceWindow dump (" << this << "):" << endl;
    cout << indent << "  device  = " << device << endl;
    cout << indent << "  dlrList = " << dlrList << endl;
    cout << indent << "  address of callback = " << &deviceChangedCallback
	 << endl;

    device->dump (nextIndent);
}
#endif
