/*---*- C++ -*------------------------------------------------------------
 * $Id: DeviceWindow.h,v 2.5 1993/11/05 19:55:11 carlson Exp $
 *
 * Header file for DeviceWindow class.
 *
 * Revision History:
 *	$Log: DeviceWindow.h,v $
 * Revision 2.5  1993/11/05  19:55:11  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.4  1993/10/22  00:33:21  carlson
 * Added prototype for destructor.
 *
 * Revision 2.3  1993/10/14  23:07:13  carlson
 * Added prototype for dump method.
 *
 * Revision 2.2  1993/08/10  22:04:28  chris
 * Added methods for handling all the callbacks.
 * Need to include Device.h now.
 *
 * Revision 2.1  1993/07/29  01:24:26  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _DEVICEWINDOW_H_
#define _DEVICEWINDOW_H_

#include <Vk/VkComponent.h>
#include "SmartList.h"
#include "Device.h"

class DeviceWindow : public VkComponent
{
  public:
    DeviceWindow (Widget parent, Device *dev, SmartList *dlrs = NULL);
    ~DeviceWindow ();

    virtual const char*	className () { return "DeviceWindow"; }
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    Device		*device;
    SmartList		*dlrList;
    Widget		w_data_port;
    Widget		w_dialer_port;
    Widget		w_baud;
    Widget		w_chat_script;
    Widget		w_modem_control;
    Widget		w_proto;

    static void		deviceChangedCallback (Device *, Device::Field,
					       void *);
    static void		dataPChangedCallback (Widget, XtPointer, XtPointer);
    static void		dialerPChangedCallback (Widget, XtPointer, XtPointer);
    static void		baudChangedCallback (Widget, XtPointer, XtPointer);
    static void		chatChangedCallback (Widget, XtPointer, XtPointer);
    static void		modemChangedCallback (Widget, XtPointer, XtPointer);
    static void		protoChangedCallback (Widget, XtPointer, XtPointer);

    void		deviceChanged (Device::Field, Device *);
    void		dataPChanged (XmAnyCallbackStruct *);
    void		dialerPChanged (XmAnyCallbackStruct *);
    void		baudChanged (XmAnyCallbackStruct *);
    void		chatChanged (XmAnyCallbackStruct *);
    void		modemChanged (XmAnyCallbackStruct *);
    void		protoChanged (XmAnyCallbackStruct *);
};

#endif
