/*---*- C++ -*------------------------------------------------------------
 * $Id: Devices.h,v 2.8 1994/02/28 22:30:09 carlson Exp $
 *
 * Header file describing the Devices class.
 *
 * Revision History:
 *	$Log: Devices.h,v $
 * Revision 2.8  1994/02/28  22:30:09  carlson
 * Added place to keep any errors encounted and a way to read it.
 *
 * Revision 2.7  1994/02/18  18:40:14  chrisc
 * Modified package manipulation methods to call SmartList callbacks.
 * Rearranged some lines.
 *
 * Revision 2.6  1994/02/10  22:55:48  chrisc
 * Made package access functions inline.
 * Added methods to remove a package.
 *
 * Revision 2.5  1994/01/18  04:02:59  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.4  1993/11/16  02:21:11  carlson
 * Added modified flag.
 * Added virtual methods to set modified flag.
 * Added prototypes for new methods to check and clear modified flag.
 *
 * Revision 2.3  1993/11/05  19:55:11  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.2  1993/10/06  17:39:17  chris
 * Modified all parameters for PathName to use references.
 * Added declarations for save methods.
 *
 * Revision 2.1  1993/07/27  23:59:53  chris
 * Updated to only include class for Devices.
 * Made to follow new format, using PathName as input.
 *
 * Revision 1.3  1993/05/17  06:14:14  chris
 * Moved stuff around so public items are on top.
 * Constructor no longer needs file list or pointer to dialers file.
 * Added functions readFile and makeNew to handle the reading of the file.
 *
 * Revision 1.2  1993/05/07  00:17:35  chris
 * All items are protected, none are private.
 * No longer pass Dialers to instantiate.
 *
 * Revision 1.1  1993/05/05  16:09:40  chris
 * Initial revision
 *
 *------------------------------------------------------------------------*/

#ifndef _DEVICES_H_
#define _DEVICES_H_

#include "SmartList.h"
#include "PathName.h"

class Package;

/*------------------------------------------------------------------------
 * Devices class
 *------------------------------------------------------------------------*/

class Devices : public LinkData, public SmartList
{
  public:
    Devices () { modified = 0; }
    Devices (PathName& filename);
    ~Devices () { }

    virtual void	addPackage (LinkData *le) {
	packages.add (le); callCallbacks (SmartList::OTHER); }
    LinkData*		findPackage (const char *name) {
	return packages.find (name); }
    LinkData*		firstPackage (void) {
	return packages.first (); }
    LinkData*		nextPackage (void) {
	return packages.next (); }
    void		removePackage (void) {
	packages.removeEntry (); callCallbacks (SmartList::OTHER); }
    void		removePackage (LinkData *ld) {
	packages.removeEntry (ld); callCallbacks (SmartList::OTHER); }
    void		save (PathName& filename);
    void		save (const char *name = NULL);

    int			wasModified (void);
    void		clearModified (void);
    int			error (void) { return _errno; }

    virtual void	add (LinkData *pe1) {
	modified = 1; SmartList::add (pe1); }
    virtual void	clean (void) {
	modified = 1; SmartList::clean (); }
    virtual void	addBefore (LinkData *pe1) {
	modified = 1; SmartList::addBefore (pe1); }
    virtual void	addAfter (LinkData *pe1) {
	modified = 1; SmartList::addAfter (pe1); }
    virtual void	removeEntry (void) {
	modified = 1; SmartList::removeEntry (); }
    virtual void	removeEntry (LinkData *d) {
	modified = 1; SmartList::removeEntry (d); }

  protected:
    int			modified;
    int			_errno;

    virtual void	create (PathName& filename);
    virtual LinkData*	makeNewDevice (const char *line);
    virtual void	deleteEntry (LinkData *e);
    void		readFile (PathName& filename);

    SmartList		packages;
};

#endif
