/*---*- C++ -*------------------------------------------------------------
 * $Id: DevicesWindow.h,v 2.11 1994/02/26 03:56:33 carlson Exp $
 *
 * Header for DevicesWindow class
 *
 * Revision History:
 *	$Log: DevicesWindow.h,v $
 * Revision 2.11  1994/02/26  03:56:33  carlson
 * Added place to keep the parent widget.  This is used by the dialogs.
 *
 * Revision 2.10  1994/02/18  18:48:37  chrisc
 * Added method to find the selected item from the list.
 * Added method to update the package and item list.
 * Added methods for handling delete button.
 *
 * Revision 2.9  1994/01/17  23:15:25  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.8  1994/01/04  20:19:43  carlson
 * Added prototype for new callback methods.
 *
 * Revision 2.7  1993/11/16  02:20:19  carlson
 * Added a method to check if modified.
 *
 * Revision 2.6  1993/11/15  21:25:12  carlson
 * Added current device being displayed.
 * Added callbacks when device is updated.
 *
 * Revision 2.5  1993/11/05  19:55:11  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.4  1993/10/12  00:49:59  carlson
 * Added prototype for dump method.
 *
 * Revision 2.3  1993/10/06  17:39:16  chris
 * Added declaration for 'save' method.
 *
 * Revision 2.2  1993/08/12  00:59:29  chris
 * Added inline method for setting the defaults after instantiation.
 *
 * Revision 2.1  1993/07/29  01:05:19  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _DEVICESWINDOW_H_
#define _DEVICESWINDOW_H_

#include <Vk/VkComponent.h>

#include "SmartList.h"
#include "PathName.h"
#include "Device.h"
#include "Devices.h"

class DeviceWindow;

class DevicesWindow : public VkComponent
{
  public:
    DevicesWindow (Widget parent, Widget device_parent, Devices *devs,
		   SmartList *dlrs = NULL);
    ~DevicesWindow ();

    void		setDefaults (SmartList *dlrs) { dlrList = dlrs; }
    void		save (void);
    int			wasModified (void) { return devices->wasModified (); }

    virtual const char*	className () { return "DevicesWindow"; }
    virtual void	addPackage (LinkData *le);
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    Widget		_parent;
    Devices		*devices;
    SmartList		*dlrList;
    Widget		parent_for_device;
    Widget		pkg_list;
    Widget		device_list;
    DeviceWindow	*current;	// Current DeviceWindow displayed
    Device		*c_dev;		// Device displayed in current

    Device*		getSelected (char *);
    void		updateLists (void);

    static void		selectCallback (Widget, XtPointer, XtPointer);
    static void		devChangedCallback (Device *, Device::Field, void *);
    static void		listChangedCallback (SmartList*, SmartList::ACTION,
					     void*);
    static void		newCallback (Widget, XtPointer, XtPointer);
    static void		deleteCallback (Widget, XtPointer, XtPointer);

    void		select_cb (char *);
    void		devChanged (Device *, Device::Field);
    void		new_cb ();
    void		delete_cb ();
};

#endif
