/*------------------------------------------------------------------------
 * $Id: Dialcode.c++,v 2.6 1993/11/16 18:03:03 carlson Exp $
 *
 * Methods for Dialcode classes
 *
 * Revision History:
 *	$Log: Dialcode.c++,v $
// Revision 2.6  1993/11/16  18:03:03  carlson
// Include systemfiles.h and we get to remove prototype for gettoken.
// Replaced COPY macro with better one used in Host and Device.
// Added support for modified flag.
//
// Revision 2.5  1993/10/22  00:25:55  carlson
// Added removeChangeCallback method.
//
// Revision 2.4  1993/10/18  19:58:45  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/10/06  18:27:46  chris
// Added method for generating a line to be put back into the Dialcodes file.
//
// Revision 2.2  1993/08/11  18:55:06  chris
// Added destructor.
// Added methods for setting the script.
// Added callback capability for when script is changed.
//
// Revision 2.1  1993/07/28  15:26:35  chris
// Initial revision.
// Broken out from Dialcodes.c++.
//
 *------------------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>
#include <iostream.h>

#include "Dialcode.h"
#include "systemfiles.h"

#define COPY(from,to)	(to) = Copy (from)
#define DELETE(field)	if (field) delete [] field

/*========================================================================
 * Dialcode class
 *========================================================================*/

/*------------------------------------------------------------------------
 * Dialcode::Dialcode \(const char *\)
 *------------------------------------------------------------------------*/

Dialcode::Dialcode (const char *line)
{
    char	*cptr;
    char	*newline, *nexttoken;

    callbacks = NULL;
    modified = 0;

    /*----
     * Make a duplicate of the line so we can edit it without screwing
     * up the original.
     *----*/

    newline = new char [strlen (line) + 1];
    nexttoken = newline;
    strcpy (newline, line);

    cptr = gettoken (&nexttoken, " \t\n");
    setKey (cptr);

    cptr = gettoken (&nexttoken, "\n");
    COPY (cptr, code);

    delete [] newline;
}

/*------------------------------------------------------------------------
 * Dialcode::~Dialcode \(\)
 *------------------------------------------------------------------------*/

Dialcode::~Dialcode ()
{
    DELETE (code);
}

/*------------------------------------------------------------------------
 * Dialcode::line \(void\)
 *------------------------------------------------------------------------*/

char *Dialcode::line (void)
{
    static char		buffer[256];

    sprintf (buffer, "%s %s", theKey (), code);
    return buffer;
}

/*------------------------------------------------------------------------
 * Dialcode::setScript \(char *\)
 *------------------------------------------------------------------------*/

void Dialcode::setScript (char *ns)
{
    callbackStruct	*cbs;

    if (code)
	DELETE (code);

    COPY (ns, code);
    modified = 1;

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
		(*((callbackFunction)cbs->cbf))(this, 0, cbs->clientData);
}

/*------------------------------------------------------------------------
 * Dialcode::addChangeCallback \(callbackFunction *, void *\)
 *------------------------------------------------------------------------*/

void Dialcode::addChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp;

    cbs = new callbackStruct;
    cbs->cbf = cbf;
    cbs->clientData = cd;
    cbs->next = NULL;

    if (callbacks)
    {
	for (cbstemp = callbacks; cbstemp->next; cbstemp = cbstemp->next)
	    ;
	cbstemp->next = cbs;
    }
    else
	callbacks = cbs;
}

/*------------------------------------------------------------------------
 * Dialcode::removeChangeCallback \(callbackFunction, void *\)
 *------------------------------------------------------------------------*/

void Dialcode::removeChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp;

    if (callbacks)
    {
	for (cbs = NULL, cbstemp = callbacks; cbstemp; cbstemp = cbstemp->next)
	{
	    if ((cbstemp->cbf == cbf) && (cbstemp->clientData == cd))
	    {
		if (cbs)
		    cbs->next = cbstemp->next;
		else
		    callbacks = cbstemp->next;
		delete cbstemp;
		break;
	    }
	    cbs = cbstemp;
	}
    }
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * Dialcode::dump \(char *\)
 *------------------------------------------------------------------------*/

void Dialcode::dump (char *indent)
{
    char		*nextIndent;
    int			i;
    callbackStruct	*cb;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "Dialcode dump (" << this << "):" << endl;
    cout << indent << "  dialcode  = " << theKey () << endl;
    cout << indent << "  code      = " << code << endl;
    cout << endl;
    cout << indent << "  Callbacks:" << endl;
    for (cb = callbacks; cb; cb = cb->next)
    {
	cout << indent << "  . function = " << cb->cbf << endl;
	cout << indent << "    data     = " << cb->clientData << endl;
    }
}
#endif
