/*------------------------------------------------------------------------
 * $Id: DialcodeWindow.c++,v 2.7 1994/01/20 20:17:40 carlson Exp $
 *
 * Methods for DialcodeWindow class.
 *
 * Revision History:
 *	$Log: DialcodeWindow.c++,v $
// Revision 2.7  1994/01/20  20:17:40  carlson
// Modified to support new kind of help facility.
//
// Revision 2.6  1993/11/16  18:39:50  carlson
// Added help callbacks to widgets.
//
// Revision 2.5  1993/10/22  00:33:21  carlson
// Added destructor that removes any callbacks set up.
//
// Revision 2.4  1993/10/18  19:58:46  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/09/28  13:33:14  chris
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
//
// Revision 2.2  1993/08/11  18:55:06  chris
// Added callback for when Dialcode class is changed.
// Added callback for when text field is changed.
//
// Revision 2.1  1993/07/29  20:39:35  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "DialcodeWindow.h"
#include "Dialcode.h"
#include "HelpDialog.h"

#include <iostream.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/List.h>

/*------------------------------------------------------------------------
 * DialcodeWindow::dialcodeChangedCallback \(Dialcode *, void *\)
 *------------------------------------------------------------------------*/

void DialcodeWindow::dialcodeChangedCallback (Dialcode *h, int data,
					      void *clientData)
{
    DialcodeWindow		*dw;

    dw = (DialcodeWindow *)clientData;
    dw->dialcodeChanged (data, h);
}

/*------------------------------------------------------------------------
 * DialcodeWindow::dialcodeChanged \(Dialcode *\)
 *------------------------------------------------------------------------*/

void DialcodeWindow::dialcodeChanged (int, Dialcode *)
{
    XtVaSetValues (w_code, XmNvalue, (char *)dialcode->script (), NULL);
}

/*------------------------------------------------------------------------
 * DialcodeWindow::codeChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DialcodeWindow::codeChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    DialcodeWindow		*hw;

    hw = (DialcodeWindow *)clientData;
    hw->codeChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * DialcodeWindow::codeChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DialcodeWindow::codeChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_code, XmNvalue, &str, NULL);
    dialcode->setScript (str);
}

/*------------------------------------------------------------------------
 * DialcodeWindow::DialcodeWindow (const char *)
 *------------------------------------------------------------------------*/

DialcodeWindow::DialcodeWindow (Widget parent, Dialcode *dlcd)
: VkComponent (dlcd->theKey ())
{
    Arg		wargs[10];
    int		n;
    XmString	xmstr;
    Widget	label;
    char	wait[2];

    wait[1] = '\0';
    dialcode = dlcd;

    /*----
     * Create a row/column widget to hold labels and text for dialcode
     * data.  This way, everything should line up properly.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, 2); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    _baseWidget = XmCreateRowColumn (parent, "dialcodeWindow", wargs, n);
    installDestroyHandler ();

    /*----
     * Create information.
     * For the RowColumn widget, we have to give all the labels
     * first, then all the text portions.
     *----*/

    n = 0;
    label = XmCreateLabel (_baseWidget, "dlcdNameLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlcd_name_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "codeLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlcd_code_help");

/*---
 *---*/

    n = 0;
    xmstr = XmStringCreateSimple ((char *)dialcode->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    label = XmCreateLabel (_baseWidget, "dlcdNameText", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlcd_name_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, dialcode->script ()); n++;
    w_code = XmCreateTextField (_baseWidget, "codeText", wargs, n);
    XtManageChild (w_code);

    XtAddCallback (w_code, XmNactivateCallback, codeChangedCallback,
		   this);
    XtAddCallback (w_code, XmNhelpCallback, aHelpCallback, "dlcd_code_help");

    dialcode->addChangeCallback (DialcodeWindow::dialcodeChangedCallback,
				 (void *)this);
}

/*------------------------------------------------------------------------
 * DialcodeWindow::~DialcodeWindow \(\)
 *------------------------------------------------------------------------*/

DialcodeWindow::~DialcodeWindow ()
{
    dialcode->removeChangeCallback (DialcodeWindow::dialcodeChangedCallback,
				    (void *)this);
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * DialcodeWindow::dump \(char *\)
 *------------------------------------------------------------------------*/

void DialcodeWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "DialcodeWindow dump (" << this << "):" << endl;
    cout << indent << "  dialcode = " << dialcode << endl;

    if (dialcode)
	dialcode->dump (nextIndent);
}
#endif
