/*------------------------------------------------------------------------
 * $Id: Dialcodes.c++,v 2.7 1994/02/28 22:36:34 carlson Exp $
 *
 * Methods for Dialcodes classes
 *
 * Revision History:
 *	$Log: Dialcodes.c++,v $
// Revision 2.7  1994/02/28  22:36:34  carlson
// Added keeping track of errors with I/O.
// Changed the way that I create the temporary file name because tempnam must
//   make its name in /usr/var/tmp which is frequently on a different file
//   system and won't allow a simple rename.
// Added code to change the mode of the file created to be the same as the
//   original.
//
// Revision 2.6  1994/02/10  22:55:15  chrisc
// Removed package access functions and made them inline.
//
// Revision 2.5  1994/01/18  00:38:14  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.4  1993/11/16  18:16:28  carlson
// Removed unnecessary include of malloc.h.
// Include systemfiles.h and we get to remove prototypes for read_line and
//   get_line.
// Added code to support modified flag.
// Added methods to check and clear modified flag.
//
// Revision 2.3  1993/11/05  19:55:12  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.2  1993/10/06  18:29:41  chris
// Modified all parameters for PathName to use references.
// Added two methods for saving the file.
//
// Revision 2.1  1993/07/27  23:59:53  chris
// Updated to only include class for Dialcodes.
// Made to follow new format, using PathName as input.
//
// Revision 1.2  1993/05/17  05:42:53  chris
// No longer require a file name with the constructor.  Functions are
//   now done in makeNew and readFile methods.
//
// Revision 1.1  1993/05/05  19:35:10  chris
// Initial revision
//
 *------------------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "Dialcodes.h"
#include "Dialcode.h"
#include "systemfiles.h"

/*========================================================================
 * Dialcodes class
 *========================================================================*/

/*------------------------------------------------------------------------
 * Dialcodes::Dialcodes (PathName&)
 *------------------------------------------------------------------------*/

Dialcodes::Dialcodes (PathName& filename)
: LinkData (), SmartList ()
{
    create (filename);
    readFile (filename);
    modified = 0;
}

/*------------------------------------------------------------------------
 * Dialcodes::create (PathName&)
 *------------------------------------------------------------------------*/

void Dialcodes::create (PathName& filename)
{
    if (top)
	clean ();

    setKey ((char *)filename.name ());
}

/*------------------------------------------------------------------------
 * Dialcodes::readFile (PathName&)
 *------------------------------------------------------------------------*/

void Dialcodes::readFile (PathName& filename)
{
    LinkData	*dialcode;
    ifstream	*dialcode_file;
    char	*cptr;

    _errno = 0;
    dialcode_file = new ifstream (filename.name ());
    if (dialcode_file->fail ())
    {
	_errno = errno;
	return;
    }

    while (!dialcode_file->eof ())
    {
	cptr = read_line (dialcode_file);
	if (*cptr)
	{
	    if ((*cptr == ' ') || (*cptr == '\t') || (*cptr == '\n'))
		continue;
	    dialcode = makeNewDialcode (cptr);
	    add (dialcode);
	}
    }

    delete dialcode_file;
}

/*------------------------------------------------------------------------
 * Dialcodes::save (PathName&)
 *------------------------------------------------------------------------*/

void Dialcodes::save (PathName& filename)
{
    ifstream		*ifile;
    ofstream		*ofile;
    char		*tmpfname, *cptr, *tptr;
    Dialcode		*dialcodeline, *dialcode;
    int			flag;
    struct stat		statBuf;

    _errno = 0;
    if (!wasModified ())
	return;

    tmpfname = new char [(filename.directory ()).length () + 14];
    strcpy (tmpfname, (filename.directory ()).name ());
    strcat (tmpfname, "/DialcodessXXXXXX");
    mktemp (tmpfname);
    ofile = new ofstream (tmpfname);
    if (ofile->fail ())
    {
	_errno = errno;
	return;
    }

    ifile = new ifstream (filename.name ());

    /*----
     * Scan through the input file copying comments and blank lines
     * until we come to a legitimate dialcode entry.
     *----*/

    while (!ifile->eof ())
    {
	cptr = get_line (ifile);
	if (ifile->eof () && (*cptr == '\0'))
	    break;

	if ((*cptr == '#') || (*cptr == '\0'))
	    *ofile << cptr << endl;
	else
	{
	    /*----
	     * Create an instance of a Dialcode class using the entry
	     * found so we can use it for comparisons.
	     *----*/

	    dialcodeline = new Dialcode (cptr);

	    /*----
	     * Search our list to see if we have an entry with the
	     * same dialcode name.  If we do, put *all* of our entries
	     * with the same dialcode name out here.
	     *
	     * Note:  We use the ListEntry flag to indicate those
	     * entries that we\'ve written.  We assume all the
	     * entries are currently set to 0.  We also assume that
	     * if an entry matches the name but the flag is set,
	     * then all entries we have internally have been written.
	     *----*/

	    dialcode = (Dialcode *)find (dialcodeline->theKey ());
	    if (dialcode)
	    {
		if (lastFlag ())
		    continue;
		tptr = dialcode->line ();
		*ofile << tptr << endl;
		setLastFlag (1);
		while ((dialcode = (Dialcode *)next ()) &&
		       (strcmp (dialcode->theKey (),
				dialcodeline->theKey ()) == 0))
		{
		    tptr = dialcode->line ();
		    *ofile << tptr << endl;
		    setLastFlag (1);
		}
	    }

	    delete dialcodeline;
	}
    }
    ifile->close ();

    /*----
     * Scan through the list of dialcodes we have and write any that
     * have not been written yet.  While we're scanning, clear all
     * the flags.
     *----*/

    for (dialcode = (Dialcode *)first (); dialcode;
	 dialcode = (Dialcode *)next ())
    {
	flag = lastFlag ();
	setLastFlag (0);
	if (flag)
	    continue;
	tptr = dialcode->line ();
	*ofile << tptr << endl;
    }
    ofile->close ();

    /*----
     * Now we're done.
     * Find out the mode settings of the original file.
     * Make sure we have write access to the file.
     * Rename the new file over the old file.
     * Then set the mode settings of the new file back to what they were.
     *----*/

    if (stat (filename.name (), &statBuf))
	_errno = errno;

    if (!_errno)
	if (chmod (filename.name (),
		   statBuf.st_mode | (S_IWUSR | S_IWGRP | S_IWOTH)))
	    _errno = errno;

    if (!_errno)
	if (rename (tmpfname, filename.name ()))
	    _errno = errno;

    if (!_errno)
    {
	if (chmod (filename.name (), statBuf.st_mode))
	    _errno = errno;
	clearModified ();
    }

    delete [] tmpfname;
}

/*------------------------------------------------------------------------
 * Dialcodes::save (const char*)
 *------------------------------------------------------------------------*/

void Dialcodes::save (const char *fname)
{
    PathName		filename;

    if (fname)
	filename = (char *)fname;
    else
	filename = (char *)theKey ();

    save (filename);
}

/*------------------------------------------------------------------------
 * Dialcodes::makeNewDialcode (const char *)
 *------------------------------------------------------------------------*/

LinkData* Dialcodes::makeNewDialcode (const char *line)
{
    Dialcode	*dialcode;

    dialcode = new Dialcode (line);

    return dialcode;
}

/*------------------------------------------------------------------------
 * Dialcodes::deleteEntry (LinkData *)
 *------------------------------------------------------------------------*/

void Dialcodes::deleteEntry (LinkData *e)
{
    delete e;
}

/*------------------------------------------------------------------------
 * Dialcodes::wasModified (void)
 *------------------------------------------------------------------------*/

int Dialcodes::wasModified (void)
{
    Dialcode		*dlcd;

    if (modified == 0)
    {
	for (dlcd = (Dialcode *)first (); dlcd; dlcd = (Dialcode *)next ())
	{
	    if (dlcd->wasModified ())
	    {
		modified = 1;
		break;
	    }
	}
    }

    return modified;
}

/*------------------------------------------------------------------------
 * Dialcodes::clearModified (void)
 *------------------------------------------------------------------------*/

void Dialcodes::clearModified (void)
{
    Dialcode		*dlcd;

    for (dlcd = (Dialcode *)first (); dlcd; dlcd = (Dialcode *)next ())
	dlcd->clearModified ();

    modified = 0;
}
