/*------------------------------------------------------------------------
 * $Id: DialcodesSetup.c++,v 2.2 1994/02/28 22:30:07 carlson Exp $
 *
 * Methods for DialcodesSetup class.
 *
 * Revision History:
 *	$Log: DialcodesSetup.c++,v $
// Revision 2.2  1994/02/28  22:30:07  carlson
// When doing a save, check if an error occurred and generate a dialog if it does.
//
// Revision 2.1  1994/02/23  23:47:24  carlson
// Cleaned up some functions to be more like all the rest.
// Added menubar and menu options.
// Keep track of Dialcode that we have open.
//
// Revision 1.2  1994/02/18  19:02:07  chrisc
// Needed to null out the current variable whenever I deleted the class.
//
// Revision 1.1  1994/02/17  16:57:11  chrisc
// Initial revision
//
 *------------------------------------------------------------------------*/

#include "DialcodesSetup.h"
#include "DialcodeWindow.h"
#include "Dialcode.h"
#include "HelpDialog.h"
#include "SmartList.h"

#include <Vk/VkPromptDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>
#include <Vk/VkHelpPane.h>
#include <Vk/VkMenuBar.h>

#include <iostream.h>
#include <stdio.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/SeparatoG.h>

/*------------------------------------------------------------------------
 * DialcodesSetup::selectCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialcodesSetup::selectCallback (Widget, XtPointer clientData,
				     XtPointer callData)
{
    XmListCallbackStruct*	cbStruct;
    DialcodesSetup*		sw;
    char			*text;

    cbStruct = (XmListCallbackStruct *)callData;
    sw = (DialcodesSetup *)clientData;

    XmStringGetLtoR (cbStruct->item, XmFONTLIST_DEFAULT_TAG, &text);
    sw->select_cb (text);
}

/*------------------------------------------------------------------------
 * DialcodesSetup::select_cb (char *)
 *------------------------------------------------------------------------*/

void DialcodesSetup::select_cb (char *text)
{
    Dialcode		*dlcd;
    char		line[100];

    dlcd = (Dialcode *)_dialcodes->find (text);
    if (!dlcd)
    {
	strcpy (line, "Unable to find dialcode ");
	strcat (line, text);
	strcat (line, " in list.");
	theErrorDialog->postAndWait (line);
	return;
    }

    if (current)
    {
	delete current;
	current = NULL;
    }

    c_dlcd = dlcd;
    current = new DialcodeWindow (dialcode, dlcd);
    current->show ();
}

/*------------------------------------------------------------------------
 * DialcodesSetup::newCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialcodesSetup::newCallback (Widget, XtPointer clientData, XtPointer)
{
    DialcodesSetup*		sw;

    sw = (DialcodesSetup *)clientData;
    sw->new_cb ();
}

/*------------------------------------------------------------------------
 * DialcodesSetup::new_cb ()
 *------------------------------------------------------------------------*/

void DialcodesSetup::new_cb ()
{
    VkPromptDialog::VkDialogReason	reason;
    XmString				xmstr;

    reason = thePromptDialog->postAndWait ("Enter name of dialcode to add:");
    if (reason == VkPromptDialog::OK)
    {
	if (current)
	    delete current;

	c_dlcd = new Dialcode (thePromptDialog->text ());
	_dialcodes->add (c_dlcd);
	xmstr = XmStringCreateSimple ((char *)c_dlcd->theKey ());
	XmListAddItemUnselected (dialcode_list, xmstr, 2);
	XmListSelectPos (dialcode_list, 2, False);
	XmStringFree (xmstr);
	current = new DialcodeWindow (dialcode, c_dlcd);
	current->show ();
    }
}

/*----------------------------------------------------------------------
 * deleteCallback
 *
 * Static method called by X when the "Delete" button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void DialcodesSetup::deleteCallback (Widget, XtPointer clientData, XtPointer)
{
    DialcodesSetup*		dlcw;

    dlcw = (DialcodesSetup *)clientData;
    dlcw->delete_cb ();
}

/*----------------------------------------------------------------------
 * delete_cb
 *
 * Method called by DialcodesSetup::deleteCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void DialcodesSetup::delete_cb ()
{
    int			*position_list, position_count;
    XmString		*xmstr;
    char		*text, line[80];
    Dialcode		*aDialcode;

    if (!XmListGetSelectedPos (dialcode_list, &position_list, &position_count))
    {
	theErrorDialog->postAndWait ("You need to select the\ndialcode "
				     "to delete.");
	return;
    }
    XFree (position_list);

    XtVaGetValues (dialcode_list, XmNselectedItems, &xmstr, NULL);
    XmStringGetLtoR (xmstr[0], XmFONTLIST_DEFAULT_TAG, &text);
    aDialcode = (Dialcode *)_dialcodes->find (text);

    if (!aDialcode)
    {
	strcpy (line, "Unable to find dialcode ");
	strcat (line, text);
	strcat (line, ".\nReport this to SGI.");
	theErrorDialog->postAndWait (line);
	return;
    }

    strcpy (line, "Are you sure you want\nto delete ");
    strcat (line, text);
    strcat (line, "?");
    if (theQuestionDialog->postAndWait (line) == VkDialogManager::OK)
    {
	if (aDialcode == c_dlcd)
	{
	    delete current;
	    current = NULL;
	}

	/*----
	 * Removing the entry will automatically remove it from
	 * the list window because of callbacks we have set
	 * up.
	 *----*/

	_dialcodes->removeEntry (aDialcode);
    }
}

/*----------------------------------------------------------------------
 * saveCallback
 *
 * Static method called by X when the "Save" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void DialcodesSetup::saveCallback (Widget, XtPointer clientData, XtPointer)
{
    DialcodesSetup *obj = (DialcodesSetup *) clientData;
    obj->save_cb ();
}

/*----------------------------------------------------------------------
 * save_cb
 *
 * Method called by DialcodesSetup::saveCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void DialcodesSetup::save_cb ()
{
    char		line[128];

    _dialcodes->save ();
    if (_dialcodes->error ())
    {
	strcpy (line, "*** Error saving file:\n");
	strcat (line, strerror (_dialcodes->error ()));
	theErrorDialog->postAndWait (line);
    }
}

/*----------------------------------------------------------------------
 * printCallback
 *
 * Static method called by X when the "Print" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void DialcodesSetup::printCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    DialcodesSetup *obj = (DialcodesSetup *) clientData;
    obj->print_cb ();
}

/*----------------------------------------------------------------------
 * print_cb
 *
 * Method called by DialcodesSetup::printCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void DialcodesSetup::print_cb ()
{
#ifdef DEBUG
    cout << "print_cb callback" << "\n";
#endif
}

/*----------------------------------------------------------------------
 * quitCallback
 *
 * Static method called by X when the "Quit" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void DialcodesSetup::quitCallback (Widget, XtPointer clientData,
				 XtPointer)
{
    DialcodesSetup			*dlcw;

    dlcw = (DialcodesSetup *)clientData;
    dlcw->quit_cb ();
}

/*------------------------------------------------------------------------
 * DialcodesSetup::quit_cb ()
 *------------------------------------------------------------------------*/

void DialcodesSetup::quit_cb ()
{
    if (_dialcodes->wasModified ())
    {
	if (theQuestionDialog->postAndWait ("File was modified\nReally quit?")
	    == VkDialogManager::OK)
	    delete this;
    }
    else
	delete this;
}

/*------------------------------------------------------------------------
 * DialcodesSetup::onWindowCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialcodesSetup::onWindowCallback (Widget, XtPointer clientData,
					 XtPointer)
{
    DialcodesSetup		*dlcw;

    dlcw = (DialcodesSetup *)clientData;
    dlcw->onWindow_cb ();
}

/*------------------------------------------------------------------------
 * DialcodesSetup::onWindow_cb ()
 *------------------------------------------------------------------------*/

void DialcodesSetup::onWindow_cb ()
{
    theHelpDialog->displayHelp ("DialcodesSetupOW");
}

/*------------------------------------------------------------------------
 * DialcodesSetup::~DialcodesSetup ()
 *------------------------------------------------------------------------*/

DialcodesSetup::~DialcodesSetup ()
{
    if (current)
	delete current;
}

/*------------------------------------------------------------------------
 * DialcodesSetup::DialcodesSetup (PathName &)
 *------------------------------------------------------------------------*/

DialcodesSetup::DialcodesSetup (PathName &file)
: VkWindow (file.tail ())
{
    _dialcodes = new Dialcodes (file);
    current = NULL;
    setTitle ("Dialcodes");
}

/*------------------------------------------------------------------------
 * DialcodesSetup::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget DialcodesSetup::setUpInterface (Widget parent)
{
    Arg			wargs[15];
    int			n;
    XmString		xmstr;
    LinkData		*le;
    Widget		top;
    Widget		temp, dialcode_list_label;
    Widget		form1;
    VkMenuBar		*menubar;
    VkHelpPane		*helpMenu;

    XtVaSetValues (XtParent (parent),
		   XmNallowShellResize, True,
		   NULL);

    /*----
     * Create a rowColumn into which to put all the stuff.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_TIGHT); n++;
    top = XmCreateRowColumn (parent, "dialcodesWindow", wargs, n);

    /*----
     * Create a form to put the list and buttons in.
     *----*/

    n = 0;
    form1 = XmCreateForm (top, "dlcdlForm", wargs, n);
    XtManageChild (form1);

    /*----
     * Provide a list of dialcodes to select from.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    dialcode_list_label = XmCreateLabel (form1, "dialcodeListLabel", wargs, n);
    XtManageChild (dialcode_list_label);

    XtAddCallback (dialcode_list_label, XmNhelpCallback, aHelpCallback,
		   "dlcdw_help");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialcode_list_label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNlistSizePolicy, XmVARIABLE); n++;
    XtSetArg (wargs[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    XtSetArg (wargs[n], XmNvisibleItemCount, 4); n++;
    dialcode_list = XmCreateScrolledList (form1, "dialcodeList", wargs, n);
    XtManageChild (dialcode_list);

    for (le = _dialcodes->first (); le; le = _dialcodes->next ())
    {
	xmstr = XmStringCreateSimple ((char *)le->theKey ());
	XmListAddItemUnselected (dialcode_list, xmstr, 0);
	XmStringFree (xmstr);
    }

    XtAddCallback (dialcode_list, XmNsingleSelectionCallback, selectCallback,
		   this);
    XtAddCallback (dialcode_list, XmNhelpCallback, aHelpCallback,
		   "dlcdw_help");

    /*----
     * Create "New" button for creating new dialcodes.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialcode_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form1, "new", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, newCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dlcdw_help");

    /*----
     * Create "Delete" button for creating new dialcodes.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialcode_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, temp); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form1, "delete", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, deleteCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dlcdw_help");

    n = 0;
    XtSetArg (wargs[n], XmNorientation, XmVERTICAL); n++;
    temp = XmCreateSeparatorGadget (top, "sep1", wargs, n);
    XtManageChild (temp);

    n = 0;
    dialcode = XmCreateForm (top, "dialcode", wargs, n);
    XtManageChild (dialcode);

    /*----
     * Add 'Files' menu pane.
     *----*/

    fileMenuPane = addMenuPane ("files");
    temp = fileMenuPane->baseWidget ();
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dcw_file_help");

    fileMenuPane->addAction ("save", saveCallback, this);
    fileMenuPane->addAction ("print", printCallback, this);
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("quit", quitCallback, this);

    /*----
     * Add items to the Help menu.
     *----*/

    menubar = menu ();
    helpMenu = menubar->helpPane ();
    helpMenu->addSeparator ();
    helpMenu->addAction ("window", onWindowCallback, this);

    /*----
     * Deactivate menu items that don't work yet.
     *----*/

    fileMenuPane->deactivateItem ("print");

    return top;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * DialcodesSetup::dump (char *)
 *------------------------------------------------------------------------*/

void DialcodesSetup::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "DialcodesSetup dump (" << this << "):" << endl;
    cout << indent << "  dialcodes = " << _dialcodes << endl;
    cout << indent << "  current   = " << current << endl;

    if (current)
	current->dump (nextIndent);
}
#endif

/*------------------------------------------------------------------------
 * DialcodesSetup::save (void)
 *------------------------------------------------------------------------*/

void DialcodesSetup::save (void)
{
    _dialcodes->save ();
}
