/*---*- C++ -*------------------------------------------------------------
 * $Id: Dialer.h,v 2.6 1993/11/16 17:42:16 carlson Exp $
 *
 * Header file describing the Dialer class.
 *
 * Revision History:
 *	$Log: Dialer.h,v $
 * Revision 2.6  1993/11/16  17:42:16  carlson
 * Added support for modified flag.
 * Cleaned up some.
 *
 * Revision 2.5  1993/10/22  00:25:55  carlson
 * Added prototype for removeChangeCallback method.
 *
 * Revision 2.4  1993/10/18  18:21:50  carlson
 * Added prototype for dump method.
 *
 * Revision 2.3  1993/10/06  18:07:03  chris
 * Added declaration for method for outputing a line.
 * Changed all protected members to be private.
 *
 * Revision 2.2  1993/08/11  16:22:55  chris
 * Added enum Field for identifying what element was changed when a
 *   callback is called.
 * Added callback capabilities.
 * Added methods for changing elements.
 * Changed names of elements.
 *
 * Revision 2.1  1993/07/27  23:20:32  chris
 * Initial revision.
 * Broken out from Dialers.h.
 *
 *------------------------------------------------------------------------*/

#ifndef _DIALER_H_
#define _DIALER_H_

#include "linkList.h"

/*------------------------------------------------------------------------
 * Dialer class
 *------------------------------------------------------------------------*/

class Dialer;
class Dialer : public LinkData
{
  public:
    enum Field { W4CARRIERCHAR, PAUSECHAR, CHATSCRIPT };
    typedef void (*callbackFunction)(Dialer*, Field, void*);

    Dialer (const char *line);
    ~Dialer ();

    const char		wait4CarrierChar (void) { return w4carrier; }
    const char		pauseChar (void) { return pause; }
    const char*		chat_script (void) { return chat; }
    char*		line (void);

    void		setWait4CarrierChar (char nw4);
    void		setPauseChar (char np);
    void		setChatScript (char *nc);

    int			wasModified (void) { return modified; }
    void		clearModified (void) { modified = 0; }

    void		addChangeCallback (callbackFunction ncbf,
					   void *clientData);
    void		removeChangeCallback (callbackFunction ncbf,
					      void *clientData);

    char*		expandChatScript (char *script = NULL);

#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    /*----
     * w4carrier	Character in the dial string that requests a wait
     *			for carrier or until register <S7> seconds.
     * pause		Character in the dial string that request a pause
     *			for register <S8> seconds.
     *----*/
    char		w4carrier;
    char		pause;
    char		*chat;

    int			modified;

    struct callbackStruct
    {
	callbackFunction	cbf;
	void			*clientData;
	callbackStruct		*next;
    };
    callbackStruct		*callbacks;
};

#endif
