/*------------------------------------------------------------------------
 * $Id: DialerWindow.c++,v 2.7 1994/01/20 20:17:39 carlson Exp $
 *
 * Methods for DialerWindow class.
 *
 * Revision History:
 *	$Log: DialerWindow.c++,v $
// Revision 2.7  1994/01/20  20:17:39  carlson
// Modified to support new kind of help facility.
//
// Revision 2.6  1993/11/16  18:39:50  carlson
// Added help callbacks to widgets.
//
// Revision 2.5  1993/10/22  00:33:21  carlson
// Added destructor that removes any callbacks set up.
//
// Revision 2.4  1993/10/14  23:45:23  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/09/28  13:40:17  chris
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
//
// Revision 2.2  1993/08/11  17:18:35  chris
// Added callbacks for when text fields are changed.
// Added callback for when an element in the Dialer class is changed.
// Changed code which directly accessed the elements in the Dialer class to
//   use methods to get the data.
//
// Revision 2.1  1993/07/29  02:08:23  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "DialerWindow.h"
#include "HelpDialog.h"

#include <iostream.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/List.h>

/*------------------------------------------------------------------------
 * DialerWindow::dialerChangedCallback \(Dialer *, Field, void *\)
 *------------------------------------------------------------------------*/

void DialerWindow::dialerChangedCallback (Dialer *d, Dialer::Field field,
					  void *clientData)
{
    DialerWindow		*dw;

    dw = (DialerWindow *)clientData;
    dw->dialerChanged (field, d);
}

/*------------------------------------------------------------------------
 * DialerWindow::dialerChanged \(Field, Dialer *\)
 *------------------------------------------------------------------------*/

void DialerWindow::dialerChanged (Dialer::Field field, Dialer *)
{
    char		str[2];

    str[1] = '\0';

    switch (field)
    {
      case Dialer::W4CARRIERCHAR:
	str[0] = dialer->wait4CarrierChar ();
	XtVaSetValues (w_wait7, XmNvalue, str, NULL);
	break;
      case Dialer::PAUSECHAR:
	str[0] = dialer->pauseChar ();
	XtVaSetValues (w_wait8, XmNvalue, str, NULL);
	break;
      case Dialer::CHATSCRIPT:
	XtVaSetValues (w_chat_script, XmNvalue, (char *)dialer->chat_script (),
		       NULL);
	break;
    }
}

/*------------------------------------------------------------------------
 * Dialer::wait7ChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DialerWindow::wait7ChangedCallback (Widget, XtPointer clientData,
					 XtPointer callData)
{
    DialerWindow		*dw;

    dw = (DialerWindow *)clientData;
    dw->wait7Changed ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * Dialer::wait8ChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DialerWindow::wait8ChangedCallback (Widget, XtPointer clientData,
					 XtPointer callData)
{
    DialerWindow		*dw;

    dw = (DialerWindow *)clientData;
    dw->wait8Changed ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * Dialer::chatChangedCallback \(Widget, XtPointer, XtPointer\)
 *------------------------------------------------------------------------*/

void DialerWindow::chatChangedCallback (Widget, XtPointer clientData,
					XtPointer callData)
{
    DialerWindow		*dw;

    dw = (DialerWindow *)clientData;
    dw->chatChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * Dialer::wait7Changed \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DialerWindow::wait7Changed (XmAnyCallbackStruct *)
{
    char		*str, *cptr;

    XtVaGetValues (w_wait7, XmNvalue, &str, NULL);
    for (cptr = str; *cptr && isspace (*cptr); cptr++);
    dialer->setWait4CarrierChar (*cptr);
}

/*------------------------------------------------------------------------
 * Dialer::wait8Changed \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DialerWindow::wait8Changed (XmAnyCallbackStruct *)
{
    char		*str, *cptr;

    XtVaGetValues (w_wait8, XmNvalue, &str, NULL);
    for (cptr = str; *cptr && isspace (*cptr); cptr++);
    dialer->setPauseChar (*cptr);
}

/*------------------------------------------------------------------------
 * Dialer::chatChanged \(XmAnyCallbackStruct *\)
 *------------------------------------------------------------------------*/

void DialerWindow::chatChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_chat_script, XmNvalue, &str, NULL);
    dialer->setChatScript (str);
}

/*------------------------------------------------------------------------
 * DialerWindow::DialerWindow (const char *)
 *------------------------------------------------------------------------*/

DialerWindow::DialerWindow (Widget parent, Dialer *dlr)
: VkComponent (dlr->theKey ())
{
    Arg		wargs[10];
    int		n;
    XmString	xmstr;
    Widget	label;
    char	wait[2];

    wait[1] = '\0';
    dialer = dlr;

    /*----
     * Create a row/column widget to hold labels and text for dialer
     * data.  This way, everything should line up properly.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, 2); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    _baseWidget = XmCreateRowColumn (parent, "dialerWindow", wargs, n);
    installDestroyHandler ();

    /*----
     * Create information.
     * For the RowColumn widget, we have to give all the labels
     * first, then all the text portions.
     *----*/

    n = 0;
    label = XmCreateLabel (_baseWidget, "dlrNameLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlr_name_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "waits7Label", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlr_w7_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "waits8Label", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlr_w8_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "chatLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlr_cht_help");

/*---
 *---*/

    n = 0;
    xmstr = XmStringCreateSimple ((char *)dialer->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    label = XmCreateLabel (_baseWidget, "dlrNameText", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlr_name_help");

    n = 0;
    wait[0] = dialer->wait4CarrierChar ();
    XtSetArg (wargs[n], XmNvalue, wait); n++;
    w_wait7 = XmCreateTextField (_baseWidget, "waits7Text", wargs, n);
    XtManageChild (w_wait7);

    XtAddCallback (w_wait7, XmNactivateCallback, wait7ChangedCallback,
		   this);
    XtAddCallback (w_wait7, XmNhelpCallback, aHelpCallback, "dlr_w7_help");

    n = 0;
    wait[0] = dialer->pauseChar ();
    XtSetArg (wargs[n], XmNvalue, wait); n++;
    w_wait8 = XmCreateTextField (_baseWidget, "waits8Text", wargs, n);
    XtManageChild (w_wait8);

    XtAddCallback (w_wait8, XmNactivateCallback, wait8ChangedCallback,
		   this);
    XtAddCallback (w_wait8, XmNhelpCallback, aHelpCallback, "dlr_w8_help");

    n = 0;
    XtSetArg (wargs[n], XmNvalue, dialer->chat_script ()); n++;
    w_chat_script = XmCreateTextField (_baseWidget, "chatText", wargs, n);
    XtManageChild (w_chat_script);

    XtAddCallback (w_chat_script, XmNactivateCallback, chatChangedCallback,
		   this);
    XtAddCallback (w_chat_script, XmNhelpCallback, aHelpCallback,
		   "dlr_cht_help");

    dialer->addChangeCallback (DialerWindow::dialerChangedCallback,
			       (void *)this);
}

/*------------------------------------------------------------------------
 * DialerWindow::~DialerWindow \(\)
 *------------------------------------------------------------------------*/

DialerWindow::~DialerWindow ()
{
    dialer->removeChangeCallback (DialerWindow::dialerChangedCallback,
				  (void *)this);
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * DialerWindow::dump \(char *\)
 *------------------------------------------------------------------------*/

void DialerWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "DialerWindow dump (" << this << "):" << endl;
    cout << indent << "  dialer = " << dialer << endl;
    cout << indent << "  address of callback = " << &dialerChangedCallback
	 << endl;

    dialer->dump (nextIndent);
}
#endif
