/*------------------------------------------------------------------------
 * $Id: Dialers.c++,v 2.7 1994/02/28 22:36:34 carlson Exp $
 *
 * Methods for handling dialers class.
 *
 * Revision History:
 *	$Log: Dialers.c++,v $
// Revision 2.7  1994/02/28  22:36:34  carlson
// Added keeping track of errors with I/O.
// Changed the way that I create the temporary file name because tempnam must
//   make its name in /usr/var/tmp which is frequently on a different file
//   system and won't allow a simple rename.
// Added code to change the mode of the file created to be the same as the
//   original.
//
// Revision 2.6  1994/02/10  22:55:26  chrisc
// Removed package access functions and made them inline.
//
// Revision 2.5  1994/01/18  00:38:14  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.4  1993/11/16  18:16:28  carlson
// Removed unnecessary include of malloc.h.
// Include systemfiles.h and we get to remove prototypes for read_line and
//   get_line.
// Updated ifdef'd out code (in case if I want to include it).
// Added code to support modified flag.
// Added methods to check and clear modified flag.
//
// Revision 2.3  1993/11/05  20:09:30  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.2  1993/10/06  18:07:03  chris
// Modified all parameters for PathName to use references.
// Added two methods for saving the file.
//
// Revision 2.1  1993/07/27  23:59:53  chris
// Updated to only include class for Dialers.
// Made to follow new format, using PathName as input.
//
// Revision 1.2  1993/05/17  05:59:53  chris
// No longer pass a file list to construct Dialers class.
// Added functions makeNew and readFile to perform the reading of the file.
//
// Revision 1.1  1993/05/05  19:35:10  chris
// Initial revision
//
 *------------------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "Dialers.h"
#include "Dialer.h"
#include "systemfiles.h"

/*========================================================================
 * Dialers class
 *========================================================================*/

/*------------------------------------------------------------------------
 * Dialers::Dialers (PathName&)
 *------------------------------------------------------------------------*/

Dialers::Dialers (PathName& filename) 
: LinkData (), SmartList ()
{
    create (filename);
    readFile (filename);
    modified = 0;
}

/*------------------------------------------------------------------------
 * Dialers::create (PathName&)
 *------------------------------------------------------------------------*/

void Dialers::create (PathName& filename)
{
    if (top)
	clean ();

    setKey ((char *)filename.name ());
}

/*------------------------------------------------------------------------
 * Dialers::readFile (PathName&)
 *------------------------------------------------------------------------*/

void Dialers::readFile (PathName& filename)
{
    LinkData	*dialer;
    ifstream	*dialer_file;
    char	*cptr;

    _errno = 0;
#if 0
    dialer = new makeNewDialer ("212 ==--");
    add (dialer);
    dialer = new makeNewDialer ("801 ==--");
    add (dialer);
#endif

    dialer_file = new ifstream (filename.name ());
    if (dialer_file->fail ())
    {
	_errno = errno;
	return;
    }

    while (!dialer_file->eof ())
    {
	cptr = read_line (dialer_file);
	if (*cptr)
	{
	    if ((*cptr == ' ') || (*cptr == '\t') || (*cptr == '\n'))
		continue;
	    dialer = makeNewDialer (cptr);
	    add (dialer);
	}
    }

    delete dialer_file;
}

/*------------------------------------------------------------------------
 * Dialers::save (PathName&)
 *------------------------------------------------------------------------*/

void Dialers::save (PathName& filename)
{
    ifstream		*ifile;
    ofstream		*ofile;
    char		*tmpfname, *cptr, *tptr;
    Dialer		*dialerline, *dialer;
    int			flag;
    struct stat		statBuf;

    _errno = errno;
    if (!wasModified ())
	return;

    tmpfname = new char [(filename.directory ()).length () + 14];
    strcpy (tmpfname, (filename.directory ()).name ());
    strcat (tmpfname, "/DialersXXXXXX");
    mktemp (tmpfname);
    ofile = new ofstream (tmpfname);
    if (ofile->fail ())
    {
	_errno = errno;
	return;
    }

    ifile = new ifstream (filename.name ());

    /*----
     * Scan through the input file copying comments and blank lines
     * until we come to a legitimate dialer entry.
     *----*/

    while (!ifile->eof ())
    {
	cptr = get_line (ifile);
	if (ifile->eof () && (*cptr == '\0'))
	    break;

	if ((*cptr == '#') || (*cptr == '\0'))
	    *ofile << cptr << endl;
	else
	{
	    /*----
	     * Create an instance of a Dialer class using the entry
	     * found so we can use it for comparisons.
	     *----*/

	    dialerline = new Dialer (cptr);

	    /*----
	     * Search our list to see if we have an entry with the
	     * same dialer name.  If we do, put *all* of our entries
	     * with the same dialer name out here.
	     *
	     * Note:  We use the ListEntry flag to indicate those
	     * entries that we\'ve written.  We assume all the
	     * entries are currently set to 0.  We also assume that
	     * if an entry matches the name but the flag is set,
	     * then all entries we have internally have been written.
	     *----*/

	    dialer = (Dialer *)find (dialerline->theKey ());
	    if (dialer)
	    {
		if (lastFlag ())
		    continue;
		tptr = dialer->line ();
		*ofile << tptr << endl;
		setLastFlag (1);
		while ((dialer = (Dialer *)next ()) &&
		       (strcmp (dialer->theKey (),
				dialerline->theKey ()) == 0))
		{
		    tptr = dialer->line ();
		    *ofile << tptr << endl;
		    setLastFlag (1);
		}
	    }

	    delete dialerline;
	}
    }
    ifile->close ();

    /*----
     * Scan through the list of dialers we have and write any that
     * have not been written yet.  While we're scanning, clear all
     * the flags.
     *----*/

    for (dialer = (Dialer *)first (); dialer; dialer = (Dialer *)next ())
    {
	flag = lastFlag ();
	setLastFlag (0);
	if (flag)
	    continue;
	tptr = dialer->line ();
	*ofile << tptr << endl;
    }
    ofile->close ();

    /*----
     * Now we're done.
     * Find out the mode settings of the original file.
     * Make sure we have write access to the file.
     * Rename the new file over the old file.
     * Then set the mode settings of the new file back to what they were.
     *----*/

    if (stat (filename.name (), &statBuf))
	_errno = errno;

    if (!_errno)
	if (chmod (filename.name (),
		   statBuf.st_mode | (S_IWUSR | S_IWGRP | S_IWOTH)))
	    _errno = errno;

    if (!_errno)
	if (rename (tmpfname, filename.name ()))
	    _errno = errno;

    if (!_errno)
    {
	if (chmod (filename.name (), statBuf.st_mode))
	    _errno = errno;
	clearModified ();
    }

    delete [] tmpfname;
}

/*------------------------------------------------------------------------
 * Dialers::save (const char*)
 *------------------------------------------------------------------------*/

void Dialers::save (const char *fname)
{
    PathName		filename;

    if (fname)
	filename = (char *)fname;
    else
	filename = (char *)theKey ();

    save (filename);
}

/*------------------------------------------------------------------------
 * Dialers::makeNewDialer (const char *)
 *------------------------------------------------------------------------*/

LinkData* Dialers::makeNewDialer (const char *line)
{
    Dialer	*dialer;

    dialer = new Dialer (line);

    return dialer;
}

/*------------------------------------------------------------------------
 * Dialers::deleteEntry (LinkData *)
 *------------------------------------------------------------------------*/

void Dialers::deleteEntry (LinkData *e)
{
    delete e;
}

/*------------------------------------------------------------------------
 * Dialers::wasModified (void)
 *------------------------------------------------------------------------*/

int Dialers::wasModified (void)
{
    Dialer		*dlr;

    if (modified == 0)
    {
	for (dlr = (Dialer *)first (); dlr; dlr = (Dialer *)next ())
	{
	    if (dlr->wasModified ())
	    {
		modified = 1;
		break;
	    }
	}
    }

    return modified;
}

/*------------------------------------------------------------------------
 * Dialers::clearModified (void)
 *------------------------------------------------------------------------*/

void Dialers::clearModified (void)
{
    Dialer		*dlr;

    for (dlr = (Dialer *)first (); dlr; dlr = (Dialer *)next ())
	dlr->clearModified ();

    modified = 0;
}
