/*------------------------------------------------------------------------
 * $Id: DialersWindow.c++,v 2.12 1994/03/03 00:27:09 carlson Exp $
 *
 * Methods for DialersWindow class.
 *
 * Revision History:
 *	$Log: DialersWindow.c++,v $
// Revision 2.12  1994/03/03  00:27:09  carlson
// Modified so that it will compile on 5.1 OS versions.  This required taking
//   out the parameters for postAndWait methods that caused dialog window
//   to appear over the parent widget.
//
// Revision 2.11  1994/02/28  22:30:07  carlson
// When doing a save, check if an error occurred and generate a dialog if it
//   does.
//
// Revision 2.10  1994/02/26  03:56:32  carlson
// Modified all dialogs to place dialog window over the current parent widget.
// Removed code to add new dialer to dialer list since this is done
//   automatically in the callback for when the list of dialers changes.
// Added code to method that handles a change in the list of dialers to select
//   the current dialer, if there is a current dialer selected.
//
// Revision 2.9  1994/02/18  18:47:21  chrisc
// Cleaned up useless code in selectCallback.
// Improved the way select callback functions.
// Added delete button and delete callback to delete a file.
// Moved all the loops that fill the package list and selection list to
//   be in a method together.  This way, they can be updated by callbacks.
// Added callback for when the items in the list change.
//
// Revision 2.8  1994/01/20  20:17:39  carlson
// Modified to support new kind of help facility.
//
// Revision 2.7  1994/01/18  00:38:14  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.6  1994/01/04  20:58:12  carlson
// Changed some error printouts to use an error dialog.
// Instead of having a <new> item in list, created a New button.
// Fixed all \( \) to just ( ).
//
// Revision 2.5  1993/11/16  19:39:05  carlson
// Added help to various buttons and labels.
// Added ability to create a new dialer.
// Changed how I handle the list of packages.
// Fixed it so dialcodes can now add a package.
//
// Revision 2.4  1993/10/14  23:45:23  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.3  1993/10/06  18:07:02  chris
// Added 'save' method.
//
// Revision 2.2  1993/09/28  13:52:35  chris
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
//
// Revision 2.1  1993/07/29  01:58:29  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#ifndef OSVERS
#define OSVERS	5100
#endif

#include "DialersWindow.h"
#include "DialerWindow.h"
#include "HelpDialog.h"

#include <Vk/VkPromptDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/SeparatoG.h>

static char	*default_line = " ==--";

/*------------------------------------------------------------------------
 * DialersWindow::selectCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialersWindow::selectCallback (Widget, XtPointer clientData,
				   XtPointer callData)
{
    XmListCallbackStruct*	cbStruct;
    DialersWindow*		sw;
    char			*text;

    cbStruct = (XmListCallbackStruct *)callData;
    sw = (DialersWindow *)clientData;

    XmStringGetLtoR (cbStruct->item, XmFONTLIST_DEFAULT_TAG, &text);

    sw->select_cb (text);
}

/*------------------------------------------------------------------------
 * DialersWindow::select_cb (char *)
 *------------------------------------------------------------------------*/

void DialersWindow::select_cb (char *text)
{
    Dialer				*dlr;
    char				line[100];

    dlr = (Dialer *)dialers->find (text);
    if (!dlr)
    {
	strcpy (line, "Unable to find dialer ");
	strcat (line, text);
	strcat (line, " in list.\nReport this to SGI.");
	theErrorDialog->postAndWait (line
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }

    /*----
     * If a dialer is already displaying, bring it down.
     *----*/

    if (current)
    {
	_dialer->removeChangeCallback (dlrChangedCallback, this);
	delete current;
    }

    _dialer = dlr;
    _dialer->addChangeCallback (dlrChangedCallback, this);
    current = new DialerWindow (parent_for_dialer, _dialer);
    current->show ();
}

/*------------------------------------------------------------------------
 * DialersWindow::newCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialersWindow::newCallback (Widget, XtPointer clientData, XtPointer)
{
    DialersWindow*		dw;

    dw = (DialersWindow *)clientData;
    dw->new_cb ();
}

/*------------------------------------------------------------------------
 * DialersWindow::new_cb ()
 *------------------------------------------------------------------------*/

void DialersWindow::new_cb ()
{
    char				line[100];


    if (thePromptDialog->postAndWait ("Enter name of dialer to add:"
#if OSVERS >= 5200
				      , True, True, False, NULL, NULL, _parent
#endif
				      ) == VkPromptDialog::OK)
    {
	if (current)
	{
	    _dialer->removeChangeCallback (dlrChangedCallback, this);
	    delete current;
	}

	strcpy (line, thePromptDialog->text ());
	strcat (line, default_line);
	_dialer = new Dialer (line);
	_dialer->addChangeCallback (dlrChangedCallback, this);
	current = new DialerWindow (parent_for_dialer, _dialer);
	dialers->add (_dialer);		// Must occur after current defined.
	current->show ();
    }
}

/*------------------------------------------------------------------------
 * DialersWindow::deleteCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void DialersWindow::deleteCallback (Widget, XtPointer clientData, XtPointer)
{
    DialersWindow*		dlrw;

    dlrw = (DialersWindow *)clientData;
    dlrw->delete_cb ();
}

/*------------------------------------------------------------------------
 * DialersWindow::delete_cb ()
 *------------------------------------------------------------------------*/

void DialersWindow::delete_cb ()
{
    int			*position_list, position_count;
    XmString		*xmstr;
    char		*text, line[80];
    Dialer		*aDialer;

    if (!XmListGetSelectedPos (dialer_list, &position_list, &position_count))
    {
	theErrorDialog->postAndWait ("You need to select the\ndialer "
				     "to delete."
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }
    XFree (position_list);

    XtVaGetValues (dialer_list, XmNselectedItems, &xmstr, NULL);
    XmStringGetLtoR (xmstr[0], XmFONTLIST_DEFAULT_TAG, &text);
    aDialer = (Dialer *)dialers->find (text);

    if (!aDialer)
    {
	strcpy (line, "Unable to find dialer ");
	strcat (line, text);
	strcat (line, ".\nReport this to SGI.");
	theErrorDialog->postAndWait (line
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }

    strcpy (line, "Are you sure you want\nto remove dialer ");
    strcat (line, text);
    strcat (line, "?");
    if (theQuestionDialog->postAndWait (line
#if OSVERS >= 5200
					, True, True, False, NULL, NULL,
					_parent
#endif
					) == VkDialogManager::OK)
    {
	if (aDialer == _dialer)
	{
	    delete current;
	    current = NULL;
	}

	/*----
	 * Removing the entry will automatically remove it from
	 * the list window because of callbacks we have set
	 * up.
	 *----*/

	dialers->removeEntry (aDialer);
    }
}

/*------------------------------------------------------------------------
 * DialersWindow::~DialersWindow ()
 *------------------------------------------------------------------------*/

DialersWindow::~DialersWindow ()
{
    if (current)
    {
	_dialer->removeChangeCallback (dlrChangedCallback, this);
	delete current;
    }
    dialers->removeChangeCallback (listChangedCallback, this);
}

/*------------------------------------------------------------------------
 * DialersWindow::DialersWindow (PathName)
 *------------------------------------------------------------------------*/

DialersWindow::DialersWindow (Widget parent, Widget dialer_parent,
			      Dialers *dlrs)
: VkComponent (dlrs->theKey ())
{
    Arg			wargs[15];
    int			n;
    XmString		xmstr;
    Widget		temp, label, pkg_list_label, dialer_list_label;
    Widget		rc1, form1, form2;

    _parent = parent;
    dialers = dlrs;
    parent_for_dialer = dialer_parent;
    current = NULL;

    /*----
     * Create a form into which to put all the stuff.  We have to
     * specify its size, for some reason, or it never gets set.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNwidth, 370); n++;
    XtSetArg (wargs[n], XmNheight, 160); n++;
    XtSetArg (wargs[n], XmNallowOverlap, False); n++;
    _baseWidget = XmCreateForm (parent, (char *)dialers->theKey (), wargs, n);
    installDestroyHandler ();

    /*----
     * Create a label to indicate what Dialers we are showing here.
     *----*/
    
    n = 0;
    xmstr = XmStringCreateSimple ((char *)dialers->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabel (_baseWidget, "label", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "dlrw_help");

    /*----
     * Because of display problems, I was hoping that I could get
     * it to look better by creating a row/column to contain the
     * rest of the information in.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_TIGHT); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    rc1 = XmCreateRowColumn (_baseWidget, "lists", wargs, n);
    XtManageChild (rc1);

    n = 0;
    form1 = XmCreateForm (rc1, "plForm", wargs, n);
    XtManageChild (form1);

    /*----
     * List packages that use this file.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    pkg_list_label = XmCreateLabel (form1, "pkgListLabel", wargs, n);
    XtManageChild (pkg_list_label);
    
    XtAddCallback (pkg_list_label, XmNhelpCallback, aHelpCallback,
		   "dlrw_help");

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNnumColumns, 6); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, pkg_list_label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNentryAlignment, XmALIGNMENT_CENTER); n++;
    XtSetArg (wargs[n], XmNborderWidth, 1); n++;
    pkg_list = XmCreateRowColumn (form1, "pkgList", wargs, n);
    XtManageChild (pkg_list);
    
    XtAddCallback (pkg_list, XmNhelpCallback, aHelpCallback, "dlrw_help");

    /*----
     * Put a separator between the two areas.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNorientation, XmVERTICAL); n++;
    temp = XmCreateSeparatorGadget (rc1, "separator", wargs, n);
    XtManageChild (temp);

    n = 0;
    form2 = XmCreateForm (rc1, "dlrlForm", wargs, n);
    XtManageChild (form2);

    /*----
     * Provide a list of dialers to select from.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    dialer_list_label = XmCreateLabel (form2, "dialerListLabel", wargs, n);
    XtManageChild (dialer_list_label);
    
    XtAddCallback (dialer_list_label, XmNhelpCallback, aHelpCallback,
		   "dlrw_help");

    n = 0;
    XtSetArg (wargs[n], XmNlistSizePolicy, XmVARIABLE); n++;
    XtSetArg (wargs[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    XtSetArg (wargs[n], XmNvisibleItemCount, 4); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialer_list_label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    dialer_list = XmCreateScrolledList (form2, "dialerList", wargs, n);
    XtManageChild (dialer_list);

    updateLists ();

#if 0					// Used for single click
    XtAddCallback (dialer_list, XmNsingleSelectionCallback, selectCallback,
		   this);
#else
    XtAddCallback (dialer_list, XmNdefaultActionCallback, selectCallback,
		   this);
#endif
    XtAddCallback (dialer_list, XmNhelpCallback, aHelpCallback, "dlrw_help");
    dialers->addChangeCallback (listChangedCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialer_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form2, "new", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, newCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dlrw_new");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, dialer_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, temp); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form2, "delete", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, deleteCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dlrw_delete");
}

/*------------------------------------------------------------------------
 * DialersWindow::addPackage (LinkData *)
 *------------------------------------------------------------------------*/

void DialersWindow::addPackage (LinkData *le)
{
    Arg		wargs[5];
    int		n;
    XmString	xmstr;
    Widget	temp;

    if (_baseWidget)
    {
	n = 0;
	xmstr = XmStringCreateSimple ((char *)le->theKey ());
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
	temp = XmCreateLabel (pkg_list, (char *)le->theKey (), wargs, n);
	XtManageChild (temp);
	XmStringFree (xmstr);
    }

    dialers->addPackage (le);
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * DialersWindow::dump (char *)
 *------------------------------------------------------------------------*/

void DialersWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "DialersWindow dump (" << this << "):" << endl;
    cout << indent << "  dialers = " << dialers << endl;
    cout << indent << "  current = " << current << endl;

    if (current)
	current->dump (nextIndent);
}
#endif

/*------------------------------------------------------------------------
 * DialersWindow::save (void)
 *------------------------------------------------------------------------*/

void DialersWindow::save (void)
{
    char		line[128];

    dialers->save ();
    if (dialers->error ())
    {
	strcpy (line, "*** Error saving file:\n");
	strcat (line, strerror (dialers->error ()));
	theErrorDialog->postAndWait (line);
    }
}

/*------------------------------------------------------------------------
 * DialersWindow::dlrChangedCallback (Dialer *, Dialer::Field, void *)
 *------------------------------------------------------------------------*/

void DialersWindow::dlrChangedCallback (Dialer *dialer, Dialer::Field field,
					void *client_data)
{
    DialersWindow		*dlrw;

    dlrw = (DialersWindow *)client_data;
    dlrw->dlrChanged (dialer, field);
}

/*------------------------------------------------------------------------
 * DialersWindow::dlrChanged (Dialer *, Dialer::Field)
 *------------------------------------------------------------------------*/

void DialersWindow::dlrChanged (Dialer *dialer, Dialer::Field)
{
    int				*pos_list, pos;
    char			dialer_string[80];
    XmString			xmstr;

    if (XmListGetSelectedPos (dialer_list, &pos_list, &pos))
    {
	xmstr = XmStringCreateSimple ((char *)dialer->theKey ());
	XmListReplaceItemsPos (dialer_list, &xmstr, 1, pos_list[0]);
	XmStringFree (xmstr);
	XmListSelectPos (dialer_list, pos_list[0], False);
    }
}

/*------------------------------------------------------------------------
 * DialersWindow::listChangedCallback (SmartList *, SmartList::ACTION, void *)
 *------------------------------------------------------------------------*/

void DialersWindow::listChangedCallback (SmartList *, SmartList::ACTION,
					void *client_data)
{
    DialersWindow		*dlrw;

    dlrw = (DialersWindow *)client_data;
    dlrw->updateLists ();
}

/*------------------------------------------------------------------------
 * DialersWindow::updateLists (void)
 *------------------------------------------------------------------------*/

void DialersWindow::updateLists (void)
{
    Dialer		*aDialer;
    char		dialer_string[80];
    XmString		xmstr;
    WidgetList		wPackages;
    int			nPackages, i, n;
    LinkData		*ld;
    Widget		temp;
    Arg			wargs[10];

    /*----
     * Update the package list.
     *	1. Find out what children exist in the package list.
     *	2. Loop through the packages and add any that are missing.
     *----*/

    XtVaGetValues (pkg_list,
		   XmNchildren, &wPackages,
		   XmNnumChildren, &nPackages, NULL);

    for (ld = dialers->firstPackage (); ld; ld = dialers->nextPackage ())
    {
	for (i = 0; i < nPackages; i++)
	    if (strcmp (ld->theKey (), XtName (wPackages[i])) == 0)
		break;
	if (i != nPackages)
	    continue;
	n = 0;
	XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
	temp = XmCreateLabel (pkg_list, (char *)ld->theKey (), wargs, n);
	XtManageChild (temp);

	XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "dlrw_help");
    }

    /*----
     * Now loop through the children and remove any that are
     * not in the package list.
     *----*/

    for (i = 0; i < nPackages; i++)
    {
	ld = dialers->findPackage (XtName (wPackages[i]));
	if (!ld)
	    XtDestroyWidget (wPackages[i]);
    }

    /*----
     * Update the dialer list.
     *	1. Empty it out.
     *	2. Fill it.
     *----*/

    XmListDeleteAllItems (dialer_list);

    for (aDialer = (Dialer *)dialers->first (); aDialer;
	 aDialer = (Dialer *)dialers->next ())
    {
	xmstr = XmStringCreateSimple ((char *)aDialer->theKey ());
	XmListAddItemUnselected (dialer_list, xmstr, 0);
	XmStringFree (xmstr);
    }

    if (current)
    {
	xmstr = XmStringCreateSimple ((char *)_dialer->theKey ());
	XmListSelectItem (dialer_list, xmstr, False);
	XmStringFree (xmstr);
    }
}
