/*---*- C++ -*------------------------------------------------------------
 * $Id: DialersWindow.h,v 2.8 1994/02/26 03:56:33 carlson Exp $
 *
 * Header for DialersWindow class
 *
 * Revision History:
 *	$Log: DialersWindow.h,v $
 * Revision 2.8  1994/02/26  03:56:33  carlson
 * Added a place to keep the parent widget.  This is used by the dialogs.
 *
 * Revision 2.7  1994/02/18  18:50:45  chrisc
 * Added method to find the selected item from the list.
 * Added method to update the package and item list.
 * Added methods for handling delete button.
 * Added method to handle callback when a dialer's information changed.
 *
 * Revision 2.6  1994/01/17  23:42:46  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.5  1994/01/04  20:47:15  carlson
 * Added prototype for new callback methods.
 *
 * Revision 2.4  1993/11/16  19:35:41  carlson
 * Fixed include of linkList.h to use SmartList.h.
 * Added code to handle modified flag.
 * Cleaned up some.
 *
 * Revision 2.3  1993/10/14  23:45:24  carlson
 * Added prototype for dump method.
 *
 * Revision 2.2  1993/10/06  18:07:03  chris
 * Added declaration for 'save' method.
 *
 * Revision 2.1  1993/07/29  01:58:29  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _DIALERSWINDOW_H_
#define _DIALERSWINDOW_H_

#include <Vk/VkComponent.h>

#include "SmartList.h"
#include "PathName.h"
#include "Dialer.h"
#include "Dialers.h"

class DialerWindow;

class DialersWindow : public VkComponent
{
  public:
    DialersWindow (Widget parent, Widget dialer_parent, Dialers *devs);
    ~DialersWindow ();

    void		save (void);
    int			wasModified (void) { return dialers->wasModified (); }

    virtual const char*	className () { return "DialersWindow"; }
    virtual void	addPackage (LinkData *le);
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    Widget		_parent;
    Dialers		*dialers;
    Widget		parent_for_dialer;
    Widget		pkg_list;
    Widget		dialer_list;
    DialerWindow	*current;	// Current DialerWindow displayed
    Dialer		*_dialer;	// Dialer displayed in current

    void		updateLists (void);

    static void		selectCallback (Widget, XtPointer, XtPointer);
    static void		dlrChangedCallback (Dialer *, Dialer::Field, void *);
    static void		listChangedCallback (SmartList*, SmartList::ACTION,
					     void*);
    static void		newCallback (Widget, XtPointer, XtPointer);
    static void		deleteCallback (Widget, XtPointer, XtPointer);

    void		select_cb (char *text);
    void		dlrChanged (Dialer *, Dialer::Field);
    void		new_cb ();
    void		delete_cb ();
};

#endif
