/*------------------------------------------------------------------------
 * $Id: EditModem.c++,v 2.15 1994/02/25 20:49:29 carlson Exp $
 *
 * Methods for EditModem class.
 *
 * Revision History:
 *	$Log: EditModem.c++,v $
// Revision 2.15  1994/02/25  20:49:29  carlson
// Added a 'Copy' menu option to provide the ability to create new modems from the
//   descriptions of other modems.
// Changed the filter pattern for finding modem description files to require .mdm at
//   the end of the file name.
// Modified the import function to use the class PathName.
// Decided to give a constant name to this class window so that we can refer to it
//   in the resource files.
// Changed the names of the menu options and added the label strings to the resource
//   files.
//
// Revision 2.14  1994/02/15  04:15:47  chrisc
// Changed all references to myApplication to uucpApplication.
//
// Revision 2.13  1994/02/03  04:05:42  chrisc
// Added stuff for delete button and new sequence for setting the number of
//   rings.
//
// Revision 2.12  1994/02/02  23:41:14  chrisc
// Added callback methods for 'import' menu option.  This required calling
//   VkFileSelectionDialog and importFile.
// Activated the import menu button.
//
// Revision 2.11  1994/01/21  02:54:05  carlson
// Added methods to handle the "On Window" help menu option.
// Gave real names to the help page to print out for each help callback.
// Added the File menu and added the "On Window" help menu option.
//
// Revision 2.10  1994/01/20  20:20:31  carlson
// Added temporary help callbacks.
// Changed from VkSimpleWindow to VkWindow.
//
// Revision 2.9  1994/01/18  22:18:31  carlson
// Before saving modemFile, be sure to update all the fields.
// Added methods to save a field set to the appropriate sequence record
//   (readAField).
// Added a method to read all the field sets and update them (readAllFields).
//
// Revision 2.8  1994/01/14  23:54:45  carlson
// Removed type changed callbacks and methods.  They weren't used anyway.
//
// Revision 2.7  1994/01/14  08:08:42  carlson
// Since 'prefix' doesn't have an 'expect' clause, the global callback
//   failed.  Now it knows how to handle only one textfield input.
//
// Revision 2.6  1994/01/13  20:15:29  carlson
// Removed a number of callback functions and combined them into one.
// Take advantage of new methods provided by ModemFile class.
// Added widgets to support prefix and minimum.
// Always instantiate ModemFile.  Other methods depend on it.
//
// Revision 2.5  1994/01/12  02:17:02  carlson
// Removed dupstr since Copy in systemfiles.c++ does the same thing.
// Modified code that picks out the file name only to handle paths.
// No longer hide when window manager says quit or delete.  Delete the class.
// Removed afterRealizeHook.
//
// Revision 2.4  1994/01/11  22:24:02  carlson
// Removed the reading and writing of file and moved into its own class.
//
// Revision 2.3  1994/01/11  21:06:33  carlson
// Added new translations to text widgets that list defined registers and
//   baud settings.
// Created a little function to duplicate a string using the 'new' operator.
// Modified both quit routines to call unregisterWindow4Help and to delete
//   the current class instance.
// Call writeFile in button callbacks that require it.
// Added lots more callbacks.
// Created method to update the register list.
// Created method to update the baud list.
// Created method to update all the special sequences.
// Created methods to write an output file.
// Documented readFile.
// Added ability to read sequences from file.
// Use dupstr to create a new string for each token found in input file.
// Added stuff for handling baud in modem.
// Added (and commented out) callbacks for help.
// Added support for tab groups.
// Rearranged the buttons and widgets.
//
// Revision 2.2  1994/01/10  19:03:34  carlson
// Changed handling the variable fileName to handle it as a PathName type.
// Added support for maintaining register settings for the modem.
// Added method to read a file describing modem info.
//
// Revision 2.1  1994/01/06  03:44:23  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include "uucp.h"
#include "EditModem.h"
#include "HelpDialog.h"
#include "systemfiles.h"

#include <unistd.h>
#include <iostream.h>
#include <string.h>
#include <ctype.h>

#include <Vk/VkErrorDialog.h>
#include <Vk/VkMenu.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkMenuBar.h>
#include <Vk/VkHelpPane.h>
#include <Vk/VkFileSelectionDialog.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/SeparatoG.h>
#include <Xm/ScrolledW.h>

void registerWindow4Help (Widget);
void unregisterWindow4Help (Widget);
void importFile (const char *, ModemFile *);

static const char	*textTranslations =
"<BtnDown>:		select-adjust()\n"
"<BtnDown>(2):		activate()\n";

/*------------------------------------------------------------------------
 * EditModem::onWindowCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::onWindowCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*mw;

    mw = (EditModem *)clientData;
    mw->onWindow_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::onWindow_cb ()
 *------------------------------------------------------------------------*/

void EditModem::onWindow_cb ()
{
    theHelpDialog->displayHelp ("editModem");
}

/*------------------------------------------------------------------------
 * EditModem::okCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::okCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->ok_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::ok_cb ()
 *------------------------------------------------------------------------*/

void EditModem::ok_cb ()
{
    readAllFields ();
    modemFile->save ();

    unregisterWindow4Help (parentw);

    delete this;
}

/*------------------------------------------------------------------------
 * EditModem::saveCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::saveCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->save_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::save_cb ()
 *------------------------------------------------------------------------*/

void EditModem::save_cb ()
{
    readAllFields ();
    modemFile->save ();
}

/*------------------------------------------------------------------------
 * EditModem::quitCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::quitCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->quit_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::quit_cb ()
 *------------------------------------------------------------------------*/

void EditModem::quit_cb ()
{
    unregisterWindow4Help (parentw);

    delete this;
}

/*------------------------------------------------------------------------
 * EditModem::copyCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::copyCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->copy_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::copy_cb ()
 *------------------------------------------------------------------------*/

void EditModem::copy_cb ()
{
    PathName			fileName;

    theFileSelectionDialog->setDirectory (uucpApplication->directory ());
    theFileSelectionDialog->setFilterPattern ("*.mdm");
    theFileSelectionDialog->centerOnScreen (True);
    if (theFileSelectionDialog->postAndWait ("Select file to copy from:")
	== VkDialogManager::OK)
    {
	fileName = theFileSelectionDialog->fileName ();
	modemFile->insertFile (fileName);
	updateRegisterList ();
    }
}

/*------------------------------------------------------------------------
 * EditModem::importCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::importCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->import_cb ();
}

/*------------------------------------------------------------------------
 * EditModem::import_cb ()
 *
 * Currently, this routine just handles one type of modem, a Telebit.
 * Sometime in the future, we can add other types of modems that it
 * can read.
 *------------------------------------------------------------------------*/

void EditModem::import_cb ()
{
    PathName			fileName;

    theFileSelectionDialog->setDirectory (uucpApplication->directory ());
    theFileSelectionDialog->setFilterPattern ("*.mdm");
    theFileSelectionDialog->centerOnScreen (True);
    if (theFileSelectionDialog->postAndWait ("Select file to import:")
	== VkDialogManager::OK)
    {
	fileName = theFileSelectionDialog->fileName ();
	importFile (fileName.name (), modemFile);
	updateRegisterList ();
    }
}

/*------------------------------------------------------------------------
 * EditModem::setRegisterCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::setRegisterCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->setRegister ();
}

/*------------------------------------------------------------------------
 * EditModem::setRegister ()
 *------------------------------------------------------------------------*/

void EditModem::setRegister ()
{
    char		*reg, *val, *doc;
    char		*valNew, *docNew;
    Registers::Type	type;
    Registers		*Reg;
    Widget		lastType;

    XtVaGetValues (numberT, XmNvalue, &reg, NULL);
    XtVaGetValues (valueT, XmNvalue, &val, NULL);
    XtVaGetValues (docText, XmNvalue, &doc, NULL);
    XtVaGetValues (typeT, XmNmenuHistory, &lastType, NULL);

    valNew = Copy (val);
    XtFree (val);

    docNew = doc + strlen (doc) - 1;
    while (*docNew == '\n')
	*docNew-- = '\0';
    docNew = Copy (doc);
    XtFree (doc);

    type = (strcmp (XtName (lastType), "register") == 0)
	? Registers::REGISTER : Registers::FLAG;

    Reg = modemFile->findRegister (reg);
    if (Reg)
    {
	Reg->setType (type);

	val = Reg->getValue ();
	delete [] val;
	Reg->setValue (valNew);

	doc = Reg->getDocumentation ();
	delete [] doc;
	Reg->setDocumentation (docNew);
    }
    else
    {
	Reg = new Registers (reg, type, valNew, docNew);
	modemFile->addRegister (Reg);
    }

    updateRegisterList ();

    XmTextSetSelection (numberT, 0, XmTextGetLastPosition (numberT),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (numberT);
    XmTextSetSelection (valueT, 0, XmTextGetLastPosition (valueT),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (valueT);
    XmTextSetSelection (docText, 0, XmTextGetLastPosition (docText),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (docText);

    XmProcessTraversal (numberT, XmTRAVERSE_CURRENT);

    if (reg)
	XtFree (reg);
}

/*------------------------------------------------------------------------
 * EditModem::delRegisterCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::delRegisterCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->delRegister ();
}

/*------------------------------------------------------------------------
 * EditModem::delRegister ()
 *------------------------------------------------------------------------*/

void EditModem::delRegister ()
{
    char		*rname, *ctmp;
    Registers		*reg;

    XtVaGetValues (numberT, XmNvalue, &rname, NULL);

    if (strlen (rname) > 0)
    {
	reg = modemFile->findRegister (rname);
	if (reg)
	{
	    modemFile->deleteRegister (reg);
	    updateRegisterList ();

	    XmTextSetSelection (numberT, 0, XmTextGetLastPosition (numberT),
			XtLastTimestampProcessed (uucpApplication->display ()));
	    XmTextRemove (numberT);
	    XmTextSetSelection (valueT, 0, XmTextGetLastPosition (valueT),
			XtLastTimestampProcessed (uucpApplication->display ()));
	    XmTextRemove (valueT);
	    XmTextSetSelection (docText, 0, XmTextGetLastPosition (docText),
			XtLastTimestampProcessed (uucpApplication->display ()));
	    XmTextRemove (docText);
	}
    }
    else
    {
	rname = XmTextGetSelection (rList);
	if (rname)
	{
	    ctmp = strchr (rname, '\t');
	    if (ctmp)
		*ctmp = '\0';

	    reg = modemFile->findRegister (rname);
	    if (reg)
	    {
		modemFile->deleteRegister (reg);
		updateRegisterList ();
	    }
	    XtFree (rname);
	}
    }
}

/*------------------------------------------------------------------------
 * EditModem::updateRegisterList (void)
 *------------------------------------------------------------------------*/

void EditModem::updateRegisterList (void)
{
    char		line[128], *ctmp, *dtmp;
    Registers		*reg;

    XmTextSetEditable (rList, True);
    XmTextSetSelection (rList, 0, XmTextGetLastPosition (rList),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (rList);

    for (reg = modemFile->firstRegister (); reg;
	 reg = modemFile->nextRegister ())
    {
	strcpy (line, (char *)reg->theKey ());
	strcat (line, "\t");
	ctmp = (reg->getType () == Registers::REGISTER) ?
	    "reg" : (reg->getType () == Registers::FLAG) ? "flag" : "???";
	strcat (line, ctmp);
	strcat (line, "\t");
	strcat (line, reg->getValue ());
	strcat (line, "\t");
	ctmp = reg->getDocumentation ();
	dtmp = strchr (ctmp, '\n');
	if (dtmp)
	    strncat (line, ctmp, dtmp - ctmp);
	else
	    strcat (line, ctmp);
	strcat (line, "\n");

	XmTextInsert (rList, XmTextGetLastPosition (rList), line);
    }

    XmTextSetEditable (rList, False);
}

/*------------------------------------------------------------------------
 * EditModem::rNumChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::rNumChangedCallback (Widget w, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->rNumChanged (w);
}

/*------------------------------------------------------------------------
 * EditModem::rNumChanged (Widget)
 *------------------------------------------------------------------------*/

void EditModem::rNumChanged (Widget w)
{
    char		*regNum;
    Registers		*reg;
    Widget		last;

    XtVaGetValues (numberT, XmNvalue, &regNum, NULL);

    reg = modemFile->findRegister (regNum);
    if (reg)
    {
	XtVaSetValues (valueT, XmNvalue, reg->getValue (), NULL);
	XtVaSetValues (docText, XmNvalue, reg->getDocumentation (), NULL);
	if (reg->getType () == Registers::REGISTER)
	    last = regT;
	else if (reg->getType () == Registers::FLAG)
	    last = flgT;
	else
	    last = regT;
	XtVaSetValues (typeT, XmNmenuHistory, last, NULL);
    }

    XmProcessTraversal (w, XmTRAVERSE_RIGHT);

    if (regNum)
	XtFree (regNum);
}

/*------------------------------------------------------------------------
 * EditModem::rListCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::rListCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->rListActivate ();
}

/*------------------------------------------------------------------------
 * EditModem::rListActivate ()
 *------------------------------------------------------------------------*/

void EditModem::rListActivate ()
{
    char			*text, *ctmp;
    Registers			*reg;
    Widget			last;

    text = XmTextGetSelection (rList);
    ctmp = strchr (text, '\t');
    if (ctmp)
	*ctmp = '\0';

    XtVaSetValues (numberT, XmNvalue, text, NULL);
    reg = modemFile->findRegister (text);
    if (reg)
    {
	XtVaSetValues (valueT, XmNvalue, reg->getValue (), NULL);
	XtVaSetValues (docText, XmNvalue, reg->getDocumentation (), NULL);
	if (reg->getType () == Registers::REGISTER)
	    last = regT;
	else if (reg->getType () == Registers::FLAG)
	    last = flgT;
	else
	    last = regT;
	XtVaSetValues (typeT, XmNmenuHistory, last, NULL);
    }

    XmProcessTraversal (numberT, XmTRAVERSE_CURRENT);

    if (text)
	XtFree (text);
}

/*------------------------------------------------------------------------
 * EditModem::sChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::sChangedCallback (Widget w, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->sChanged (w);
}

/*------------------------------------------------------------------------
 * EditModem::sChanged (Widget)
 *------------------------------------------------------------------------*/

void EditModem::sChanged (Widget w)
{
    char		*myName, *tag;
    Widget		sendw, expectw;

    myName = XtName (w);
    sendw = NULL;

    if (strncmp (myName, "ab", 2) == 0)
    {
	sendw = abSend;
	expectw = abExpect;
	tag = "check";
    }
    else if (strncmp (myName, "sr", 2) == 0)
    {
	sendw = srSend;
	expectw = srExpect;
	tag = "setR";
    }
    else if (strncmp (myName, "sf", 2) == 0)
    {
	sendw = sfSend;
	expectw = sfExpect;
	tag = "setF";
    }
    else if (strncmp (myName, "vr", 2) == 0)
    {
	sendw = vrSend;
	expectw = vrExpect;
	tag = "save";
    }
    else if (strncmp (myName, "pr", 2) == 0)
    {
	sendw = prSend;
	expectw = NULL;
	tag = "prefix";
    }
    else if (strncmp (myName, "bm", 2) == 0)
    {
	sendw = bmSend;
	expectw = bmExpect;
	tag = "minimum";
    }
    else if (strncmp (myName, "an", 2) == 0)
    {
	sendw = anSend;
	expectw = anExpect;
	tag = "minimum";
    }
    else if (strncmp (myName, "sb", 2) == 0)
    {
	sendw = sbSend;
	expectw = sbExpect;
	tag = "speed";
    }

    if (sendw)
	readAField (sendw, expectw, tag);

    XmProcessTraversal (w, XmTRAVERSE_NEXT_TAB_GROUP);
}

/*------------------------------------------------------------------------
 * EditModem::readAllFields (void)
 *------------------------------------------------------------------------*/

void EditModem::readAllFields (void)
{
    readAField (abSend, abExpect, "check");
    readAField (srSend, srExpect, "setR");
    readAField (sfSend, sfExpect, "setF");
    readAField (vrSend, vrExpect, "save");
    readAField (prSend, NULL,     "prefix");
    readAField (bmSend, bmExpect, "minimum");
    readAField (anSend, anExpect, "answer");
    readAField (sbSend, sbExpect, "speed");
}

/*------------------------------------------------------------------------
 * EditModem::readAField (Widget, Widget, char *)
 *------------------------------------------------------------------------*/

void EditModem::readAField (Widget sendw, Widget expectw, char *tag)
{
    char		*send, *expect;
    char		*sNew, *eNew;
    Sequences		*seq;

    XtVaGetValues (sendw, XmNvalue, &send, NULL);
    sNew = Copy (parse_string (send));
    XtFree (send);

    if (expectw)
    {
	XtVaGetValues (expectw, XmNvalue, &expect, NULL);
	eNew = Copy (expect);
	XtFree (expect);
    }
    else
	eNew = Copy ("");

    seq = modemFile->findSequence (tag);
    if (seq)
    {
	send = seq->getSend ();
	if (send)
	    delete [] send;
	seq->setSend (sNew);

	expect = seq->getExpect ();
	if (expect)
	    delete [] expect;
	seq->setExpect (eNew);
    }
    else
    {
	seq = new Sequences (tag, sNew, eNew);
	modemFile->addSequence (seq);
    }
}

/*------------------------------------------------------------------------
 * EditModem::setBaudCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::setBaudCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->setBaud ();
}

/*------------------------------------------------------------------------
 * EditModem::setBaud ()
 *------------------------------------------------------------------------*/

void EditModem::setBaud ()
{
    char		*baud, *sym;
    char		*baudNew, *symNew;
    Sequences		*seq;

    XtVaGetValues (bRate, XmNvalue, &baud, NULL);
    XtVaGetValues (bSymbol, XmNvalue, &sym, NULL);

    baudNew = Copy (baud);
    XtFree (baud);

    symNew = Copy (sym);
    XtFree (sym);

    seq = modemFile->getBaudSequence (baudNew);
    if (seq)
    {
	sym = seq->getExpect ();
	delete [] sym;
	seq->setExpect (symNew);
    }
    else
    {
	seq = new Sequences ("baud", baudNew, symNew);
	modemFile->addSequence (seq);
    }

    updateBaudList ();

    XmTextSetSelection (bRate, 0, XmTextGetLastPosition (bRate),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (bRate);
    XmTextSetSelection (bSymbol, 0, XmTextGetLastPosition (bSymbol),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (bSymbol);

    XmProcessTraversal (bRate, XmTRAVERSE_CURRENT);
}

/*------------------------------------------------------------------------
 * EditModem::updateBaudList (void)
 *------------------------------------------------------------------------*/

void EditModem::updateBaudList (void)
{
    char		line[128], *ctmp, *dtmp;
    Sequences		*seq;

    XmTextSetEditable (bList, True);
    XmTextSetSelection (bList, 0, XmTextGetLastPosition (bList),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (bList);

    for (seq = modemFile->firstSequence (); seq;
	 seq = modemFile->nextSequence ())
    {
	if (strcmp (seq->theKey (), "baud"))
	    continue;

	strcpy (line, seq->getSend ());
	strcat (line, "\t");
	strcat (line, seq->getExpect ());
	ctmp = seq->getDocumentation ();
	if (ctmp)
	{
	    strcat (line, "\t");
	    dtmp = strchr (ctmp, '\n');
	    if (dtmp)
		strncat (line, ctmp, dtmp - ctmp);
	    else
		strcat (line, ctmp);
	}
	strcat (line, "\n");

	XmTextInsert (bList, XmTextGetLastPosition (bList), line);
    }

    XmTextSetEditable (bList, False);
}

/*------------------------------------------------------------------------
 * EditModem::updateSequences (void)
 *------------------------------------------------------------------------*/

void EditModem::updateSequences (void)
{
    char		line[128];
    const char		*ctmp;
    Sequences		*seq;
    Widget		send, expect;

    for (seq = modemFile->firstSequence (); seq;
	 seq = modemFile->nextSequence ())
    {
	send = NULL;
	ctmp = seq->theKey ();
	if (strcmp (ctmp, "baud") == 0)
	    continue;
	else if (strcmp (ctmp, "check") == 0)
	{
	    send = abSend;
	    expect = abExpect;
	}
	else if (strcmp (ctmp, "setR") == 0)
	{
	    send = srSend;
	    expect = srExpect;
	}
	else if (strcmp (ctmp, "setF") == 0)
	{
	    send = sfSend;
	    expect = sfExpect;
	}
	else if (strcmp (ctmp, "save") == 0)
	{
	    send = vrSend;
	    expect = vrExpect;
	}
	else if (strcmp (ctmp, "speed") == 0)
	{
	    send = sbSend;
	    expect = sbExpect;
	}
	else if (strcmp (ctmp, "prefix") == 0)
	{
	    send = prSend;
	    expect = prExpect;
	}
	else if (strcmp (ctmp, "answer") == 0)
	{
	    send = anSend;
	    expect = anExpect;
	}
	else if (strcmp (ctmp, "minimum") == 0)
	{
	    send = bmSend;
	    expect = bmExpect;
	}

	if (send)
	{
	    XtVaSetValues (send, XmNvalue, unparse_string (seq->getSend ()),
			   NULL);
	    XtVaSetValues (expect, XmNvalue, seq->getExpect (), NULL);
	}
    }
}

/*------------------------------------------------------------------------
 * EditModem::bNumChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::bNumChangedCallback (Widget w, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->bNumChanged (w);
}

/*------------------------------------------------------------------------
 * EditModem::bNumChanged (Widget)
 *------------------------------------------------------------------------*/

void EditModem::bNumChanged (Widget w)
{
    char		*seqNum;
    Sequences		*seq;

    XtVaGetValues (bRate, XmNvalue, &seqNum, NULL);

    seq = modemFile->getBaudSequence (seqNum);
    if (seq)
	XtVaSetValues (bSymbol, XmNvalue, seq->getExpect (), NULL);

    XmProcessTraversal (w, XmTRAVERSE_RIGHT);

    if (seqNum)
	XtFree (seqNum);
}

/*------------------------------------------------------------------------
 * EditModem::bListCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void EditModem::bListCallback (Widget, XtPointer clientData, XtPointer)
{
    EditModem		*em;

    em = (EditModem *)clientData;
    em->bListActivate ();
}

/*------------------------------------------------------------------------
 * EditModem::bListActivate ()
 *------------------------------------------------------------------------*/

void EditModem::bListActivate ()
{
    char			*text, *ctmp;
    Sequences			*seq;

    text = XmTextGetSelection (bList);
    ctmp = strchr (text, '\t');
    if (ctmp)
	*ctmp = '\0';

    XtVaSetValues (bRate, XmNvalue, text, NULL);

    seq = modemFile->getBaudSequence (text);
    if (seq)
	XtVaSetValues (bSymbol, XmNvalue, seq->getExpect (), NULL);

    XmProcessTraversal (bRate, XmTRAVERSE_CURRENT);

    if (text)
	XtFree (text);
}

/*------------------------------------------------------------------------
 * EditModem::EditModem (const char *)
 *------------------------------------------------------------------------*/

EditModem::EditModem (const char *name) : VkWindow ("editModem")
{
    char		*tmp;
    const char		*ctmp;

    fileName = name;
    modemFile = NULL;

    ctmp = strrchr (name, '/');
    if (ctmp)
	ctmp++;
    else
	ctmp = name;
    tmp = Copy (ctmp);
    tmp[strlen (tmp) - 4] = '\0';
    setTitle (tmp);
    delete [] tmp;
}

/*------------------------------------------------------------------------
 * EditModem::~EditModem ()
 *------------------------------------------------------------------------*/

EditModem::~EditModem ()
{
    unregisterWindow4Help (parentw);

    if (modemFile)
	delete modemFile;
}

/*------------------------------------------------------------------------
 * EditModem::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget EditModem::setUpInterface (Widget parent)
{
    Arg			wargs[20];
    int			n;
    Widget		form1, rc1, rc2, temp, label, form2;
    Widget		number, value, type;
    Widget		popup, docLabel, menuw;
    VkMenuBar		*menubar;
    VkHelpPane		*helpMenu;
    XmTextScanType	scanTypeArray[2];
    XmString		xmstr;

    registerWindow4Help (parentw = parent);

    /*----
     * Create outer form.
     *----*/

    n = 0;
    form1 = XmCreateForm (parent, "editModemForm", wargs, n);

    /*----
     * Create register information window.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form1, "registerLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "editModemReg");

    /*-
     * Register number
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    number = XmCreateLabelGadget (form1, "number", wargs, n);
    XtManageChild (number);

    XtAddCallback (number, XmNhelpCallback, aHelpCallback, "editModemReg");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, number); n++;
    XtSetArg (wargs[n], XmNtopOffset, 4); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    numberT = XmCreateTextField (form1, "numberT", wargs, n);
    XtManageChild (numberT);

    XtAddCallback (numberT, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (numberT, XmNactivateCallback, rNumChangedCallback, this);

    /*-
     * Register value
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, numberT); n++;
    XtSetArg (wargs[n], XmNleftOffset, 2); n++;
    value = XmCreateLabelGadget (form1, "value", wargs, n);
    XtManageChild (value);

    XtAddCallback (value, XmNhelpCallback, aHelpCallback, "editModemReg");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, value); n++;
    XtSetArg (wargs[n], XmNtopOffset, 4); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, numberT); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    valueT = XmCreateTextField (form1, "valueT", wargs, n);
    XtManageChild (valueT);

    XtAddCallback (valueT, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (valueT, XmNactivateCallback,
		   (XtCallbackProc)XmProcessTraversal,
		   (XtPointer)XmTRAVERSE_DOWN);

    /*-
     * Register type
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, valueT); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    type = XmCreateLabelGadget (form1, "type", wargs, n);
    XtManageChild (type);

    XtAddCallback (type, XmNhelpCallback, aHelpCallback, "editModemReg");

    n = 0;
    popup = XmCreateSimplePulldownMenu (form1, "typeMenu", wargs, n);

    n = 0;
    regT = XmCreatePushButtonGadget (popup, "register", wargs, n);
    XtManageChild (regT);

    n = 0;
    flgT = XmCreatePushButtonGadget (popup, "flag", wargs, n);
    XtManageChild (flgT);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, type); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, valueT); n++;
    XtSetArg (wargs[n], XmNsubMenuId, popup); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    typeT = XmCreateOptionMenu (form1, "typeButton", wargs, n);
    XtManageChild (typeT);

    XtAddCallback (typeT, XmNhelpCallback, aHelpCallback, "editModemReg");

    /*----
     * Create a row column to hold the Set and Delete buttons in.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNtopOffset, 6); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, typeT); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNisAligned, True); n++;
    XtSetArg (wargs[n], XmNorientation, XmVERTICAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    rc1 = XmCreateRowColumn (form1, "regButtons", wargs, n);
    XtManageChild (rc1);

    /*----
     * Set button
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    setReg = XmCreatePushButtonGadget (rc1, "setReg", wargs, n);
    XtManageChild (setReg);

    XtAddCallback (setReg, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (setReg, XmNactivateCallback, setRegisterCallback, this);

    /*----
     * Delete button
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    delReg = XmCreatePushButtonGadget (rc1, "delReg", wargs, n);
    XtManageChild (delReg);

    XtAddCallback (delReg, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (delReg, XmNactivateCallback, delRegisterCallback, this);

    /*----
     * Put in label and text window for documentation on register.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, numberT); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    docLabel = XmCreateLabelGadget (form1, "docLabel", wargs, n);
    XtManageChild (docLabel);

    XtAddCallback (docLabel, XmNhelpCallback, aHelpCallback, "editModemReg");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, docLabel); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNcolumns, 50); n++;
    XtSetArg (wargs[n], XmNrows, 3); n++;
    XtSetArg (wargs[n], XmNwordWrap, True); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    XtSetArg (wargs[n], XmNtraversalOn, True); n++;
    docText = XmCreateText (form1, "docText", wargs, n);
    XtManageChild (docText);

    XtAddCallback (docText, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (docText, XmNactivateCallback,
		   (XtCallbackProc)XmProcessTraversal,
		   (XtPointer)XmTRAVERSE_RIGHT);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, docText); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNrows, 4); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    scanTypeArray[0] = scanTypeArray[1] = XmSELECT_LINE;
    XtSetArg (wargs[n], XmNselectionArray, scanTypeArray); n++;
    XtSetArg (wargs[n], XmNselectionArrayCount, 2); n++;
    XtSetArg (wargs[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    XtSetArg (wargs[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    rList = XmCreateScrolledText (form1, "regList", wargs, n);
    XtManageChild (rList);

    XtUninstallTranslations (rList);
    XtOverrideTranslations (rList, XtParseTranslationTable (textTranslations));

    XtAddCallback (rList, XmNhelpCallback, aHelpCallback, "editModemReg");
    XtAddCallback (rList, XmNactivateCallback, rListCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, rList); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    temp = XmCreateSeparatorGadget (form1, "sep1", wargs, n);
    XtManageChild (temp);

    /*----
     * Create needed sequences window.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form1, "sequencesLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "editModemSeq");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNisAligned, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, 9); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    rc2 = XmCreateRowColumn (form1, "rowcolumn", wargs, n);
    XtManageChild (rc2);

    /*-
     * Labels above text fields.
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreateLabelGadget (rc2, "sequence", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSeq");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreateLabelGadget (rc2, "send", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSend");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreateLabelGadget (rc2, "expect", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemExpect");

    /*-
     * Prefix string
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "prefix", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemCM");

    n = 0;
    prSend = XmCreateTextField (rc2, "prSend", wargs, n);
    XtManageChild (prSend);

    XtAddCallback (prSend, XmNhelpCallback, aHelpCallback, "editModemCM");
    XtAddCallback (prSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    xmstr = XmStringCreateSimple ("");
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    prExpect = XmCreateLabelGadget (rc2, "prExpect", wargs, n);
    XtManageChild (prExpect);

    XtAddCallback (prExpect, XmNhelpCallback, aHelpCallback, "editModemCM");

    /*-
     * Answerback
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "answerback", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemAB");

    n = 0;
    abSend = XmCreateTextField (rc2, "abSend", wargs, n);
    XtManageChild (abSend);

    XtAddCallback (abSend, XmNhelpCallback, aHelpCallback, "editModemAB");
    XtAddCallback (abSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    abExpect = XmCreateTextField (rc2, "abExpect", wargs, n);
    XtManageChild (abExpect);

    XtAddCallback (abExpect, XmNhelpCallback, aHelpCallback, "editModemAB");
    XtAddCallback (abExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Set register
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "setRegister", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSR");

    n = 0;
    srSend = XmCreateTextField (rc2, "srSend", wargs, n);
    XtManageChild (srSend);

    XtAddCallback (srSend, XmNhelpCallback, aHelpCallback, "editModemSR");
    XtAddCallback (srSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    srExpect = XmCreateTextField (rc2, "srExpect", wargs, n);
    XtManageChild (srExpect);

    XtAddCallback (srExpect, XmNhelpCallback, aHelpCallback, "editModemSR");
    XtAddCallback (srExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Set flag
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "setFlag", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSF");

    n = 0;
    sfSend = XmCreateTextField (rc2, "sfSend", wargs, n);
    XtManageChild (sfSend);

    XtAddCallback (sfSend, XmNhelpCallback, aHelpCallback, "editModemSF");
    XtAddCallback (sfSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    sfExpect = XmCreateTextField (rc2, "sfExpect", wargs, n);
    XtManageChild (sfExpect);

    XtAddCallback (sfExpect, XmNhelpCallback, aHelpCallback, "editModemSF");
    XtAddCallback (sfExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Save registers
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "saveRegisters", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSV");

    n = 0;
    vrSend = XmCreateTextField (rc2, "vrSend", wargs, n);
    XtManageChild (vrSend);

    XtAddCallback (vrSend, XmNhelpCallback, aHelpCallback, "editModemSV");
    XtAddCallback (vrSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    vrExpect = XmCreateTextField (rc2, "vrExpect", wargs, n);
    XtManageChild (vrExpect);

    XtAddCallback (vrExpect, XmNhelpCallback, aHelpCallback, "editModemSV");
    XtAddCallback (vrExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Minimum required
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "minimum", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemMN");

    n = 0;
    bmSend = XmCreateTextField (rc2, "bmSend", wargs, n);
    XtManageChild (bmSend);

    XtAddCallback (bmSend, XmNhelpCallback, aHelpCallback, "editModemMN");
    XtAddCallback (bmSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    bmExpect = XmCreateTextField (rc2, "bmExpect", wargs, n);
    XtManageChild (bmExpect);

    XtAddCallback (bmExpect, XmNhelpCallback, aHelpCallback, "editModemMN");
    XtAddCallback (bmExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Enable phone answer
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "answer", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemAN");

    n = 0;
    anSend = XmCreateTextField (rc2, "anSend", wargs, n);
    XtManageChild (anSend);

    XtAddCallback (anSend, XmNhelpCallback, aHelpCallback, "editModemAN");
    XtAddCallback (anSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    anExpect = XmCreateTextField (rc2, "anExpect", wargs, n);
    XtManageChild (anExpect);

    XtAddCallback (anExpect, XmNhelpCallback, aHelpCallback, "editModemAN");
    XtAddCallback (anExpect, XmNactivateCallback, sChangedCallback, this);

    /*-
     * Set baud
     *-*/

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
    temp = XmCreateLabelGadget (rc2, "setBaud", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSB");

    n = 0;
    sbSend = XmCreateTextField (rc2, "sbSend", wargs, n);
    XtManageChild (sbSend);

    XtAddCallback (sbSend, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (sbSend, XmNactivateCallback, sChangedCallback, this);

    n = 0;
    sbExpect = XmCreateTextField (rc2, "sbExpect", wargs, n);
    XtManageChild (sbExpect);

    XtAddCallback (sbExpect, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (sbExpect, XmNactivateCallback, sChangedCallback, this);

    /*----
     * Create window to display how baud is identified to modem.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftOffset, 40); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    form2 = XmCreateForm (form1, "baudForm", wargs, n);
    XtManageChild (form2);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form2, "baud", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "editModemSB");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    bRate = XmCreateTextField (form2, "baudText", wargs, n);
    XtManageChild (bRate);

    XtAddCallback (bRate, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (bRate, XmNactivateCallback, bNumChangedCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, bRate); n++;
    temp = XmCreateLabelGadget (form2, "symbol", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSB");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, bRate); n++;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    bSymbol = XmCreateTextField (form2, "symbolText", wargs, n);
    XtManageChild (bSymbol);

    XtAddCallback (bSymbol, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (bSymbol, XmNactivateCallback, setBaudCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, bSymbol); n++;
    setBd = XmCreatePushButtonGadget (form2, "setBaud", wargs, n);
    XtManageChild (setBd);

    XtAddCallback (setBd, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (setBd, XmNactivateCallback, setBaudCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, bRate); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNrows, 4); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    XtSetArg (wargs[n], XmNselectionArray, scanTypeArray); n++;
    XtSetArg (wargs[n], XmNselectionArrayCount, 2); n++;
    XtSetArg (wargs[n], XmNscrollBarDisplayPolicy, XmAS_NEEDED); n++;
    XtSetArg (wargs[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    bList = XmCreateScrolledText (form2, "baudList", wargs, n);
    XtManageChild (bList);

    XtUninstallTranslations (bList);
    XtOverrideTranslations (bList, XtParseTranslationTable (textTranslations));

    XtAddCallback (bList, XmNhelpCallback, aHelpCallback, "editModemSB");
    XtAddCallback (bList, XmNactivateCallback, bListCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, form2); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    temp = XmCreateSeparatorGadget (form1, "sep2", wargs, n);
    XtManageChild (temp);

    /*----
     * Create buttons at bottom.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    rc1 = XmCreateRowColumn (form1, "buttons", wargs, n);
    XtManageChild (rc1);

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButton (rc1, "ok", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemSE");
    XtAddCallback (temp, XmNactivateCallback, okCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButton (rc1, "save", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemS");
    XtAddCallback (temp, XmNactivateCallback, saveCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButton (rc1, "quit", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "editModemQ");
    XtAddCallback (temp, XmNactivateCallback, quitCallback, this);

    modemFile = new ModemFile (fileName);
    updateRegisterList ();
    updateSequences ();
    updateBaudList ();

    XtVaSetValues (form1, XmNinitialFocus, numberT, NULL);

    /*----
     * Add menu panes
     *----*/

    fileMenuPane = addMenuPane ("files");
    menuw = fileMenuPane->baseWidget ();
    XtAddCallback (menuw, XmNhelpCallback, aHelpCallback, "editModemFile");

    fileMenuPane->addAction ("save", saveCallback, this);
    fileMenuPane->addAction ("saveAs", saveCallback, this);
    fileMenuPane->addAction ("import", importCallback, this);
    fileMenuPane->addAction ("copy", copyCallback, this);
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("quit", quitCallback, this);

    menubar = menu ();
    helpMenu = menubar->helpPane ();
    helpMenu->addSeparator ();
    helpMenu->addAction ("window", onWindowCallback, this);

    fileMenuPane->deactivateItem ("saveAs");

    return form1;
}

/*------------------------------------------------------------------------
 * EditModem::handleWmDeleteMessage ()
 *------------------------------------------------------------------------*/

void EditModem::handleWmDeleteMessage ()
{
    delete this;
}

/*------------------------------------------------------------------------
 * EditModem::handleWmQuitMessage ()
 *------------------------------------------------------------------------*/

void EditModem::handleWmQuitMessage ()
{
    delete this;
}
