/*---*- C++ -*------------------------------------------------------------
 * $Id: EditModem.h,v 2.13 1994/02/25 20:37:18 carlson Exp $
 *
 * Header file for EditModem class.
 *
 * Revision History:
 *	$Log: EditModem.h,v $
 * Revision 2.13  1994/02/25  20:37:18  carlson
 * Added prototypes for the callback methods to handle the 'Copy' option.
 *
 * Revision 2.12  1994/02/03  04:03:07  chrisc
 * Added stuff for delete button and new sequence for setting the number of
 *   rings.
 *
 * Revision 2.11  1994/02/02  23:40:49  chrisc
 * Added callback methods for 'import' menu option.
 *
 * Revision 2.10  1994/01/20  23:47:03  carlson
 * Added methods to handle the "On Window" menu option added to the Help menu.
 *
 * Revision 2.9  1994/01/19  20:53:10  carlson
 * Changed to a regular VkWindow and not a VkSimpleWindow to support a
 *   Help menu later.
 *
 * Revision 2.8  1994/01/18  21:26:30  carlson
 * Added prototypes for methods to read fields before saving them.
 *
 * Revision 2.7  1994/01/14  23:52:56  carlson
 * No longer used these callbacks so I got rid of them.
 *
 * Revision 2.6  1994/01/13  19:01:17  carlson
 * Added widgets for new text fields holding prefix and minimum sequences.
 * Removed a bunch of callbacks and combined them into one.
 * Removed a bunch of routines and compiled them into one.
 *
 * Revision 2.5  1994/01/12  02:13:51  carlson
 * Got rid of afterRealizeHook.
 *
 * Revision 2.4  1994/01/11  22:12:15  carlson
 * Removed the reading and writing of file and moved into its own class.
 *
 * Revision 2.3  1994/01/11  20:46:23  carlson
 * Added prototype for writeFile method.
 * Added lots more widgets because we need to save them.
 * Added prototypes for routines to update the various text widgets.
 * Added prototypes for lots more callback functions.
 *
 * Revision 2.2  1994/01/10  19:03:11  carlson
 * Changed fileName variable to be a PathName type.
 * Added prototypes for methods to read a file.
 * Added linked list describing register settings.
 *
 * Revision 2.1  1994/01/06  02:26:14  carlson
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#include <Vk/VkWindow.h>

#include "ModemFile.h"

class VkSubMenu;

class EditModem : public VkWindow
{
  public:
    EditModem (const char *name);
    ~EditModem ();

    virtual const char	*className () { return "EditModem"; }

  protected:
    virtual Widget	setUpInterface (Widget parent);
    virtual void	handleWmDeleteMessage ();
    virtual void	handleWmQuitMessage ();

  private:
    Widget		parentw;
    Widget		numberT;
    Widget		valueT;
    Widget		typeT, regT, flgT;
    Widget		setReg;
    Widget		delReg;
    Widget		docText;
    Widget		rList;
    Widget		prSend, prExpect;
    Widget		abSend, abExpect;
    Widget		srSend, srExpect;
    Widget		sfSend, sfExpect;
    Widget		vrSend, vrExpect;
    Widget		bmSend, bmExpect;
    Widget		anSend, anExpect;
    Widget		sbSend, sbExpect;
    Widget		bRate, bSymbol;
    Widget		setBd;
    Widget		bList;
    PathName		fileName;
    ModemFile		*modemFile;
    VkSubMenu		*fileMenuPane;

    void		updateRegisterList (void);
    void		updateBaudList (void);
    void		updateSequences (void);
    void		readAllFields (void);
    void		readAField (Widget, Widget, char *);

    static void		onWindowCallback (Widget, XtPointer, XtPointer);

    static void		okCallback (Widget, XtPointer, XtPointer);
    static void		saveCallback (Widget, XtPointer, XtPointer);
    static void		quitCallback (Widget, XtPointer, XtPointer);
    static void		importCallback (Widget, XtPointer, XtPointer);
    static void		copyCallback (Widget, XtPointer, XtPointer);

    static void		setRegisterCallback (Widget, XtPointer, XtPointer);
    static void		delRegisterCallback (Widget, XtPointer, XtPointer);
    static void		rNumChangedCallback (Widget, XtPointer, XtPointer);
    static void		rListCallback (Widget, XtPointer, XtPointer);
    static void		sChangedCallback (Widget, XtPointer, XtPointer);
    static void		setBaudCallback (Widget, XtPointer, XtPointer);
    static void		bNumChangedCallback (Widget, XtPointer, XtPointer);
    static void		bListCallback (Widget, XtPointer, XtPointer);

    void		onWindow_cb ();

    void		ok_cb ();
    void		save_cb ();
    void		quit_cb ();
    void		import_cb ();
    void		copy_cb ();

    void		setRegister ();
    void		delRegister ();
    void		rNumChanged (Widget);
    void		rListActivate ();
    void		sChanged (Widget);
    void		setBaud ();
    void		bNumChanged (Widget);
    void		bListActivate ();
};
