/*------------------------------------------------------------------------
 * $Id: HelpAPI.c++,v 2.6 1994/02/08 20:51:20 chrisc Exp $
 *
 * Functions that are called by Viewkit to handle the "Help" button on
 * the menubar.  These stubs make it possible for me to write my own
 * help facility, without having to go through Insight.
 *
 * Revision History:
 *	$Log: HelpAPI.c++,v $
// Revision 2.6  1994/02/08  20:51:20  chrisc
// Added code to allow the displaying of an index.
// Removed parameters that aren't used except for debug.
//
// Revision 2.5  1994/01/20  20:24:35  carlson
// Modified to support new kind of help facility.
//
// Revision 2.4  1993/11/11  21:16:41  carlson
// Found out that "click on help" works automatically without my code.
// Cleaned up some print statements.
//
// Revision 2.3  1993/11/04  00:58:07  carlson
// Added better comment separators.
// Rewrote help message handler to allow multiple shell windows.
// Added functions to register and unregister top-level shell windows to
//   be included in the search for help callbacks.
//
// Revision 2.2  1993/11/03  17:11:57  carlson
// Finally figured out how to create 'theHelpDialog' so that this file
//   doesn't have to 'show', 'open' and 'raise' the window each time and
//   it doesn't have to figure out whether it has been created or not.
//
// Revision 2.1  1993/11/03  02:54:25  carlson
// Revision lifted from vcal and modified for my use.
//
 *------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

#include <iostream.h>

#include <Vk/VkHelp.h>
#include <Vk/VkApp.h>
#include <Vk/VkErrorDialog.h>

#include "HelpDialog.h"

Widget isRegistered (char *);

extern "C" {
    
/*------------------------------------------------------------------------
 * Called during startup of the application.
 *     parm1 = display.
 *	   parm2 = Applications class name
 *	   parm3 = "."
 *------------------------------------------------------------------------*/

int SGIHelpInit(Display *, char *, char *)
{
#ifdef DEBUGp
    cout << "SIGHelpInit called" << endl;
#endif
    return 1;
}

/*------------------------------------------------------------------------
 * Called whenever the user selects one of the help menu functions.
 *	"Click for Help"
 *	    type = "Sysfiles.form.cu.UUCPFiles.dialers"
 *	    c2   = NULL
 *	    c3   = NULL
 *	I discovered that this only occurs for those widgets that don\'t
 *	have a helpCallback setup for them.  Otherwise, it automatically
 *	calls the helpCallbacks.
 *
 *	"Overview"
 *	    type = "overview"
 *	    c2 = NULL
 *	    c3 = NULL
 *
 *	"Keys & Shortcuts"
 *	    type = "keys"
 *	    c2 = NULL
 *	    c3 = NULL
 *------------------------------------------------------------------------*/

int SGIHelpMsg(char *type, char*, char*)
{
    char		tmp1[100], tmp2[100];

#ifdef DEBUGp
    cout << "SGIHelpMsg called" << endl;
    cout << "   Type = " << type << endl;
    if (c2)
	cout << "   c2   = " << c2 << endl;
    if (c3)
	cout << "   c3   = " << c3 << endl;
#endif

    theApplication->busy();
    
    if (strcmp (type, "overview") == 0)
    {
	theHelpDialog->displayHelp ("overviewHelp");
    }
    else if (strcmp (type, "keys") == 0)
    {
	theHelpDialog->displayHelp ("keysHelp");
    }
    else if (strcmp (type, "index") == 0)
    {
	theHelpDialog->displayIndex ();
    }
    else
    {
#ifdef DEBUG1
	Widget			requested, mainw;
	VkSimpleWindow		*win;
	XtCallbackStatus	cb_stat;
	char			*cptr;

	strcpy (tmp2, type);
	win = theApplication->mainWindow ();
	mainw = win->mainWindowWidget ();

	if (cptr = strchr (type, '-'))
	{
	    cptr = strchr (type, '.');
	    if (cptr)
	    {
		*cptr = '\0';
		strcpy (tmp1, type);
		tmp2[0] = '*';
		strcpy (&tmp2[1], cptr+1);
		requested = isRegistered (tmp1);
		if (requested)
		    mainw = requested;
	    }
	}

	requested = XtNameToWidget (XtParent (mainw), tmp2);
	if (requested)
	{
	    cb_stat = XtHasCallbacks (requested, XmNhelpCallback);
	    if (cb_stat == XtCallbackNoList)
	    {
		sprintf (tmp1, "Widget %s has no help.\n", tmp2);
		theErrorDialog->postModal (tmp1);
	    }
	    else if (cb_stat == XtCallbackHasNone)
	    {
		sprintf (tmp1, "Widget %s doesn't have help set up yet.\n",
			 tmp2);
		theErrorDialog->postModal (tmp1);
	    }
	    else
	    {
		XtCallCallbacks (requested, XmNhelpCallback, NULL);
	    }
	}
	else
	{
	    sprintf (tmp1, "Couldn't find widget '%s'\n", tmp2);
	    theErrorDialog->postModal (tmp1);
	}
#endif
#ifdef DEBUGp
	cout << "Click for help occurred for widget " << type << endl;
#endif
    }
    
    theApplication->notBusy();
    return 1;
}

/*------------------------------------------------------------------------
 * Called whenever the user selects Help/Index.
 *	c1 = UUCP
 *	c2 = NULL
 *------------------------------------------------------------------------*/

int SGIHelpIndexMsg(char *c1, char *c2)
{
#ifdef DEBUGp
    cout << "SGIHelpIndexMsg called:\n" << endl;
    cout << "  c1 = " << c1 << endl;
    cout << "  c2 = " << c2 << endl;
#endif
    SGIHelpMsg("index", c1, c2);
    return 1;
}

}

static Widget		registeredWindows[50];
static int		lastRW = 0;

/*------------------------------------------------------------------------
 * registerWindow4Help
 *------------------------------------------------------------------------*/

void registerWindow4Help (Widget w)
{
    registeredWindows[lastRW++] = w;

    if (lastRW >= 50)
    {
	fprintf (stderr, "Help!  Too many windows!!!\n");
	theApplication->quitYourself ();
    }
}

/*------------------------------------------------------------------------
 * unregisterWindow4Help
 *------------------------------------------------------------------------*/

void unregisterWindow4Help (Widget w)
{
    int		i;

    for (i = 0; i < lastRW; i++)
    {
	if (registeredWindows[i] == w)
	    break;
    }

    if (lastRW)
	lastRW--;

    for ( ; i < lastRW; i++)
	registeredWindows[i] = registeredWindows[i+1];
}

/*------------------------------------------------------------------------
 * isRegistered
 *------------------------------------------------------------------------*/

Widget isRegistered (char *str)
{
    Widget	retval = NULL;
    int		i;

    for (i = 0; i < lastRW; i++)
    {
	if (strcmp (XtName (registeredWindows[i]), str) == 0)
	{
	    retval = registeredWindows[i];
	    break;
	}
    }

    return retval;
}
