/*------------------------------------------------------------------------
 * $Id: HelpDialog.c++,v 2.9 1994/02/18 18:52:48 chrisc Exp $
 *
 * Methods for HelpDialog class.
 *
 * This file was lifted from vcal.
 *
 * Revision History:
 *	$Log: HelpDialog.c++,v $
// Revision 2.9  1994/02/18  18:52:48  chrisc
// Added call to function to place text so first line is at top of window.
//
// Revision 2.8  1994/02/15  04:16:12  chrisc
// Changed all references to myApplication to uucpApplication.
//
// Revision 2.7  1994/02/08  20:52:15  chrisc
// Added a method for displaying an index of topics and selecting one.
//
// Revision 2.6  1994/01/20  22:05:47  carlson
// Added resource to make cursor not shown in the window.
// Fixed bug where last page of help file would cause error message to be
//   printed with the help text found.
//
// Revision 2.5  1994/01/20  20:42:49  carlson
// Modified to read help messages from a file rather than char strings.
//
// Revision 2.4  1993/11/11  21:16:41  carlson
// Added the method handleWmQuitMessage because this was added to ViewKit.
//   This keeps the window from bringing down the whole application when
//   the user quits it.
//
// Revision 2.3  1993/11/04  01:38:45  carlson
// Changed the size of the dialog window to be only 20 lines.
//
// Revision 2.2  1993/11/03  18:16:21  carlson
// Added a function that can be accessed by any widget as a callback to
//   bring up the help dialog widget.  All the widget has to do is provide
//   the message to be used in the dialog.
// When setting the help message, also 'show', 'open' and 'raise' the dialog.
//
// Revision 2.1  1993/11/03  00:57:34  carlson
// Revision lifted from vcal and modified for my use.
//
 *------------------------------------------------------------------------*/

#include "HelpDialog.h"
#include "uucp.h"
#include "VkSelectionDialog.h"

#include <stdio.h>
#include <fstream.h>
#include <iostream.h>
#include <limits.h>
#include <string.h>

#include <Xm/Text.h>

HelpDialog		*theHelpDialog;

/*------------------------------------------------------------------------
 * aHelpCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void aHelpCallback (Widget, XtPointer clientData, XtPointer)
{
    char		*text;

    text = (char *)clientData;
    if (text)
	theHelpDialog->displayHelp (text);
}

/*------------------------------------------------------------------------
 * HelpDialog::HelpDialog (const char *)
 *------------------------------------------------------------------------*/

HelpDialog::HelpDialog(const char *docName)
: VkSimpleWindow(docName)
{
    char		line[PATH_MAX];

    strcpy (line, uucpApplication->directory ());
    strcat (line, "/UUCPHelpText");
    fileName = new char [strlen (line) + 1];
    strcpy (fileName, line);
}

/*------------------------------------------------------------------------
 * HelpDialog::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget
HelpDialog::setUpInterface(Widget parent)
{
    Arg			wargs[10];
    int			n;
    Widget		scroll, hs;
    
    n = 0;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNcolumns, 80); n++;
    XtSetArg (wargs[n], XmNrows, 20); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    text = XmCreateScrolledText(parent, "helpText", wargs, n);
    XtManageChild(text);
    
    scroll = XtParent(text);
    n = 0;
    XtSetArg(wargs[n], XmNhorizontalScrollBar, &hs);  n++;
    XtGetValues(scroll, wargs, n);
    if (hs) {
	XtUnmanageChild(hs);
    }
    
    return XtParent(text);
}

/*------------------------------------------------------------------------
 * HelpDialog::handleWmDeleteMessage ()
 *------------------------------------------------------------------------*/

void
HelpDialog::handleWmDeleteMessage()
{
    hide();
}

/*------------------------------------------------------------------------
 * HelpDialog::handleWmQuitMessage ()
 *------------------------------------------------------------------------*/

void
HelpDialog::handleWmQuitMessage()
{
    hide();
}

/*------------------------------------------------------------------------
 * HelpDialog::displayHelp (char *)
 *------------------------------------------------------------------------*/

void
HelpDialog::displayHelp(char *helpText)
{
    enum State { INIT, SEARCHING, FOUND, DONE };

    ifstream		*file;
    char		line[128];
    enum State		_state;

    show ();
    open ();
    raise ();

    uucpApplication->busyCursor ();

    /*----
     * First step, remove anything that might be there already.
     *----*/

    XmTextSetEditable (text, True);
    XmTextSetSelection (text, 0, XmTextGetLastPosition (text),
		XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (text);

    /*----
     * Next step is to read the file and find the text field that was
     * requested.
     *----*/

    file = new ifstream (fileName);
    if (file->fail ())
    {
	XmTextInsert (text, XmTextGetLastPosition (text),
		      "  *** Unable to open file ");
	XmTextInsert (text, XmTextGetLastPosition (text), fileName);
	XmTextInsert (text, XmTextGetLastPosition (text), " ***\n");
	XmTextInsert (text, XmTextGetLastPosition (text),
		      "  *** No help is available ***\n");
    }
    else
    {
	for (_state = INIT; !file->eof () && (_state != DONE); )
	{
	    file->getline (line, sizeof (line));

	    switch (_state)
	    {
	      case INIT:		// Ignore up to first ^L
		if (line[0] == '\f')
		    _state = SEARCHING;
		break;

	      case SEARCHING:
		if (strcmp (helpText, line) == 0)
		    _state = FOUND;
		else
		    _state = INIT;
		break;

	      case FOUND:
		if (line[0] == '\f')
		    _state = DONE;
		else
		{
		    strcat (line, "\n");
		    XmTextInsert (text, XmTextGetLastPosition (text), line);
		}
		break;

	      case DONE:		// Should never occur.
		break;
	    }
	}

	if ((_state != DONE) && (_state != FOUND))
	{
	    XmTextInsert (text, XmTextGetLastPosition (text),
			  "  *** Unable to find help for ");
	    XmTextInsert (text, XmTextGetLastPosition (text), helpText);
	    XmTextInsert (text, XmTextGetLastPosition (text), " ***\n");
	}

	delete file;
    }

    /*----
     * Finish up and raise the window.
     *----*/

    XmTextSetTopCharacter (text, 0);
    XmTextSetEditable (text, False);

    uucpApplication->normalCursor ();
}

/*------------------------------------------------------------------------
 * HelpDialog::displayIndex (void)
 *------------------------------------------------------------------------*/

void HelpDialog::displayIndex (void)
{
    ifstream		*file;
    char		line[128];
    char		**listName, **listTitle, *cptr;
    int			nitems, n;

    file = new ifstream (fileName);
    if (file->fail ())
    {
	show ();
	open ();
	raise ();

	XmTextSetEditable (text, True);
	XmTextSetSelection (text, 0, XmTextGetLastPosition (text),
			XtLastTimestampProcessed (uucpApplication->display ()));
	XmTextRemove (text);

	XmTextInsert (text, XmTextGetLastPosition (text),
		      "  *** Unable to open file ");
	XmTextInsert (text, XmTextGetLastPosition (text), fileName);
	XmTextInsert (text, XmTextGetLastPosition (text), " ***\n");
	XmTextInsert (text, XmTextGetLastPosition (text),
		      "  *** No help is available ***\n");

	XmTextSetEditable (text, False);
    }
    else
    {
	uucpApplication->busyCursor ();

	for (nitems = 0; !file->eof (); )
	{
	    file->getline (line, sizeof (line));

	    if (line[0] == '\f')
		nitems++;
	}

	delete file;

	listName = new char* [nitems+1];
	listTitle = new char* [nitems+1];

	file = new ifstream (fileName);

	for (n = 0; !file->eof (); )
	{
	    file->getline (line, sizeof (line));

	    if (line[0] == '\f')
	    {
		file->getline (line, sizeof (line));
		listName[n] = new char [strlen (line) + 1];
		strcpy (listName[n], line);

		file->getline (line, sizeof (line));
		cptr = line + strlen (line) - 1;
		while ((cptr > line) && (*cptr == ':'))
		    *cptr-- = '\0';

		listTitle[n] = new char [strlen (line) + 1];
		strcpy (listTitle[n], line);

		listTitle[++n] = NULL;
	    }
	}

	delete file;

	uucpApplication->normalCursor ();
	theSelectionDialog->setList (listTitle);
	theSelectionDialog->mustMatch (True);
	if (theSelectionDialog->postAndWait ("Select item to show:") ==
	    VkDialogManager::OK)
	{
	    cptr = theSelectionDialog->getSelection ();
	    for (n = 0; n < nitems; n++)
		if (strcmp (cptr, listName[n]) == 0)
		    break;

	    displayHelp (listName[n]);
	}

	for (n = 0; n < nitems; n++)
	{
	    delete [] listName[n];
	    delete [] listTitle[n];
	}
	delete [] listName;
	delete [] listTitle;
    }

}
