/*---*- C++ -*------------------------------------------------------------
 * $Id: Host.h,v 2.9 1993/11/16 01:09:27 carlson Exp $
 *
 * Header file for describing the hosts defined in the Systems file.
 *
 * Revision History:
 *	$Log: Host.h,v $
 * Revision 2.9  1993/11/16  01:09:27  carlson
 * Added flag to indicate we've been modified.
 *
 * Revision 2.8  1993/10/19  23:51:43  carlson
 * Added prototype for removeChangeCallback method.
 *
 * Revision 2.7  1993/10/14  23:07:12  carlson
 * Added prototype for dump method.
 *
 * Revision 2.6  1993/10/06  00:17:45  chris
 * Added some comments.
 * Added a declaration for the Host class (this fixed a warning message
 *   from cfe indicating I had wrong arguments for the function to be
 *   called as a callback).
 * Added declarations for the methods line and isMatch.
 * All protected elements are now private.
 *
 * Revision 2.5  1993/07/30  23:45:51  chris
 * Added enumeration which will identify which structure element got
 *   changed when the changed callback is called.
 * Changed the name of all the structure elements so that methods could be
 *   created with their names.
 * Created methods for accessing the structure elements.
 * Added methods for changing the structure elements.
 * Added a method for setting up callbacks for when elements get changed.
 * Added private structure for holding callback information.
 * Created top of callback tree.
 *
 * Revision 2.4  1993/07/27  22:48:43  chris
 * Only define ANYBAUD if it isn't already defined.
 *
 * Revision 2.3  1993/07/27  01:05:29  chris
 * Made all elements public for now.
 *
 * Revision 2.2  1993/07/07  23:37:21  chris
 * Changed the symbol which protects against duplicate includes.
 *
 * Revision 2.1  1993/06/30  00:31:25  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _HOST_H_
#define _HOST_H_

#include "linkList.h"

#ifndef ANYBAUD
#define ANYBAUD		-1
#endif

/*------------------------------------------------------------------------
 * class Host
 *	When instantiating, this class must be provided a line from
 *	the "Systems" file that describes a host.  The line will be
 *	parsed and placed in the various private locations to be
 *	referenced or set through the methods.
 *
 *	The line can be returned \(as it should appear in the "Systems"
 *	file\) by using the method \'line\'.
 *
 *	A callback can be set up using \'addChangeCallback\' which
 *	will run a function when any of the values is changed \(using
 *	one of the methods\).
 *------------------------------------------------------------------------*/

class Host;
class Host : public LinkData
{
  public:
    enum Field { SCHEDULE, DEVICE, BAUD, PHONE, CHATSCRIPT };
    typedef void (*callbackFunction)(Host*, Field, void*);

    Host (const char *line);
    ~Host ();

    const char*		schedule (void) { return s; }
    const char*		device (void) { return d; }
    int			baud (void) { return b; }
    const char*		phone (void) { return p; }
    const char*		chat_script (void) { return c; }
    char*		line (void);

    void		setSchedule (char *ns);
    void		setDevice (char *nd);
    void		setBaud (int nb);
    void		setBaud (char *nb);
    void		setPhone (char *np);
    void		setChatScript (char *nc);

    int			wasModified (void) { return modified; }
    void		clearModified (void) { modified = 0; }

    int			isMatch (char *, char *, int);
    int			isMatch (Host *);

    void		addChangeCallback (callbackFunction ncbf,
					   void *clientData);
    void		removeChangeCallback (callbackFunction ncbf,
					      void *clientData);

#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    char	*s;				// Scheduled times
    char	*d;				// Device string
    int		b;				// Baud rate
    char	*p;				// Phone number
    char	*c;				// Chat script

    int		modified;		// True if data has been modified

    struct callbackStruct
    {
	callbackFunction	cbf;
	void			*clientData;
	callbackStruct		*next;
    };
    callbackStruct	*callbacks;
};

#endif
