/*------------------------------------------------------------------------
 * $Id: HostWindow.c++,v 2.21 1994/03/03 00:27:09 carlson Exp $
 *
 * Methods for HostWindow class.
 *
 * Revision History:
 *	$Log: HostWindow.c++,v $
// Revision 2.21  1994/03/03  00:27:09  carlson
// Modified so that it will compile on 5.1 OS versions.  This required taking
//   out the parameters for postAndWait methods that caused dialog window
//   to appear over the parent widget.
//
// Revision 2.20  1994/02/26  03:12:52  carlson
// Removed extraneous class identifiers.
// Remember what the parent widget is for the dialogs.
// Rewrote code that builds the option buttons for the device.  It needed to
//   be more intelligent and allow for multiple devices with the same name and
//   only some may not be available.
// Made dialogs appear over the parent widget.
// Moved 'setDefaults' method to here (rather than have it be inline) so that
//   we can adjust callbacks.
//
// Revision 2.19  1994/02/15  04:16:38  chrisc
// No longer pass dialcodes list when creating this class.  Pick it up
//   from the application.
//
// Revision 2.18  1994/01/20  20:17:38  carlson
// Modified to support new kind of help facility.
//
// Revision 2.17  1994/01/18  05:41:55  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.16  1993/11/15  18:51:50  carlson
// Modified so that the current device is shown in the option menu.
// Modified so when new devices are created, they show up in the device list.
// Had to move some of the work done at startup to be part of updateDeviceList.
// Added callbacks for when dialers or dialcodes are updated.
//
// Revision 2.15  1993/11/09  23:28:03  carlson
// Fixed bug where changed device didn't show up.
// Added code to setup callbacks for other files that might change.
// Fixed bug where I did an extra XmStringFree.
// Added code where I forgot to set the current_dev.
//
// Revision 2.14  1993/11/05  20:09:30  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.13  1993/11/04  23:16:30  carlson
// Added help callbacks to all buttons and labels.
// Moved the section of code that creates the buttons for the option menu
//   used to select the Device into its own method.  This way, it can be
//   changed during the running of the program.
//
// Revision 2.12  1993/10/22  22:25:18  carlson
// Have to check if there are values in the elements of the Host file
//   before puting them into the text boxes.  If we create a new host,
//   it won't have any of these things filled in.
// *** Suggestions ***
//   Need to verify good data before writing file.
//
// Revision 2.11  1993/10/19  23:51:43  carlson
// Added a destructor that removes any callbacks set up.
//
// Revision 2.10  1993/10/14  23:45:24  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.9  1993/09/09  02:11:34  chris
// Use an option menu to display and select one of the available devices.
//   This required removing the code that updated the label when it was
//   changed, removing some extensive comments, removing the menu bar and
//   changing the inserting of labels to inserting menu histories.
//   Also had to identify the one that is currently selected.
//
// Revision 2.8  1993/08/24  19:10:22  chris
// Modified device so that it is a button that creates a pulldown menu of
//   legal devices to set.
// Changed the method for the device callback to take the widget that caused
//   the callback.  This way, we can use the label of the widget to set the
//   new device.
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
// Added comments to explain what I'm doing for the device button.
//
// Revision 2.7  1993/08/02  19:59:20  chris
// Attempt to resolve warning.  Didn't work - submitted a bug.
//
// Revision 2.6  1993/07/31  00:18:18  chris
// Removed inclusion of Host.h because it is done in HostWindow.h.
// Added a static function for Host class to call when an element changes.
// Added a method for Host class to call when an element changes.
// Added static functions and methods for callbacks for all the text fields
//   for when their contents change.
// Changed all direct access to Host variables to call methods.
//
// Revision 2.5  1993/07/29  00:32:25  chris
// Changed text windows to text fields.
// Made device text field uneditable and removed the cursor.
// Saved the widget for the expanded phone label.
//
// Revision 2.4  1993/07/28  19:39:48  chris
// Added code to display the expanded phone number.
// Created a function to expand the phone number.
//
// Revision 2.3  1993/07/27  20:17:19  chris
// Constructor now accepts pointers to other lists.
// Saved these pointers in class variables.
//
// Revision 2.2  1993/07/27  19:35:10  chris
// Modified to be a VkComponent instead of a VkWindow.
// No longer be derived from Host.
// Removed all the callbacks for the menus.
// setUpInterface is now the constructor.
// Added labels for the name of the system.
// Changed all the names of the labels to start lowercase.
// Fixed baud to actually display the baud.
// Removed definition for the menu bar.
//
// Revision 2.1  1993/07/09  00:28:26  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#ifndef OSVERS
#define OSVERS	5100
#endif

#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <iostream.h>

#include "uucp.h"
#include "HostWindow.h"
#include "Dialcode.h"
#include "Device.h"
#include "HelpDialog.h"

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/TextF.h>
#include <Xm/List.h>
#include <Xm/PushBG.h>
#include <Xm/CascadeBG.h>

#include <Vk/VkWarningDialog.h>

/*------------------------------------------------------------------------
 * HostWindow::hostChangedCallback (Host *, void *)
 *------------------------------------------------------------------------*/

void HostWindow::hostChangedCallback (Host *h, Host::Field field,
				      void *clientData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->hostChanged (field, h);
}

/*------------------------------------------------------------------------
 * HostWindow::hostChanged (Host *)
 *------------------------------------------------------------------------*/

void HostWindow::hostChanged (Host::Field field, Host *)
{
    XmString		xmstr;
    char		baud_rate[50];

    switch (field)
    {
      case Host::SCHEDULE:
	XtVaSetValues (w_schedule, XmNvalue, (char *)host->schedule (), NULL);
	break;

      case Host::DEVICE:
	updateDeviceList ();
	break;

      case Host::BAUD:
	if (host->baud () != ANYBAUD)
	    sprintf (baud_rate, "%d", host->baud ());
	else
	    strcpy (baud_rate, "Any");
	XtVaSetValues (w_baud,     XmNvalue, baud_rate,                 NULL);
	break;

      case Host::PHONE:
	XtVaSetValues (w_phone,    XmNvalue, (char *)host->phone (),    NULL);
	xmstr = XmStringCreateSimple ((char *)expandPhone ());
	XtVaSetValues (w_ephone,   XmNlabelString, xmstr,               NULL);
	break;

      case Host::CHATSCRIPT:
	XtVaSetValues (w_chat_script, XmNvalue, (char *)host->chat_script (),
		       NULL);
	break;
    }
}

/*------------------------------------------------------------------------
 * HostWindow::schedChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void HostWindow::schedChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->schedChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * HostWindow::devChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void HostWindow::devChangedCallback (Widget w, XtPointer clientData, XtPointer)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->devChanged (w);
}

/*------------------------------------------------------------------------
 * HostWindow::baudChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void HostWindow::baudChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->baudChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * HostWindow::phoneChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void HostWindow::phoneChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->phoneChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * HostWindow::chatChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void HostWindow::chatChangedCallback (Widget, XtPointer clientData,
				       XtPointer callData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->chatChanged ((XmAnyCallbackStruct *)callData);
}

/*------------------------------------------------------------------------
 * HostWindow::schedChanged (XmAnyCallbackStruct *)
 *------------------------------------------------------------------------*/

void HostWindow::schedChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_schedule, XmNvalue, &str, NULL);
    host->setSchedule (str);
}

/*------------------------------------------------------------------------
 * HostWindow::devChanged (XmAnyCallbackStruct *)
 *------------------------------------------------------------------------*/

void HostWindow::devChanged (Widget w)
{
    XmString		xmstr;
    char		*str;

    XtVaGetValues (w, XmNlabelString, &xmstr, NULL);
    if (XmStringGetLtoR (xmstr, XmFONTLIST_DEFAULT_TAG, &str))
	host->setDevice (str);
}

/*------------------------------------------------------------------------
 * HostWindow::baudChanged (XmAnyCallbackStruct *)
 *------------------------------------------------------------------------*/

void HostWindow::baudChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_baud, XmNvalue, &str, NULL);
    host->setBaud (str);
    updateDeviceList ();
}

/*------------------------------------------------------------------------
 * HostWindow::phoneChanged (XmAnyCallbackStruct *)
 *------------------------------------------------------------------------*/

void HostWindow::phoneChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_phone, XmNvalue, &str, NULL);
    host->setPhone (str);
}

/*------------------------------------------------------------------------
 * HostWindow::chatChanged (XmAnyCallbackStruct *)
 *------------------------------------------------------------------------*/

void HostWindow::chatChanged (XmAnyCallbackStruct *)
{
    char		*str;

    XtVaGetValues (w_chat_script, XmNvalue, &str, NULL);
    host->setChatScript (str);
}

/*------------------------------------------------------------------------
 * HostWindow::HostWindow (const char *)
 *------------------------------------------------------------------------*/

HostWindow::HostWindow (Widget parent, Host *h, SmartList *dev,
			SmartList *dlrs)
: VkComponent (h->theKey ())
{
    Arg		wargs[10];
    int		n;
    XmString	xmstr;
    Widget	label;
    char	baud_rate[50];

    _parent = parent;
    dev_popup = NULL;
    w_device = NULL;

    host = h;
    devList = dev;
    dlrList = dlrs;
    dlcdsList = uucpApplication->dialcodes ();

    devList->addChangeCallback (devListCallback, (void *)this);
    dlrList->addChangeCallback (dlrListCallback, (void *)this);
    if (dlcdsList)
	dlcdsList->addChangeCallback (dlcdListCallback,
				      (void *)this);

    /*----
     * Create a row/column widget to hold labels and text for host
     * data.  This way, everything should line up properly.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, 2); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    _baseWidget = XmCreateRowColumn (parent, "hostWindow", wargs, n);
    installDestroyHandler ();

    /*----
     * Create information.
     * For the RowColumn widget, we have to give all the labels
     * first, then all the text portions.
     *----*/

    n = 0;
    label = XmCreateLabel (_baseWidget, "sysNameLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_sys_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "scheduleLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_sch_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "deviceLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_dev_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "baudLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_bd_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "phoneLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_phn_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "ePhoneLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_ephn_help");

    n = 0;
    label = XmCreateLabel (_baseWidget, "chatLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_cht_help");

/*----
 *----*/

    n = 0;
    xmstr = XmStringCreateSimple ((char *)host->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    label = XmCreateLabel (_baseWidget, "sysNameText", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "hw_sys_help");

    n = 0;
    if (host->schedule ())
    {
	XtSetArg (wargs[n], XmNvalue, (char *)host->schedule ()); n++;
    }
    w_schedule = XmCreateTextField (_baseWidget, "scheduleText", wargs, n);
    XtManageChild (w_schedule);

    XtAddCallback (w_schedule, XmNactivateCallback, schedChangedCallback,
		   this);
    XtAddCallback (w_schedule, XmNhelpCallback, aHelpCallback, "hw_sch_help");

    /*----
     * Create an option menu for selecting a device to use.
     *----*/

    updateDeviceList ();

    n = 0;
    if (current_dev)
	XtSetArg (wargs[n], XmNmenuHistory, current_dev); n++;
    XtSetArg (wargs[n], XmNsubMenuId, dev_popup); n++;
    w_device = XmCreateOptionMenu (_baseWidget, "deviceText", wargs, n);
    XtManageChild (w_device);

    XtAddCallback (w_device, XmNhelpCallback, aHelpCallback, "hw_dev_help");

    n = 0;
    if (host->baud () != ANYBAUD)
	sprintf (baud_rate, "%d", host->baud ());
    else
	strcpy (baud_rate, "Any");
    XtSetArg (wargs[n], XmNvalue, baud_rate); n++;
    w_baud = XmCreateTextField (_baseWidget, "baudText", wargs, n);
    XtManageChild (w_baud);

    XtAddCallback (w_baud, XmNactivateCallback, baudChangedCallback,
		   this);
    XtAddCallback (w_baud, XmNhelpCallback, aHelpCallback, "hw_bd_help");

    n = 0;
    if (host->phone ())
    {
	XtSetArg (wargs[n], XmNvalue, (char *)host->phone ()); n++;
    }
    w_phone = XmCreateTextField (_baseWidget, "phoneText", wargs, n);
    XtManageChild (w_phone);

    XtAddCallback (w_phone, XmNactivateCallback, phoneChangedCallback,
		   this);
    XtAddCallback (w_phone, XmNhelpCallback, aHelpCallback, "hw_phn_help");

    n = 0;
    xmstr = XmStringCreateSimple ((char *)expandPhone ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    w_ephone = XmCreateLabel (_baseWidget, "ePhoneText", wargs, n);
    XtManageChild (w_ephone);
    XmStringFree (xmstr);

    XtAddCallback (w_ephone, XmNhelpCallback, aHelpCallback, "hw_ephn_help");

    n = 0;
    if (host->chat_script ())
    {
	XtSetArg (wargs[n], XmNvalue, (char *)host->chat_script ()); n++;
    }
    w_chat_script = XmCreateTextField (_baseWidget, "chatText", wargs, n);
    XtManageChild (w_chat_script);

    XtAddCallback (w_chat_script, XmNactivateCallback, chatChangedCallback,
		   this);
    XtAddCallback (w_chat_script, XmNhelpCallback, aHelpCallback,
		   "hw_cht_help");

    host->addChangeCallback (hostChangedCallback,
			     (void *)this);
}

/*------------------------------------------------------------------------
 * HostWindow::~HostWindow ()
 *------------------------------------------------------------------------*/

HostWindow::~HostWindow ()
{
    devList->removeChangeCallback (devListCallback, (void *)this);
    dlrList->removeChangeCallback (dlrListCallback, (void *)this);
    if (dlcdsList)
	dlcdsList->removeChangeCallback (dlcdListCallback,
					 (void *)this);
    host->removeChangeCallback (hostChangedCallback,
				(void *)this);
}

/*------------------------------------------------------------------------
 * HostWindow::expandPhone ()
 *------------------------------------------------------------------------*/

const char* HostWindow::expandPhone (void)
{
    static char		expanded[80];
    char		token[20];
    char		*c1, *c2;
    Dialcode		*dc;

    if (host->phone () == NULL)
	return "";

    expanded[0] = '\0';
    for (c1 = (char *)host->phone (), c2 = token; *c1; c1++)
    {
	if (isalpha (*c1))
	    *c2++ = *c1;
	else
	    break;
    }
    *c2 = '\0';

    if (token[0] != '\0')
    {
	if (!dlcdsList)
	{
	    dlcdsList = uucpApplication->dialcodes ();
	    if (dlcdsList)
		dlcdsList->addChangeCallback (dlcdListCallback,
					      (void *)this);
	}

	if (dlcdsList)
	{
	    dc = (Dialcode *)dlcdsList->find (token);
	    if (dc)
	    {
		strcpy (expanded, dc->script ());
		strcat (expanded, c1);
	    }
	}
    }

    if (expanded[0] == '\0')
	strcpy (expanded, (char *)host->phone ());

    return expanded;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * HostWindow::dump (char *)
 *------------------------------------------------------------------------*/

void HostWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "HostWindow dump (" << this << "):" << endl;
    cout << indent << "  host       = " << host << endl;
    cout << indent << "  devList    = " << devList << endl;
    cout << indent << "  dlrList    = " << dlrList << endl;
    cout << indent << "  dlcdsList  = " << dlcdsList << endl;
    cout << indent << "  address of callback = " << &hostChangedCallback
	 << endl;
    host->dump (nextIndent);
}
#endif

/*------------------------------------------------------------------------
 * HostWindow::updateDeviceList (void)
 *------------------------------------------------------------------------*/

void HostWindow::updateDeviceList (void)
{
    Widget		item;
    Device		*aDev;
    char		*cptr;
    Boolean		legalDev;

    if (dev_popup)
	XtDestroyWidget (dev_popup);

    dev_popup = XmCreateSimplePulldownMenu (_baseWidget, "deviceMenu", NULL,
					    0);

    /*----
     * Now scan the list of devices and create buttons for each.
     *----*/

    cptr = NULL;
    current_dev = NULL;
    item = NULL;
    legalDev = False;
    for (aDev = (Device *)devList->first (); aDev;
	 aDev = (Device *)devList->next ())
    {
	/*----
	 * Make sure we don't list device names twice (even though
	 * there may be more than one device name in the devices file
	 * with the same name).  Also, desensitize those devices
	 * that we can't use because the baud rate is wrong.
	 *----*/

	if ((cptr == NULL) || strcmp (cptr, aDev->theKey ()))
	{
	    if (item && !legalDev)
		XtSetSensitive (item, False);
	    legalDev = False;

	    cptr = (char *)aDev->theKey ();
	    item = XmCreatePushButtonGadget (dev_popup, cptr, NULL, 0);
	    XtManageChild (item);

	    XtAddCallback (item, XmNactivateCallback, devChangedCallback,
			   this);
	    /*----
	     * If this entry is the currently defined device, save the
	     * widget.
	     *----*/

	    if ((current_dev == NULL) && host->device ())
		if (strcmp (cptr, (char *)host->device ()) == 0)
		    current_dev = item;
	}

	/*----
	 * Determine if this device is usable.  If yes, set the flag.
	 * We'll desensitize it later.
	 *----*/

	if ((aDev->baud () == ANYBAUD) || (host->baud () == ANYBAUD) ||
	    (aDev->baud () == host->baud ()))
	    legalDev = True;
    }

    if (item && !legalDev)
	XtSetSensitive (item, False);

    /*----
     * If we didn't find the device that this host points to,
     * add this device anyway but make it insensitive.
     *----*/

    if (current_dev == NULL)
    {
	cptr = (char *)host->device ();
	item = XmCreatePushButtonGadget (dev_popup, cptr, NULL, 0);
	XtManageChild (item);
	XtSetSensitive (item, False);

	current_dev = item;
    }

    if (!XtIsSensitive (current_dev))
	theWarningDialog->postAndWait ("deviceSelectWarning"
#if OSVERS >= 5200
				       , True, False, False, NULL, NULL,
				       _parent
#endif
				       );

    if (w_device)
    {
	XtVaSetValues (w_device, XmNsubMenuId, dev_popup, NULL);
	XtVaSetValues (w_device, XmNmenuHistory, current_dev, NULL);
    }
}

/*------------------------------------------------------------------------
 * HostWindow::devListCallback (SmartList *, SmartList::ACTION, void *)
 *------------------------------------------------------------------------*/

void HostWindow::devListCallback (SmartList *, SmartList::ACTION action,
				  void *clientData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->devListChanged (action);
}

/*------------------------------------------------------------------------
 * HostWindow::devListChanged (SmartList::ACTION)
 *------------------------------------------------------------------------*/

void HostWindow::devListChanged (SmartList::ACTION)
{
#ifdef DEBUGp
    cout << "devListChanged callback occurred in " << host->theKey () << endl;
#endif
    updateDeviceList ();
}

/*------------------------------------------------------------------------
 * HostWindow::dlrListCallback (SmartList *, SmartList::ACTION, void *)
 *------------------------------------------------------------------------*/

void HostWindow::dlrListCallback (SmartList *, SmartList::ACTION action,
				  void *clientData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->dlrListChanged (action);
}

/*------------------------------------------------------------------------
 * HostWindow::dlrListChanged (SmartList::ACTION)
 *------------------------------------------------------------------------*/

void HostWindow::dlrListChanged (SmartList::ACTION)
{
    cout << "dlrListChanged callback occurred in " << host->theKey () << endl;
}

/*------------------------------------------------------------------------
 * HostWindow::dlcdListCallback (SmartList *, SmartList::ACTION, void *)
 *------------------------------------------------------------------------*/

void HostWindow::dlcdListCallback (SmartList *, SmartList::ACTION action,
				  void *clientData)
{
    HostWindow		*hw;

    hw = (HostWindow *)clientData;
    hw->dlcdListChanged (action);
}

/*------------------------------------------------------------------------
 * HostWindow::dlcdListChanged (SmartList::ACTION)
 *------------------------------------------------------------------------*/

void HostWindow::dlcdListChanged (SmartList::ACTION)
{
    cout << "dlcdListChanged callback occurred in " << host->theKey () << endl;
}

/*------------------------------------------------------------------------
 * HostWindow::setDefaults (SmartList *, SmartList *)
 *------------------------------------------------------------------------*/

void HostWindow::setDefaults (SmartList *devs, SmartList *dlrs)
{
    devList->removeChangeCallback (devListCallback, (void *)this);
    dlrList->removeChangeCallback (dlrListCallback, (void *)this);
    devList = devs;
    dlrList = dlrs;
    devList->addChangeCallback (devListCallback, (void *)this);
    dlrList->addChangeCallback (dlrListCallback, (void *)this);
}
