/*---*- C++ -*------------------------------------------------------------
 * $Id: HostWindow.h,v 2.15 1994/02/26 01:10:39 carlson Exp $
 *
 * Header file for HostWindow class.
 *
 * Revision History:
 *	$Log: HostWindow.h,v $
 * Revision 2.15  1994/02/26  01:10:39  carlson
 * Made 'setDefaults' not inline.
 * Added place to keep the parent widget for dialogs.
 *
 * Revision 2.14  1994/02/15  04:18:38  chrisc
 * Removed parameter to add dialcodes list from methods that expected it.
 *
 * Revision 2.13  1993/11/09  02:23:39  carlson
 * Added prototypes for callbacks.
 *
 * Revision 2.12  1993/11/05  20:09:30  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.11  1993/11/04  22:03:54  carlson
 * Added space to hold the pupup and currently selected button widgets
 *   for the device.  This allows me to place the building of this menu
 *   in its own method.
 *
 * Revision 2.10  1993/10/19  23:51:43  carlson
 * Added prototype for destructor.
 *
 * Revision 2.9  1993/10/14  23:07:13  carlson
 * Added prototype for dump method.
 *
 * Revision 2.8  1993/08/24  01:42:31  chris
 * Changed the parameter to the method devChanged to be the widget that
 *   caused the callback to be called.
 *
 * Revision 2.7  1993/08/12  15:23:45  chris
 * Added inline method for setting the defaults after instantiation.
 *
 * Revision 2.6  1993/07/31  00:11:16  chris
 * Include Host.h because we need to reference an internal identifier.
 * Added prototypes for all callback functions and methods.
 *
 * Revision 2.5  1993/07/29  00:03:35  chris
 * Added widget for expanded phone label.
 *
 * Revision 2.4  1993/07/28  17:53:08  chris
 * Added method for expanding the phone number.
 *
 * Revision 2.3  1993/07/27  20:23:09  chris
 * Constructor now accepts pointers to other lists.
 * Created variables to hold these pointers in.
 *
 * Revision 2.2  1993/07/27  01:02:06  chris
 * Changed from VkWindow to VkComponent.
 * Removed all callbacks.
 * No longer derived from Host.
 *
 * Revision 2.1  1993/07/08  02:50:15  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _HOSTWINDOW_H_
#define _HOSTWINDOW_H_

#include <Vk/VkComponent.h>
#include "SmartList.h"
#include "Host.h"

class HostWindow : public VkComponent
{
  public:
    HostWindow (Widget parent, Host *h, SmartList *dev = NULL,
		SmartList *dlrs = NULL);
    ~HostWindow ();
    const char*		expandPhone (void);

    void		setDefaults (SmartList *devs, SmartList *dlrs);

    virtual const char*	className () { return "HostWindow"; }
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    Widget		_parent;
    Host		*host;
    SmartList		*devList;
    SmartList		*dlrList;
    SmartList		*dlcdsList;
    Widget		w_schedule;
    Widget		w_device;	// Option menu widget
    Widget		dev_popup;	// Pulldown menu widget
    Widget		current_dev;	// Current pushbutton widget
    Widget		w_baud;
    Widget		w_phone;
    Widget		w_ephone;
    Widget		w_chat_script;

    void		updateDeviceList (void);

    static void		hostChangedCallback (Host *, Host::Field, void *);
    static void		schedChangedCallback (Widget, XtPointer, XtPointer);
    static void		devChangedCallback (Widget, XtPointer, XtPointer);
    static void		baudChangedCallback (Widget, XtPointer, XtPointer);
    static void		phoneChangedCallback (Widget, XtPointer, XtPointer);
    static void		chatChangedCallback (Widget, XtPointer, XtPointer);

    static void		devListCallback (SmartList*, SmartList::ACTION, void*);
    static void		dlrListCallback (SmartList*, SmartList::ACTION, void*);
    static void		dlcdListCallback (SmartList*, SmartList::ACTION,
					  void*);

    void		hostChanged (Host::Field, Host *);
    void		schedChanged (XmAnyCallbackStruct *);
    void		devChanged (Widget);
    void		baudChanged (XmAnyCallbackStruct *);
    void		phoneChanged (XmAnyCallbackStruct *);
    void		chatChanged (XmAnyCallbackStruct *);

    void		devListChanged (SmartList::ACTION);
    void		dlrListChanged (SmartList::ACTION);
    void		dlcdListChanged (SmartList::ACTION);
};

#endif
