/*---*- C++ -*------------------------------------------------------------
 * $Id: ModemFile.h,v 2.6 1994/02/25 20:40:33 carlson Exp $
 *
 * Header file for ModemFile class.
 *
 * Revision History:
 *	$Log: ModemFile.h,v $
 * Revision 2.6  1994/02/25  20:40:33  carlson
 * Modified readFile and writeFile to take a parameter telling it which file to read
 *   or write.
 * Added prototypes for insertFile methods.
 *
 * Revision 2.5  1994/02/03  04:06:05  chrisc
 * Added prototypes for methods to handle deleting of a register.
 * Added prototypes for methods to support a sequence for setting the
 *   answering ability of the modem.
 *
 * Revision 2.4  1994/01/17  20:09:26  carlson
 * Added methods to get list of bauds.
 *
 * Revision 2.3  1994/01/13  19:36:33  carlson
 * Added a number of methods to retrieve specific records from the file.
 *   These methods will also provide expansions if requested.
 *
 * Revision 2.2  1994/01/12  00:40:46  carlson
 * Added conditional compilation to avoid duplicate includes.
 *
 * Revision 2.1  1994/01/11  22:12:15  carlson
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _MODEMFILE_H_
#define _MODEMFILE_H_

#include "PathName.h"
#include "Registers.h"
#include "Sequences.h"

class ModemFile
{
  public:
    ModemFile (PathName &name);
    ModemFile (const char *name);
    ~ModemFile () { }

    int			insertFile (PathName &name);
    int			insertFile (const char *name);

    int			error (void) { return _error; }
    void		save (void) { writeFile (fileName); }

    Registers		*firstRegister (void);
    Registers		*nextRegister (void);
    Registers		*findRegister (const char *reg);
    void		deleteRegister (const char *reg);
    void		deleteRegister (Registers *reg);

    Sequences		*firstSequence (void);
    Sequences		*nextSequence (void);
    Sequences		*findSequence (const char *seq);

    char		*firstBaud (void);
    char		*nextBaud (void);

    void		addRegister (Registers *reg);
    void		addSequence (Sequences *seq);

    Sequences		*getBaudSequence (char *);
    Sequences		*getBaudSequence (int);

    char		*getCheckSend (void);
    char		*getCheckExpect (void);
    char		*getSetRSend (Registers *);
    char		*getSetRExpect (void);
    char		*getSetFSend (Registers *);
    char		*getSetFExpect (void);
    char		*getSaveSend (void);
    char		*getSaveExpect (void);
    char		*getSpeedSend (char *);
    char		*getSpeedSend (int);
    char		*getSpeedExpect (void);
    char		*getPrefix (char *);
    char		*getMinimumSend (void);
    char		*getMinimumExpect (void);
    char		*getBaudCode (char *);
    char		*getBaudCode (int);
    char		*getAnswerSend (char *);
    char		*getAnswerSend (int);
    char		*getAnswerExpect (void);

  private:
    LinkList		registers;
    LinkList		sequences;
    int			_error;
    PathName		fileName;
    Sequences		*lastBaud;

    void		readFile (PathName &);
    void		writeFile (PathName &);

    char		*expandRegister (char *str, char *rnum, char *rval);
    char		*expandFlag (char *str, char *fstr, char *fval);
    char		*expandBaud (char *str, char *baud);
    char		*expandAnswer (char *str, int rings);
};

#endif
