/*------------------------------------------------------------------------
 * $Id: ModemSetup.c++,v 2.15 1994/02/25 23:03:43 carlson Exp $
 *
 * Methods for ModemSetup class.
 *
 * Revision History:
 *	$Log: ModemSetup.c++,v $
// Revision 2.15  1994/02/25  23:03:43  carlson
// Added help menu item to provide help for errors.
// Cleaned up a bunch of messages.
// Changed the names of the menu buttons so we can name them from resource files.
// Decided to give this window a constant name so we can refer to it with resources.
//
// Revision 2.14  1994/02/24  00:10:39  carlson
// Modified to remove information about ringer.  This should be in setting up inittab.
//
// Revision 2.13  1994/02/15  04:19:51  chrisc
// Changed all references to myApplication to uucpApplication.
//
// Revision 2.12  1994/02/03  04:07:53  chrisc
// Added buttons and widgets to support the two methods of setting a
//   modem to answer the phone.  Currently, these buttons change the
//   state of the class but don't do anything to the modem.
//
// Revision 2.11  1994/01/24  17:16:11  carlson
// Added "On Window" callback methods.
// Added quit callback methods.
// Changed from VkSimpleWindow to VkWindow so we can have a menu bar.
// Added/changed help callback stuff.
// Added menubar and file pulldown menu.
//
// Revision 2.10  1994/01/20  20:20:31  carlson
// Added temporary help callbacks.
//
// Revision 2.9  1994/01/19  02:27:14  carlson
// Added a message when all tests are to be performed to indicate if all the
//   tests completed successfully.
// Added a label to the individual test buttons.
//
// Revision 2.8  1994/01/19  01:51:34  carlson
// Destructor now closes the terminal device if it is open.
// If a new device is specified, also closes the terminal device currently
//   opened.
// No longer perform SAVE function from automatic setup.  Make this user
//   requestable only.
// Put messages in interaction window that are also sent to status window.
// Changed what verbosity is required to get certain messages.
// No longer open and close terminal device every time we do a test.  This
//   can cause modem to reset and thus foul the test.
// Only set the interface speed if it isn't already set to the speed
//   requested.
// If full verbosity not set but some is, provide some feedback whether
//   responses from modem succeeded or failed.
// Added code to download register settings.
// Added code to download flag settings.
// Added code to save modem state.
// Moved 'Save' button from test area to buttons at bottom.
// Added methods for handling Window manager messages.
//
// Revision 2.7  1994/01/18  22:26:58  carlson
// Forgot to initialize _test_state when instantiating.
// No longer always update device string.  Now, update it only if it changes.
//   This way, when it changes, we automatically reset the known baud to
//   require the user to re-determine it.
// Enhanced some of the messages sent to the screen.
// Completed developing the speed setting test.
// Added new method to compute the baud rate the user wants to use.
//
// Revision 2.6  1994/01/17  20:17:09  carlson
// Removed all the stuff that had to do with handling baud rates and terminal
//   functions and moved them into a separate class.
// Now 'test' takes more steps because we separated them in the Terminal class.
// Added a method called test_input which just reads characters and tests
//   them against what is expected.
//
// Revision 2.5  1994/01/14  23:43:01  carlson
// Defined a symbol indicating the amount of time to timeout per character.
// Moved the setting up of the signal handler to the instantiation method.
// Added an option menu that selects verbosity level.
// Got rid of a number of callbacks to make one do all the work (since they
//   were so similar).
// Changed where some messages are sent.
// Added a bunch of messages when errors occur.
// Reworked finding baud to use new steps in test_current_baud.
// Modified test_current_baud to set baud as a separate step.
// Started doing work on setting interface speed test.
// Added some comments to help figure out what widgets are being built.
// Cleaned up some comments.
// Modified some functions to return a success/failure enum value.
// Moved the steps for writing to the tty to a separate method.
// Some print statements aren't done unless the verbosity is high enough.
// Massive cleanups on how/when things are done.
//
// Revision 2.4  1994/01/14  08:04:49  carlson
// Set alarm time to 15 seconds.  *** This should be changed back later ***
// When an interrupt or error occurs, close the device.
// Modified cases where we test a baud to make sure to only try baud rates
//   that are supported by the modem.
// Found that errno is not an integer but a function call.  This caused
//   problems with trying to set up the line to print.  Modified all methods
//   that referenced errno to save it right after executing the function
//   that set it.
// Changed a lot of setting up the line to print to the interaction window
//   to just write to the window.
// Set up conditional compiles that won't really write to or read from the
//   device.  Just does the other setup.
// Added printout when a timeout occurs.
// Removed the convert_string method and made it a function in systemfiles.
// Modified the setting up of the device to take advantage of the initial
//   settings.  This way, if a user selects ttyf2 as opposed to ttym2 or
//   ttyd2, they will get different line settings.
// Set up the signal handler and alarm at a different step.
// Don't try and flush the device unless a legal baud was set.
// Added code to get the command to send.
// When writing, make sure the write works.
// Added code to get the string to expect in return.
//
// Revision 2.3  1994/01/13  05:40:16  carlson
// Started adding code to perform the initial test of the modem.
// Added a function (not a method) for handling alarm signals.
// Modified instantiation code to try and get baud rate information from
//   the file /usr/include/sys/ttydev.h.
// Set up the states in 'test' and got them pretty much finished.
// Added methods for supporting the device.
//
// Revision 2.2  1994/01/13  02:02:23  carlson
// Added includes to be able to use warning and info dialogs.
// Defined states that are common and used in _state.
// Initialized _state and _workProcID.
// Created another method that gets the device string so it can be called
//   by other methods.
// Created a workProcCallback method that is called by the X workProc feature.
//   This allows us to have our tests running but still update the windows
//   and handle keyboard/mouse events.
// Added a debug method that prints the current state.
// Got rid of all the _cb routines and combined into one, setup_test.
// Each test is now a method.
// Filled in some dummy steps for each test.
//
// Revision 2.1  1994/01/12  16:53:54  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include "uucp.h"
#include "ModemSetup.h"
#include "systemfiles.h"
#include "HelpDialog.h"
#include "Registers.h"
#include "Sequences.h"

#include <unistd.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <termio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#include <Vk/VkErrorDialog.h>
#include <Vk/VkWarningDialog.h>
#include <Vk/VkInfoDialog.h>
#include <Vk/VkMenu.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkMenuBar.h>
#include <Vk/VkHelpPane.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/LabelG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>
#include <Xm/SeparatoG.h>
#include <Xm/ScrolledW.h>
#include <Xm/ToggleBG.h>

// #define TEST_WRITE

void registerWindow4Help (Widget);
void unregisterWindow4Help (Widget);

#define INITIAL_STATE		0
#define DONE_STATE		-1
#define NO_STATE		-2

#define MAX_UINT		((unsigned int)-1)

/*------------------------------------------------------------------------
 * ModemSetup::onWindowCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::onWindowCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemSetup		*mw;

    mw = (ModemSetup *)clientData;
    mw->onWindow_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::onWindow_cb ()
 *------------------------------------------------------------------------*/

void ModemSetup::onWindow_cb ()
{
    theHelpDialog->displayHelp ("modemSetup");
}

/*------------------------------------------------------------------------
 * ModemSetup::onErrorCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::onErrorCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemSetup		*mw;

    mw = (ModemSetup *)clientData;
    mw->onError_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::onError_cb ()
 *------------------------------------------------------------------------*/

void ModemSetup::onError_cb ()
{
    theHelpDialog->displayHelp ("modemSetupErrors");
}

/*------------------------------------------------------------------------
 * ModemSetup::quitCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::quitCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemSetup		*em;

    em = (ModemSetup *)clientData;
    em->quit_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::quit_cb ()
 *------------------------------------------------------------------------*/

void ModemSetup::quit_cb ()
{
    unregisterWindow4Help (parentw);

    delete this;
}

/*------------------------------------------------------------------------
 * ModemSetup::ModemSetup (const char *)
 *------------------------------------------------------------------------*/

ModemSetup::ModemSetup (const char *name) : VkWindow ("modemSetup")
{
    char		*tmp;
    const char		*ctmp;

    fileName = name;
    devString = NULL;
    _error = 0;
    _interrupt = 0;
    _state = NO_STATE;
    _workProcID = 0;
    _verbosity = 0;
    _baud = 0;
    _test_state = INIT;

    ctmp = strrchr (name, '/');
    if (ctmp)
	ctmp++;
    else
	ctmp = name;
    tmp = Copy (ctmp);
    tmp[strlen (tmp) - 4] = '\0';
    setTitle (tmp);
    delete [] tmp;
}

/*------------------------------------------------------------------------
 * ModemSetup::~ModemSetup ()
 *------------------------------------------------------------------------*/

ModemSetup::~ModemSetup ()
{
    if (terminal.currentState () != Terminal::CLOSED)
    {
	clearInteraction ();
	clearStatus ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	XSync (uucpApplication->display (), False);
	sleep (2);
    }

    unregisterWindow4Help (parentw);

    if (modemFile)
	delete modemFile;
}

/*------------------------------------------------------------------------
 * ModemSetup::clearInteraction (void)
 *------------------------------------------------------------------------*/

void ModemSetup::clearInteraction (void)
{
    XmTextSetEditable (interaction, True);
    XmTextSetSelection (interaction, 0, XmTextGetLastPosition (interaction),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (interaction);
    XmTextSetEditable (interaction, False);
}

/*------------------------------------------------------------------------
 * ModemSetup::appendInteraction (const char *)
 *------------------------------------------------------------------------*/

void ModemSetup::appendInteraction (const char *line)
{
    XmTextSetEditable (interaction, True);
    XmTextInsert (interaction, XmTextGetLastPosition (interaction),
		  (char *)line);
    XmTextShowPosition (interaction, XmTextGetLastPosition (interaction));
    XmTextSetEditable (interaction, False);
}

/*------------------------------------------------------------------------
 * ModemSetup::clearStatus (void)
 *------------------------------------------------------------------------*/

void ModemSetup::clearStatus (void)
{
    XmTextSetEditable (status, True);
    XmTextSetSelection (status, 0, XmTextGetLastPosition (status),
			XtLastTimestampProcessed (uucpApplication->display ()));
    XmTextRemove (status);
    XmTextSetEditable (status, False);
}

/*------------------------------------------------------------------------
 * ModemSetup::appendStatus (const char *)
 *------------------------------------------------------------------------*/

void ModemSetup::appendStatus (const char *line)
{
    XmTextSetEditable (status, True);
    XmTextInsert (status, XmTextGetLastPosition (status), (char *)line);
    XmTextShowPosition (status, XmTextGetLastPosition (status));
    XmTextSetEditable (status, False);
}

/*------------------------------------------------------------------------
 * ModemSetup::deviceChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::deviceChangedCallback (Widget, XtPointer clientData,
					XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;
    ms->deviceChanged_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::deviceChanged_cb (void)
 *------------------------------------------------------------------------*/

void ModemSetup::deviceChanged_cb (void)
{
    getDevice ();
}

/*------------------------------------------------------------------------
 * ModemSetup::getDevice ()
 *------------------------------------------------------------------------*/

void ModemSetup::getDevice ()
{
    char		*dptr;

    XtVaGetValues (device, XmNvalue, &dptr, NULL);

    /*----
     * Check if it has changed.
     *----*/

    if (!devString || strcmp (dptr, devString))
    {
	if (devString)
	    XtFree (devString);

	devString = dptr;
	if (devString[0] != '/')
	{
	    theErrorDialog->postAndWait
		("Currently a device specification must\n"
		 "be a fully qualified path name.");
	    XtFree (devString);
	    devString = NULL;
	}
	_baud = 0;			// Make sure to compute baud again.
	if (terminal.currentState () != Terminal::CLOSED)
	{
	    if (_verbosity > 0)
		appendInteraction ("  Closing down device.\n");
	    terminal.close ();
	}
    }
}

/*------------------------------------------------------------------------
 * ModemSetup::ringsChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::ringsChangedCallback (Widget w, XtPointer clientData,
					XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;
    ms->ringsChanged_cb (XtName (w));
}

/*------------------------------------------------------------------------
 * ModemSetup::ringsChanged_cb (char *)
 *------------------------------------------------------------------------*/

void ModemSetup::ringsChanged_cb (char *wname)
{
    char		*cptr;

    if (ringer == COUNT)
    {
	XtVaGetValues (numRings, XmNvalue, &cptr, NULL);
	for (wname = cptr; *wname; wname++)
	    if (!isdigit (*wname))
		break;
	if (*wname)
	{
	    theErrorDialog->postAndWait ("Rings must be all digits.");
	    return;
	}

	ringCount = atoi (cptr);
    }
    else
    {
	if (strcmp (wname, "on"))
	    ringCount = 1;
	else if (strcmp (wname, "off"))
	    ringCount = 0;
    }
}

/*------------------------------------------------------------------------
 * ModemSetup::workProcCallback (XtPointer)
 *------------------------------------------------------------------------*/

Boolean ModemSetup::workProcCallback (XtPointer clientData)
{
    ModemSetup		*ms;
    Boolean		ret_val;

    ms = (ModemSetup *)clientData;
    ret_val = ms->workProc ();

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::workProc ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::workProc ()
{
    Boolean		ret_val = True;

#ifdef DEBUG1
    debugOutput ("workProc being called");
#endif

    switch (_test_state)
    {
      case INIT:
	XtRemoveWorkProc (_workProcID);
	break;

      case SETUP:
	ret_val = setup ();
	if (ret_val)
	    _test_state = INIT;
	break;

      case TEST:
	ret_val = test ();
	if (ret_val)
	    _test_state = INIT;
	break;

      case SPEED:
	ret_val = speed ();
	if (ret_val)
	    _test_state = INIT;
	break;

      case REGS:
	ret_val = regs ();
	if (ret_val)
	    _test_state = INIT;
	break;

      case FLAGS:
	ret_val = flags ();
	if (ret_val)
	    _test_state = INIT;
	break;

      case SAVE:
	ret_val = save ();
	if (ret_val)
	    _test_state = INIT;
	break;
    }

#ifdef DEBUG1
    debugOutput ("workProc exiting");
#endif

    return ret_val;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * ModemSetup::debugOutput (char *)
 *------------------------------------------------------------------------*/

void ModemSetup::debugOutput (char *msg)
{
    char		line[100];
    int			n;

#define CASE(a)		case a: strcat (line,#a); break

    sprintf (line, "  %s\n", msg);
    appendStatus (line);


    strcpy (line, "    state = ");
    switch (_state)
    {
	CASE (NO_STATE);
	CASE (DONE_STATE);
	CASE (INITIAL_STATE);

      default:
	n = strlen (line);
	sprintf (&line[n], "%d", _state);
    }

    strcat (line, " test_state = ");
    switch (_test_state)
    {
	CASE (INIT);
	CASE (SETUP);
	CASE (TEST);
	CASE (SPEED);
	CASE (REGS);
	CASE (FLAGS);
	CASE (SAVE);
    }

    n = strlen (line);
    sprintf (&line[n], " error = %s", (_error) ? "True" : "False");

    n = strlen (line);
    sprintf (&line[n], " interrupt  = %s\n", (_interrupt) ? "True" : "False");

    appendStatus (line);
}
#endif

/*------------------------------------------------------------------------
 * ModemSetup::verbChangedCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::verbChangedCallback (Widget w, XtPointer clientData,
				      XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;
    ms->verb_cb (atoi (XtName (w)));
}

/*------------------------------------------------------------------------
 * ModemSetup::verb_cb (int)
 *------------------------------------------------------------------------*/

void ModemSetup::verb_cb (int val)
{
    _verbosity = val;
}

/*------------------------------------------------------------------------
 * ModemSetup::testCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::testCallback (Widget w, XtPointer clientData, XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;

    if (strcmp (XtName (w), "setup") == 0)
	ms->setup_test (SETUP);
    else if (strcmp (XtName (w), "test") == 0)
	ms->setup_test (TEST);
    else if (strcmp (XtName (w), "speed") == 0)
	ms->setup_test (SPEED);
    else if (strcmp (XtName (w), "regs") == 0)
	ms->setup_test (REGS);
    else if (strcmp (XtName (w), "flags") == 0)
	ms->setup_test (FLAGS);
    else if (strcmp (XtName (w), "save") == 0)
	ms->setup_test (SAVE);
}

/*------------------------------------------------------------------------
 * ModemSetup::setup_test (enum Test)
 *------------------------------------------------------------------------*/

void ModemSetup::setup_test (enum Test what)
{
    if (_test_state != INIT)
    {
	theErrorDialog->postAndWait ("Test currently being performed.\n"
				     "Cannot start another test until\n"
				     "the current one is complete.");
    }
    else
    {
	getDevice ();
	if (!devString)
	{
	    theErrorDialog->postAndWait ("You must specify the device where\n"
					 "the modem is attached.");
	}
	else
	{
	    _error = 0;
	    _interrupt = 0;
	    _test_state = what;
	    if (what != SETUP)
		_state = INITIAL_STATE;
	    _workProcID = XtAppAddWorkProc (uucpApplication->appContext (),
					    workProcCallback, this);
	    clearInteraction ();
	    clearStatus ();
	}
    }
}

/*------------------------------------------------------------------------
 * ModemSetup::setup ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::setup ()
{
    static enum Test		my_state = INIT;
    Boolean			ret_val = False;

    switch (my_state)
    {
      case INIT:
	my_state = TEST;
	_state = INITIAL_STATE;
	break;

      case SETUP:			// To keep the compiler happy.
	break;

      case TEST:
	if (test ())
	{
	    my_state = SPEED;
	    _state = INITIAL_STATE;
	}
	break;

      case SPEED:
	if (speed ())
	{
	    my_state = REGS;
	    _state = INITIAL_STATE;
	}
	break;

      case REGS:
	if (regs ())
	{
	    my_state = FLAGS;
	    _state = INITIAL_STATE;
	}
	break;

      case FLAGS:
	if (flags ())
	{
	    my_state = SAVE;
	    _state = INITIAL_STATE;
	}
	break;

      case SAVE:			// Don't automatically do this.
	my_state = INIT;
	_state = NO_STATE;
	break;
    }

    if (my_state == INIT)
    {
	if (_state == NO_STATE)
	    appendStatus ("\nAll tests completed.\n");
	ret_val = True;
    }
    else if (_error)
    {
	terminal.close ();
	theErrorDialog->postAndWait ("Error occurred!");
	my_state = INIT;
	_state = NO_STATE;
	ret_val = True;
    }
    else if (_interrupt)
    {
	terminal.close ();
	theInfoDialog->postAndWait ("Process interrupted.");
	my_state = INIT;
	_state = NO_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::test ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::test ()
{
    Boolean		ret_val = False;
    char		line[128], *cptr;

#ifdef DEBUG1
    sprintf (line, "    -- Current state = %d\n", _state);
    appendInteraction (line);
#endif

    if (_interrupt)
    {
	terminal.close ();
	appendStatus ("*** 'Test' interrupted ***\n");
	_state = NO_STATE;
    }

    switch (_state)
    {
	/*-----------------------------------------------------------
	 * This could happen if _interrupt set or we get called
	 * by mistake.  Just make sure we don't get called again.
	 *----*/

      case NO_STATE:
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Initialize for this test.
	 *
	 * Send message to Interaction window indicating test to
	 * be performed.
	 *----*/

      case INITIAL_STATE:
	_error = 0;
	appendStatus ("Testing if modem responds and determine the\n");
	appendStatus ("  baud rate that the modem is currently set:\n");
	if (_verbosity > 0)
	{
	    appendInteraction ("Testing if modem responds and determine "
			       "the\n");
	    appendInteraction ("  baud rate that the modem is currently "
			       "set:\n");
	}
	_state = 1;
	break;

	/*-----------------------------------------------------------
	 * Open device.
	 *----*/

      case 1:
	if (terminal.setDevice (devString))
	{
	    if (terminal.open ())
		_state = 2;
	    else
	    {
		appendInteraction ("  *** Error opening ");
		appendInteraction (devString);
		appendInteraction (" ***\n");

		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_error = 1;
	    }
	}
	else
	{
	    appendInteraction ("   *** Error setting device ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _error = 1;
	}
	break;

	/*-----------------------------------------------------------
	 * Set up the terminal for communication.
	 *----*/

      case 2:
	if (_verbosity > 0)
	    appendInteraction ("  Setting up device.\n");

	if (terminal.init ())
	    _state = 3;
	else
	{
	    appendInteraction ("  *** Unable to initialize ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _error = 1;
	}
	break;

	/*-----------------------------------------------------------
	 * Initiate loop through bauds to find one that works.
	 * Set the baud to the first one defined for the modem.
	 *----*/

      case 3:
	cptr = modemFile->firstBaud ();
	if (cptr)
	{
	    if (_verbosity > 0)
	    {
		appendInteraction ("  Setting baud to ");
		appendInteraction (cptr);
		appendInteraction ("\n");
	    }

	    _baud = atoi (cptr);
	    if (terminal.setBaud (_baud))
		_state = 4;
	    else
	    {
		if (terminal.errVal () < 0)
		    _state = 9;
		else if (_verbosity > 1)
		{
		    appendInteraction ("  Unable to set baud to ");
		    appendInteraction (cptr);
		    appendInteraction ("\n");

		    appendInteraction ("  ");
		    appendInteraction (terminal.strerror ());
		    appendInteraction ("\n");
		}
	    }
	}
	else
	{
	    appendInteraction ("  *** No baud rates defined for modem.\n");
	    _error = 1;
	}
	break;

	/*-----------------------------------------------------------
	 * Send the string to check if modem exists.
	 *----*/

      case 4:
	if (_verbosity > 0)
	    appendInteraction ("  Sending check string.\n");

	cptr = modemFile->getPrefix (modemFile->getCheckSend ());
	if (_verbosity > 2)
	{
	    appendInteraction ("     SEND > ");
	    appendInteraction (convert_string (cptr));
	    appendInteraction ("\n");
	}

	if (terminal.write (cptr))
	    _state = 5;			// Ok, now try and read.
	else
	{
	    if (_verbosity > 1)
	    {
		appendInteraction ("  *** Write failure ***\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
	    }
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Now read in what was returned (if anything).
	 *----*/

      case 5:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getCheckExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 10;		// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    _state = 6;			// Try initializing modem.
	}
	break;

	/*-----------------------------------------------------------
	 * Try sending the string to initialize the modem.
	 *----*/

      case 6:
	if (_verbosity > 0)
	    appendInteraction ("  Trying to reset modem.\n");

	cptr = modemFile->getPrefix (modemFile->getMinimumSend ());
	if (_verbosity > 2)
	{
	    appendInteraction ("     SEND > ");
	    appendInteraction (convert_string (cptr));
	    appendInteraction ("\n");
	}

	if (terminal.write (cptr))
	    _state = 7;			// Ok, now try and read.
	else
	{
	    if (_verbosity > 1)
	    {
		appendInteraction ("  *** Write failure ***\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
	    }
	    _state = 8;
	}
	break;

	/*-----------------------------------------------------------
	 * Check if response is good for initialize.
	 *----*/

      case 7:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getMinimumExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 10;		// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    _state = 8;			// Reset the device
	}
	break;

	/*-----------------------------------------------------------
	 * Clear the device, then try another baud.
	 *----*/

      case 8:
	if (_verbosity > 0)
	    appendInteraction ("  Clearing device.\n");

	if (terminal.clear ())
	    _state = 9;
	else
	{
	    appendInteraction ("  *** Unable to hang up line. ***\n");
	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _error = 1;
	}
	break;

	/*-----------------------------------------------------------
	 * Figure out another baud rate to try.
	 *----*/

      case 9:
	cptr = modemFile->nextBaud ();
	if (cptr)
	{
	    if (_verbosity > 0)
	    {
		appendInteraction ("  Setting baud to ");
		appendInteraction (cptr);
		appendInteraction ("\n");
	    }

	    _baud = atoi (cptr);
	    if (terminal.setBaud (_baud))
		_state = 4;
	    else
	    {
		if ((terminal.errVal () >= 0) && (_verbosity > 1))
		{
		    appendInteraction ("  Unable to set baud to ");
		    appendInteraction (cptr);
		    appendInteraction ("\n");

		    appendInteraction ("  ");
		    appendInteraction (terminal.strerror ());
		    appendInteraction ("\n");
		}
	    }
	}
	else
	    _state = 11;			// Failure.
	break;

	/*-----------------------------------------------------------
	 * Test was successful.
	 *----*/

      case 10:
	sprintf (line, "Modem responded to baud %d.  OK\n", _baud);
	appendStatus (line);
	_state = DONE_STATE;
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Test was unsuccessful.
	 *----*/

      case 11:
	if (_verbosity > 0)
	    appendInteraction ("  Resetting terminal parameters.\n");
	terminal.uninit ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	appendStatus ("Unable to find a baud that the modem responds on.\n");
	_baud = 0;
	_error = 1;
	break;
    }

    if (_error)
    {
	appendStatus ("Test failed.\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::speed ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::speed ()
{
    Boolean		ret_val = False;
    char		line[128], *cptr;
    static int		newBaud;

#ifdef DEBUG1
    sprintf (line, "    -- Current state = %d\n", _state);
    appendInteraction (line);
#endif

    if (_interrupt)
    {
	terminal.close ();
	appendStatus ("*** 'Speed' interrupted ***\n");
	_state = NO_STATE;
    }

    switch (_state)
    {
	/*----
	 * This could happen if _interrupt set or we get called
	 * by mistake.  Just make sure we don't get called again.
	 *----*/

      case NO_STATE:
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Initialize for this test.
	 *
	 * Verify a baud has been picked.  If not, inform user to
	 * run the 'test'.
	 *
	 * If this modem has the ability to set the interface speed,
	 * then let's try and set it.  Otherwise, skip this test.
	 *
	 * Also, figure out what baud the user has chosen the modem
	 * should communicate with.
	 *----*/

      case INITIAL_STATE:
	_error = 0;

	if (_baud)
	{
	    cptr = modemFile->getSpeedSend (-1);
	    if (cptr)
	    {
		newBaud = computeNewBaud ();
		if (!newBaud)
		{
		    if (_verbosity > 0)
			appendInteraction ("The interface speed is not to be "
					   "changed from the default.\n");
		    _state = DONE_STATE;
		    ret_val = True;
		    break;
		}

		if (newBaud != _baud)
		{
		    appendStatus ("Setting interface speed to modem:\n");
		    if (_verbosity > 0)
			appendInteraction ("Setting interface speed to "
					   "modem:\n");
		    _state = 5;
		}
		else
		{
		    if (_verbosity > 0)
			appendInteraction ("Unnecessary to change baud.\n");
		    _state = DONE_STATE;
		    ret_val = True;
		}
	    }
	    else
	    {
		if (_verbosity > 0)
		    appendInteraction ("Modem can't set interface speed.\n");
		_state = DONE_STATE;
		ret_val = True;
	    }
	}
	else
	{
	    appendInteraction ("  *** No baud rate has been set up ***\n"
			       "    Please run 'Test' first.\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 *----*/

      case 1:
	break;

	/*-----------------------------------------------------------
	 * Open the device.
	 *----*/

      case 2:
	if (_verbosity > 0)
	    appendInteraction ("  Opening device.\n");

	if (terminal.open ())
	    _state = 3;
	else
	{
	    appendInteraction ("  *** Error opening ");
	    appendInteraction (devString);
	    appendInteraction (" ****\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set up the terminal for communication.
	 *----*/

      case 3:
	if (_verbosity > 0)
	    appendInteraction ("  Setting up device.\n");

	if (terminal.init ())
	    _state = 4;
	else
	{
	    appendInteraction ("  *** Unable to initialize ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set the baud rate.
	 *----*/

      case 4:
	if (_verbosity > 0)
	{
	    sprintf (line, "  Setting baud to %d\n", _baud);
	    appendInteraction (line);
	}
	if (terminal.setBaud (_baud))
	    _state = 5;
	else
	{
	    sprintf (line, "  *** Unable to set baud to %d\n", _baud);
	    appendInteraction (line);

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Send string to set the interface baud.
	 * We can't expect a response until we change our baud.
	 *----*/

      case 5:
	if (_verbosity > 0)
	    appendInteraction ("  Sending command to change baud.\n");

	cptr = modemFile->getSpeedSend (newBaud);
	if (cptr)
	{
	    cptr = modemFile->getPrefix (cptr);
	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }

	    if (terminal.write (cptr))
		_state = 6;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	else
	{
	    sprintf (line, "  *** Unable to find command to set speed "
		     "for baud %d\n", newBaud);
	    appendInteraction (line);
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set new baud rate.
	 *----*/

      case 6:
	sleep (2);			// We need to give the modem time.
	if (_verbosity > 0)
	{
	    sprintf (line, "  Setting new baud rate to %d\n", newBaud);
	    appendInteraction (line);
	}

	if (terminal.setBaud (newBaud))
	{
	    _baud = newBaud;
	    _state = 7;
	}
	else
	{
	    sprintf (line, "  *** Unable to set baud to %d\n", newBaud);
	    appendInteraction (line);

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");

	    appendInteraction ("  It is recommended to run 'Test' again.\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Now make sure we can still talk to the modem.
	 * Send the check string.
	 *----*/

      case 7:
	if (_verbosity > 0)
	    appendInteraction ("  Sending check string.\n");

	cptr = modemFile->getPrefix (modemFile->getCheckSend ());
	if (_verbosity > 2)
	{
	    appendInteraction ("     SEND > ");
	    appendInteraction (convert_string (cptr));
	    appendInteraction ("\n");
	}

	if (terminal.write (cptr))
	    _state = 8;			// Ok, now try and read.
	else
	{
	    appendInteraction ("  *** Write failure\n");
	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Read what was returned.
	 *----*/

      case 8:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getCheckExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 10;		// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    _state = 9;			// Try initializing modem.
	}
	break;

	/*-----------------------------------------------------------
	 * Unable to set interface speed.
	 *----*/

      case 9:
	if (_verbosity > 0)
	    appendInteraction ("  Resetting terminal parameters.\n");
	terminal.uninit ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	_baud = 0;
	_error = 1;
	break;

	/*-----------------------------------------------------------
	 * Test was successful.
	 *----*/

      case 10:
	appendStatus ("Interface speed set successfully.  OK\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    if (_error)
    {
	appendStatus ("Setting interface speed failed.\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::regs ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::regs ()
{
    Boolean		ret_val = False;
    char		line[128], *cptr;
    static Registers	*curReg;
    static int		tries;

#ifdef DEBUG1
    sprintf (line, "    -- Current state = %d\n", _state);
    appendInteraction (line);
#endif

    if (_interrupt)
    {
	terminal.close ();
	appendStatus ("*** 'Regs' interrupted ***\n");
	_state = NO_STATE;
    }

    switch (_state)
    {
	/*-----------------------------------------------------------
	 * This could happen if _interrupt set or we get called
	 * by mistake.  Just make sure we don't get called again.
	 *----*/

      case NO_STATE:
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Initialize for this test.
	 * Send message to Interaction window.
	 *----*/

      case INITIAL_STATE:
	_error = 0;
	if (_baud)
	{
	    appendStatus ("Setting modem registers:\n");
	    if (_verbosity > 0)
		appendInteraction ("Setting modem registers:\n");
	    _state = 4;
	}
	else
	{
	    appendInteraction ("  *** No baud rate has been set up ***\n"
			       "Please run 'Test' first.\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Open the device.
	 *----*/

      case 1:
	if (_verbosity > 0)
	    appendInteraction ("  Opening device.\n");

	if (terminal.open ())
	    _state = 2;
	else
	{
	    appendInteraction ("  *** Error opening ");
	    appendInteraction (devString);
	    appendInteraction (" ****\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set up the terminal for communication.
	 *----*/

      case 2:
	if (_verbosity > 0)
	    appendInteraction ("  Setting up device.\n");

	if (terminal.init ())
	    _state = 3;
	else
	{
	    appendInteraction ("  *** Unable to initialize ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set the baud rate.
	 *----*/

      case 3:
	if (_verbosity > 0)
	{
	    sprintf (line, "  Setting baud to %d\n", _baud);
	    appendInteraction (line);
	}
	if (terminal.setBaud (_baud))
	    _state = 4;
	else
	{
	    sprintf (line, "  *** Unable to set baud to %d\n", _baud);
	    appendInteraction (line);

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Initiate sending of registers to modem.
	 *----*/

      case 4:
	if (_verbosity > 0)
	    appendInteraction ("  Starting to set registers.\n");

	tries = 0;
	curReg = modemFile->firstRegister ();
	if (curReg && (curReg->getType () == Registers::REGISTER))
	{
	    if (_verbosity > 1)
	    {
		appendInteraction ("    Setting register ");
		appendInteraction (curReg->theKey ());
		appendInteraction ("\n");
	    }
	    cptr = modemFile->getPrefix (modemFile->getSetRSend (curReg));
	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }
	    if (terminal.write (cptr))
		_state = 5;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	else if (curReg)
	    _state = 6;
	else
	    _state = 10;
	break;

	/*-----------------------------------------------------------
	 * Check response is OK.
	 *----*/

      case 5:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getSetRExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 6;			// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    if (tries > 3)
	    {
		appendInteraction ("  *** Error with response after setting "
				   "register ");
		appendInteraction (curReg->theKey ());
		appendInteraction (" ***\n");
		_state = 9;
	    }
	    else
	    {
		tries++;
		if (_verbosity > 1)
		{
		    appendInteraction ("    Setting register ");
		    appendInteraction (curReg->theKey ());
		    appendInteraction (" again.\n");
		}
		cptr = modemFile->getPrefix (modemFile->getSetRSend (curReg));
		if (_verbosity > 2)
		{
		    appendInteraction ("     SEND > ");
		    appendInteraction (convert_string (cptr));
		    appendInteraction ("\n");
		}
		if (!terminal.write (cptr))
		{
		    appendInteraction ("  *** Write failure\n");
		    appendInteraction ("  ");
		    appendInteraction (terminal.strerror ());
		    appendInteraction ("\n");
		    _state = 9;
		}
	    }
	}
	break;

	/*-----------------------------------------------------------
	 * Send next register.
	 *----*/

      case 6:
	tries = 0;
	curReg = modemFile->nextRegister ();
	if (!curReg)
	{
	    _state = 10;
	    break;
	}

	if (curReg->getType () == Registers::REGISTER)
	{
	    if (_verbosity > 0)
		appendInteraction ("  Sending next register.\n");

	    if (_verbosity > 1)
	    {
		appendInteraction ("    Setting register ");
		appendInteraction (curReg->theKey ());
		appendInteraction ("\n");
	    }
	    cptr = modemFile->getPrefix (modemFile->getSetRSend (curReg));
	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }
	    if (terminal.write (cptr))
		_state = 5;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	break;

	/*-----------------------------------------------------------
	 * Error setting the registers.
	 *----*/

      case 9:
	if (_verbosity > 0)
	    appendInteraction ("  Resetting terminal parameters.\n");
	terminal.uninit ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	_baud = 0;
	_error = 1;
	break;

	/*-----------------------------------------------------------
	 * Test was successful.
	 *----*/

      case 10:
	appendStatus ("Registers set successfully.  OK\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    if (_error)
    {
	appendStatus ("Setting registers failed.\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::flags ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::flags ()
{
    Boolean		ret_val = False;
    char		line[128], *cptr;
    static Registers	*curReg;
    static int		tries;

#ifdef DEBUG1
    sprintf (line, "    -- Current state = %d\n", _state);
    appendInteraction (line);
#endif

    if (_interrupt)
    {
	terminal.close ();
	appendStatus ("*** 'Flags' interrupted ***\n");
	_state = NO_STATE;
    }

    switch (_state)
    {
	/*-----------------------------------------------------------
	 * This could happen if _interrupt set or we get called
	 * by mistake.  Just make sure we don't get called again.
	 *----*/

      case NO_STATE:
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Initialize for this test.
	 * Send message to Interaction window.
	 *----*/

      case INITIAL_STATE:
	_error = 0;
	if (_baud)
	{
	    appendStatus ("Setting modem flags:\n");
	    if (_verbosity > 0)
		appendInteraction ("Setting modem flags:\n");
	    _state = 4;
	}
	else
	{
	    appendInteraction ("  *** No baud rate has been set up ***\n"
			       "Please run 'Test' first.\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Open the device.
	 *----*/

      case 1:
	if (_verbosity > 0)
	    appendInteraction ("  Opening device.\n");

	if (terminal.open ())
	    _state = 2;
	else
	{
	    appendInteraction ("  *** Error opening ");
	    appendInteraction (devString);
	    appendInteraction (" ****\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set up the terminal for communication.
	 *----*/

      case 2:
	if (_verbosity > 0)
	    appendInteraction ("  Setting up device.\n");

	if (terminal.init ())
	    _state = 3;
	else
	{
	    appendInteraction ("  *** Unable to initialize ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set the baud rate.
	 *----*/

      case 3:
	if (_verbosity > 0)
	{
	    sprintf (line, "  Setting baud to %d\n", _baud);
	    appendInteraction (line);
	}
	if (terminal.setBaud (_baud))
	    _state = 4;
	else
	{
	    sprintf (line, "  *** Unable to set baud to %d\n", _baud);
	    appendInteraction (line);

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Initiate sending of flags to modem.
	 *----*/

      case 4:
	if (_verbosity > 0)
	    appendInteraction ("  Starting to set flags.\n");

	tries = 0;
	curReg = modemFile->firstRegister ();
	if (curReg && (curReg->getType () == Registers::FLAG))
	{
	    if (_verbosity > 1)
	    {
		appendInteraction ("    Setting flag ");
		appendInteraction (curReg->theKey ());
		appendInteraction ("\n");
	    }
	    cptr = modemFile->getPrefix (modemFile->getSetFSend (curReg));
	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }
	    if (terminal.write (cptr))
		_state = 5;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	else if (curReg)
	    _state = 6;
	else
	    _state = 10;
	break;

	/*-----------------------------------------------------------
	 * Check response is OK.
	 *----*/

      case 5:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getSetFExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 6;			// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    if (tries > 3)
	    {
		appendInteraction ("  *** Error with response after setting "
				   "flag ");
		appendInteraction (curReg->theKey ());
		appendInteraction ("\n");
		_state = 9;
	    }
	    else
	    {
		tries++;
		if (_verbosity > 1)
		{
		    appendInteraction ("    Setting flag ");
		    appendInteraction (curReg->theKey ());
		    appendInteraction (" again.\n");
		}
		cptr = modemFile->getPrefix (modemFile->getSetFSend (curReg));
		if (_verbosity > 2)
		{
		    appendInteraction ("     SEND > ");
		    appendInteraction (convert_string (cptr));
		    appendInteraction ("\n");
		}
		if (!terminal.write (cptr))
		{
		    appendInteraction ("  *** Write failure\n");
		    appendInteraction ("  ");
		    appendInteraction (terminal.strerror ());
		    appendInteraction ("\n");
		    _state = 9;
		}
	    }
	}
	break;

	/*-----------------------------------------------------------
	 * Send next flag.
	 *----*/

      case 6:
	tries = 0;
	curReg = modemFile->nextRegister ();
	if (!curReg)
	{
	    _state = 10;
	    break;
	}

	if (curReg && (curReg->getType () == Registers::FLAG))
	{
	    if (_verbosity > 0)
		appendInteraction ("  Sending next flag.\n");

	    if (_verbosity > 1)
	    {
		appendInteraction ("    Setting flag ");
		appendInteraction (curReg->theKey ());
		appendInteraction ("\n");
	    }
	    cptr = modemFile->getPrefix (modemFile->getSetFSend (curReg));
	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }
	    if (terminal.write (cptr))
		_state = 5;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	break;

	/*-----------------------------------------------------------
	 * Error setting the flags.
	 *----*/

      case 9:
	if (_verbosity > 0)
	    appendInteraction ("  Resetting terminal parameters.\n");
	terminal.uninit ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	_baud = 0;
	_error = 1;
	break;

	/*-----------------------------------------------------------
	 * Test was successful.
	 *----*/

      case 10:
	appendStatus ("Flags set successfully.  OK\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    if (_error)
    {
	appendStatus ("Setting flags failed.\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::save ()
 *------------------------------------------------------------------------*/

Boolean ModemSetup::save ()
{
    Boolean		ret_val = False;
    char		line[128], *cptr;
    static int		tries;

#ifdef DEBUG1
    sprintf (line, "    -- Current state = %d\n", _state);
    appendInteraction (line);
#endif

    if (_interrupt)
    {
	terminal.close ();
	appendStatus ("*** 'Save' interrupted ***\n");
	_state = NO_STATE;
    }

    switch (_state)
    {
	/*-----------------------------------------------------------
	 * This could happen if _interrupt set or we get called
	 * by mistake.  Just make sure we don't get called again.
	 *----*/

      case NO_STATE:
	ret_val = True;
	break;

	/*-----------------------------------------------------------
	 * Initialize for this test.
	 * Send message to Interaction window.
	 *----*/

      case INITIAL_STATE:
	_error = 0;
	if (_baud)
	{
	    appendStatus ("Saving modem setup:\n");
	    if (_verbosity > 0)
		appendInteraction ("Saving modem setup:\n");
	    _state = 4;
	}
	else
	{
	    appendInteraction ("  *** No baud rate has been set up ***\n"
			       "Please run 'Test' first.\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Open the device.
	 *----*/

      case 1:
	if (_verbosity > 0)
	    appendInteraction ("  Opening device.\n");

	if (terminal.open ())
	    _state = 2;
	else
	{
	    appendInteraction ("  *** Error opening ");
	    appendInteraction (devString);
	    appendInteraction (" ****\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set up the terminal for communication.
	 *----*/

      case 2:
	if (_verbosity > 0)
	    appendInteraction ("  Setting up device.\n");

	if (terminal.init ())
	    _state = 3;
	else
	{
	    appendInteraction ("  *** Unable to initialize ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Set the baud rate.
	 *----*/

      case 3:
	if (_verbosity > 0)
	{
	    sprintf (line, "  Setting baud to %d\n", _baud);
	    appendInteraction (line);
	}
	if (terminal.setBaud (_baud))
	    _state = 4;
	else
	{
	    sprintf (line, "  *** Unable to set baud to %d\n", _baud);
	    appendInteraction (line);

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    _state = 9;
	}
	break;

	/*-----------------------------------------------------------
	 * Send sequence to save registers.
	 *----*/

      case 4:
	tries = 0;
	cptr = modemFile->getPrefix (modemFile->getSaveSend ());
	if (cptr)
	{
	    if (_verbosity > 0)
		appendInteraction ("  Sending sequence to save modem "
				   "state.\n");

	    if (_verbosity > 2)
	    {
		appendInteraction ("     SEND > ");
		appendInteraction (convert_string (cptr));
		appendInteraction ("\n");
	    }
	    if (terminal.write (cptr))
		_state = 5;
	    else
	    {
		appendInteraction ("  *** Write failure\n");
		appendInteraction ("  ");
		appendInteraction (terminal.strerror ());
		appendInteraction ("\n");
		_state = 9;
	    }
	}
	else
	{
	    appendInteraction ("This modem doesn't have a sequence for "
			       "saving its state.\n");
	    _state = 10;
	}
	break;

	/*-----------------------------------------------------------
	 * Check response is OK.
	 *----*/

      case 5:
	if (_verbosity > 0)
	    appendInteraction ("  Reading response.");
	if (_verbosity > 2)
	    appendInteraction ("\n");

	if (test_input (modemFile->getSaveExpect ()))
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  OK\n");
	    _state = 10;		// Ok, we have success.
	}
	else
	{
	    if (_verbosity > 0 && _verbosity < 3)
		appendInteraction ("  FAILED\n");
	    if (tries > 3)
	    {
		appendInteraction ("  *** Error with response ***\n");
		_state = 9;
	    }
	    else
	    {
		tries++;
		if (_verbosity > 1)
		    appendInteraction ("  Sending sequence to save modem "
				       "state again.\n");
		cptr = modemFile->getPrefix (modemFile->getSaveSend ());
		if (_verbosity > 2)
		{
		    appendInteraction ("     SEND > ");
		    appendInteraction (convert_string (cptr));
		    appendInteraction ("\n");
		}
		if (!terminal.write (cptr))
		{
		    appendInteraction ("  *** Write failure\n");
		    appendInteraction ("  ");
		    appendInteraction (terminal.strerror ());
		    appendInteraction ("\n");
		    _state = 9;
		}
	    }
	}
	break;

	/*-----------------------------------------------------------
	 * Error trying to save the modem state.
	 *----*/

      case 9:
	if (_verbosity > 0)
	    appendInteraction ("  Resetting terminal parameters.\n");
	terminal.uninit ();
	if (_verbosity > 0)
	    appendInteraction ("  Closing down device.\n");
	terminal.close ();
	_baud = 0;
	_error = 1;
	break;

	/*-----------------------------------------------------------
	 * Saveing the modem state was successful.
	 *----*/

      case 10:
	appendStatus ("Modem state saved successfully.  OK\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    if (_error)
    {
	appendStatus ("Saving the modem state failed.\n");
	_state = DONE_STATE;
	ret_val = True;
    }

    return ret_val;
}

/*------------------------------------------------------------------------
 * ModemSetup::doneCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::doneCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;
    ms->done_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::done_cb (void)
 *------------------------------------------------------------------------*/

void ModemSetup::done_cb (void)
{
    delete this;
}

/*------------------------------------------------------------------------
 * ModemSetup::interruptCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemSetup::interruptCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemSetup		*ms;

    ms = (ModemSetup *)clientData;
    ms->interrupt_cb ();
}

/*------------------------------------------------------------------------
 * ModemSetup::interrupt_cb (void)
 *------------------------------------------------------------------------*/

void ModemSetup::interrupt_cb (void)
{
    _interrupt = 1;
}

/*------------------------------------------------------------------------
 * ModemSetup::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget ModemSetup::setUpInterface (Widget parent)
{
    Arg			wargs[15];
    int			n;
    Widget		form, rc, temp, label, setup, popup;
    Widget		menuw;
    VkMenuBar		*menubar;
    VkHelpPane		*helpMenu;
    char		*tptr;

    registerWindow4Help (parentw = parent);

    if (access (fileName.name (), R_OK) == 0)
	modemFile = new ModemFile (fileName);

    ringer = NONE;
    tptr = modemFile->getAnswerSend (-1);
    if (tptr)
    {
	if (strstr (tptr, "%n"))
	    ringer = COUNT;
	else if (strstr (tptr, "%b"))
	    ringer = ONOFF;
    }
    ringer = NONE;

    n = 0;
    form = XmCreateForm (parent, "modemSetupForm", wargs, n);

    /*----
     * Put together the first pane which contains:
     *	Label		To identify device text.
     *	TextField	For user to enter device name.
     *	PushButton	To start setup process.
     *	OptionMenu	To select verbosity.
     *     either
     *  RadioButton	To enable/disable answering.
     *     or
     *  TextField	To select number of rings to answer.
     *	Separator
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNisAligned, False); n++;
    XtSetArg (wargs[n], XmNnumColumns, (ringer == NONE) ? 1 : 2); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    rc = XmCreateRowColumn (form, "textrc", wargs, n);
    XtManageChild (rc);

    n = 0;
    label = XmCreateLabelGadget (rc, "deviceLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "modemSetupDevice");

    n = 0;
    XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
    device = XmCreateTextField (rc, "deviceText", wargs, n);
    XtManageChild (device);

    XtAddCallback (device, XmNactivateCallback, deviceChangedCallback, this);
    XtAddCallback (device, XmNhelpCallback, aHelpCallback, "modemSetupDevice");

    switch (ringer)
    {
      case NONE:
	break;

      case COUNT:
	n = 0;
	label = XmCreateLabelGadget (rc, "numRingsLabel", wargs, n);
	XtManageChild (label);

	XtAddCallback (label, XmNhelpCallback, aHelpCallback,
		       "modemSetupDevice");

	n = 0;
	XtSetArg (wargs[n], XmNnavigationType, XmNONE); n++;
	numRings = XmCreateTextField (rc, "numRings", wargs, n);
	XtManageChild (numRings);

	XtAddCallback (numRings, XmNactivateCallback, ringsChangedCallback,
		       this);
	XtAddCallback (numRings, XmNhelpCallback, aHelpCallback,
		       "modemSetupDevice");
	break;

      case ONOFF:
	n = 0;
	label = XmCreateLabelGadget (rc, "answerLabel", wargs, n);
	XtManageChild (label);

	XtAddCallback (label, XmNhelpCallback, aHelpCallback,
		       "modemSetupDevice");

	n = 0;
	XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
	ringsOn = XmCreateRadioBox (rc, "ringsOnOff", wargs, n);
	XtManageChild (ringsOn);

	XtAddCallback (ringsOn, XmNhelpCallback, aHelpCallback,
		       "modemSetupDevice");

	n = 0;
	temp = XmCreateToggleButtonGadget (ringsOn, "on", wargs, n);
	XtManageChild (temp);

	XtAddCallback (temp, XmNarmCallback, ringsChangedCallback, this);

	n = 0;
	temp = XmCreateToggleButtonGadget (ringsOn, "off", wargs, n);
	XtManageChild (temp);

	XtAddCallback (temp, XmNarmCallback, ringsChangedCallback, this);
	break;
    }

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, rc); n++;
    XtSetArg (wargs[n], XmNleftOffset, 30); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    setup = XmCreatePushButton (form, "setup", wargs, n);
    XtManageChild (setup);

    XtAddCallback (setup, XmNactivateCallback, testCallback, this);
    XtAddCallback (setup, XmNhelpCallback, aHelpCallback, "modemSetupSetup");

    n = 0;
    popup = XmCreateSimplePulldownMenu (form, "verbosityMenu", wargs, n);

    n = 0;
    temp = XmCreatePushButtonGadget (popup, "0", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, verbChangedCallback, this);

    n = 0;
    temp = XmCreatePushButtonGadget (popup, "1", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, verbChangedCallback, this);

    n = 0;
    temp = XmCreatePushButtonGadget (popup, "2", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, verbChangedCallback, this);

    n = 0;
    temp = XmCreatePushButtonGadget (popup, "3", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, verbChangedCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, setup); n++;
    XtSetArg (wargs[n], XmNleftOffset, 10); n++;
    XtSetArg (wargs[n], XmNsubMenuId, popup); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    temp = XmCreateOptionMenu (form, "verbosity", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupVerbose");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, rc); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    temp = XmCreateSeparatorGadget (form, "sep1", wargs, n);
    XtManageChild (temp);

    /*----
     * Build second pane which contains:
     *	RowColumn	Into which buttons for each test are placed.
     *	PushButtonG	To select initial test.
     *	PushButtonG	To set interface speed.
     *	PushButtonG	To download the registers.
     *	PushButtonG	To download the flags.
     *	Separator
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    temp = XmCreateLabelGadget (form, "tests", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupTests");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_TIGHT); n++;
    rc = XmCreateRowColumn (form, "rowcolumn", wargs, n);
    XtManageChild (rc);

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButtonGadget (rc, "test", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, testCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupTest");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButtonGadget (rc, "speed", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, testCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupSpeed");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButtonGadget (rc, "regs", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, testCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupRegs");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButtonGadget (rc, "flags", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, testCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupFlags");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, rc); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    temp = XmCreateSeparatorGadget (form, "sep2", wargs, n);
    XtManageChild (temp);

    /*----
     * Create third pane
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form, "interLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "modemSetupInter");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNcolumns, 60); n++;
    XtSetArg (wargs[n], XmNrows, 10); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    interaction = XmCreateScrolledText (form, "interaction", wargs, n);
    XtManageChild (interaction);

    XtAddCallback (interaction, XmNhelpCallback, aHelpCallback,
		   "modemSetupInter");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, interaction); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form, "statusLabel", wargs, n);
    XtManageChild (label);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "modemSetupStatus");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNcolumns, 60); n++;
    XtSetArg (wargs[n], XmNrows, 5); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    XtSetArg (wargs[n], XmNtraversalOn, False); n++;
    status = XmCreateScrolledText (form, "status", wargs, n);
    XtManageChild (status);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "modemSetupStatus");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, status); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    temp = XmCreateSeparatorGadget (form, "sep3", wargs, n);
    XtManageChild (temp);

    /*----
     * Create fourth pane
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    rc = XmCreateRowColumn (form, "buttons", wargs, n);
    XtManageChild (rc);

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButton (rc, "done", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, doneCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemSetupDone");

    n = 0;
    XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
    temp = XmCreatePushButton (rc, "interrupt", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, interruptCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback,
		   "modemSetupInterrupt");

    if (modemFile->getSaveSend ())
    {
	n = 0;
	XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
	temp = XmCreatePushButton (rc, "save", wargs, n);
	XtManageChild (temp);

	XtAddCallback (temp, XmNactivateCallback, testCallback, this);
	XtAddCallback (temp, XmNhelpCallback, aHelpCallback,
		       "modemSetupSave");
    }

    XtVaSetValues (form, XmNinitialFocus, device, NULL);

    /*----
     * Add menu panes
     *----*/

    fileMenuPane = addMenuPane ("files");
    menuw = fileMenuPane->baseWidget ();
    XtAddCallback (menuw, XmNhelpCallback, aHelpCallback, "modemSetupFile");

    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("quit", quitCallback, this);

    menubar = menu ();
    helpMenu = menubar->helpPane ();
    helpMenu->addSeparator ();
    helpMenu->addAction ("window", onWindowCallback, this);
    helpMenu->addAction ("error", onErrorCallback, this);

    return form;
}

/*------------------------------------------------------------------------
 * ModemSetup::handleWmDeleteMessage ()
 *------------------------------------------------------------------------*/

void ModemSetup::handleWmDeleteMessage ()
{
    delete this;
}

/*------------------------------------------------------------------------
 * ModemSetup::handleWmQuitMessage ()
 *------------------------------------------------------------------------*/

void ModemSetup::handleWmQuitMessage ()
{
    delete this;
}

/*-------------------------------------------------------------------------
 * ModemSetup::test_input (char *)
 *
 * Check the response from the modem.
 *
 * Description:
 *	Reads input from the tty device and checks it with the string
 *	passed as input to test_input.  Returns SUCCESS/FAILURE
 *	depending on results.
 *-------------------------------------------------------------------------*/

Boolean ModemSetup::test_input (char *expect)
{
    char		in_line[128];
    char		*ptr1, *ptr2, *ptr3;
    int 		cnt;

    if (_verbosity > 2)
	appendInteraction ("    RECEIVED > ");

    /*----
     * Loop until we either have some kind of problem or the characters
     * read match what we expect.
     *
     * The loop is because we will get characters one at a time from
     * the read.
     *----*/

    ptr1 = expect;
    ptr3 = in_line;
    do {
	cnt = terminal.read (ptr3, (sizeof (in_line) - (ptr3 - in_line)));

	/*----
	 * Next, handle situation where a read error occurred.
	 *----*/

	if (cnt < 0) {
	    appendInteraction ("  *** Input error occurred on ");
	    appendInteraction (devString);
	    appendInteraction (" ***\n");

	    appendInteraction ("  ");
	    appendInteraction (terminal.strerror ());
	    appendInteraction ("\n");
	    return False;
	}

	/*----
	 * Terminate the string received and print to user if
	 * verbosity is high enough.
	 *----*/

	ptr3[cnt] = '\0';
	if (_verbosity > 2)
	    appendInteraction (convert_string (ptr3));

	/*----
	 * Loop through characters received to see if they
	 * match what we expect.  If not, reset compare pointer
	 * (ptr1) to beginning of string expected and continue
	 * with loop (either within the received string or to
	 * read more characters).  If a match occurs, exit
	 * both loops.
	 *----*/

	for (ptr2 = ptr3; *ptr2; ptr2++) {
	    if (*ptr1 == *ptr2) {
		ptr1++;
		if (*ptr1 == '\0')
		    break;
	    } else {
		ptr1 = expect;
	    }
	}
	if (*ptr1 == '\0')
	    break;

	ptr3 += cnt;

    } while (cnt > 0);

    if (_verbosity > 2)
	appendInteraction ("\n");

    return (*ptr1 != '\0') ? False : True;
}

/*------------------------------------------------------------------------
 * ModemSetup::computeNewBaud (void)
 *------------------------------------------------------------------------*/

int ModemSetup::computeNewBaud (void)
{
    enum States { SCAN, REG, VAL};
    char		*speedStr, *setRStr, *cptr1, *cptr2;
    enum States		cnb_state;
    char		reg[10], val[10], *rptr, *vptr;
    Boolean		success;
    Registers		*regClass;
    Sequences		*seqClass;


    /*----
     * First, check if the speed string is a register setting.
     *----*/

    speedStr = modemFile->getSpeedSend (-1);
    setRStr = modemFile->getSetRSend (NULL);

    cptr1 = speedStr;
    cptr2 = setRStr;
    cnb_state = SCAN;
    reg[0] = val[0] = '\0';
    success = True;
    rptr = reg;
    vptr = val;

    while (success && (*cptr1 || *cptr2))
    {
	switch (cnb_state)
	{
	  case SCAN:
	    if (*cptr2 == '%')
	    {
		if (*(++cptr2) == 'r')
		{
		    cptr2++;
		    if (reg[0])
			return 0;	// We've done register already
		    cnb_state = REG;
		}
		else if (*cptr2 == 'v')
		{
		    cptr2++;
		    if (val[0])
			return 0;	// We've done value already
		    cnb_state = VAL;
		}
		else if (cptr2[-1] == '%')
		{			// Uh, oh, should ignore this.
		    continue;
		}
		else
		    success = False; // What else is there?
	    }
	    else
	    {
		if (*cptr1 == *cptr2)
		    cptr1++, cptr2++;
		else
		    success = False;
	    }
	    break;

	  case REG:
	    if (*cptr1 == *cptr2)
	    {
		*rptr = '\0';
		cnb_state = SCAN;
		cptr1++, cptr2++;
	    }
	    else
		*rptr++ = *cptr1++;
	    break;

	  case VAL:
	    if (*cptr1 == *cptr2)
	    {
		*vptr = '\0';
		cnb_state = SCAN;
		cptr1++, cptr2++;
	    }
	    else
		*vptr++ = *cptr1++;
	    break;
	}
    }

    /*----
     * If we succeeded in matching the strings.
     *----*/

    if (success)
    {
	/*----
	 * If we got a register and value.
	 *----*/

	if (reg[0] && val[0])
	{

	    /*----
	     * If that register is to be set.
	     *----*/

	    regClass = modemFile->findRegister (reg);
	    if (regClass)
	    {
		vptr = regClass->getValue ();
		for (seqClass = modemFile->firstSequence (); seqClass;
		     seqClass = modemFile->nextSequence ())
		{
		    if (strcmp (seqClass->theKey (), "baud"))
			continue;
		    if (strcmp (seqClass->getExpect (), vptr) == 0)
			return atoi (seqClass->getSend ());
		}
	    }
	    return 0;
	}
    }

    /*----
     * If not, check if the speed string is a flag setting.
     *----*/

    setRStr = modemFile->getSetFSend (NULL);

    cptr1 = speedStr;
    cptr2 = setRStr;
    cnb_state = SCAN;
    reg[0] = val[0] = '\0';
    success = True;
    rptr = reg;
    vptr = val;

    while (success && (*cptr1 || *cptr2))
    {
	switch (cnb_state)
	{
	  case SCAN:
	    if (*cptr2 == '%')
	    {
		if (*(++cptr2) == 'f')
		{
		    cptr2++;
		    if (reg[0])
			return 0;	// We've done register already
		    cnb_state = REG;
		}
		else if (*cptr2 == 'v')
		{
		    cptr2++;
		    if (val[0])
			return 0;	// We've done value already
		    cnb_state = VAL;
		}
		else if (cptr2[-1] == '%')
		{			// Uh, oh, should ignore this.
		    continue;
		}
		else
		    success = False; // What else is there?
	    }
	    else
	    {
		if (*cptr1 == *cptr2)
		    cptr1++, cptr2++;
		else
		    success = False;
	    }
	    break;

	  case REG:
	    if (*cptr1 == *cptr2)
	    {
		*rptr = '\0';
		cnb_state = SCAN;
		cptr1++, cptr2++;
	    }
	    else
		*rptr++ = *cptr1++;
	    break;

	  case VAL:
	    if (*cptr1 == *cptr2)
	    {
		*vptr = '\0';
		cnb_state = SCAN;
		cptr1++, cptr2++;
	    }
	    else
		*vptr++ = *cptr1++;
	    break;
	}
    }

    *rptr = *vptr = '\0';

    /*----
     * If we succeeded in matching the strings.
     *----*/

    if (success)
    {
	/*----
	 * If we got a register and value.
	 *----*/

	if (reg[0] && val[0])
	{

	    /*----
	     * If that register is to be set.
	     *----*/

	    regClass = modemFile->findRegister (reg);
	    if (regClass)
	    {
		vptr = regClass->getValue ();
		for (seqClass = modemFile->firstSequence (); seqClass;
		     seqClass = modemFile->nextSequence ())
		{
		    if (strcmp (seqClass->theKey (), "baud"))
			continue;
		    if (strcmp (seqClass->getExpect (), vptr) == 0)
			return atoi (seqClass->getSend ());
		}
	    }
	}
    }

    return 0;
}
