/*---*- C++ -*------------------------------------------------------------
 * $Id: ModemSetup.h,v 2.11 1994/02/25 19:41:40 carlson Exp $
 *
 * Header file for ModemSetup class.
 *
 * Revision History:
 *	$Log: ModemSetup.h,v $
 * Revision 2.11  1994/02/25  19:41:40  carlson
 * Added prototypes for callbacks to handle help for errors.
 *
 * Revision 2.10  1994/02/03  04:08:51  chrisc
 * Added things to support handling the enabling and disabling of a modem
 *   to answer the phone.
 *
 * Revision 2.9  1994/01/24  17:23:32  carlson
 * Changed to a VkWindow instead of VkSimpleWindow.
 * Added "On Window" help option.
 * Added quit menu option from "File" menubar.
 *
 * Revision 2.8  1994/01/19  01:23:49  carlson
 * Added prototypes for methods to handle window manager messages.
 *
 * Revision 2.7  1994/01/18  18:54:30  carlson
 * Added method to compute the new baud.
 *
 * Revision 2.6  1994/01/17  19:45:40  carlson
 * Removed all the terminal functions and moved them into their own class.
 * Added a Terminal class element.
 * Added a variable to maintain what baud we found.
 *
 * Revision 2.5  1994/01/14  23:22:55  carlson
 * Changed around how test_current_baud works.  It now has four steps
 *   instead of three.  Baud rate can be set independent of doing check.
 * Changed a number of routines to return an enum of success or failure.
 * Got rid of some callbacks and made into just one.
 * Added verbosity level.
 *
 * Revision 2.4  1994/01/14  02:10:32  carlson
 * Moved the method convert_string to be a function in systemfiles.
 *
 * Revision 2.3  1994/01/13  05:31:08  carlson
 * Added a number of methods for testing the device.
 * Added an enumeration to be used to tell test_current_baud what step
 *   it is to perform.
 * Added some variables used to support the device and baud rates.
 *
 * Revision 2.2  1994/01/13  02:02:23  carlson
 * Created an enumeration of the various tests to be performed.  This is
 *   used by the workProc to figure out which function to call.
 * Made _state an int so that it can have negative values to identify modes
 *   outside of a current state.
 * Added prototypes for methods to handle workProc.
 * Added the variable _test_state to identify the current function that is
 *   doing a test now.
 * Added a debug method for printing the current states.
 * No longer need the callback methods.  setup_test performs the same
 *   function as all of them did.
 * Test functions now return a boolean to indicate when they're done.
 *
 * Revision 2.1  1994/01/12  16:53:08  carlson
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _MODEMSETUP_H_
#define _MODEMSETUP_H_

#include <Vk/VkWindow.h>

#include "ModemFile.h"
#include "Terminal.h"

class VkSubMenu;

class ModemSetup : public VkWindow
{
  public:
    ModemSetup (const char *name);
    ~ModemSetup ();

    virtual const char	*className () { return "ModemSetup"; }

  protected:
    virtual Widget	setUpInterface (Widget parent);
    virtual void        handleWmDeleteMessage ();
    virtual void        handleWmQuitMessage ();

    void		clearInteraction (void);
    void		appendInteraction (const char *);

    void		clearStatus (void);
    void		appendStatus (const char *);

  private:
    enum Test { INIT, SETUP, TEST, SPEED, REGS, FLAGS, SAVE };
    enum Ringer { NONE, ONOFF, COUNT };

    Widget		parentw;
    Widget		device;
    Widget		numRings;
    Widget		ringsOn;
    Widget		interaction;
    Widget		status;
    PathName		fileName;
    ModemFile		*modemFile;
    char		*devString;
    int			_error;
    int			_interrupt;
    int			_state;
    enum Test		_test_state;
    XtWorkProcId	_workProcID;
    Terminal		terminal;
    unsigned int	_verbosity;
    unsigned int	_baud;
    VkSubMenu		*fileMenuPane;
    Ringer		ringer;
    int			ringCount;

    static Boolean	workProcCallback (XtPointer);
    Boolean		workProc ();
#ifdef DEBUG
    void		debugOutput (char *);
#endif
    void		getDevice ();
    Boolean		test_input (char *);
    int			computeNewBaud (void);

    static void		onWindowCallback (Widget, XtPointer, XtPointer);
    static void		onErrorCallback (Widget, XtPointer, XtPointer);

    static void		deviceChangedCallback (Widget, XtPointer, XtPointer);
    static void		ringsChangedCallback (Widget, XtPointer, XtPointer);
    static void		testCallback (Widget, XtPointer, XtPointer);
    static void		doneCallback (Widget, XtPointer, XtPointer);
    static void		interruptCallback (Widget, XtPointer, XtPointer);
    static void		verbChangedCallback (Widget, XtPointer, XtPointer);
    static void		quitCallback (Widget, XtPointer, XtPointer);

    void		deviceChanged_cb (void);
    void		ringsChanged_cb (char *);
    void		setup_test (enum Test);
    void		done_cb (void);
    void		interrupt_cb (void);
    void		verb_cb (int);
    void		quit_cb ();

    void		onWindow_cb ();
    void		onError_cb ();

    Boolean		setup ();
    Boolean		test ();
    Boolean		speed ();
    Boolean		regs ();
    Boolean		flags ();
    Boolean		save ();
};

#endif
