/*------------------------------------------------------------------------
 * $Id: ModemsWindow.c++,v 2.8 1994/02/15 04:20:10 chrisc Exp $
 *
 * Methods for the ModemsWindow class.
 *
 * Revision History:
 *	$Log: ModemsWindow.c++,v $
// Revision 2.8  1994/02/15  04:20:10  chrisc
// Changed all references to myApplication to uucpApplication.
//
// Revision 2.7  1994/01/20  22:14:57  carlson
// Added help callbacks for various widgets.
//
// Revision 2.6  1994/01/19  19:49:34  carlson
// Modified to have char strings big enough for entire file paths rather
//   than just for file names.
//
// Revision 2.5  1994/01/13  19:01:17  carlson
// Added code to handle opening a new modem when the file already exists.
//
// Revision 2.4  1994/01/12  00:53:58  carlson
// Added code to instantiate ModemSetup class.
// Added code to handle the directory before the file names.
//
// Revision 2.3  1994/01/11  22:35:57  carlson
// Removed the handlers for Window manager delete and quit messages.
// Delete this class rather than hide it.
//
// Revision 2.2  1994/01/06  01:23:08  carlson
// Added code to open EditModem window.
// Added select callback for when item is double clicked in list window.
// Added code for new callback.
// Added edit button and callback.
// Added destructor to unregister the window with help.
//
// Revision 2.1  1994/01/05  02:38:43  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include "uucp.h"
#include "ModemsWindow.h"
#include "EditModem.h"
#include "ModemSetup.h"
#include "HelpDialog.h"

#include <Vk/VkPromptDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkWarningDialog.h>

#include <unistd.h>
#include <iostream.h>
#include <dirent.h>
#include <limits.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/MainW.h>
#include <Xm/List.h>

void registerWindow4Help (Widget);
void unregisterWindow4Help (Widget);

/*------------------------------------------------------------------------
 * ModemsWindow::selectCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemsWindow::selectCallback (Widget, XtPointer clientData,
			       XtPointer callData)
{
    XmListCallbackStruct	*cbStruct;
    ModemsWindow		*mw;
    char			*text;

    cbStruct = (XmListCallbackStruct *)callData;
    mw = (ModemsWindow *)clientData;

    if (!XmStringGetLtoR (cbStruct->item, XmFONTLIST_DEFAULT_TAG, &text))
	theErrorDialog->postAndWait ("Unable to get list item!\n"
				     "Report this to SGI.");
    else
	mw->select_cb (text);
}

/*------------------------------------------------------------------------
 * ModemsWindow::select_cb (char *)
 *------------------------------------------------------------------------*/

void ModemsWindow::select_cb (char *modem)
{
    char		fileName[PATH_MAX];
    ModemSetup		*setup;

    strcpy (fileName, uucpApplication->directory ());
    strcat (fileName, "/");
    strcat (fileName, modem);
    strcat (fileName, ".mdm");

    if (access (fileName, R_OK) == 0)
    {
	setup = new ModemSetup (fileName);
	setup->show ();
	delete this;
    }
    else
    {
	char		msg[PATH_MAX];

	strcpy (msg, "Can't find file ");
	strcat (msg, fileName);
	theErrorDialog->postAndWait (msg);
    }
}

/*------------------------------------------------------------------------
 * ModemsWindow::okCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemsWindow::okCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemsWindow		*mw;

    mw = (ModemsWindow *)clientData;
    mw->ok_cb ();
}

/*------------------------------------------------------------------------
 * ModemsWindow::ok_cb ()
 *------------------------------------------------------------------------*/

void ModemsWindow::ok_cb ()
{
    int			*position_list, position_count;
    XmString		*selectedItems;
    char		*text = NULL;

    if (XmListGetSelectedPos (list, &position_list, &position_count))
    {
	XtVaGetValues (list, XmNselectedItems, &selectedItems, NULL);
	if (!XmStringGetLtoR (selectedItems[0], XmFONTLIST_DEFAULT_TAG, &text))
	{
	    theErrorDialog->postAndWait ("Unable to get selected list item!\n"
					 "Report this to SGI.");
	    text = NULL;
	}
	XFree (position_list);
    }

    if (text)
	select_cb (text);
    else
	delete this;
}

/*------------------------------------------------------------------------
 * ModemsWindow::newCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemsWindow::newCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemsWindow	*mw;

    mw = (ModemsWindow *)clientData;
    mw->new_cb ();
}

/*------------------------------------------------------------------------
 * ModemsWindow::new_cb ()
 *------------------------------------------------------------------------*/

void ModemsWindow::new_cb ()
{
    VkPromptDialog::VkDialogReason	reason;
    char				line[100];
    int					n, doit;
    EditModem				*editm;

    doit = False;
    reason = thePromptDialog->postAndWait ("Enter name of modem to add:");
    if (reason == VkPromptDialog::OK)
    {
	strcpy (line, uucpApplication->directory ());
	strcat (line, "/");
	strcat (line, thePromptDialog->text ());
	n = strlen (line);
	if (strcmp (&line[n-4], ".mdm"))
	    strcat (line, ".mdm");
	if (access (line, F_OK) == 0)
	{
	    reason = theWarningDialog->postAndWait ("File exists!\n"
						    "Overwrite?");
	    if (reason == VkWarningDialog::OK)
	    {
		if (unlink (line))
		    theErrorDialog->postAndWait ("Unable to remove file.");
		else
		    doit = True;
	    }
	}
	else
	    doit = True;
    }

    if (doit)
    {
	editm = new EditModem (line);
	editm->show ();

	delete this;
    }
}

/*------------------------------------------------------------------------
 * ModemsWindow::editCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemsWindow::editCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemsWindow	*mw;

    mw = (ModemsWindow *)clientData;
    mw->edit_cb ();
}

/*------------------------------------------------------------------------
 * ModemsWindow::edit_cb ()
 *------------------------------------------------------------------------*/

void ModemsWindow::edit_cb ()
{
    int				*position_list, position_count;
    XmString			*selectedItems;
    char			*text = NULL;
    EditModem			*editm;
    char			line[100];
    int				n;

    if (XmListGetSelectedPos (list, &position_list, &position_count))
    {
	XtVaGetValues (list, XmNselectedItems, &selectedItems, NULL);
	if (!XmStringGetLtoR (selectedItems[0], XmFONTLIST_DEFAULT_TAG, &text))
	{
	    theErrorDialog->postAndWait ("Unable to get selected list item!\n"
					 "Report this to SGI.");
	    text = NULL;
	}
	XFree (position_list);
    }

    if (text)
    {
	strcpy (line, uucpApplication->directory ());
	strcat (line, "/");
	strcat (line, text);
	n = strlen (line);
	if (strcmp (&line[n-4], ".mdm"))
	    strcat (line, ".mdm");
	editm = new EditModem (line);
	editm->show ();
    }

    delete this;
}

/*------------------------------------------------------------------------
 * ModemsWindow::cancelCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void ModemsWindow::cancelCallback (Widget, XtPointer clientData, XtPointer)
{
    ModemsWindow	*mw;

    mw = (ModemsWindow *)clientData;
    mw->cancel_cb ();
}

/*------------------------------------------------------------------------
 * ModemsWindow::cancel_cb ()
 *------------------------------------------------------------------------*/

void ModemsWindow::cancel_cb ()
{
    delete this;
}

/*------------------------------------------------------------------------
 * ModemsWindow::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget ModemsWindow::setUpInterface (Widget parent)
{
    Arg			wargs[10];
    int			n;
    struct dirent	*anEntry;
    DIR			*dir;
    Widget		temp, rc;
    char		fileName[PATH_MAX], *cptr;
    XmString		xmstr;

    registerWindow4Help (parentw = parent);

    /*----
     * Make sure the parent window (which is an XmMainWindow) shows
     * the separators between segments, automatically creates scrollbars
     * when needed and places the command window below the workspace
     * window.
     *----*/

    XtVaSetValues (parent,
		   XmNshowSeparator, True,
		   XmNscrollBarDisplayPolicy, XmAUTOMATIC,
		   XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE,
		   NULL);

    n = 0;
    XtSetArg (wargs[n], XmNlistSizePolicy, XmVARIABLE); n++;
    XtSetArg (wargs[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    XtSetArg (wargs[n], XmNvisibleItemCount, 4); n++;
    list = XmCreateScrolledList (parent, "modemList", wargs, n);

    dir = opendir (uucpApplication->directory ());
    if (dir == NULL)
	theErrorDialog->postAndWait ("Unable to open directory.");
    else
    {
	while ((anEntry = readdir (dir)))
	{
	    n = anEntry->d_reclen - sizeof (struct dirent) + 1;
	    memcpy (fileName, anEntry->d_name, n);
	    fileName[n] = '\0';

	    n = strlen (fileName);
	    cptr = fileName + n - 4;
	    if (n > 4 && strcmp (cptr, ".mdm") == 0)
	    {
		fileName[n-4] = '\0';
		xmstr = XmStringCreateSimple (fileName);
		XmListAddItemUnselected (list, xmstr, 0);
		XmStringFree (xmstr);
	    }
	}

	closedir (dir);
    }

    XtAddCallback (list, XmNdefaultActionCallback, selectCallback, this);
    XtAddCallback (list, XmNhelpCallback, aHelpCallback, "modemsWindow");

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    rc = XmCreateRowColumn (parent, "buttons", wargs, n);
    XtManageChild (rc);

    n = 0;
    temp = XmCreatePushButton (rc, "ok", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, okCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemsWindow");

    n = 0;
    temp = XmCreatePushButton (rc, "new", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, newCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemsWindow");

    n = 0;
    temp = XmCreatePushButton (rc, "edit", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, editCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemsWindow");

    n = 0;
    temp = XmCreatePushButton (rc, "cancel", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, cancelCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "modemsWindow");

    XtVaSetValues (parent, XmNcommandWindow, rc, NULL);

    return list;
}

/*------------------------------------------------------------------------
 * ModemsWindow::~ModemsWindow ()
 *------------------------------------------------------------------------*/

ModemsWindow::~ModemsWindow ()
{
    unregisterWindow4Help (parentw);
}
