/*------------------------------------------------------------------------
 * $Id: PackageSubWindow.c++,v 2.21 1994/03/02 23:06:59 carlson Exp $
 *
 * Top-level window which is generated from PackageWindow when one of
 * the files are selected.
 *
 * Revision History:
 *	$Log: PackageSubWindow.c++,v $
// Revision 2.21  1994/03/02  23:06:59  carlson
// Changed the symbol to reference legal drop sites because the symbols
//   I used are only defined in 5.2 and later.
//
// Revision 2.20  1994/02/25  23:03:44  carlson
// Must have forgotten to remove the deactivation of the save all menu option.
//
// Revision 2.19  1994/02/24  21:13:17  carlson
// Removed some writes to cout and cerr and changed them to raise an error
//   dialog.
//
// Revision 2.18  1994/02/23  23:33:54  carlson
// Removed include of DialcodesWindow.h.
// Cleaned up defining the menu pulldowns.
//
// Revision 2.17  1994/02/22  21:12:33  chrisc
// Added dummy label for when there are no files specified.
//
// Revision 2.16  1994/02/18  18:54:11  chrisc
// At some places when I delete the current child, I need to null out the
//   variable.
// Removed automatic variables for VkDialog stuff and just check the return
//   value directly.
// Changed around the way the quit asks the user if they really want to
//   quit because some file has changed.  This cleaned up a lot of code.
// Use theQuestionDialog instead of theWarningDialog so the user has two
//   buttons to choose from.
// Found a bug in my transfer function where it added the class directly
//   to the list and should have wrapped it in a FileName structure.
// Added code to transfer function that makes sure to add the package to
//   the list of packages using the child.
// No longer use command and message windows of parent window.  It seemed too
//   cumbersome and didn't add anything.
// Put everybody (including children classes) in an outer form.
//
// Revision 2.15  1994/02/17  21:04:12  chrisc
// Added support for an "On Window" help item.
//
// Revision 2.14  1994/02/17  06:01:47  chrisc
// Made name of shell a constant so it can be referred to in resources.
// Had to set the title by hand, now.
// Added functionality to "Save All" menu option.
// Cleaned up setting help callback for menubar.
// Fixed names of all menu options so they can be referred to in resources.
//
// Revision 2.13  1994/02/15  04:22:29  chrisc
// Changed all references to myApplication to uucpApplication.
// Removed all references to dialcodes.  It is now done separately in the
//   main window.
//
// Revision 2.12  1994/02/15  01:15:03  chrisc
// Fixed bug where _newFilePrompt wasn't initialized.
//
// Revision 2.11  1994/02/10  22:57:15  chrisc
// Modified remove method to also remove the package from the list of
//   packages that use the particular file.
//
// Revision 2.10  1994/02/10  21:24:11  chrisc
// Removed all references to selectedFile because there are better ways to do
//   this.
// Changed the name of delete buttons and methods to remove so it is more
//   explanitory as to what it is doing.
// Added 'Remove' function to pulldown menu.
// Made 'fileList' defined in 'setUpInterface' a part of the class so I don't
//   have to recompute it all the time.
// Use the method 'uucpDirectory' in myApplication instead of 'directory'
//   since the former is to refer to where files should be found and the
//   latter is for where help and vadmin files should be found.
// Using the new fileList variable, I was able to remove a whole chunk of
//   code from transfer_cb and make one small one do the same thing.
// Removed the ordering methods since they don't seem to do anything useful.
// Moved the help callback to the row-column widget so it doesn't have to be
//   provided for each button.
//
// Revision 2.9  1994/02/10  04:13:03  chrisc
// Added capability to select a file button without actually having it open
//   a subwindow.  I thought this might be important for the ability to
//   move the order of the files.
// Now a double click is required to open a file.
// Added drag and drop feature to files.  Now they can be reordered or even
//   moved from one package to another.
// *** Maybe next time I can remove the ordering methods ***
// Added code to allow for me to tell the RowColumn widget how to order the
//   buttons.  This is probably not necessary.
//
// Revision 2.8  1994/01/20  20:17:38  carlson
// Modified to support new kind of help facility.
//
// Revision 2.7  1994/01/18  05:41:54  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.6  1993/11/16  20:40:01  carlson
// Added code to be able to warn when closing a window that hasn't been
//   saved for Dialers and Dialcodes.
//
// Revision 2.5  1993/11/16  02:20:18  carlson
// Added code to check if device was modified before closing window.
//
// Revision 2.4  1993/11/16  01:37:05  carlson
// Added code to make sure System hasn't been modified before we quit.
//   This required making a method for the quit callback.
//
// Revision 2.3  1993/11/15  23:09:30  carlson
// Don't mess with default resources yet.
// Don't deactivate the "Save" option for default files!!!
//
// Revision 2.2  1993/11/13  00:39:47  carlson
// Fixed spelling error.
//
// Revision 2.1  1993/11/11  20:47:05  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include "PackageSubWindow.h"
#include "SystemWindow.h"
#include "DevicesWindow.h"
#include "DialersWindow.h"
#include "System.h"
#include "Devices.h"
#include "Dialers.h"
#include "Dialcodes.h"
#include "HelpDialog.h"
#include "uucp.h"

#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkWarningDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>
#include <Vk/VkHelpPane.h>
#include <Vk/VkMenuBar.h>

#include <limits.h>
#if defined(DEBUG1) || defined (DEBUG)
#include <iostream.h>
#endif

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/MainW.h>
#include <Xm/DragDrop.h>
#include <Xm/SeparatoG.h>
#include <Xm/AtomMgr.h>
#include <Xm/LabelG.h>

void registerWindow4Help (Widget);
void unregisterWindow4Help (Widget);

/*========================================================================
 * PackageSubWindow class
 *========================================================================*/

XtResource	PackageSubWindow::_resources [] = 
{
    {
	"newFilePrompt", "NewFilePrompt", XmRString,
	sizeof (char *),
	XtOffset (PackageSubWindow*, _newFilePrompt),
	XmRString,
	(XtPointer)"Select file to include:"
    }
};

/*------------------------------------------------------------------------
 * PackageSubWindow::PackageSubWindow (const char*, fileType, PackageWindow*)
 *
 *	Widget structure:
 *		+---------------------------+  Main        - (parent)
 *		|+-------------------------+|  RowColumn   - "work"
 *		||+----------+ +----------+||  PushButton  - System file
 *		|||          | |          |||
 *		||+----------+ +----------+||
 *		||     ...          ...    ||
 *		|+-------------------------+|
 *		|+-------------------------+|  Form        - "command"
 *		||                         ||     (parent for SystemWindow)
 *		|+-------------------------+|
 *		|+-------------------------+|  Form        - "message"
 *		||                         ||     (parent for HostWindow)
 *		|+-------------------------+|
 *		+---------------------------+
 *------------------------------------------------------------------------*/

PackageSubWindow::PackageSubWindow (const char* name, fileType t,
				    PackageWindow* pw)
: VkWindow ("packageFile")
{
    type = t;
    package = pw;
    current = NULL;
    dummy = NULL;
#if 0
    getResources (_resources, XtNumber (_resources));
#else
    _newFilePrompt = "Enter file to add:";
#endif
    setTitle (name);
}

/*------------------------------------------------------------------------
 * PackageSubWindow::~PackageSubWindow ()
 *------------------------------------------------------------------------*/

PackageSubWindow::~PackageSubWindow ()
{
    unregisterWindow4Help (parentw);

    if (current)
    {
	delete current;
	current = NULL;
    }
}

/*----------------------------------------------------------------------
 * saveCallback
 *
 * Static method called by X when the "Save" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::saveCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->save_cb ();
}

/*----------------------------------------------------------------------
 * save_cb
 *
 * Method called by PackageSubWindow::saveCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::save_cb ()
{
    SystemWindow	*s;
    DevicesWindow	*dev;
    DialersWindow	*dlr;

    switch (type)
    {
      case SYSTEMS:
	s = (SystemWindow *)current;
	if (s)
	    s->save ();
	break;
      case DEVICES:
	dev = (DevicesWindow *)current;
	if (dev)
	    dev->save ();
	break;
      case DIALERS:
	dlr = (DialersWindow *)current;
	if (dlr)
	    dlr->save ();
	break;
    }
}

/*----------------------------------------------------------------------
 * saveAllCallback
 *
 * Static method called by X when the "SaveAll" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::saveAllCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->saveAll_cb ();
}

/*----------------------------------------------------------------------
 * saveAll_cb
 *
 * Method called by PackageSubWindow::saveAllCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::saveAll_cb ()
{
    System	*s;
    Devices	*dev;
    Dialers	*dlr;

    switch (type)
    {
      case SYSTEMS:
	for (s = (System *)fileList->first (); s;
	     s = (System *)fileList->next ())
	    s->save ();
	break;
      case DEVICES:
	for (dev = (Devices *)fileList->first (); dev;
	     dev = (Devices *)fileList->next ())
	    dev->save ();
	break;
      case DIALERS:
	for (dlr = (Dialers *)fileList->first (); dlr;
	     dlr = (Dialers *)fileList->next ())
	    dlr->save ();
	break;
    }
}

/*----------------------------------------------------------------------
 * printCallback
 *
 * Static method called by X when the "Print" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::printCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->print_cb ();
}

/*----------------------------------------------------------------------
 * print_cb
 *
 * Method called by PackageSubWindow::printCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::print_cb ()
{
#ifdef DEBUG
    cout << "print_cb callback" << "\n";
#endif
}

/*----------------------------------------------------------------------
 * newCallback
 *
 * Static method called by X when the "New" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::newCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->new_cb ();
}

/*----------------------------------------------------------------------
 * new_cb
 *
 * Method called by PackageSubWindow::newCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::new_cb ()
{
    const char		*fname;
    PathName		fileName, tail;
    int			n;
    Arg			wargs[10];
    XmString		xmstr;
    Widget		temp;
    CallbackData	*cbData;
    Package::FileName	*fn;

    theFileSelectionDialog->setDirectory (package->dirPath.name ());

    switch (type)
    {
      case SYSTEMS:
	theFileSelectionDialog->setFilterPattern ("Systems*");
	break;

      case DEVICES:
	theFileSelectionDialog->setFilterPattern ("Devices*");
	break;

      case DIALERS:
	theFileSelectionDialog->setFilterPattern ("Dialers*");
	break;
    }

    if (theFileSelectionDialog->postAndWait (_newFilePrompt)
	== VkDialogManager::OK)
    {
	fname = theFileSelectionDialog->fileName ();
	fileName = (char *)fname;
	
	switch (type)
	{
	  case SYSTEMS:
	    fn = (Package::FileName *)package->addSystemFile (fileName);
	    break;
	    
	  case DEVICES:
	    fn = (Package::FileName *)package->addDeviceFile (fileName);
	    break;
	    
	  case DIALERS:
	    fn = (Package::FileName *)package->addDialerFile (fileName);
	    break;
	}
	
	n = 0;
	tail = (PathName &)fileName.tail ();
	xmstr = XmStringCreateSimple ((char *)tail.name ());
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	temp = XmCreatePushButton (wList, (char *)tail.name (), wargs, n);
	XtManageChild (temp);
	XmStringFree (xmstr);
	
	cbData = new CallbackData (fn, this);
	XtAddCallback (temp, XmNactivateCallback, fsCallback, cbData);

	if (dummy)
	{
	    XtDestroyWidget (dummy);
	    dummy = NULL;
	}
    }
}

/*----------------------------------------------------------------------
 * removeCallback
 *
 * Static method called by X when the "Remove" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::removeCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->remove_cb ();
}

/*----------------------------------------------------------------------
 * remove_cb
 *
 * Method called by PackageSubWindow::removeCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::remove_cb ()
{
    Widget		lastButton;
    XmString		xmstr;
    char		*buttonName;
    PathName		fullName;
    char		line[128];
    LinkData		*ld;
    Package::FileName	*fn;
    System		*sys;
    Devices		*dev;
    Dialers		*dlrs;
    int			nChildren;

    XtVaGetValues (wList,
		   XmNmenuHistory, &lastButton,
		   XmNnumChildren, &nChildren, NULL);
    if (lastButton)
    {
	XtVaGetValues (lastButton, XmNlabelString, &xmstr, NULL);
	XmStringGetLtoR (xmstr, XmFONTLIST_DEFAULT_TAG, &buttonName);

	fullName = uucpApplication->uucpDirectory ();
	fullName += buttonName;
	fullName.expand ();

	ld = fileList->find (fullName);
	if (!ld)
	{
	    strcpy (line, "Internal error with removing ");
	    strcat (line, fullName.name ());
	    theErrorDialog->postAndWait (line);
	    return;
	}

	strcpy (line, "Remove ");
	strcat (line, fullName.name ());
	strcat (line, "\n from search list for ");
	strcat (line, package->theKey ());
	strcat (line, "?");
	if (theQuestionDialog->postAndWait (line) == VkDialogManager::OK)
	{
	    if (current)
	    {
		delete current;
		current = NULL;
	    }

	    fileList->removeEntry (ld);
	    if (nChildren == 1)
	    {
		dummy = XmCreateLabelGadget (wList, "none", NULL, 0);
		XtManageChild (dummy);
	    }
	    XtDestroyWidget (lastButton);

	    fn = (Package::FileName *)ld;
	    switch (type)
	    {
	      case SYSTEMS:
		sys = (System *)fn->data;
		sys->removePackage (package);
		break;

	      case DEVICES:
		dev = (Devices *)fn->data;
		dev->removePackage (package);
		break;

	      case DIALERS:
		dlrs = (Dialers *)fn->data;
		dlrs->removePackage (package);
		break;
	    }
	}
    }
    else
    {
	theErrorDialog->postAndWait ("Don't know which file to remove.");
    }
}

#ifdef DEBUG
/*----------------------------------------------------------------------
 * dumpCallback
 *
 * Static method called by X when the "Dump" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::dumpCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow *obj = (PackageSubWindow *) clientData;
    obj->dump_cb ();
}

/*----------------------------------------------------------------------
 * dump_cb
 *
 * Method called by PackageSubWindow::dumpCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void PackageSubWindow::dump_cb ()
{
    dump ("");
}
#endif

/*----------------------------------------------------------------------
 * quitCallback
 *
 * Static method called by X when the "Quit" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void PackageSubWindow::quitCallback (Widget, XtPointer clientData,
				 XtPointer)
{
    PackageSubWindow			*sw;

    sw = (PackageSubWindow *)clientData;
    sw->quit_cb ();
}

/*------------------------------------------------------------------------
 * PackageSubWindow::quit_cb ()
 *------------------------------------------------------------------------*/

void PackageSubWindow::quit_cb ()
{
    char			*typeTxt = NULL;
    char			line[80];
    SystemWindow		*sw;
    DevicesWindow		*dw;
    DialersWindow		*dlw;

    switch (type)
    {
      case SYSTEMS:
	sw = (SystemWindow *)current;
	if (sw && sw->wasModified ())
	    typeTxt = "Systems";
	break;

      case DEVICES:
	dw = (DevicesWindow *)current;
	if (dw && dw->wasModified ())
	    typeTxt = "Devices";
	break;

      case DIALERS:
	dlw = (DialersWindow *)current;
	if (dlw && dlw->wasModified ())
	    typeTxt = "Dialers";
	break;
    }

    if (typeTxt)
    {
	strcpy (line, typeTxt);
	strcat (line, " file was modified.\nAre you sure you want to quit?");

	if (theQuestionDialog->postAndWait (line) == VkDialogManager::OK)
	{
	    switch (type)
	    {
	      case SYSTEMS:
		package->sysPW = NULL;
		break;

	      case DEVICES:
		package->devPW = NULL;
		break;

	      case DIALERS:
		package->dlrPW = NULL;
		break;
	    }
	    delete this;
	}
    }
    else
	delete this;
}

/*------------------------------------------------------------------------
 * PackageSubWindow::fsCallback
 *	File select callback function.
 *------------------------------------------------------------------------*/

void PackageSubWindow::fsCallback (Widget, XtPointer clientData,
				   XtPointer callData)
{
    XmPushButtonCallbackStruct	*pb_data;
    CallbackData		*cbData;

    pb_data = (XmPushButtonCallbackStruct *)callData;
    cbData = (CallbackData *)clientData;
    cbData->This->fs_cb (cbData->fileData, pb_data->click_count);
}

/*------------------------------------------------------------------------
 * PackageSubWindow::fs_cb
 *------------------------------------------------------------------------*/

void PackageSubWindow::fs_cb (Package::FileName *fileData, int click_count)
{
    SystemWindow	*sw;
    DevicesWindow	*dw;
    DialersWindow	*dlw;

    if (click_count > 1)
    {
	if (current)
	{
	    delete current;
	    current = NULL;
	}

	switch (type)
	{
	  case SYSTEMS:
	    sw = new SystemWindow (command, message, (System *)fileData->data,
				   &package->devList, &package->dlrList);
	    current = sw;
	    break;

	  case DEVICES:
	    dw = new DevicesWindow (command, message,
				    (Devices *)fileData->data,
				    &package->dlrList);
	    current = dw;
	    break;

	  case DIALERS:
	    dlw = new DialersWindow (command, message,
				     (Dialers *)fileData->data);
	    current = dlw;
	    break;
	}

	if (current)
	    current->show ();
    }
}

/*------------------------------------------------------------------------
 * PackageSubWindow::onWindowCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void PackageSubWindow::onWindowCallback (Widget, XtPointer clientData,
					 XtPointer)
{
    PackageSubWindow		*mw;

    mw = (PackageSubWindow *)clientData;
    mw->onWindow_cb ();
}

/*------------------------------------------------------------------------
 * PackageSubWindow::onWindow_cb ()
 *------------------------------------------------------------------------*/

void PackageSubWindow::onWindow_cb ()
{
    theHelpDialog->displayHelp ("PackageSubWindowOW");
}

/*------------------------------------------------------------------------
 * PackageSubWindow::dropCallback (Widget, XtPointer, XtPointer)
 *
 * Things I've learned about drop callbacks:
 *	1. clientData - is always NULL.  There is no way to get it to
 *	   be anything else because it is passed as NULL from Xlib.
 *	2. The drop callback will be called when anything is dropped.
 *	   It is then up to the callback routine to determine if it
 *	   will handle the requested callback.  If the importTargets
 *	   don't match the exportTargets or if the operations supported
 *	   don't match one that is requested, dropSiteStatus will be
 *	   initially filled with DROP_SITE_INVALID.  Otherwise it will
 *	   be filled with DROP_SITE_VALID.
 *	3. A drop can still be accepted by changing the dropSiteStatus.
 *	4. The drop does not complete until XmDropTransferStart is called
 *	   and the resource XmNtransferStatus is set to XmTRANSFER_SUCCESS.
 * 	   This can be done when the widget is created.
 *	5. Pushbutton drag contexts have their client data pre-filled
 *	   with something.
 *	6. It appears that it is possible to change the operation.  If it
 *	   initially was an XmDROP_COPY, changing it to XmDROP_MOVE should
 *	   work.
 *	7. Pushbutton drag has XmDROP_COPY as the requested operation.
 *------------------------------------------------------------------------*/

void PackageSubWindow::dropCallback (Widget w, XtPointer, XtPointer callData)
{
    XmDropProcCallback		cbData;
    Widget			dragContext, dropTransfer;
    Arg				wargs[10];
    int				n;
    XmDropTransferEntryRec	transferRec;
    Atom			atoms[1];
    TransferData		*td;

    /*----
     * Cast callData to the proper structure pointer.
     * Separate drag context widget.
     *----*/

    cbData = (XmDropProcCallback)callData;
    dragContext = cbData->dragContext;

#ifdef DEBUG1
    cout << "Drop callback occurred:" << endl;
    cout << "   widget     = " << w << endl;
    XtVaGetValues (w, XmNuserData, &dropTransfer, NULL);
    cout << "     userData = " << dropTransfer << endl;
    cout << "   clientData = " << clientData << endl;
    cout << "   callData {" << endl;
    cout << "      reason        = " << cbData->reason << endl;
    cout << "      dragContext   = " << cbData->dragContext << endl;
    cout << "      x             = " << cbData->x << endl;
    cout << "      y             = " << cbData->y << endl;
    cout << "      ds_status     = ";
    switch (cbData->dropSiteStatus)
    {
      case XmNO_DROP_SITE:
	cout << "NO_DROP_SITE" << endl;
	break;

      case XmVALID_DROP_SITE:
	cout << "DROP_SITE_VALID" << endl;
	break;

      case XmINVALID_DROP_SITE:
	cout << "DROP_SITE_INVALID" << endl;
	break;

      default:
	cout << "0x" << hex << (int)cbData->dropSiteStatus << endl;
	break;
    }

    cout << "      ds_operation  = 0x" << hex << (int)cbData->operation <<
	endl;
    cout << "      ds_operations = 0x" << hex << (int)cbData->operations <<
	endl;
    cout << "      ds_action     = 0x" << hex << (int)cbData->dropAction <<
	endl;
    cout << "   }" << endl;

    XtVaGetValues (dragContext, XmNclientData, &dropTransfer, NULL);
    if (dropTransfer)
	cout << "  clientdata was " << dropTransfer << endl;
    else
	XtVaSetValues (dragContext, XmNclientData, 5, NULL);
#endif

    /*----
     * If the status indicates that we were not a valid drop site,
     * it must be because the data being passed is not of type
     * COMPOUND_TEXT, since we accept all kinds of drops.  If this
     * is the case, we're going to just ignore it.
     *----*/

    if (cbData->dropSiteStatus != XmVALID_DROP_SITE)
    {
	n = 0;
	XtSetArg (wargs[n], XmNtransferStatus, XmTRANSFER_FAILURE); n++;
	XtSetArg (wargs[n], XmNtransferProc, transferCallback); n++;
	dropTransfer = XmDropTransferStart (dragContext, wargs, n);
	return;
    }

    /*----
     * Create a structure of information to pass to transfer_cb.
     * Save info that we've received here.
     *	    Note that we saved the 'this' pointer in the userData
     *	    portion of the RowColumn widget that was set up to
     *	    accept drops.
     *----*/

    td = new TransferData;
    atoms[0] = XInternAtom (uucpApplication->display (), "COMPOUND_TEXT",
			    False);
    XtVaGetValues (w, XmNuserData, &td->This, NULL);
    td->x = cbData->x;
    td->y = cbData->y;
    td->button = NULL;
    transferRec.client_data = td;
    transferRec.target = atoms[0];

    /*----
     * Here we indicate that we are ready to start the transfer.
     * We get a transfer widget but I really don't know what to do
     * with it.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNdropTransfers, &transferRec); n++;
    XtSetArg (wargs[n], XmNnumDropTransfers, 1); n++;
    XtSetArg (wargs[n], XmNtransferProc, transferCallback); n++;
    dropTransfer = XmDropTransferStart (dragContext, wargs, n);

    /*----
     * Finally, we tell the dragdrop mechanism that we want to do
     * a move, rather than a copy.
     *----*/

    cbData->operation = (unsigned char)XmDROP_MOVE;
}

/*------------------------------------------------------------------------
 * PackageSubWindow::transferCallback (Widget, XtPointer, Atom, Atom,
 *				       XtPointer, unsigned long*, int*)
 *
 * Widget	Drop transfer widget
 * XtPointer	Client data from transfer record.
 * Atom		Selection atom = _MOTIF_DROP.
 * Atom		Type
 * XtPointer	value
 * unsigned long *length
 * int*		format
 *------------------------------------------------------------------------*/

void PackageSubWindow::transferCallback (
#ifdef DEBUG1
					 Widget w,
					 XtPointer clientData,
					 Atom *selection, Atom *type,
					 XtPointer value,
					 unsigned long *length,
					 int *format
#else
					 Widget,
					 XtPointer clientData,
					 Atom *, Atom *,
					 XtPointer value,
					 unsigned long *length,
					 int *
#endif
					 )
{
    PackageSubWindow		*psw;
    char			name[50];
    TransferData		*td;
#ifdef DEBUG1
    char			*str;
#endif

    /*----
     * For the buttons that I'm expecting, the value should be
     * just a char* value.  I copy it into a local buffer for
     * transfer_cb and null terminate it.  (I'm not sure whether
     * X null terminated it but this way I'm sure).
     *----*/

    strncpy (name, (char *)value, (unsigned int)*length);
    name[*length] = '\0';

#ifdef DEBUG1
    cout << "transfer callback" << endl;
    cout << "   widget     = " << w << endl;
    cout << "   clientData = " << clientData << endl;
//    cout << "   selection  = 0x" << hex << *selection << endl;
//    cout << "   type       = 0x" << hex << *type << endl;
    str = XmGetAtomName (uucpApplication->display (), *selection);
    cout << "   selection  = " << str << endl;
    str = XmGetAtomName (uucpApplication->display (), *type);
    cout << "   type       = " << str << endl;
    cout << "   value      = " << value << endl;
    cout << "   length     = " << *length << endl;
    cout << "   format     = " << *format << endl;

    cout << "   value string = " << name << endl;
#endif

    /*----
     * Get my structure and fill in the last piece of information
     * to pass to transfer_cb.
     *----*/

    td = (TransferData *)clientData;
    td->button = name;

    /*----
     * Get instance of class and call transfer_cb for it.
     *----*/

    psw = td->This;
    psw->transfer_cb (td);

    /*----
     * If we don't do this, it will never get deleted.
     *----*/

    delete td;
}

/*------------------------------------------------------------------------
 * PackageSubWindow::transfer_cb (TransferData *)
 *------------------------------------------------------------------------*/

void PackageSubWindow::transfer_cb (TransferData *td)
{
    LinkData					*ld;
    WidgetList					wl;
    Cardinal					nc, i, j;
    Position					cx, cy;
    Dimension					cw, ch;
    char					*cname, *tname;
    XmString					xmstr;
    int						newChild;
    Widget					movedChild;
    PathName					fullName, fullCName;
    Package::FileName				*fn;
    enum POSITION { NONE, BEFORE, AFTER }	pos;

#ifdef DEBUG1
    cout << "transfer_cb called" << endl;
    cout << "   transferData = " << td << endl;
    cout << "     x          = " << td->x << endl;
    cout << "     y          = " << td->y << endl;
    cout << "     button     = " << td->button << endl;
#endif

    fullName = uucpApplication->uucpDirectory ();
    fullName += td->button;
    fullName.expand ();

    /*----
     * First, let's find out where this button is supposed to go.
     * Scan the children and find out where this X,Y coordinate
     * relates.
     *
     * When done with this loop:
     *	    pos = BEFORE	New button should be placed before
     *				the 'i'th child.
     *	    pos = AFTER		New button should be placed at end.
     *----*/

    XtVaGetValues (wList, XmNchildren, &wl, XmNnumChildren, &nc, NULL);

    pos = NONE;
    movedChild = NULL;
    for (j = 0; j < nc; j++)
    {
	XtVaGetValues (wl[j],
		       XmNheight, &ch, XmNwidth, &cw,
		       XmNx, &cx, XmNy, &cy,
		       XmNlabelString, &xmstr, NULL);
	XmStringGetLtoR (xmstr, XmFONTLIST_DEFAULT_TAG, &tname);
	if (strcmp (td->button, tname) == 0)
	    movedChild = wl[j];
	if (td->x < (cx + (cw / 2)))
	{
	    cname = new char [strlen (tname) + 1];
	    strcpy (cname, tname);
	    i = j;
	    pos = BEFORE;
	}
    }
    if (pos == NONE)
    {
	pos = AFTER;
	i = nc - 1;
	XtVaGetValues (wl[i], XmNlabelString, &xmstr, NULL);
	XmStringGetLtoR (xmstr, XmFONTLIST_DEFAULT_TAG, &tname);
	cname = new char [strlen (tname) + 1];
	strcpy (cname, tname);
    }
    fullCName = uucpApplication->uucpDirectory ();
    fullCName += cname;
    fullCName.expand ();

    /*----
     * The next thing we need to do is find out if the button being
     * moved is from this instance or from another instance of this
     * PackageSubWindow.  If it is from this instance, then we need
     * to remove the entry so we can re-install it.
     *----*/

    newChild = False;
    fn = (Package::FileName*)fileList->find (fullName);
    if (fn)
	fileList->removeEntry ();
    else
    {
	ld = package->allFiles[(int)type].find (fullName);
	if (!ld)
	    return;
	fn = new Package::FileName (fullName);
	fn->data = ld;
	newChild = True;
    }

    fileList->find (fullCName);
    if (pos == BEFORE)
	fileList->addBefore (fn);
    else
	fileList->addAfter (fn);

    if (newChild)
    {
	Arg				wargs[10];
	Widget				temp;
	CallbackData			*cbData;
	PathName			tail;
	int				n;
	System				*sys;
	Devices				*dev;
	Dialers				*dlrs;

	if (pos == AFTER)
	    i++;

	n = 0;
	tail = (PathName &)fullName.tail ();
	xmstr = XmStringCreateSimple ((char *)tail.name ());
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	XtSetArg (wargs[n], XmNuserData, this); n++;
	XtSetArg (wargs[n], XmNpositionIndex, i); n++;
	temp = XmCreatePushButton (wList, (char *)tail.name (), wargs, n);
	XtManageChild (temp);
	XmStringFree (xmstr);

	cbData = new CallbackData (fn, this);
	XtAddCallback (temp, XmNactivateCallback, fsCallback, cbData);
	XtAddCallback (temp, XmNhelpCallback, aHelpCallback,
		       "psw_button_help");

	if (dummy)
	{
	    XtDestroyWidget (dummy);
	    dummy = NULL;
	}

	switch (type)
	{
	  case SYSTEMS:
	    sys = (System *)ld;
	    sys->addPackage (package);
	    break;

	  case DEVICES:
	    dev = (Devices *)ld;
	    dev->addPackage (package);
	    break;

	  case DIALERS:
	    dlrs = (Dialers *)ld;
	    dlrs->addPackage (package);
	    break;

	}
    }
    else if (!movedChild)
	theErrorDialog->postAndWait ("Internal error in "
				     "PackageSubWindow::transfer_cb");
    else
	XtVaSetValues (movedChild, XmNpositionIndex, i, NULL);

    delete [] cname;
}

/*------------------------------------------------------------------------
 * PackageSubWindow::setUpInterface (Widget)
 *	This method is used to set up the widgets in the window.  Since
 *	the parent is a MainWindow, we\'ll take advantage of this by
 *	using the work region (with one scroll bar) to display the
 *	buttons for the files, we\'ll place the command window below
 *	the work region and make it actually a SystemWindow, and we\'ll
 *	use the message window as the HostWindow.
 *------------------------------------------------------------------------*/

Widget PackageSubWindow::setUpInterface (Widget parent)
{
    Arg			wargs[10];
    int			n;
    XmString		xmstr;
    Widget		temp, outerForm;
    CallbackData	*cbData;
    PathName		fname, tail;
    LinkData		*ld;
    Atom		atoms[1];
    VkMenuBar		*menubar;
    VkHelpPane		*helpMenu;

    XtVaSetValues (XtParent (parent),
		   XmNallowShellResize, True,
		   NULL);

    registerWindow4Help (parentw = parent);

    n = 0;
    outerForm = XmCreateForm (parent, "packageSub", wargs, n);

    /*----
     * Create rowColumn for putting buttons in for each file
     * in the file list.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNuserData, this); n++;
    wList = XmCreateRowColumn (outerForm, "fileList", wargs, n);
    XtManageChild (wList);

    XtAddCallback (wList, XmNhelpCallback, aHelpCallback, "psw_button_help");

#ifdef DEBUG1
    cout << "Row column widget = " << wList << endl;
#endif

    atoms[0] = XInternAtom (uucpApplication->display (), "COMPOUND_TEXT",
			    False);

    n = 0;
    XtSetArg (wargs[n], XmNimportTargets, atoms); n++;
    XtSetArg (wargs[n], XmNnumImportTargets, 1); n++;
    XtSetArg (wargs[n], XmNdropSiteOperations,
	      XmDROP_MOVE | XmDROP_COPY | XmDROP_LINK); n++;
    XtSetArg (wargs[n], XmNdropProc, dropCallback); n++;
    XmDropSiteRegister (wList, wargs, n);

    /*----
     * Loop through the file list and make a pushbutton for
     * each.
     *----*/

    switch (type)
    {
      case SYSTEMS:
	fileList = &package->systems;
	break;
      case DEVICES:
	fileList = &package->devices;
	break;
      case DIALERS:
	fileList = &package->dialers;
	break;
    }

    cbData = NULL;			// Use this to know first time
    for (ld = fileList->first (); ld; ld = fileList->next ())
    {
	n = 0;
	fname = ld->theKey ();
	tail = (PathName &)fname.tail ();
	xmstr = XmStringCreateSimple ((char *)tail.name ());
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	XtSetArg (wargs[n], XmNuserData, this); n++;
	temp = XmCreatePushButton (wList, (char *)tail.name (), wargs, n);
	XtManageChild (temp);
	XmStringFree (xmstr);

	if (cbData == NULL)
	    XtVaSetValues (wList, XmNmenuHistory, temp, NULL);

	cbData = new CallbackData ((Package::FileName *)ld, this);
	XtAddCallback (temp, XmNactivateCallback, fsCallback, cbData);
    }

    /*----
     * If there were no files to put in the window, let's put
     * a dummy label there to inform the user.
     *----*/

    if (!cbData)
    {
	n = 0;
	dummy = XmCreateLabelGadget (wList, "none", wargs, n);
	XtManageChild (dummy);
    }

    /*----
     * Put in the separator gadgets and create forms for the children
     * windows which will be generated if the user selects on of the
     * file buttons.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, wList); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    temp = XmCreateSeparatorGadget (outerForm, "sep1", wargs, n);
    XtManageChild (temp);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    command = XmCreateForm (outerForm, "command", wargs, n);
    XtManageChild (command);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, command); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    temp = XmCreateSeparatorGadget (outerForm, "sep1", wargs, n);
    XtManageChild (temp);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    message = XmCreateForm (outerForm, "message", wargs, n);
    XtManageChild (message);

    /*----
     * Add a menu pane
     *----*/
    
    fileMenuPane = addMenuPane ("files");
    temp = fileMenuPane->baseWidget ();
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "psw_file_help");

    fileMenuPane->addAction ("save", saveCallback, this);
    fileMenuPane->addAction ("saveAll", saveAllCallback, this);
    fileMenuPane->addAction ("print", printCallback, this);
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("new", newCallback, this);
    fileMenuPane->addAction ("remove", removeCallback, this);
#ifdef DEBUG
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("dump", dumpCallback, this);
#endif
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("quit", quitCallback, this);

    // Add items to the Help menu

    menubar = menu ();
    helpMenu = menubar->helpPane ();
    helpMenu->addSeparator ();
    helpMenu->addAction ("window", onWindowCallback, this);

    if (strcmp (package->theKey (), "default") == 0)
    {
	fileMenuPane->deactivateItem ("saveAll");
	fileMenuPane->deactivateItem ("new");
	fileMenuPane->deactivateItem ("remove");
    }

    /*----
     * Deactivate menu items that don't do anything yet.
     *----*/

    fileMenuPane->deactivateItem ("print");

    return outerForm;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * PackageSubWindow::dump (char *)
 *------------------------------------------------------------------------*/

void PackageSubWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;
    SystemWindow	*sw;
    DevicesWindow	*dw;
    DialersWindow	*dlw;
    char		*typec;
    
    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");
    
    cout << indent << "PackageSubWindow (" << this << "):" << endl;
    cout << indent << "  _newFilePrompt = '" << _newFilePrompt << "'" << endl;
    typec = "Unknown";
    switch (type)
    {
      case SYSTEMS:
	typec = "SYSTEMS";
	break;
      case DEVICES:
	typec = "DEVICES";
	break;
      case DIALERS:
	typec = "DIALERS";
	break;
    }
    cout << indent << "  type           = " << typec << endl;
    cout << indent << "  PackageWindow  = " << package << endl;
    cout << indent << "  current        = " << current << endl;

    package->dumpMin (nextIndent);

    if (current)
    {
	switch (type)
	{
	  case SYSTEMS:
	    sw = (SystemWindow *)current;
	    sw->dump (nextIndent);
	    break;
	    
	  case DEVICES:
	    dw = (DevicesWindow *)current;
	    dw->dump (nextIndent);
	    break;
	    
	  case DIALERS:
	    dlw = (DialersWindow *)current;
	    dlw->dump (nextIndent);
	    break;
	}
    }
}
#endif
