/*---*- C++ -*------------------------------------------------------------
 * $Id: PackageSubWindow.h,v 2.8 1994/02/22 21:14:39 chrisc Exp $
 *
 * Header file for PackageSubWindow class.
 *
 * Revision History:
 *	$Log: PackageSubWindow.h,v $
 * Revision 2.8  1994/02/22  21:14:39  chrisc
 * Added place to keep dummy label widget for when no files are specified.
 *
 * Revision 2.7  1994/02/17  21:04:35  chrisc
 * Added prototypes for "On Window" help callbacks.
 *
 * Revision 2.6  1994/02/17  06:05:28  chrisc
 * Added method for calling the save all callback from another function.
 *
 * Revision 2.5  1994/02/15  04:23:37  chrisc
 * No longer support dialcodes file type.
 *
 * Revision 2.4  1994/02/10  21:29:03  chrisc
 * Added fileList from setUpInterface so it doesn't have to be recomputed
 *   all the time.
 * Removed the selectedFile variable.
 * Changed the name of the delete callbacks to be remove so it is more
 *   descriptive of what it is doing.
 * Removed the order methods since we don't really need them.
 *
 * Revision 2.3  1994/02/10  04:17:29  chrisc
 * Added structure to pass when transferring a button.
 * Added prototypes for methods required to transfer buttons and to specify
 *   the ordering of the buttons.
 *
 * Revision 2.2  1993/11/16  01:31:50  carlson
 * Did some clean up.
 * Added a quit_cb method.
 *
 * Revision 2.1  1993/11/11  18:41:08  carlson
 * Broken out from PackageWindow.h.
 *
 *------------------------------------------------------------------------*/

#ifndef _PACKAGESUBWINDOW_H_
#define _PACKAGESUBWINDOW_H_

#include <Vk/VkWindow.h>
#include <Vk/VkMenu.h>
#include <Vk/VkSubMenu.h>

#include "PackageWindow.h"

/*------------------------------------------------------------------------
 * PackageSubWindow class
 *	This class creates a new window into which all the information
 *	about one of the file lists created above is displayed and
 *	acted upon.
 *------------------------------------------------------------------------*/

class PackageSubWindow : public VkWindow
{
  public:
    enum fileType { SYSTEMS, DEVICES, DIALERS };
    struct CallbackData {
	Package::FileName	*fileData;
	PackageSubWindow	*This;

	CallbackData (Package::FileName *f, PackageSubWindow *p) {
	    fileData = f; This = p; }
    };
    struct TransferData {
	PackageSubWindow	*This;
	Position		x;
	Position		y;
	char			*button;
    };

    PackageSubWindow (const char *name, fileType t, PackageWindow *pw);
    ~PackageSubWindow ();

    void		saveAll (void) { saveAll_cb (); }

    virtual const char* className () { return "PackageSubWindow"; }
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  protected:
    char		*_newFilePrompt;
    virtual Widget	setUpInterface (Widget parent);

  private:
    fileType		type;
    SmartList		*fileList;	// Pointer to systems/devices...
    PackageWindow*	package;
    Widget		parentw;	// Gotta have this
    Widget		wList;		// RowColumn of list of files
    Widget		command;	// System|Devices|Dialers...Window
    Widget		message;	// Host|Device|Dialer...Window
    VkComponent*	current;
    VkSubMenu*		fileMenuPane;
    Widget		dummy;

    static XtResource	_resources[];

    static void		fsCallback (Widget, XtPointer, XtPointer);
    static void		saveCallback (Widget, XtPointer, XtPointer);
    static void		saveAllCallback (Widget, XtPointer, XtPointer);
    static void		printCallback (Widget, XtPointer, XtPointer);
    static void		newCallback (Widget, XtPointer, XtPointer);
    static void		removeCallback (Widget, XtPointer, XtPointer);
#ifdef DEBUG
    static void		dumpCallback (Widget, XtPointer, XtPointer);
#endif
    static void		quitCallback (Widget, XtPointer, XtPointer);
    static void		onWindowCallback (Widget, XtPointer, XtPointer);
    static void		dropCallback (Widget, XtPointer, XtPointer);
    static void		transferCallback (Widget, XtPointer, Atom*, Atom*,
					  XtPointer, unsigned long*, int*);

    void		fs_cb (Package::FileName *fileData, int click_count);
    void		save_cb ();
    void		saveAll_cb ();
    void		print_cb ();
    void		new_cb ();
    void		remove_cb ();
#ifdef DEBUG
    void		dump_cb ();
#endif
    void		quit_cb ();
    void		onWindow_cb ();
    void		transfer_cb (TransferData *);
};

#endif
