/*------------------------------------------------------------------------
 * $Id: PackageWindow.c++,v 2.40 1994/02/26 03:42:09 carlson Exp $
 *
 * Methods for PackageWindow class.
 *
 * Revision History:
 *	$Log: PackageWindow.c++,v $
// Revision 2.40  1994/02/26  03:42:09  carlson
// Modified the methods to handle the systems, devices or dialers list
//   changing to work better.
// Modified the method to update the expanded list of systems, devices and
//   dialers to work better.
// All of this was to support multiple hosts, devices and dialers with the
//   same name but something else different.
//
// Revision 2.39  1994/02/23  23:33:55  carlson
// Removed include of DialcodesWindow.h.
//
// Revision 2.38  1994/02/18  18:59:59  chrisc
// Changed name of some of the ifdef'd symbols.
//
// Revision 2.37  1994/02/16  22:24:08  chrisc
// For the PackageSubWindows, show the window only once, when it is created
//   but if it is selected again, raise and open (de-iconify) it.
//
// Revision 2.36  1994/02/16  22:18:52  chrisc
// Moved the showing and raising of the various PackageSubWindows to be
//   outside of the test if the class has been created.  This way, if it
//   has been but the user has lost the window, it will automatically be
//   raised to the top.
//
// Revision 2.35  1994/02/15  04:24:01  chrisc
// Removed all references to dialcodes.  It is now taken care of separately
//   from the main window.
//
// Revision 2.34  1994/01/20  20:17:38  carlson
// Modified to support new kind of help facility.
//
// Revision 2.33  1994/01/18  03:31:31  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.32  1994/01/04  21:03:58  carlson
// Ifdef'd out some print statements.
//
// Revision 2.31  1993/12/29  01:38:30  carlson
// Tried adding a comma to the resources.
// Enabled debug printout if I decide to enable resource checking again.
//
// Revision 2.30  1993/11/16  01:09:26  carlson
// Added code to indicate we've been modified.
//
// Revision 2.29  1993/11/12  23:51:44  carlson
// Renamed some of the class members to make more sense.
//
// Revision 2.28  1993/11/11  18:40:20  carlson
// Broke out PackageSubWindow stuff into its own file.
// Fixed bug where it hangs whenever creating a new file.
//
// Revision 2.27  1993/11/11  01:03:23  carlson
// Added code to get callbacks for all the lists generated.
// Put conditionals around a bunch of print statements.
//
// Revision 2.26  1993/11/05  20:09:29  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.25  1993/11/05  00:14:23  carlson
// Removed a debug print statement that I forgot.
//
// Revision 2.24  1993/11/04  00:59:11  carlson
// Added help to PackageSubWindow buttons and labels.
//
// Revision 2.23  1993/11/03  18:58:09  carlson
// Added help callbacks for all the things that you can get help for.
// Decided to move all the callback definitions so they are with the
//   creation of the widgets.
//
// Revision 2.22  1993/11/02  17:40:23  carlson
// Forgot to surround some functions with conditional compilation based on
//   the variable DEBUG.  This caused errors when trying to compile without
//   the variable defined.
//
// Revision 2.21  1993/10/19  23:16:31  carlson
// Now the minimum dump will also dump the Package.
// If this package is the default package, deactivate menu buttons that
//   allow the user to change it.
//
// Revision 2.20  1993/10/19  21:27:05  carlson
// Modified dump method to call Package::dump and print all the names of
//   the elements in each of the file lists.
// Added a method to PackageWindow that just dumps information, like the
//   method dump, except doesn't recurse down to other classes.
// Modified dump output to actually tell what type package it is.
//
// Revision 2.19  1993/10/19  00:30:35  carlson
// Messed around with trying to get resources to work.  Gave up.
//
// Revision 2.18  1993/10/18  19:59:25  carlson
// Added a resource list for additional resources.  This isn't finished
//   yet because I was waiting for information on when to call one of the
//   ViewKit methods to update the resources.
// Added code to add a new system file.  This code will be duplicated (as
//   soon as I'm sure it works) for all the other files.
// For PackageWindow, added a 'dump' method which prints all the contents
//   to stdout for debug purposes.
// Added a set of methods for a new menu item to dump the contents of all
//   the classes.
//
// Revision 2.17  1993/10/08  18:41:06  carlson
// No longer need all_files list for addDialcodes because it is retained from
//   when the package was created.
// Before adding elements to the host/device/dialers/dialcodes list, make
//   sure we aren't duplicating any.
//
// Revision 2.16  1993/10/07  22:53:28  carlson
// Started setting up to allow "New" menu option to work.
// Added the ability to pass the path when instantiated.
//
// Revision 2.15  1993/10/07  20:14:47  carlson
// Removed all the commented out code.
//
// Revision 2.14  1993/10/07  20:06:58  chris
// Had to move the label at the bottom of each package to be written first.
//   This fixed a bug when creating a new package where the row-column
//   widget is not expanded to its full size.
// Added some code to print the value of the label but commented it out.
//   I left it in so that I can refer to this someday.  I'll remove it the
//   next time I check this in.
// Removed code for OK and CANCEL callbacks that are never used.
//
// Revision 2.13  1993/10/06  18:27:46  chris
// Added code to allow Save function to work on all other types of files.
//
// Revision 2.12  1993/10/06  02:16:32  chris
// Added some documentation.
// Added code to perform save function when selected from the menu.
//
// Revision 2.11  1993/09/28  13:46:18  chris
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
//
// Revision 2.10  1993/08/12  21:07:23  chris
// Moved the code which broke out the default package into its component
//   default lists into its own method.
// Call this new method from the create method and from the virtual method
//   for setting the default package after instantiation.
//
// Revision 2.9  1993/07/29  02:44:38  chris
// Added code to handle the other files; Devices, Dialers and Dialcodes.
// Save the pointer to the PackageSubWindows so we know if they already
//   exist.  This way we don't create the window more than once.
//
// Revision 2.8  1993/07/28  19:37:32  chris
// Added code to create broken out devices, dialers and dialcodes.
// Modified breakout code to break out from default Package if one isn't
//   defined for this one.
// Created virtual addDialcodes so we make the button sensitive.
// No longer pass the lists of files but a pointer to the entire package.
// Modified PackageSubWindow to get a pointer to the package it comes from.
//
// Revision 2.7  1993/07/27  20:52:34  chris
// Added code to create a linked list of the actual entries in Systems
//   files so these can be passed to lower level routines.
// When creating SystemWindow, pass pointers to all the lists created.
// When created, get pointers to all the lists.
//
// Revision 2.6  1993/07/27  19:39:31  chris
// Added destructor which will also delete 'current', if it exists.
// Don't just hide this window when 'quit' selected, delete it.
// Pass the Widget for the message window to the SystemWindow constructor.
//
// Revision 2.5  1993/07/27  00:02:55  chris
// Tried rewriting this so that SystemWindow isn't a derived class of
//   System.  This way, multiple SystemWindows can be up for one System
//   class.
// Create the PackageWindow class immediately.
// Don't need to create PackageSubWindows ahead of time anymore.
// No longer need a method for makeNewSystem.
// Changed my mind about requiring two clicks to select a file.
// PackageSubWindow now requires a file type (Systems, Devices, ...) to
//   keep track of what kind of file it is working with.
// Cleaned up code for opening sub window.
//
// Revision 2.4  1993/07/23  23:12:48  chris
// Massive changes to make it so that each file doesn't have its own
//   window.  Also, this way, we can keep information that is package
//   dependent together.
// Added documentation.
// Created methods for PackageSubWindow - a window which will display all
//   the information related to a package-file.
// Create PackageSubWindows for each file supported by a package.
// Removed old way of accessing system stuff and added new way.
//
// Revision 2.3  1993/07/08  02:43:43  chris
// Modified to just raise a window if already visible.
//
// Revision 2.2  1993/07/07  18:09:24  chris
// Needed to include SystemWindow.h and VkSelectionDialog.h.
// Removed commented out code for defining list_type.
// Need to instantiate Package before calling routines for instantiating
//   Systems.  This is because the virtual method makeNewSystem isn't
//   updated to the one for PackageWindow until after instantiation.
// Provided a special makeNewSystem method for this class.
// Added callbacks for pressing the OK or CANCEL buttons on the
//   VkSelectionDialog widget.
// Added code for when the user selects one of the file groups.
//
// Revision 2.1  1993/06/30  20:40:10  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

	/* Define this to display messages when SmartList callbacks called */
// #define DISP_CBS

#include "PackageWindow.h"
#include "PackageSubWindow.h"
#include "SystemWindow.h"
#include "DevicesWindow.h"
#include "DialersWindow.h"
#include "System.h"
#include "Devices.h"
#include "Dialers.h"
#include "Dialcodes.h"
#include "HelpDialog.h"

#include <Vk/VkFileSelectionDialog.h>

#include <iostream.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Label.h>

/*========================================================================
 * PackageWindow class
 *========================================================================*/

XtResource	PackageWindow::_resources[] = {{
    "verbose", "Verbose", XmRBoolean, sizeof (Boolean),
    XtOffset (PackageWindow*, _verbose), XmRString, (XtPointer) "True"},
};

/*------------------------------------------------------------------------
 * PackageWindow::PackageWindow (Widget, const char *, SmartList *)
 *	Constructor for this class.  Creates a widget which represents
 *	a software package described in the "Sysfiles" file.
 *
 *	Widget structure:
 *		+----------------------------+  Form       - key
 *		|+--------------------------+|  RowColumn  - "files"
 *		||+-----------+ +----------+||  PushButton - "systems"
 *		|||           | |          |||               "devices"
 *		||+-----------+ +----------+||		     "dialers"
 *		||+-----------+ +----------+||
 *		|||           | |          |||
 *		||+-----------+ +----------+||
 *		||     ...           ...    ||
 *		|+--------------------------+|
 *		|+--------------------------+|  Label      - "label"
 *		||                          ||
 *		|+--------------------------+|
 *		+----------------------------+
 *------------------------------------------------------------------------*/

PackageWindow::PackageWindow (Widget parent, const char *line,
			      SmartList *all_files, Package *dflt, PathName *dp)
: Package (line), VkComponent (theKey ())
{
    Arg			wargs[15];
    int			n;
    XmString		xmstr;
    char		title[50];
    FileName		*fn;
    System		*sys;
    Devices		*dev;
    Dialers		*dlrs;

    sysPW = NULL;
    devPW = NULL;
    dlrPW = NULL;

    /*----
     * If a directory path was provided, save it.
     * Then run Package::create to read the line and set things up.
     *----*/

    if (dp)
	dirPath = *dp;
    create (line, all_files, dflt);

    /*----
     * Create main window for this class.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNborderWidth, 1); n++;
    _baseWidget = XmCreateForm (parent, (char *)theKey (), wargs, n);
    installDestroyHandler ();

#if 0
    getResources (_resources, XtNumber (_resources));
    n = _verbose;
#ifdef DEBUG
    cout << "PackageWindow (" << theKey () << "): _verbose = "
	 << n << endl;
#endif
#endif

    /*----
     * Now create the label at the bottom.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    xmstr = XmStringCreateLtoR ((char *)theKey (), XmSTRING_DEFAULT_CHARSET);
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    w_label = XmCreateLabel (_baseWidget, "label", wargs, n);
    XtManageChild (w_label);
    XmStringFree (xmstr);

    XtAddCallback (w_label, XmNhelpCallback, aHelpCallback, "pkg_help");

    /*----
     * Create row/column widget to keep Systems, Devices, Dialers
     * and Dialcodes buttons in.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNbottomWidget, w_label); n++;
    XtSetArg (wargs[n], XmNentryClass, xmPushButtonWidgetClass); n++;
    XtSetArg (wargs[n], XmNisHomogeneous, True); n++;
    XtSetArg (wargs[n], XmNnumColumns, 2); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    w_buttons = XmCreateRowColumn (_baseWidget, "UUCPFiles", wargs, n);
    XtManageChild (w_buttons);

    /*----
     * Create button widgets inside of row/column.
     *----*/

    n = 0;
    w_systems = XmCreatePushButton (w_buttons, "systems", wargs, n);
    XtManageChild (w_systems);

    XtAddCallback (w_systems, XmNactivateCallback, buttonCallback, this);
    XtAddCallback (w_systems, XmNhelpCallback, aHelpCallback, "pkg_sys_help");

    n = 0;
    w_devices = XmCreatePushButton (w_buttons, "devices", wargs, n);
    XtManageChild (w_devices);

    XtAddCallback (w_devices, XmNactivateCallback, buttonCallback, this);
    XtAddCallback (w_devices, XmNhelpCallback, aHelpCallback, "pkg_dev_help");

    n = 0;
    w_dialers = XmCreatePushButton (w_buttons, "dialers", wargs, n);
    XtManageChild (w_dialers);

    XtAddCallback (w_dialers, XmNactivateCallback, buttonCallback, this);
    XtAddCallback (w_dialers, XmNhelpCallback, aHelpCallback, "pkg_dlr_help");

    /*----
     * Call this routine to create lists containing all the default systems,
     * devices and dialers split out.  Most of the children don't care
     * what file they are in.
     *----*/

    breakOutDefaults ();

    for (fn = (FileName *)systems.first (); fn;
	 fn = (FileName *)systems.next ())
    {
	sys = (System *)fn->data;
	sys->addChangeCallback (sysChangeCallback, this);
    }

    for (fn = (FileName *)devices.first (); fn;
	 fn = (FileName *)devices.next ())
    {
	dev = (Devices *)fn->data;
	dev->addChangeCallback (devChangeCallback, this);
    }

    for (fn = (FileName *)dialers.first (); fn;
	 fn = (FileName *)dialers.next ())
    {
	dlrs = (Dialers *)fn->data;
	dlrs->addChangeCallback (dlrChangeCallback, this);
    }

    modified = False;
}

/*------------------------------------------------------------------------
 * PackageWindow::~PackageWindow ()
 *------------------------------------------------------------------------*/

PackageWindow::~PackageWindow ()
{
    FileName		*fn;
    System		*sys;
    Devices		*dev;
    Dialers		*dlrs;
    SmartList		*ll;
    
    if (systems.first () || (defaults == NULL))
	ll = &systems;
    else
	ll = &defaults->systems;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	sys = (System *)fn->data;
	sys->removeChangeCallback (sysChangeCallback, this);
    }
    
    if (devices.first () || (defaults == NULL))
	ll = &devices;
    else
	ll = &defaults->devices;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	dev = (Devices *)fn->data;
	dev->removeChangeCallback (devChangeCallback, this);
    }
    
    if (dialers.first () || (defaults == NULL))
	ll = &dialers;
    else
	ll = &defaults->dialers;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	dlrs = (Dialers *)fn->data;
	dlrs->removeChangeCallback (dlrChangeCallback, this);
    }
}

/*------------------------------------------------------------------------
 * PackageWindow::setDefaults (Package *)
 *------------------------------------------------------------------------*/

void PackageWindow::setDefaults (Package *dflt)
{
    Package::setDefaults (dflt);

    breakOutDefaults ();
}

/*------------------------------------------------------------------------
 * PackageWindow::sysChangeCallback (SmartList*, SmartList::ACTION, void*)
 *------------------------------------------------------------------------*/

void PackageWindow::sysChangeCallback (SmartList *sl, SmartList::ACTION action,
				       void *clientData)
{
    PackageWindow	*pw;

    pw = (PackageWindow *)clientData;
    pw->sysChanged (sl, action);
}

/*------------------------------------------------------------------------
 * PackageWindow::sysChanged (SmartList*, SmartList::ACTION)
 *------------------------------------------------------------------------*/

void PackageWindow::sysChanged (SmartList *sl, SmartList::ACTION action)
{
    LinkData		*ld1, *ld2;

#ifdef DISP_CBS
    cout << "sysChanged for package " << theKey () << endl;
#endif

    switch (action)
    {
      case SmartList::ADD:
	for (ld1 = sl->first (); ld1; ld1 = sl->next ())
	{
	    for (ld2 = hostList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = hostList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		hostList.add (ld1);
		modified = True;
	    }
	}
	break;

      case SmartList::DELETE:
	for (ld1 = hostList.first (); ld1; ld1 = hostList.next ())
	{
	    for (ld2 = sl->find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = sl->next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		hostList.removeEntry (ld1);
		modified = True;
	    }
	}
	break;

      default:
	break;
    }
}

/*------------------------------------------------------------------------
 * PackageWindow::devChangeCallback (SmartList*, SmartList::ACTION, void*)
 *------------------------------------------------------------------------*/

void PackageWindow::devChangeCallback (SmartList *sl, SmartList::ACTION action,
				       void *clientData)
{
    PackageWindow	*pw;

    pw = (PackageWindow *)clientData;
    pw->devChanged (sl, action);
}

/*------------------------------------------------------------------------
 * PackageWindow::devChanged (SmartList*, SmartList::ACTION)
 *------------------------------------------------------------------------*/

void PackageWindow::devChanged (SmartList *sl, SmartList::ACTION action)
{
    LinkData		*ld1, *ld2;

#ifdef DISP_CBS
    cout << "devChanged for package " << theKey () << endl;
#endif

    switch (action)
    {
      case SmartList::ADD:
	for (ld1 = sl->first (); ld1; ld1 = sl->next ())
	{
	    for (ld2 = devList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = devList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		devList.add (ld1);
		modified = True;
	    }
	}
	break;

      case SmartList::DELETE:
	for (ld1 = devList.first (); ld1; ld1 = devList.next ())
	{
	    for (ld2 = sl->find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = sl->next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		devList.removeEntry (ld1);
		modified = True;
	    }
	}
	break;

      default:
	break;
    }
}

/*------------------------------------------------------------------------
 * PackageWindow::dlrChangeCallback (SmartList*, SmartList::ACTION, void*)
 *------------------------------------------------------------------------*/

void PackageWindow::dlrChangeCallback (SmartList *sl, SmartList::ACTION action,
				       void *clientData)
{
    PackageWindow	*pw;

    pw = (PackageWindow *)clientData;
    pw->dlrChanged (sl, action);
}

/*------------------------------------------------------------------------
 * PackageWindow::dlrChanged (SmartList*, SmartList::ACTION)
 *------------------------------------------------------------------------*/

void PackageWindow::dlrChanged (SmartList *sl, SmartList::ACTION action)
{
    LinkData		*ld1, *ld2;

#ifdef DISP_CBS
    cout << "dlrChanged for package " << theKey () << endl;
#endif

    switch (action)
    {
      case SmartList::ADD:
	for (ld1 = sl->first (); ld1; ld1 = sl->next ())
	{
	    for (ld2 = dlrList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = dlrList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		dlrList.add (ld1);
		modified = True;
	    }
	}
	break;

      case SmartList::DELETE:
	for (ld1 = dlrList.first (); ld1; ld1 = dlrList.next ())
	{
	    for (ld2 = sl->find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = sl->next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
	    {
		dlrList.removeEntry (ld1);
		modified = True;
	    }
	}
	break;

      default:
	break;
    }
}

/*------------------------------------------------------------------------
 * PackageWindow::breakOutDefaults (void)
 *------------------------------------------------------------------------*/

void PackageWindow::breakOutDefaults (void)
{
    LinkData		*ld1, *ld2;
    FileName		*fn;
    System		*sys;
    Devices		*dev;
    Dialers		*dlrs;
    SmartList		*ll;
    
    /*----
     * Break out all the file entries and put into one list.
     *
     * These lists are supposed to be a list of all the data that
     * this package is to use for its information.  Note that hostList
     * is a list of *hosts* for this package (taken from the Systems
     * file\[s\]) while devList is a list of *devices* - not the
     * files.
     *----*/
    
    if (systems.first () || (defaults == NULL))
	ll = &systems;
    else
	ll = &defaults->systems;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	sys = (System *)fn->data;
#ifdef DEBUG1
	cout << "Adding callback to System " << sys << endl;
#endif
	sys->addChangeCallback (sysChangeCallback, this);
	for (ld1 = sys->first (); ld1; ld1 = sys->next ())
	{				/* Loop through hosts */
	    for (ld2 = hostList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = hostList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
		hostList.add (ld1);
	}
    }
    
    if (devices.first () || (defaults == NULL))
	ll = &devices;
    else
	ll = &defaults->devices;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	dev = (Devices *)fn->data;
	dev->addChangeCallback (devChangeCallback, this);
	for (ld1 = dev->first (); ld1; ld1 = dev->next ())
	{				/* Loop through hosts */
	    for (ld2 = devList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = devList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
		devList.add (ld1);
	}
    }
    
    if (dialers.first () || (defaults == NULL))
	ll = &dialers;
    else
	ll = &defaults->dialers;
    for (fn = (FileName *)ll->first (); fn; fn = (FileName *)ll->next ())
    {					/* Loop through files */
	dlrs = (Dialers *)fn->data;
	dlrs->addChangeCallback (dlrChangeCallback, this);
	for (ld1 = dlrs->first (); ld1; ld1 = dlrs->next ())
	{				/* Loop through hosts */
	    for (ld2 = dlrList.find (ld1->theKey ());
		 ld2 && (strcmp (ld1->theKey (), ld2->theKey ()) == 0);
		 ld2 = dlrList.next ())
	    {
		if (ld1 == ld2)
		    break;
	    }
	    if (ld1 != ld2)
		dlrList.add (ld1);
	}
    }
}

/*------------------------------------------------------------------------
 * buttonCallback
 *	This is a static function and thus doesn\'t know to which
 *	instance of the PackageWindow class it belongs.  The
 *	clientData is a pointer to the class instance.  Using this, it
 *	calls the appropriate button_cb, passing it the widget and
 *	click count.
 *------------------------------------------------------------------------*/

void PackageWindow::buttonCallback (Widget w, XtPointer clientData,
				    XtPointer callData)
{
    XmPushButtonCallbackStruct	*pb_data;
    PackageWindow		*pw;

    pb_data = (XmPushButtonCallbackStruct *)callData;
    pw = (PackageWindow *)clientData;

    pw->buttons_cb (w, pb_data->click_count);
}

/*------------------------------------------------------------------------
 * buttons_cb
 *	This method is called by buttonCallback and does know to which
 *	PackageWindow instance it belongs.  It figures out which
 *	button was clicked and calls the appropriate method.
 *------------------------------------------------------------------------*/

void PackageWindow::buttons_cb (Widget w, int click_count)
{
    if (w == w_systems)
	systems_cb (click_count);
    else if (w == w_devices)
	devices_cb (click_count);
    else if (w == w_dialers)
	dialers_cb (click_count);
}

/*------------------------------------------------------------------------
 * systems_cb
 *------------------------------------------------------------------------*/

void PackageWindow::systems_cb (int)
{
    char		title[50];

#ifdef DEBUG1
    cout << "Systems for " << key << " clicked." << endl;
#endif

    if (!sysPW) {
	strcpy (title, theKey ());
	strcat (title, " - Systems");
	sysPW = new PackageSubWindow (title, PackageSubWindow::SYSTEMS, this);
	sysPW->show ();
    }
    sysPW->raise ();
    sysPW->open ();
}

/*------------------------------------------------------------------------
 * devices_cb
 *------------------------------------------------------------------------*/

void PackageWindow::devices_cb (int)
{
    char		title[50];

#ifdef DEBUG1
    cout << "Devices for " << key << " clicked." << endl;
#endif

    if (!devPW) {
	strcpy (title, theKey ());
	strcat (title, " - Devices");
	devPW = new PackageSubWindow (title, PackageSubWindow::DEVICES, this);
	devPW->show ();
    }
    devPW->raise ();
    devPW->open ();
}

/*------------------------------------------------------------------------
 * dialers_cb
 *------------------------------------------------------------------------*/

void PackageWindow::dialers_cb (int)
{
    char		title[50];

#ifdef DEBUG1
    cout << "Dialers for " << key << " clicked.\n" << flush;
#endif

    if (!dlrPW) {
	strcpy (title, theKey ());
	strcat (title, " - Dialers");
	dlrPW = new PackageSubWindow (title, PackageSubWindow::DIALERS, this);
	dlrPW->show ();
    }
    dlrPW->raise ();
    dlrPW->open ();
}

/*------------------------------------------------------------------------
 * PackageWindow::addSystemFile (PathName&)
 *------------------------------------------------------------------------*/

LinkData* PackageWindow::addSystemFile (PathName& file_name)
{
    FileName		*fn;
    System		*sys;
    LinkData		*le2, *le3;

    fn = (FileName *)Package::addSystemFile (file_name);
    sys = (System *)fn->data;
#ifdef DEBUG1
    cout << "Adding callback to System " << sys << endl;
#endif
    sys->addChangeCallback (sysChangeCallback, this);
    for (le2 = sys->first (); le2; le2 = sys->next ())
    {
	le3 = hostList.find (le2->theKey ());
	if (!le3)
	    hostList.add (le2);
    }

    modified = True;

    return fn;
}

/*------------------------------------------------------------------------
 * PackageWindow::addDeviceFile (PathName&)
 *------------------------------------------------------------------------*/

LinkData* PackageWindow::addDeviceFile (PathName& file_name)
{
    LinkData		*ld;

    ld = Package::addDeviceFile (file_name);
    breakOutDefaults ();

    modified = True;

    return ld;
}

/*------------------------------------------------------------------------
 * PackageWindow::addDialerFile (PathName&)
 *------------------------------------------------------------------------*/

LinkData* PackageWindow::addDialerFile (PathName& file_name)
{
    LinkData		*ld;

    ld = Package::addDialerFile (file_name);
    breakOutDefaults ();

    modified = True;

    return ld;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * PackageWindow::dump (char *)
 *------------------------------------------------------------------------*/

void PackageWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;
    LinkData		*ld;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "PackageWindow (" << this << "):" << endl;
    Package::dump (indent);
    cout << indent << "  package   = " << theKey () << endl;
    cout << indent << "  hostList   = " << &hostList << endl;
    for (ld = hostList.first (); ld; ld = hostList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << indent << "  devList   = " << &devList << endl;
    for (ld = devList.first (); ld; ld = devList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << indent << "  dlrList   = " << &dlrList << endl;
    for (ld = dlrList.first (); ld; ld = dlrList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << endl;
    cout << indent << "  sysPW = " << sysPW << endl;
    if (sysPW)
	sysPW->dump (nextIndent);
    cout << indent << "  devPW = " << devPW << endl;
    if (devPW)
	devPW->dump (nextIndent);
    cout << indent << "  dlrPW = " << dlrPW << endl;
    if (dlrPW)
	dlrPW->dump (nextIndent);
}

/*------------------------------------------------------------------------
 * PackageWindow::dumpMin (char *)
 *------------------------------------------------------------------------*/

void PackageWindow::dumpMin (char *indent)
{
    char		*nextIndent;
    int			i;
    LinkData		*ld;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "PackageWindow (" << this << "):" << endl;
    Package::dump (indent);
    cout << indent << "  package   = " << theKey () << endl;
    cout << indent << "  hostList   = " << &hostList << endl;
    for (ld = hostList.first (); ld; ld = hostList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << indent << "  devList   = " << &devList << endl;
    for (ld = devList.first (); ld; ld = devList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << indent << "  dlrList   = " << &dlrList << endl;
    for (ld = dlrList.first (); ld; ld = dlrList.next ())
	cout << nextIndent << ld->theKey () << " (" << ld
	     << ")" << endl;
    cout << endl;
    cout << indent << "  sysPW = " << sysPW << endl;
    cout << indent << "  devPW = " << devPW << endl;
    cout << indent << "  dlrPW = " << dlrPW << endl;
}
#endif
