/*---*- C++ -*------------------------------------------------------------
 * $Id: PackageWindow.h,v 2.24 1994/02/16 22:25:37 chrisc Exp $
 *
 * Header file for PackageWindow class.
 *
 * Revision History:
 *	$Log: PackageWindow.h,v $
 * Revision 2.24  1994/02/16  22:25:37  chrisc
 * Added some methods to call the hidden methods that open PackageSubWindows.
 *
 * Revision 2.23  1994/02/15  04:24:42  chrisc
 * Removed all references to dialcodes.  It is now taken care of separately
 *   from the main window.
 *
 * Revision 2.22  1993/11/16  01:09:26  carlson
 * Added flag to indicate if we've been modified.
 * Added methods to check flag and clear it.
 *
 * Revision 2.21  1993/11/12  23:51:44  carlson
 * Renamed some of the class members to make more sense.
 *
 * Revision 2.20  1993/11/11  18:38:30  carlson
 * Moved all the PackageSubWindow stuff to PackageSubWindow.h.
 *
 * Revision 2.19  1993/11/09  02:22:13  carlson
 * Added prototype for destructor.
 * Added prototypes for callbacks used for various lists.
 *
 * Revision 2.18  1993/11/05  20:09:29  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.17  1993/11/04  00:30:20  carlson
 * Added a widget to save the parent as sent to setUpInterface.  This is
 *   required because we send this widget to be registered for help and
 *   we need to unregister it when we delete this window.
 *
 * Revision 2.16  1993/10/19  21:34:41  carlson
 * Added prototype for dumpMin.
 * Reformatted things a little.
 *
 * Revision 2.15  1993/10/19  00:18:51  carlson
 * Messed around with trying to get resources to work.  Gave up.
 *
 * Revision 2.14  1993/10/14  23:26:40  carlson
 * Added prototypes for adding a new system.  When I'm sure this works, I
 *   want to duplicate it for adding other packages.
 * Added prototypes for dumping data from classes.
 * Added resource lists.
 *
 * Revision 2.13  1993/10/08  18:17:47  carlson
 * Added wList to keep the widget that contains all the file buttons.
 * Added new methods for adding system, device and dialcode files.
 *
 * Revision 2.12  1993/10/07  22:53:28  carlson
 * Allow a path to be provided when instantiated.
 *
 * Revision 2.11  1993/10/07  20:09:29  chris
 * Removed declarations for ok and cancel callbacks.
 *
 * Revision 2.10  1993/10/06  00:17:46  chris
 * Removed variable rowColumn because it wasn't being used.
 * Added some comments to describe the variables command and message.
 *
 * Revision 2.9  1993/08/12  21:07:23  chris
 * Added prototypes for 'setDefaults' and 'breakOutDefaults' methods.
 * Did some re-alignment of stuff.
 *
 * Revision 2.8  1993/07/29  02:44:38  chris
 * Added places to hold the pointers to the PackageSubWindows that are
 *   created.
 *
 * Revision 2.7  1993/07/28  19:37:32  chris
 * Modified PackageSubWindow to get a pointer to the package it comes from.
 * Keep this in a variable instead of each of the lists separately.
 * Added virtual method for addDialcodes.
 *
 * Revision 2.6  1993/07/27  20:50:17  chris
 * Constructor now takes lists for all the different file lists.
 * Created variables to hold all the lists.
 * Created variables to hold broken out lists to be passed to subclasses.
 *
 * Revision 2.5  1993/07/27  17:57:43  chris
 * Added destructor method.
 *
 * Revision 2.4  1993/07/24  00:19:13  chris
 * Created enum for keeping track of file type (so PackageSubWindow can
 *   be used for each different type of file).
 * Removed method for makeNewSystem.
 * Removed pointers to PackageSubWindows.
 *
 * Revision 2.3  1993/07/23  22:01:40  chris
 * Added PackageSubWindow class to be used to display all the info about
 *   a particular file type supported by a package.
 * Made PackageSubWindow a friend of PackageWindow.
 * Created pointers to the PackageSubWindows for each file.
 *
 * Revision 2.2  1993/07/07  17:37:33  chris
 * Added protected method makeNewSystem.
 * Added prototypes for OK and CANCEL callbacks.
 *
 * Revision 2.1  1993/06/30  19:39:12  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _PACKAGEWINDOW_H_
#define _PACKAGEWINDOW_H_

#include <Vk/VkComponent.h>

#include "Package.h"

/*------------------------------------------------------------------------
 * PackageWindow class
 *	This class is derived from the "Package" class which
 *	describes everything that is provided in the "Sysfiles"
 *	file for a particular software package \(cu, uucico, uux\).
 *	This class adds functionality by creating a widget to be
 *	installed in the parent widget allowing the user access to
 *	the data contained.
 *------------------------------------------------------------------------*/

class PackageWindow : public Package, public VkComponent
{
friend PackageSubWindow;

  public:
    PackageWindow (Widget parent, const char *line, SmartList *all_files,
		   Package *dflt = NULL, PathName *dp = NULL);
    ~PackageWindow ();

    int			wasModified (void);
    void		clearModified (void);

    void		openSystems (void) { systems_cb (2); }
    void		openDevices (void) { devices_cb (2); }
    void		openDialers (void) { dialers_cb (2); }

    virtual void	setDefaults (Package *dflt);
    virtual LinkData*	addSystemFile (PathName& file_name);
    virtual LinkData*	addDeviceFile (PathName& file_name);
    virtual LinkData*	addDialerFile (PathName& file_name);

    virtual const char*	className () { return "PackageWindow"; }

#ifdef DEBUG
    virtual void	dump (char *indent);
    virtual void	dumpMin (char *indent);
#endif

  protected:
    SmartList		hostList;		// Systems broken out
    SmartList		devList;		// Devices broken out
    SmartList		dlrList;		// Dialers broken out

    /* Resources */
    Boolean		_verbose;

    PackageSubWindow*	sysPW;
    PackageSubWindow*	devPW;
    PackageSubWindow*	dlrPW;

    /*----
     * Widgets used in this container.
     *----*/

    Widget		w_systems;		// Systems button
    Widget		w_devices;		// Devices button
    Widget		w_dialers;		// Dialers button
    Widget		w_buttons;		// Container for above buttons
    Widget		w_label;		// Label widget.

  private:
    static XtResource	_resources[];

    int			modified;

    void		breakOutDefaults (void);

    /* Callbacks */
    static void		buttonCallback (Widget w, XtPointer clientData,
					XtPointer callData);
    static void		sysChangeCallback (SmartList*, SmartList::ACTION,
					   void*);
    static void		devChangeCallback (SmartList*, SmartList::ACTION,
					   void*);
    static void		dlrChangeCallback (SmartList*, SmartList::ACTION,
					   void*);

    void		buttons_cb (Widget w, int click_count);
    void		systems_cb (int click_count);
    void		devices_cb (int click_count);
    void		dialers_cb (int click_count);

    void		sysChanged (SmartList*, SmartList::ACTION);
    void		devChanged (SmartList*, SmartList::ACTION);
    void		dlrChanged (SmartList*, SmartList::ACTION);
};

#endif
