/*------------------------------------------------------------------------
 * $Id: PathName.c++,v 2.5 1993/10/22 02:07:13 carlson Exp $
 *
 * Methods for PathName class.
 *
 * Revision History:
 *	$Log: PathName.c++,v $
// Revision 2.5  1993/10/22  02:07:13  carlson
// Added method to expand file/pathnames to be fully qualified.
// *** Suggestions ***
//   1. Expand all file names, even those that don't begin with '.' to
//      have the full path name.
//   2. If $HOME can't be found, look in the password file.
//
// Revision 2.4  1993/10/08  18:14:36  carlson
// Added method for operator= to a const char * so I don't have to cast
//   all the time.
//
// Revision 2.3  1993/10/07  22:46:28  carlson
// If the null path is provided, always make a path of at least the null
//   string.
// Added methods for hasPath and length.
//
// Revision 2.2  1993/06/30  22:28:22  chris
// Fixed the operator+ methods so they don't modify either of the
//   values provided.
//
// Revision 2.1  1993/06/30  02:57:29  chris
// Added some more functionality.
//
// Revision 1.1  1993/06/22  01:13:05  chris
// Initial revision
//
 *------------------------------------------------------------------------*/

#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <iostream.h>
#include <pwd.h>

#include "PathName.h"

/*------------------------------------------------------------------------
 * makeNew
 *------------------------------------------------------------------------*/

PathName& PathName::makeNew (char *c)
{
    int		size;

    if (path)
	delete [] path;
    if (c)
    {
	size = strlen (c);
	path = new char [size + 1];
	strcpy (path, c);
    }
    else
    {
	size = 0;
	path = new char [1];
	*path = '\0';
    }

    return *this;
}

/*------------------------------------------------------------------------
 * PathName
 *------------------------------------------------------------------------*/

PathName::PathName (PathName &p)
{
    path = NULL;
    makeNew (p.path);
}

/*------------------------------------------------------------------------
 * PathName
 *------------------------------------------------------------------------*/

PathName::PathName (char *c)
{
    path = NULL;
    makeNew (c);
}

/*------------------------------------------------------------------------
 * operator=
 *------------------------------------------------------------------------*/

PathName& PathName::operator= (PathName &p)
{
    return makeNew (p.path);
}

/*------------------------------------------------------------------------
 * operator=
 *------------------------------------------------------------------------*/

PathName& PathName::operator= (char *c)
{
    return makeNew (c);
}

/*------------------------------------------------------------------------
 * operator=
 *------------------------------------------------------------------------*/

PathName& PathName::operator= (const char *c)
{
    return makeNew ((char *)c);
}

/*------------------------------------------------------------------------
 * directory
 *------------------------------------------------------------------------*/

PathName PathName::directory ()
{
    PathName	pn;
    char	*c;
    int		size;

    c = strrchr (path, '/');
    if (c)
    {
	size = c - path;
	pn.path = new char [size + 1];
	strncpy (pn.path, path, size);
	pn.path[size] = '\0';
    }
    else
    {
	pn.path = new char [1];
	pn.path[0] = '\0';
    }

    return pn;
}

/*------------------------------------------------------------------------
 * root
 *------------------------------------------------------------------------*/

PathName PathName::root (char c)
{
    PathName	pn;
    char	*cptr;
    int		size;

    cptr = strrchr (path, c);
    if (cptr)
	size = cptr - path;
    else
	size = strlen (path);

    pn.path = new char [size + 1];
    strncpy (pn.path, path, size);
    pn.path[size] = '\0';

    return pn;
}

/*------------------------------------------------------------------------
 * tail
 *------------------------------------------------------------------------*/

PathName PathName::tail ()
{
    PathName	pn;
    char	*cptr;
    int		size;

    cptr = strrchr (path, '/');
    if (cptr)
    {
	size = strlen (++cptr);
	pn.path = new char [size + 1];
	strncpy (pn.path, cptr, size);
	pn.path[size] = '\0';
    }
    else
    {
	pn.path = new char [1];
	pn.path[0] = '\0';
    }

    return pn;
}

/*------------------------------------------------------------------------
 * extension
 *------------------------------------------------------------------------*/

PathName PathName::extension (char c)
{
    PathName	pn;
    char	*cptr;
    int		size;

    cptr = strrchr (path, c);
    if (cptr)
    {
	size = strlen (++cptr);
	pn.path = new char [size + 1];
	strncpy (pn.path, cptr, size);
	pn.path[size] = '\0';
    }
    else
    {
	pn.path = new char [1];
	pn.path[0] = '\0';
    }

    return pn;
}

/*------------------------------------------------------------------------
 * basename
 *------------------------------------------------------------------------*/

PathName PathName::basename (char *c)
{
    PathName	pn;
    char	*c1, *p1;
    int		size;

    if (c)
    {
	p1 = c1 = path + strlen (path) - strlen (c);

	while (*c && *c1)
	{
	    if (*c != *c1)
		break;
	    c++;
	    c1++;
	}

	if (*c == '\0')
	    size = p1 - path;
	else
	    size = strlen (path);
    }
    else
    {
	size = strlen (path);
    }

    pn.path = new char [size + 1];
    strncpy (pn.path, path, size);
    pn.path[size] = '\0';

    return pn.tail ();
}

/*------------------------------------------------------------------------
 * append
 *------------------------------------------------------------------------*/

PathName& PathName::append (char *c)
{
    int		size;
    char	*c1, *c2;

    if (*c == '/')
	c++;

    size = strlen (c);
    while (c[size-1] == '/')
	size--;

    c1 = new char [size + 1];
    strncpy (c1, c, size);
    c1[size] = '\0';

    c2 = new char [size + strlen (path) + 2];
    strcpy (c2, path);
    strcat (c2, "/");
    strcat (c2, c1);

    makeNew (c2);
    delete [] c1;
    delete [] c2;

    return *this;
}

/*------------------------------------------------------------------------
 * operator+
 *------------------------------------------------------------------------*/

PathName& PathName::operator+ (PathName &pn)
{
    PathName	*p2;

    p2 = new PathName (*this);
    p2->append (pn.path);

    return *p2;
}

/*------------------------------------------------------------------------
 * operator+
 *------------------------------------------------------------------------*/

PathName& PathName::operator+ (char *c)
{
    PathName	*p2;

    p2 = new PathName (*this);
    p2->append (c);

    return *p2;
}

/*------------------------------------------------------------------------
 * isFullPath
 *------------------------------------------------------------------------*/

int PathName::isFullPath (void)
{
    return (path[0] == '/');
}

/*------------------------------------------------------------------------
 * hasPath
 *------------------------------------------------------------------------*/

int PathName::hasPath (void)
{
    return (strchr (path, '/') != NULL);
}

/*------------------------------------------------------------------------
 * length
 *------------------------------------------------------------------------*/

int PathName::length (void)
{
    return strlen (path);
}

/*------------------------------------------------------------------------
 * PathName::expand \(\)
 *	If the pathname is a relative pathname or has any special
 *	characters \(like ~\), expand it out to make it a fully
 *	qualified path name.
 *------------------------------------------------------------------------*/

void PathName::expand (void)
{
    static char		*cwd = NULL;
    static char		*home = NULL;
    char		*ctmp, *tpath, *cptr;
    struct passwd	*pwent;
    char		username[9];

    /*----
     * First, if this is the first time here, compute the value for $HOME
     * and current working directory.
     *----*/

    if (home == NULL)
    {
	cwd = getenv ("HOME");
	if (cwd == NULL)
	{
	    cerr << "Unable to find HOME environment variable" << endl;
	}
	else
	{
	    home = new char [strlen (cwd) + 1];
	    strcpy (home, cwd);
	}

	cwd = getcwd (NULL, PATH_MAX);
	if (cwd == NULL)
	{
	    cerr << "Unable to get current working directory" << endl;
	    cwd = ".";
	}
    }

    /*----
     * ctmp is going to be where we create the new path.
     * tpath is a pointer into the old path while we traverse it.
     *----*/

    ctmp = new char [PATH_MAX];
    ctmp[0] = '\0';
    tpath = path;
    if (*tpath != '/')
    {

	/*----
	 * If first character is a tilde, then we either have ~/ or
	 * ~<username>.  If it is the former, prepend the home
	 * directory.  If the latter, we need to find out what the
	 * home directory for <username> is and prepend that.
	 *----*/

	if (*tpath == '~')
	{
	    tpath++;
	    if (*tpath == '/')
		strcat (ctmp, home);
	    else
	    {
		strncpy (username, tpath, 8);
		username[8] = '\0';
		cptr = strchr (username, '/');
		if (cptr)
		    *cptr = '\0';
		pwent = getpwnam (username);
		if (pwent == NULL)
		{
		    cerr << "Unable to find password entry for " << username
			 << endl;
		}
		else
		{
		    strcat (ctmp, pwent->pw_dir);
		}
	    }
	}
	else if (*tpath == '.')
	{
	    strcat (ctmp, cwd);
	    tpath++;
	}
    }

    cptr = ctmp + strlen (ctmp);
    while (*cptr = *tpath)
    {
	if ((*tpath == '/') && (*(tpath+1) == '.') && (*(tpath+2) == '/'))
	    tpath += 2;
	while ((*tpath == '/') && (*(tpath+1) == '/'))
	    tpath++;
	cptr++;
	tpath++;
    }

    delete [] path;
    path = new char [strlen (ctmp) + 1];
    strcpy (path, ctmp);
    delete [] ctmp;
}
