/*---*- C++ -*------------------------------------------------------------
 * $Id: PathName.h,v 2.5 1993/10/22 01:55:40 carlson Exp $
 *
 * Header file for PathName class.
 *
 * Revision History:
 *	$Log: PathName.h,v $
 * Revision 2.5  1993/10/22  01:55:40  carlson
 * Added prototype for method to expand pathname.
 *
 * Revision 2.4  1993/10/08  18:14:36  carlson
 * Added method for operator= to a const char * so I don't have to cast
 *   all the time.
 *
 * Revision 2.3  1993/10/07  22:46:28  carlson
 * No longer have a dummy creation routine.  Use the one with a string
 *   passing the null string.
 * Added declarations for hasPath and length.
 *
 * Revision 2.2  1993/06/30  23:09:50  chris
 * Added operator+= operators.
 *
 * Revision 2.1  1993/06/30  04:05:42  chris
 * Added some more functionality.
 *
 * Revision 1.1  1993/06/22  01:19:37  chris
 * Initial revision
 *
 *------------------------------------------------------------------------*/

#ifndef _PATHNAME_H_
#define _PATHNAME_H_

class PathName
{
  public:
    PathName (char *c = "");
    PathName (PathName &p);
    ~PathName () { if (path) delete [] path; }

    PathName&	operator= (PathName &p);
    PathName&	operator= (char *c);
    PathName&	operator= (const char *c);
    PathName	directory ();
    PathName	root (char c = '.');
    PathName	head () { return directory (); }
    PathName	tail ();
    PathName	extension (char c = '.');
    PathName	basename (char *c = 0);
    PathName	dirname () { return directory (); }
    operator	const char *() { return path; }
    const char	*name (void) { return path; }
    PathName&	operator+ (PathName &p);
    PathName&	operator+ (char *c);
    PathName&	operator+= (PathName &p) { return append (p.path); }
    PathName&	operator+= (char *c) { return append (c); }
    int		isFullPath (void);
    int		hasPath (void);
    int		length (void);
    void	expand (void);

  private:
    PathName&	makeNew (char *c);
    PathName&	append (char *c);
    char	*path;
};

#endif
