/*------------------------------------------------------------------------
 * $Id: SmartList.c++,v 2.5 1994/02/18 19:00:32 chrisc Exp $
 *
 * Derivation of the simple linked list class, LinkList.  This class
 * provides callbacks when new entries are added and when entries are
 * removed.
 *
 * Revision History:
 *	$Log: SmartList.c++,v $
// Revision 2.5  1994/02/18  19:00:32  chrisc
// Made one method to handle callbacks and have all the functions call it.
// Removed callback in one of the remove methods because it became redundant.
//
// Revision 2.4  1994/01/17  20:45:46  carlson
// Changed all methods that took LinkEntry pointers as parameters to expect
//   LinkData pointers instead.
// Got rid of 'remove' method and replaced with two removeEntry methods.
//
// Revision 2.3  1993/11/15  22:18:36  carlson
// Modified method of adding callbacks because it was possible to add a
//   callback twice.
//
// Revision 2.2  1993/11/09  19:51:56  carlson
// Added include file for iostream in case if we decide to print some
//   messages.
//
// Revision 2.1  1993/11/05  20:24:32  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include <iostream.h>
#include <string.h>

#include "SmartList.h"

/*========================================================================
 * SmartList class
 *========================================================================*/

/*------------------------------------------------------------------------
 * SmartList::callCallbacks (ACTION)
 *------------------------------------------------------------------------*/

void SmartList::callCallbacks (ACTION action)
{
    callbackStruct	*cbs;

#ifdef DEBUG1
    cout << "Callback being called to ";
    switch (action)
    {
      case ADD:
	cout << "ADD";
	break;
      case DELETE:
	cout << "DELETE";
	break;
      case MOVE:
	cout << "MOVE";
	break;
      case OTHER:
	cout << "OTHER";
	break;
    }
#endif

    if (callbacks)
	for (cbs = callbacks; cbs; cbs = cbs->next)
	    if (cbs->cbf)
	    {
#ifdef DEBUG1
		cout << " ...done";
#endif
		(*((callbackFunction)cbs->cbf))(this, action, cbs->clientData);
	    }
#ifdef DEBUG1
    cout << endl;
#endif
}

/*------------------------------------------------------------------------
 * SmartList::add (LinkData *)
 *------------------------------------------------------------------------*/

void SmartList::add (LinkData *le) 
{
    LinkList::add (le);
    callCallbacks (ADD);
}

/*------------------------------------------------------------------------
 * SmartList::clean (void)
 *------------------------------------------------------------------------*/

void SmartList::clean (void)
{
    LinkList::clean ();
    callCallbacks (DELETE);
}

/*------------------------------------------------------------------------
 * SmartList::addBefore (LinkData *)
 *------------------------------------------------------------------------*/

void SmartList::addBefore (LinkData *le)
{
    LinkList::addBefore (le);
    callCallbacks (ADD);
}

/*------------------------------------------------------------------------
 * SmartList::addAfter (LinkData *)
 *------------------------------------------------------------------------*/

void SmartList::addAfter (LinkData *le)
{
    LinkList::addAfter (le);
    callCallbacks (ADD);
}

/*------------------------------------------------------------------------
 * SmartList::removeEntry (void)
 *------------------------------------------------------------------------*/

void SmartList::removeEntry (void)
{
    LinkList::removeEntry ();
    callCallbacks (DELETE);
}

/*------------------------------------------------------------------------
 * SmartList::removeEntry (LinkData *)
 *	We don't need to call the callbacks for this one because the
 *	standard LinkList::removeEntry calls our
 *	SmartList::removeEntry (void), which will call the callbacks.
 *------------------------------------------------------------------------*/

void SmartList::removeEntry (LinkData *d)
{
    LinkList::removeEntry (d);
}

/*------------------------------------------------------------------------
 * SmartList::addChangeCallback (callbackFunction, void *)
 *------------------------------------------------------------------------*/

void SmartList::addChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp, *prev;

    cbs = new callbackStruct;
    cbs->cbf = cbf;
    cbs->clientData = cd;
    cbs->next = NULL;

    for (cbstemp = callbacks, prev = NULL; cbstemp; cbstemp = cbstemp->next)
    {
	if ((cbstemp->cbf == cbf) && (cbstemp->clientData == cd))
	{
	    delete cbs;
	    return;
	}
	prev = cbstemp;
    }

    if (prev)
	prev->next = cbs;
    else
	callbacks = cbs;
}

/*------------------------------------------------------------------------
 * SmartList::removeChangeCallback (callbackFunction, void *)
 *------------------------------------------------------------------------*/

void SmartList::removeChangeCallback (callbackFunction cbf, void *cd)
{
    callbackStruct	*cbs, *cbstemp;

    if (callbacks)
    {
	for (cbs = NULL, cbstemp = callbacks; cbstemp; cbstemp = cbstemp->next)
	{
	    if ((cbstemp->cbf == cbf) && (cbstemp->clientData == cd))
	    {
		if (cbs)
		    cbs->next = cbstemp->next;
		else
		    callbacks = cbstemp->next;
		delete cbstemp;
		break;
	    }
	    cbs = cbstemp;
	}
    }
}
