/*---*- C++ -*------------------------------------------------------------
 * $Id: SmartList.h,v 2.5 1994/02/26 01:10:38 carlson Exp $
 *
 * Header file for SmartList class.  This class is a derived class of
 * the LinkList class providing callbacks when new entries are made
 * and when entries are removed.
 *
 * Revision History:
 *	$Log: SmartList.h,v $
 * Revision 2.5  1994/02/26  01:10:38  carlson
 * Added another item to the ACTION enumeration to allow derived classes to send a
 *   callback that some child generated a callback.
 *
 * Revision 2.4  1994/02/18  19:01:22  chrisc
 * Added another action for child processes to use.
 * Added protected callback method that all methods use.
 *
 * Revision 2.3  1994/01/17  20:47:56  carlson
 * Changed all methods that took LinkEntry pointers as parameters to expect
 *   LinkData pointers instead.
 * Got rid of 'remove' method and replaced with two removeEntry methods.
 *
 * Revision 2.2  1993/11/09  02:21:12  carlson
 * Added definition for class so we avoid a compiler warning message.
 *
 * Revision 2.1  1993/11/05  20:24:32  carlson
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _SMARTLIST_H_
#define _SMARTLIST_H_

#include "linkList.h"

class SmartList;
class SmartList : public LinkList
{
  public:
    enum ACTION { ADD, DELETE, MOVE, CHILD, OTHER };
    typedef void (*callbackFunction)(SmartList*, ACTION, void*);

    SmartList () { callbacks = NULL; }
    ~SmartList () { }

    virtual void	add (LinkData *pentry);
    virtual void	clean (void);
    virtual void	addBefore (LinkData *pentry);
    virtual void	addAfter (LinkData *pentry);
    virtual void	removeEntry (void);
    virtual void	removeEntry (LinkData *d);

    void		addChangeCallback (callbackFunction ncbf,
					   void *clientData);
    void		removeChangeCallback (callbackFunction ncbf,
					      void *clientData);

  protected:
    virtual void	callCallbacks (ACTION);

  private:
    struct callbackStruct
    {
	callbackFunction	cbf;
	void			*clientData;
	callbackStruct		*next;
    };
    callbackStruct	*callbacks;
};

#endif
