/*---*- C++ -*------------------------------------------------------------
 * $Id: Sysfiles.h,v 2.7 1994/02/17 04:46:41 chrisc Exp $
 *
 * Header file describing the SysFiles class.
 *
 * Revision History:
 *	$Log: Sysfiles.h,v $
 * Revision 2.7  1994/02/17  04:46:41  chrisc
 * Added prototypes for save methods.
 * Added support for a modified flag.
 * Added a place to save the file name in.
 *
 * Revision 2.6  1994/01/17  23:46:48  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.5  1993/11/05  20:09:29  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.4  1993/10/07  21:59:58  carlson
 * Maintain the directory of this file in a variable.
 *
 * Revision 2.3  1993/07/28  18:37:04  chris
 * Keep track of defaults package in PackageSubWindow.
 *
 * Revision 2.2  1993/07/01  00:06:03  chris
 * Needed a constructor that didn't do anything so that we could create
 *   the internals AFTER the virtual pointers were set up.
 *
 * Revision 2.1  1993/06/30  03:13:23  chris
 * Updated to have new format.
 *
 * Revision 1.5  1993/05/16  20:26:27  chris
 * Moved public items to top.
 *
 * Revision 1.4  1993/05/14  06:59:56  chris
 * Make dialcode_list the same type as the others, for consistency.
 *
 * Revision 1.3  1993/05/12  17:35:03  chris
 * No longer include classes for Systems, Devices, Dialers or Dialcodes.
 *   Just keep the strings for the files.
 * Add two new methods for reading and making the Sysfiles file.
 *
 * Revision 1.2  1993/05/07  00:05:09  chris
 * No longer have private members, only protected.
 *
 * Revision 1.1  1993/05/05  19:02:20  chris
 * Initial revision
 *
 *------------------------------------------------------------------------*/

#ifndef _SYSFILES_H_
#define _SYSFILES_H_

#include "SmartList.h"
#include "PathName.h"

class Package;

/*------------------------------------------------------------------------
 * SysFile
 *	SmartList - List of all the packages.
 *------------------------------------------------------------------------*/

class SysFile : public SmartList
{
  public:
    SysFile () { }
    SysFile (const char *filename);
    SysFile (PathName& filename);
    ~SysFile ();

    void		save (PathName& filename);
    void		save (const char *filename = NULL);

    int			wasModified (void);
    void		clearModified (void);

    virtual void	add (LinkData *pe1) {
	modified = 1;
	SmartList::add (pe1); }
    virtual void	clean (void) {
	modified = 1;
	SmartList::clean (); }
    virtual void	addBefore (LinkData *pe1) {
	modified = 1;
	SmartList::addBefore (pe1); }
    virtual void	addAfter (LinkData *pe1) {
	modified = 1;
	SmartList::addAfter (pe1); }
    virtual void	removeEntry (void) {
	modified = 1;
	SmartList::removeEntry (); }
    virtual void	removeEntry (LinkData *d) {
	modified = 1;
	SmartList::removeEntry (d); }

  protected:
    int			modified;
    PathName		fileName;
    PathName		dirPath;
    Package*		defaults;
    SmartList		all_files[4];

    virtual LinkData*	makeNewPackage (const char *line);
    void		create (PathName& filename);
};

#endif
