/*---*- C++ -*------------------------------------------------------------
 * $Id: System.h,v 2.14 1994/02/28 21:25:32 carlson Exp $
 *
 * Header file for System class.
 *
 * Revision History:
 *	$Log: System.h,v $
 * Revision 2.14  1994/02/28  21:25:32  carlson
 * Added place to keep any errors encounted and a way to read it.
 *
 * Revision 2.13  1994/02/26  01:10:38  carlson
 * Moved the package list to be more consistent with my style.
 * Removed the 'deleteEntry' method since this is done by SmartList.
 *
 * Revision 2.12  1994/02/18  18:52:29  chrisc
 * Modified package manipulation methods to call SmartList callbacks.
 * Rearranged some lines.
 *
 * Revision 2.11  1994/02/10  22:56:56  chrisc
 * Made package access functions inline.
 * Added methods to remove a package.
 *
 * Revision 2.10  1994/01/18  00:24:51  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.9  1993/11/16  01:10:58  carlson
 * Added flag to indicate we've been modified.
 * Had to create virtual methods to set the flag.
 *
 * Revision 2.8  1993/11/05  20:09:29  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.7  1993/10/06  00:17:46  chris
 * Changed all references to PathName to use references instead of creating
 *   a new copy.
 * Added two versions of the method 'save', which will save the file.
 *
 * Revision 2.6  1993/07/23  23:48:25  chris
 * Added methods for getting the first and following package entries.
 *
 * Revision 2.5  1993/07/23  17:50:03  chris
 * Made addPackage a virtual function so that windowing routines can
 *   perform additional functions (like add it to the list in the window).
 * Added a function for finding a particular package (so we don't duplicate
 *   the same package in the list).
 * Made 'packages' protected so derived classes can access it.
 *
 * Revision 2.4  1993/07/08  00:15:44  chris
 * Decided to pass a constant char in.
 * Added method to read the file.
 *
 * Revision 2.3  1993/07/01  02:43:37  chris
 * Moved the 'create' method.
 *
 * Revision 2.2  1993/07/01  01:32:51  chris
 * Make packages just a linked list.
 * Added function to add packages.
 *
 * Revision 2.1  1993/06/30  00:50:56  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _SYSTEM_H_
#define _SYSTEM_H_

#include "SmartList.h"
#include "PathName.h"

/*------------------------------------------------------------------------
 * System
 *	LinkData - This class can be used in a linked list of Systems
 *		   files.
 *	SmartList - Linked list of hosts defined in this Systems file.
 *------------------------------------------------------------------------*/

class System : public LinkData, public SmartList
{
  public:
    System () { modified = 0; }
    System (PathName& filename);
    ~System () { }

    virtual void	addPackage (LinkData *le) {
	packages.add (le); callCallbacks (SmartList::OTHER); }
    LinkData*		findPackage (const char *name) {
	return packages.find (name); }
    LinkData*		firstPackage (void) {
	return packages.first (); }
    LinkData*		nextPackage (void) {
	return packages.next (); }
    void		removePackage (void) {
	packages.removeEntry (); callCallbacks (SmartList::OTHER); }
    void		removePackage (LinkData *ld) {
	packages.removeEntry (ld); callCallbacks (SmartList::OTHER); }
    void		save (PathName& filename);
    void		save (const char *name = NULL);

    int			wasModified (void);
    void		clearModified (void);
    int			error (void) { return _errno; }

    virtual void	add (LinkData *pe1) {
	modified = 1; SmartList::add (pe1); }
    virtual void	clean (void) {
	modified = 1; SmartList::clean (); }
    virtual void	addBefore (LinkData *pe1) {
	modified = 1; SmartList::addBefore (pe1); }
    virtual void	addAfter (LinkData *pe1) {
	modified = 1; SmartList::addAfter (pe1); }
    virtual void	removeEntry (void) {
	modified = 1; SmartList::removeEntry (); }
    virtual void	removeEntry (LinkData *d) {
	modified = 1; SmartList::removeEntry (d); }

  protected:
    int			modified;
    SmartList		packages;
    int			_errno;

    virtual void	create (PathName& filename);
    virtual LinkData*	makeNewHost (const char *c);
    void		readFile (PathName& filename);
};

#endif
