/*------------------------------------------------------------------------
 * $Id: SystemWindow.c++,v 2.30 1994/03/03 00:27:09 carlson Exp $
 *
 * Methods for SystemWindow class.
 *
 * Revision History:
 *	$Log: SystemWindow.c++,v $
// Revision 2.30  1994/03/03  00:27:09  carlson
// Modified so that it will compile on 5.1 OS versions.  This required taking
//   out the parameters for postAndWait methods that caused dialog window
//   to appear over the parent widget.
//
// Revision 2.29  1994/02/28  21:25:32  carlson
// When doing a save, check if an error occurred and generate a dialog if it
//   does.
//
// Revision 2.28  1994/02/26  03:58:33  carlson
// Modified all dialogs to place dialog window over the current parent widget.
// Removed code to add new host to host list since this is done automatically
//   in the callback for when the list of hosts changes.
// Added code to method that handles a change in the list of hosts to select
//   the current host, if there is a current host selected.
//
// Revision 2.27  1994/02/22  21:15:20  chrisc
// Needed to add phone number to identifiers in host list.
// Changed list to be constant in size, thus creating a horizontal scroll bar
//   for viewing the entire host information.
//
// Revision 2.26  1994/02/18  18:48:05  chrisc
// Added method 'getSelected' to do the work of finding the entry in the
//   list for the selected item.
// Cleaned up useless code in selectCallback.
// Improved the way select callback functions.
// Added delete button and delete callback to delete a file.
// Moved all the loops that fill the package list and selection list to
//   be in a method together.  This way, they can be updated by callbacks.
// Added callback for when the items in the list change.
//
// Revision 2.25  1994/02/15  04:26:52  chrisc
// Removed all references to dialcodes.  This is now done separately in
//   the main window.
//
// Revision 2.24  1994/01/20  20:17:37  carlson
// Modified to support new kind of help facility.
//
// Revision 2.23  1994/01/18  00:38:15  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.22  1994/01/04  20:47:15  carlson
// Changed some error printouts to use an error dialog.
// Instead of having a <new> item in list, created a New button.
// Fixed all \( \) to just ( ).
//
// Revision 2.21  1993/11/16  19:35:41  carlson
// No longer need to include System.h since it is included in SystemWindow.h.
// Fixed the code to make it add a package.
//
// Revision 2.20  1993/11/15  22:03:54  carlson
// Added code to put new host in host list.
//
// Revision 2.19  1993/11/13  00:38:33  carlson
// Added a complete default line that will be used in making new hosts.
//
// Revision 2.18  1993/11/11  01:45:05  carlson
// When making a new entry, we need to add the entry to the system list.
//
// Revision 2.17  1993/11/05  20:09:29  carlson
// Changed all references to LinkList to use SmartList.
//
// Revision 2.16  1993/11/04  23:50:45  carlson
// Added help callbacks for buttons and labels.
//
// Revision 2.15  1993/10/22  21:21:13  carlson
// Started adding the ability to add another host to the Systems file.
//   This is not complete yet.
// Tried to figure out a way to get labels to be centered under the packages
//   that use this system file but was generally unsuccessful.  Finally
//   decided to put a small border around the list.
// Found where I created an XmString but didn't free it.
//
// Revision 2.14  1993/10/14  23:32:31  carlson
// Added debug function 'dump' which dumps out everything about an instance
//   of a class to stdout.
//
// Revision 2.13  1993/10/06  00:44:06  chris
// Added a method for saving.  It just calls the save method for the
//   respective System class.
//
// Revision 2.12  1993/09/28  13:46:18  chris
// Changed the name of the encapsulating widget to hostWindow, instead of
//   just host.
// Removed all the definitions for the labels titles so that it is done in
//   a resource file now.  Also, removed the extraneous sizing info.
//
// Revision 2.11  1993/08/12  15:31:25  chris
// Added method for setting the defaults after instantiation.
//
// Revision 2.10  1993/07/28  17:53:08  chris
// Pass all the lists for devices, dialers and dialcodes when making a
//   HostWindow.
//
// Revision 2.9  1993/07/27  20:17:20  chris
// Constructor now accepts pointers to other lists.  Save these in the
//   class variables.
//
// Revision 2.8  1993/07/27  19:41:45  chris
// Remove some of the print statements.
// Added code to create a HostWindow when a host is selected.
// Added a destructor to delete 'current' if it exists.
// Receive pointer to Widget HostWindow is to be in.
//
// Revision 2.7  1993/07/27  00:05:04  chris
// Modified so this is no longer a derived class of System.
// Removed methods makeNewHost and deleteWindow.
// Combined createWindow with constructor.
//
// Revision 2.6  1993/07/23  23:12:48  chris
// Made selection of a host in the host-list call the select callback.
// Read Systems file when the window is created.
// Because of problems getting this stuff to display in the window properly,
//   I put wrapper widgets around the system stuff.
//
// Revision 2.5  1993/07/23  20:37:10  chris
// Made a lot of changes to this file and wanted to save them before I
//   made another significant change.
// Modified to create a VkComponent instead of its own window.  This means
//   I removed all the callbacks for menu buttons, changed the method for
//   setUpInterface to be the creation method and completely changed what
//   is displayed in the widget.
// Separated the creation of the widget from the creation of the class
//   so that base classes can read the files and update the information
//   but we can create the widgets to when we determine who the parent
//   is going to be.
// Created a deleteWindow method.
// Created an addPackage virtual method so that the packages added will
//   also be added to the display.
// Added an afterRealizeHook so I can display some stuff.
//
// Revision 2.4  1993/07/21  00:08:38  chris
// Modified to put only the file name, not full path in the title bar.
//
// Revision 2.3  1993/07/09  00:39:58  chris
// No longer pass the parent widget to HostWindow.  Just the line from
//   which will be derived the info.
// Add a callback to handle what to do when a host is selected.
// Actually put the buttons in the current widget for each host that is known.
//
// Revision 2.2  1993/07/08  00:13:54  chris
// No longer need to include PackageWindow.h.
// Need to include HostWindow.h.
// Remove commented out print statements.
// Change quitCallback to hide the SystemWindow rather than quit the
//   application.
// Changed method (which was commented out) for makeNewPackage to makeNewHost
//   and have it do the right thing.
// Modified setUpInterface to call 'readFile'.
//
// Revision 2.1  1993/07/07  01:37:21  chris
// Initial revision.
//
 *------------------------------------------------------------------------*/

#ifndef OSVERS
#define OSVERS	5100
#endif

#include "SystemWindow.h"
#include "HostWindow.h"
#include "HelpDialog.h"

#include <Vk/VkPromptDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/List.h>
#include <Xm/Label.h>
#include <Xm/SeparatoG.h>

static char	*default_line = " Any ACU 9600 - \042\042 @\\r\\c ogin:--"
				"ogin:-\\b\\d-ogin:--ogin:";

/*------------------------------------------------------------------------
 * SystemWindow::getSelected (char *)
 *------------------------------------------------------------------------*/

Host *SystemWindow::getSelected (char *text)
{
    Host		*aHost;
    int			found = 0, b;
    char		*newtext, *device, *bc, *phone;

    /*----
     * Figure out which host matches the one they selected.
     * Note that we have to check the host name, device and baud
     * rate for a match.
     *----*/

    newtext = new char [strlen (text) + 1];
    strcpy (newtext, text);

    device = strchr (newtext, ' ');
    if (device)
    {
	*device++ = '\0';

	bc = strchr (device, ' ');
	if (bc)
	{
	    *bc++ = '\0';

	    phone = strchr (bc, ' ');
	    if (phone)
	    {
		*phone++ = '\0';
	    }

	    b = atoi (bc);
	}
    }

    for (aHost = (Host *)system->find (newtext);
	 aHost && (strcmp (newtext, aHost->theKey ()) == 0);
	 aHost = (Host *)system->next ())
    {
	if ((strcmp (device, aHost->device ()) == 0) &&
	    (strcmp (phone, aHost->phone ()) == 0) &&
	    (b == aHost->baud ()))
	{
	    found = 1;
	    break;
	}
    }

    delete [] newtext;

    if (!found)
	aHost = NULL;

    return aHost;
}

/*------------------------------------------------------------------------
 * SystemWindow::selectCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void SystemWindow::selectCallback (Widget, XtPointer clientData,
				   XtPointer callData)
{
    XmListCallbackStruct*	cbStruct;
    SystemWindow*		sw;
    char			*text;

    cbStruct = (XmListCallbackStruct *)callData;
    sw = (SystemWindow *)clientData;

    XmStringGetLtoR (cbStruct->item, XmFONTLIST_DEFAULT_TAG, &text);

    sw->select_cb (text);
}

/*------------------------------------------------------------------------
 * SystemWindow::select_cb (char *)
 *------------------------------------------------------------------------*/

void SystemWindow::select_cb (char *text)
{
    Host		*host;
    char		line[100];

    host = getSelected (text);
    if (!host)
    {
	strcpy (line, "Unable to find host ");
	strcat (line, text);
	strcat (line, " in list.\nReport this to SGI.");
	theErrorDialog->postAndWait (line
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }

    /*----
     * If a host is already displaying, bring it down.
     *----*/

    if (current)
    {
	_host->removeChangeCallback (hostChangedCallback, this);
	delete current;
    }

    _host = host;
    _host->addChangeCallback (hostChangedCallback, this);
    current = new HostWindow (parent_for_host, _host, devList, dlrList);
    current->show ();
}

/*------------------------------------------------------------------------
 * SystemWindow::newCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void SystemWindow::newCallback (Widget, XtPointer clientData, XtPointer)
{
    SystemWindow*		sw;

    sw = (SystemWindow *)clientData;
    sw->new_cb ();
}

/*------------------------------------------------------------------------
 * SystemWindow::new_cb ()
 *------------------------------------------------------------------------*/

void SystemWindow::new_cb ()
{
    char				line[100];

    if (thePromptDialog->postAndWait ("Enter name of host to add:"
#if OSVERS >= 5200
				      , True, True, False, NULL, NULL, _parent
#endif
				      ) == VkPromptDialog::OK)
    {
	if (current)
	{
	    _host->removeChangeCallback (hostChangedCallback, this);
	    delete current;
	}

	strcpy (line, thePromptDialog->text ());
	strcat (line, default_line);
	_host = new Host (line);
	_host->addChangeCallback (hostChangedCallback, this);
	current = new HostWindow (parent_for_host, _host, devList, dlrList);
	system->add (_host);		// Must occur after current defined.
	current->show ();
    }
}

/*------------------------------------------------------------------------
 * SystemWindow::deleteCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void SystemWindow::deleteCallback (Widget, XtPointer clientData, XtPointer)
{
    SystemWindow*		sw;

    sw = (SystemWindow *)clientData;
    sw->delete_cb ();
}

/*------------------------------------------------------------------------
 * SystemWindow::delete_cb ()
 *------------------------------------------------------------------------*/

void SystemWindow::delete_cb ()
{
    int			*position_list, position_count;
    XmString		*xmstr;
    char		*text, line[80];
    Host		*aHost;

    if (!XmListGetSelectedPos (host_list, &position_list, &position_count))
    {
	theErrorDialog->postAndWait ("You need to select the\nhost "
				     "to delete."
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }
    XFree (position_list);

    XtVaGetValues (host_list, XmNselectedItems, &xmstr, NULL);
    XmStringGetLtoR (xmstr[0], XmFONTLIST_DEFAULT_TAG, &text);
    aHost = getSelected (text);

    if (!aHost)
    {
	strcpy (line, "Unable to find host ");
	strcat (line, text);
	strcat (line, ".\nReport this to SGI.");
	theErrorDialog->postAndWait (line
#if OSVERS >= 5200
				     , True, False, False, NULL, NULL, _parent
#endif
				     );
	return;
    }

    strcpy (line, "Are you sure you want\nto remove host ");
    strcat (line, text);
    strcat (line, "?");
    if (theQuestionDialog->postAndWait (line
#if OSVERS >= 5200
					, True, True, False, NULL, NULL,
					_parent
#endif
					) == VkDialogManager::OK)
    {
	if (aHost == _host)
	{
	    delete current;
	    current = NULL;
	}

	/*----
	 * Removing the entry will automatically remove it from
	 * the list window because of callbacks we have set
	 * up.
	 *----*/

	system->removeEntry (aHost);
    }
}

/*------------------------------------------------------------------------
 * SystemWindow::~SystemWindow ()
 *------------------------------------------------------------------------*/

SystemWindow::~SystemWindow ()
{
    if (current)
    {
	_host->removeChangeCallback (hostChangedCallback, this);
	delete current;
    }

    system->removeChangeCallback (listChangedCallback, this);
}

/*------------------------------------------------------------------------
 * SystemWindow::SystemWindow (PathName)
 *------------------------------------------------------------------------*/

SystemWindow::SystemWindow (Widget parent, Widget hosts_parent, System *sys,
			SmartList *dev, SmartList *dlrs)
: VkComponent (sys->theKey ())
{
    Arg			wargs[15];
    int			n;
    XmString		xmstr;
    Widget		temp, label, pkg_list_label, host_list_label;
    Widget		rc1, form1, form2;

    _parent = parent;
    system = sys;
    devList = dev;
    dlrList = dlrs;
    parent_for_host = hosts_parent;
    current = NULL;

    /*----
     * Create a form into which to put all the stuff.  We have to
     * specify its size, for some reason, or it never gets set.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNwidth, 415); n++;
    XtSetArg (wargs[n], XmNheight, 180); n++;
    XtSetArg (wargs[n], XmNallowOverlap, False); n++;
    _baseWidget = XmCreateForm (parent, (char *)system->theKey (), wargs, n);
    installDestroyHandler ();

    /*----
     * Create a label to indicate what System we are showing here.
     *----*/
    
    n = 0;
    xmstr = XmStringCreateSimple ((char *)system->theKey ());
    XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabel (_baseWidget, "label", wargs, n);
    XtManageChild (label);
    XmStringFree (xmstr);

    XtAddCallback (label, XmNhelpCallback, aHelpCallback, "sw_help");

    /*----
     * Because of display problems, I was hoping that I could get
     * it to look better by creating a row/column to contain the
     * rest of the information in.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_TIGHT); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    rc1 = XmCreateRowColumn (_baseWidget, "lists", wargs, n);
    XtManageChild (rc1);

    n = 0;
    form1 = XmCreateForm (rc1, "plForm", wargs, n);
    XtManageChild (form1);

    /*----
     * List packages that use this file.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    pkg_list_label = XmCreateLabel (form1, "pkgListLabel", wargs, n);
    XtManageChild (pkg_list_label);

    XtAddCallback (pkg_list_label, XmNhelpCallback, aHelpCallback,
		   "sw_help");

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    XtSetArg (wargs[n], XmNnumColumns, 6); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, pkg_list_label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNentryAlignment, XmALIGNMENT_CENTER); n++;
    XtSetArg (wargs[n], XmNborderWidth, 1); n++;
    pkg_list = XmCreateRowColumn (form1, "pkgList", wargs, n);
    XtManageChild (pkg_list);

    XtAddCallback (pkg_list, XmNhelpCallback, aHelpCallback, "sw_help");

    /*----
     * Put a separator between the two areas.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNorientation, XmVERTICAL); n++;
    temp = XmCreateSeparatorGadget (rc1, "separator", wargs, n);
    XtManageChild (temp);

    n = 0;
    form2 = XmCreateForm (rc1, "hlForm", wargs, n);
    XtManageChild (form2);

    /*----
     * Provide a list of hosts to select from.
     *----*/
    
    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    host_list_label = XmCreateLabel (form2, "hostListLabel", wargs, n);
    XtManageChild (host_list_label);

    XtAddCallback (host_list_label, XmNhelpCallback, aHelpCallback,
		   "sw_help");

    n = 0;
    XtSetArg (wargs[n], XmNlistSizePolicy, XmCONSTANT); n++;
    XtSetArg (wargs[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    XtSetArg (wargs[n], XmNvisibleItemCount, 4); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, host_list_label); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    host_list = XmCreateScrolledList (form2, "hostList", wargs, n);
    XtManageChild (host_list);

    updateLists ();

#if 0					// Used for single click
    XtAddCallback (host_list, XmNsingleSelectionCallback, selectCallback,
		   this);
#else
    XtAddCallback (host_list, XmNdefaultActionCallback, selectCallback, this);
#endif
    XtAddCallback (host_list, XmNhelpCallback, aHelpCallback, "sw_help");
    system->addChangeCallback (listChangedCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, host_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form2, "new", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, newCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "sw_new");

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, host_list); n++;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, temp); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    temp = XmCreatePushButton (form2, "delete", wargs, n);
    XtManageChild (temp);

    XtAddCallback (temp, XmNactivateCallback, deleteCallback, this);
    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "sw_delete");
}

/*------------------------------------------------------------------------
 * SystemWindow::addPackage (LinkData *)
 *------------------------------------------------------------------------*/

void SystemWindow::addPackage (LinkData *le)
{
    Arg		wargs[5];
    int		n;
    XmString	xmstr;
    Widget	temp;

    if (_baseWidget)
    {
	n = 0;
	xmstr = XmStringCreateSimple ((char *)le->theKey ());
	XtSetArg (wargs[n], XmNlabelString, xmstr); n++;
	XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
	temp = XmCreateLabel (pkg_list, (char *)le->theKey (), wargs, n);
	XtManageChild (temp);
	XmStringFree (xmstr);
    }

    system->addPackage (le);
}

/*------------------------------------------------------------------------
 * SystemWindow::setDefaults (SmartList *, SmartList *, SmartList *)
 *------------------------------------------------------------------------*/

void SystemWindow::setDefaults (SmartList *devs, SmartList *dlrs)
{
    devList = devs;
    dlrList = dlrs;
}

#ifdef DEBUG
/*------------------------------------------------------------------------
 * SystemWindow::dump (char *)
 *------------------------------------------------------------------------*/

void SystemWindow::dump (char *indent)
{
    char		*nextIndent;
    int			i;

    i = strlen (indent);
    nextIndent = new char [i + 5];
    strcpy (nextIndent, indent);
    strcat (nextIndent, "    ");

    cout << indent << "SystemWindow dump (" << this << "):" << endl;
    cout << indent << "  system    = " << system << endl;
    cout << indent << "  devList   = " << devList << endl;
    cout << indent << "  dlrList   = " << dlrList << endl;
    cout << indent << "  current   = " << current << endl;

    if (current)
	current->dump (nextIndent);
}
#endif

/*------------------------------------------------------------------------
 * SystemWindow::save (void)
 *------------------------------------------------------------------------*/

void SystemWindow::save (void)
{
    char		line[128];

    system->save ();
    if (system->error ())
    {
	strcpy (line, "*** Error saving file:\n");
	strcat (line, strerror (system->error ()));
	theErrorDialog->postAndWait (line);
    }
}

/*------------------------------------------------------------------------
 * afterRealizeHook
 *------------------------------------------------------------------------*/

void SystemWindow::afterRealizeHook ()
{
#ifdef DEBUG1
    Dimension		w, h;
    Cardinal		c;

    XtVaGetValues (XtParent (_baseWidget),
		   XmNwidth, &w,
		   XmNheight, &h,
		   XmNnumChildren, &c,
		   NULL);

    cerr << "SystemWindow: " << this << " - width = " << w << endl;
    cerr << "SystemWindow: " << this << " - height = " << h << endl;
    cerr << "SystemWindow: " << this << " - children = " << c << endl;

    XtVaGetValues (_baseWidget,
		   XmNwidth, &w,
		   XmNheight, &h,
		   NULL);
    cerr << "Form width  = " << w << endl;
    cerr << "Form height = " << h << endl;

    c = XtMakeResizeRequest (_baseWidget, 340, 210, &w, &h);
    cerr << "XtMakeResizeRequest returned " << c << endl;
    cerr << "       width = " << w << endl;
    cerr << "      height = " << h << endl;
#endif
}

/*------------------------------------------------------------------------
 * SystemWindow::hostChangedCallback (Host *, Host::Field, void *)
 *------------------------------------------------------------------------*/

void SystemWindow::hostChangedCallback (Host *host, Host::Field field,
					void *client_data)
{
    SystemWindow		*sw;

    sw = (SystemWindow *)client_data;
    sw->hostChanged (host, field);
}

/*------------------------------------------------------------------------
 * SystemWindow::hostChanged (Host *, Host::Field)
 *------------------------------------------------------------------------*/

void SystemWindow::hostChanged (Host *host, Host::Field)
{
    int				*pos_list, pos;
    char			host_string[80];
    XmString			xmstr;

    if (XmListGetSelectedPos (host_list, &pos_list, &pos))
    {
	if (host->baud () == ANYBAUD)
	    sprintf (host_string, "%s %s Any %s", host->theKey (),
		     host->device (), host->phone ());
	else
	    sprintf (host_string, "%s %s %d %s", host->theKey (),
		     host->device (), host->baud (), host->phone ());
	xmstr = XmStringCreateSimple (host_string);
	XmListReplaceItemsPos (host_list, &xmstr, 1, pos_list[0]);
	XmStringFree (xmstr);
	XmListSelectPos (host_list, pos_list[0], False);
    }
}

/*------------------------------------------------------------------------
 * SystemWindow::listChangedCallback (SmartList *, SmartList::ACTION, void *)
 *------------------------------------------------------------------------*/

void SystemWindow::listChangedCallback (SmartList *, SmartList::ACTION,
					void *client_data)
{
    SystemWindow		*sw;

    sw = (SystemWindow *)client_data;
    sw->updateLists ();
}

/*------------------------------------------------------------------------
 * SystemWindow::updateLists (void)
 *------------------------------------------------------------------------*/

void SystemWindow::updateLists (void)
{
    Host		*aHost;
    char		host_string[80];
    XmString		xmstr;
    WidgetList		wPackages;
    int			nPackages, i, n;
    LinkData		*ld;
    Widget		temp;
    Arg			wargs[10];

#if 0
    cout << "A list changed." << endl;
    cout << "   Notified " << this << endl;
    cout << "   System = " << sys->theKey () << endl;

    switch (action)
    {
      case SmartList::ADD:
	cout << "   Action = ADD" << endl;
	break;
      case SmartList::DELETE:
	cout << "   Action = DELETE" << endl;
	break;
      case SmartList::MOVE:
	cout << "   Action = MOVE" << endl;
	break;
      case SmartList::OTHER:
	cout << "   Action = OTHER" << endl;
	break;
    }
#endif

    /*----
     * Update the package list.
     *	1. Find out what children exist in the package list.
     *	2. Loop through the packages and add any that are missing.
     *----*/

    XtVaGetValues (pkg_list,
		   XmNchildren, &wPackages,
		   XmNnumChildren, &nPackages, NULL);

    for (ld = system->firstPackage (); ld; ld = system->nextPackage ())
    {
	for (i = 0; i < nPackages; i++)
	    if (strcmp (ld->theKey (), XtName (wPackages[i])) == 0)
		break;
	if (i != nPackages)
	    continue;
	n = 0;
	XtSetArg (wargs[n], XmNalignment, XmALIGNMENT_CENTER); n++;
	temp = XmCreateLabel (pkg_list, (char *)ld->theKey (), wargs, n);
	XtManageChild (temp);

	XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "sw_help");
    }

    /*----
     * Now loop through the children and remove any that are
     * not in the package list.
     *----*/

    for (i = 0; i < nPackages; i++)
    {
	ld = system->findPackage (XtName (wPackages[i]));
	if (!ld)
	    XtDestroyWidget (wPackages[i]);
    }

    /*----
     * Update the host list.
     *	1. Empty it out.
     *	2. Fill it.
     *----*/

    XmListDeleteAllItems (host_list);

    for (aHost = (Host *)system->first (); aHost;
	 aHost = (Host *)system->next ())
    {
	if (aHost->baud () == ANYBAUD)
	    sprintf (host_string, "%s %s Any %s", aHost->theKey (),
		     aHost->device (), aHost->phone ());
	else
	    sprintf (host_string, "%s %s %d %s", aHost->theKey (),
		     aHost->device (), aHost->baud (), aHost->phone ());
	xmstr = XmStringCreateSimple (host_string);
	XmListAddItemUnselected (host_list, xmstr, 0);
	XmStringFree (xmstr);
    }

    if (current)
    {
	if (_host->baud () == ANYBAUD)
	    sprintf (host_string, "%s %s Any %s", _host->theKey (),
		     _host->device (), _host->phone ());
	else
	    sprintf (host_string, "%s %s %d %s", _host->theKey (),
		     _host->device (), _host->baud (), _host->phone ());
	xmstr = XmStringCreateSimple (host_string);
	XmListSelectItem (host_list, xmstr, False);
	XmStringFree (xmstr);
    }
}
