/*---*- C++ -*------------------------------------------------------------
 * $Id: SystemWindow.h,v 2.18 1994/02/26 03:56:33 carlson Exp $
 *
 * Header for SystemWindow class.
 *
 * Revision History:
 *	$Log: SystemWindow.h,v $
 * Revision 2.18  1994/02/26  03:56:33  carlson
 * Added a place to keep the parent widget for dialogs.
 *
 * Revision 2.17  1994/02/18  18:51:27  chrisc
 * Added method to find the selected item from the list.
 * Added method to update the package and item list.
 * Added methods for handling delete button.
 *
 * Revision 2.16  1994/02/15  04:27:18  chrisc
 * Removed all references to dialcodes.  This is now done separately in
 *   the main window.
 *
 * Revision 2.15  1994/01/17  23:56:53  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.14  1993/12/31  00:04:23  carlson
 * Added prototype for new callback methods.
 *
 * Revision 2.13  1993/11/16  01:34:08  carlson
 * Added a method to return modified flag from associated system.
 *
 * Revision 2.12  1993/11/05  20:09:29  carlson
 * Changed all references to LinkList to use SmartList.
 *
 * Revision 2.11  1993/10/12  00:49:59  carlson
 * Added prototype for dump method.
 *
 * Revision 2.10  1993/10/06  00:44:06  chris
 * Added a declaration for the save method.
 *
 * Revision 2.9  1993/08/12  15:23:45  chris
 * Added prototype for 'setDefaults' method.
 *
 * Revision 2.8  1993/07/27  20:17:20  chris
 * Constructor now accepts pointers to other lists.
 * Created variables to keep these pointers in.
 *
 * Revision 2.7  1993/07/27  17:52:03  chris
 * Need to know HostWindow class exists.
 * Added parameter to constructor to identify Widget that HostWindow is
 *   to be in.
 * Make a Widget to keep this variable in.
 * Need to keep the HostWindow we create.
 *
 * Revision 2.6  1993/07/24  00:24:46  chris
 * Modified so this is no longer a derived class of System.
 * Removed methods for createWindow, deleteWindow and makeNewHost.
 * Save a pointer to the System class rather than the file name.
 *
 * Revision 2.5  1993/07/23  22:56:38  chris
 * No longer create a window but a component.
 * Added methods to create and delete window.
 * Added virtual methods for doing things after realizing widget (for test
 *   purposes).
 * Also added virtual method for handling adding a package.
 * Got rid of all the callback methods.
 *
 * Revision 2.4  1993/07/21  00:08:38  chris
 * Added value for fileName since the _name element only contains the
 *   file name and not the full path.
 *
 * Revision 2.3  1993/07/08  02:11:58  chris
 * Add prototype for selectCallback.
 *
 * Revision 2.2  1993/07/08  00:13:54  chris
 * Remove commented out constructor.
 * Modified makeNewPackage to makeNewHost and uncommented it.
 *
 * Revision 2.1  1993/07/07  01:40:42  chris
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _SYSTEMWINDOW_H_
#define _SYSTEMWINDOW_H_

#include <Vk/VkComponent.h>

#include "SmartList.h"
#include "PathName.h"
#include "System.h"
#include "Host.h"

class HostWindow;

class SystemWindow : public VkComponent
{
  public:
    SystemWindow (Widget parent, Widget hosts_parent, System *sys,
		  SmartList *dev = NULL, SmartList *dlrs = NULL);
    ~SystemWindow ();

    void		setDefaults (SmartList *dev, SmartList *dlrs);
    void		save (void);
    int			wasModified (void) { return system->wasModified (); }

    virtual const char*	className () { return "SystemWindow"; }
    virtual void	afterRealizeHook ();
    virtual void	addPackage (LinkData *le);
#ifdef DEBUG
    virtual void	dump (char *indent);
#endif

  private:
    Widget		_parent;
    System		*system;
    SmartList		*devList;
    SmartList		*dlrList;
    Widget		parent_for_host;
    Widget		pkg_list;
    Widget		host_list;
    HostWindow		*current;	// Current HostWindow displayed
    Host		*_host;		// Host displayed in current

    Host*		getSelected (char*);
    void		updateLists (void);

    static void		selectCallback (Widget, XtPointer, XtPointer);
    static void		hostChangedCallback (Host*, Host::Field, void *);
    static void		listChangedCallback (SmartList*, SmartList::ACTION,
					     void*);
    static void		newCallback (Widget, XtPointer, XtPointer);
    static void		deleteCallback (Widget, XtPointer, XtPointer);

    void		select_cb (char *text);
    void		hostChanged (Host *, Host::Field);
    void		new_cb ();
    void		delete_cb ();
};

#endif
