/*---*- C++ -*------------------------------------------------------------
 * $Id: Terminal.h,v 2.1 1994/01/19 01:51:34 carlson Exp $
 *
 * Header file for Terminal class.
 *
 * Revision History:
 *	$Log: Terminal.h,v $
 * Revision 2.1  1994/01/19  01:51:34  carlson
 * Initial revision.
 *
 *------------------------------------------------------------------------*/

#ifndef _TERMINAL_H_
#define _TERMINAL_H_

#include <termio.h>

class Terminal
{
  public:
    enum BaudTest { INIT_TERM, SET_BAUD, CHECK_OK, RETURN_TERM };
    enum StatusReturn { FAILURE = 0, SUCCESS };
    enum State { NOTSETUP, ERROR, CLOSED, OPENED };

    Terminal (const char *device);
    Terminal (void);
    ~Terminal ();

    /*----
     * Error reporting methods.
     *----*/

    int			errVal (void) { return savErrno; }
    enum State		currentState (void);
    char		*strerror (int);
    char		*strerror (void);

    /*----
     * Setup methods.
     *----*/

    enum StatusReturn	setDevice (const char *device);
    enum StatusReturn	setBaud (int);

    /*----
     * Actual work routines.
     *----*/

    enum StatusReturn	open (void);
    enum StatusReturn	init (void);
    enum StatusReturn	write (char *);
    int			read (char *, int);
    void		uninit (void);
    void		close (void);
    enum StatusReturn	clear (void);

  private:
    char		*devString;
    char		baudCodes[30];
    int			bauds[30];
    int			tty_fd;
    int			baud_ndx;
    enum StatusReturn	_error;
    int			savErrno;
    static char		*errorStrings[];
    struct termio	orig_termio, new_termio;

    void		doInitialization (void);

    int			reopen_tty (struct termio *);
    enum StatusReturn	check_tty_input (char *);
    enum StatusReturn	test_current_baud (enum BaudTest);
};

#endif
