/*------------------------------------------------------------------------
 * $Id: Tutorial.c++,v 2.6 1994/02/25 19:41:40 carlson Exp $
 *
 * Methods for handling the tutorial.
 *
 * Revision History:
 *	$Log: Tutorial.c++,v $
// Revision 2.6  1994/02/25  19:41:40  carlson
// Added a call to make sure that we show the top of a new section, not the bottom.
//
// Revision 2.5  1994/02/15  04:20:38  chrisc
// Changed all references to myApplication to uucpApplication.
//
// Revision 2.4  1994/01/05  18:16:27  carlson
// Added include of limits.h since it no longer is in uucp.h.
//
// Revision 2.3  1994/01/05  02:31:31  carlson
// Modified to get file name from resources picked up by main application.
//
// Revision 2.2  1993/12/30  02:50:43  carlson
// Added static global string with name of text file.
// Gave up trying to use 'seekg' and 'tellg'.  Just open and close file as
//   needed.
// Use \f as marker between steps instead of 'Step'.
// Created method which does all work to append text.
// Set sensitivity of buttons depending on whether they really work or not.
// Added some comments.
// Remove cursor.
// Removed unused function displayTutorial.
//
// Revision 2.1  1993/12/29  22:04:59  carlson
// Initial revision.
//
 *------------------------------------------------------------------------*/

#include "uucp.h"
#include "Tutorial.h"

#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>

#include <iostream.h>
#include <errno.h>
#include <string.h>
#include <limits.h>

static char		TutorialFileName[PATH_MAX];

/*------------------------------------------------------------------------
 * Tutorial::showStep (int)
 *------------------------------------------------------------------------*/

void Tutorial::showStep (int step)
{
    char		buff[90];

#ifdef DEBUG1
    cout << "showStep called for step " << step << endl;
#endif

    /*----
     * Remove any text currently showing in the text window.
     *----*/

    XmTextSetEditable (text, True);
    XmTextSetSelection (text, 0, XmTextGetLastPosition (text),
			XtLastTimestampProcessed (theApplication->display ()));
    XmTextRemove (text);

    /*----
     * Open the tutorial file and, if not the introduction, reposition
     * to the beginning of the step.
     *----*/

    tutorial_file->open (TutorialFileName);
    if (tutorial_file->fail ())
	return;
    if (step >= 0)
	tutorial_file->seekg (step_positions[step]);

    while (!tutorial_file->eof ())
    {
	tutorial_file->getline (buff, sizeof (buff));
	if (buff[0] == '\f')
	    break;
    XmTextSetTopCharacter (text, 0);
	appendText (buff, True);
    }
    tutorial_file->close ();

    XmTextSetTopCharacter (text, 0);
    XmTextSetEditable (text, False);

    current_step = step;
}

/*------------------------------------------------------------------------
 * Tutorial::appendText (char *, Boolean)
 *------------------------------------------------------------------------*/

void Tutorial::appendText (char *str, Boolean newline)
{
    XmTextPosition		lastPosition;
    char			*c;

    if (newline)
    {
	c = str + strlen (str) - 1;
	if (*c != '\n')
	{
	    *++c = '\n';
	    *++c = '\0';
	}
    }

    lastPosition = XmTextGetLastPosition (text);
    XmTextInsert (text, lastPosition, str);
}

/*------------------------------------------------------------------------
 * Tutorial::stepCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void Tutorial::stepCallback (Widget, XtPointer clientData, XtPointer)
{
    CallbackData		*cbData;

    cbData = (CallbackData *)clientData;
    cbData->me->step_cb (cbData->step);
}

/*------------------------------------------------------------------------
 * Tutorial::step_cb (Time, int)
 *------------------------------------------------------------------------*/

void Tutorial::step_cb (int step)
{
    showStep (step);
    if ((current_step + 1) == total_steps)
    {
	XtSetSensitive (next_button, False);
	XtSetSensitive (prev_button, True);
    }
    else if (current_step < 0)
    {
	XtSetSensitive (next_button, True);
	XtSetSensitive (prev_button, False);
    }
    else
    {
	XtSetSensitive (next_button, True);
	XtSetSensitive (prev_button, True);
    }
}

/*------------------------------------------------------------------------
 * Tutorial::nextCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void Tutorial::nextCallback (Widget, XtPointer clientData, XtPointer)
{
    Tutorial		*tut;

    tut = (Tutorial *)clientData;
    tut->next_cb ();
}

/*------------------------------------------------------------------------
 * Tutorial::next_cb ()
 *------------------------------------------------------------------------*/

void Tutorial::next_cb ()
{
#ifdef DEBUG1
    cout << "next_cb called" << endl;
#endif

    showStep (current_step + 1);
    if ((current_step + 1) == total_steps)
	XtSetSensitive (next_button, False);
    if (!XtIsSensitive (prev_button))
	XtSetSensitive (prev_button, True);
}

/*------------------------------------------------------------------------
 * Tutorial::prevCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void Tutorial::prevCallback (Widget, XtPointer clientData, XtPointer)
{
    Tutorial		*tut;

    tut = (Tutorial *)clientData;
    tut->prev_cb ();
}

/*------------------------------------------------------------------------
 * Tutorial::prev_cb ()
 *------------------------------------------------------------------------*/

void Tutorial::prev_cb ()
{
#ifdef DEBUG1
    cout << "prev_cb called" << endl;
#endif

    showStep (current_step - 1);
    if (current_step < 0)
	XtSetSensitive (prev_button, False);
    if (!XtIsSensitive (next_button))
	XtSetSensitive (next_button, True);
}

/*------------------------------------------------------------------------
 * Tutorial::quitCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void Tutorial::quitCallback (Widget, XtPointer clientData, XtPointer)
{
    Tutorial		*tut;

    tut = (Tutorial *)clientData;
    tut->quit_cb ();
}

/*------------------------------------------------------------------------
 * Tutorial::quit_cb ()
 *------------------------------------------------------------------------*/

void Tutorial::quit_cb ()
{
    hide ();
}

/*------------------------------------------------------------------------
 * Tutorial::setUpInterface (Widget)
 *------------------------------------------------------------------------*/

Widget Tutorial::setUpInterface (Widget parent)
{
    Arg			wargs[10];
    int			n;
    Widget		hs;
    char		buff[90], *cptr;
    CallbackData	*cbData;

    /*----
     * Make sure the parent window (which is an XmMainWindow) shows
     * the separators between segments, automatically creates scrollbars
     * when needed and places the command window below the workspace
     * window.
     *----*/

    XtVaSetValues (parent,
		   XmNshowSeparator, True,
		   XmNscrollBarDisplayPolicy, XmAUTOMATIC,
		   XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE,
		   NULL);

    /*----
     * Create the workspace window.  This will be a text window that
     * will display the tutorial text.  We don't want the horizontal
     * scrollbar, only the vertical one, so remove it.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNeditable, False); n++;
    XtSetArg (wargs[n], XmNcursorPositionVisible, False); n++;
    XtSetArg (wargs[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
    text = XmCreateScrolledText (parent, "text", wargs, n);
    XtManageChild (text);

    XtVaGetValues (XtParent (text), XmNhorizontalScrollBar, &hs, NULL);
    if (hs)
	XtUnmanageChild (hs);

    /*----
     * Create a rowcolumn widget to contain the buttons for traversing
     * around the tutorial.  This widget will become the command
     * window in the parent window.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNadjustLast, False); n++;
    XtSetArg (wargs[n], XmNpacking, XmPACK_COLUMN); n++;
    XtSetArg (wargs[n], XmNorientation, XmHORIZONTAL); n++;
    command = XmCreateRowColumn (parent, "buttons", wargs, n);
    XtManageChild (command);

    /*----
     * Create the buttons for Next, Previous and Quit.
     *----*/

    n = 0;
    next_button = XmCreatePushButton (command, "next", wargs, n);
    XtManageChild (next_button);

    XtAddCallback (next_button, XmNactivateCallback, nextCallback, this);

    n = 0;
    prev_button = XmCreatePushButton (command, "prev", wargs, n);
    XtManageChild (prev_button);

    XtAddCallback (prev_button, XmNactivateCallback, prevCallback, this);

    n = 0;
    hs = XmCreatePushButton (command, "quit", wargs, n);
    XtManageChild (hs);

    XtAddCallback (hs, XmNactivateCallback, quitCallback, this);

    XtVaSetValues (parent,
		   XmNcommandWindow, command,
		   NULL);

    /*----
     * Create the menubar pulldown menu to allow jumping to a particular
     * step in the tutorial.  For this callback, we need to provide the
     * step selected as well as the pointer 'this'.  For this reason, we
     * created the structure CallbackData to hold both of these items.
     *
     * First, we need a menu that will bring us back to the introduction.
     * The 'step' for the introduction will be -1.
     *----*/

    stepMenuPane = addMenuPane ("Step");
    cbData = new CallbackData;
    cbData->step = -1;
    cbData->me = this;
    stepMenuPane->addAction ("Intro", stepCallback, cbData);
    current_step = -1;

    /*----
     * Create the file name to open.
     *----*/

    strcpy (TutorialFileName, uucpApplication->directory ());
    strcat (TutorialFileName, "/");
    strcat (TutorialFileName, uucpApplication->tutorial ());

    /*----
     * Open the file containing the tutorial text.
     *		Someday, I would like this to be a resource that can
     *		be set in the resource database.
     *----*/

    tutorial_file = new ifstream (TutorialFileName);
    if (tutorial_file->fail ())
    {
	cerr << "Tutorial: Unable to open file " << TutorialFileName << endl;
	cerr << "  " << strerror (errno) << endl;
    }
    else
    {

	/*----
	 * First we need to find out how many steps there are so we
	 * can allocate and array of pointers to where these steps
	 * begin in the file.  Since we're reading the file anyway,
	 * let's go ahead and load up the introduction into the
	 * text widget.
	 *----*/

	while (!tutorial_file->eof ())
	{
	    tutorial_file->getline (buff, sizeof (buff));
	    if (buff[0] == '\f')
		break;
	    appendText (buff, True);
	}

	total_steps = 1;
	while (!tutorial_file->eof ())
	{
	    tutorial_file->getline (buff, sizeof (buff));
	    if (buff[0] == '\f')
		total_steps++;
	}

	/*----
	 * Ok, now we know how many steps there are, create an
	 * array to contain all the positions in the file where
	 * all the steps begin.  Then rewind the file and fill
	 * in the values in the array.
	 *
	 * The step positions start *after* the form feeds in
	 * the file.  Text up to ':' on the line following the
	 * form feed will be used in the menu.
	 *----*/

	step_positions = new streampos[total_steps];

	tutorial_file->close ();
	tutorial_file->open (TutorialFileName);

	for (n = 0; !tutorial_file->eof (); )
	{
	    tutorial_file->getline (buff, sizeof (buff));
	    if (buff[0] == '\f')
	    {
		if (tutorial_file->eof ())
		    break;
		step_positions[n] = tutorial_file->tellg ();
		tutorial_file->getline (buff, sizeof (buff));
		cptr = strchr (buff, ':');
		if (cptr)
		    *cptr = '\0';
		cbData = new CallbackData;
		cbData->step = n;
		cbData->me = this;
		stepMenuPane->addAction (buff, stepCallback, cbData);
		n++;
	    }
	}
	tutorial_file->close ();
    }

    XtSetSensitive (prev_button, False);

    return XtParent (text);
}

/*------------------------------------------------------------------------
 * Tutorial::handleWmDeleteMessage \(\)
 *------------------------------------------------------------------------*/

void
Tutorial::handleWmDeleteMessage()
{
    hide();
}

/*------------------------------------------------------------------------
 * Tutorial::handleWmQuitMessage \(\)
 *------------------------------------------------------------------------*/

void
Tutorial::handleWmQuitMessage()
{
    hide();
}
