/*------------------------------------------------------------------------
 * $Id: importFile.c++,v 1.2 1994/02/02 23:43:43 chrisc Exp $
 *
 * This function is a first cut at an import tool that reads the output
 * of a configuration printed by a modem.  This tool reads the output
 * of the Telebit modem command 'ATN?'.
 *
 * Revision History:
 *	$Log: importFile.c++,v $
// Revision 1.2  1994/02/02  23:43:43  chrisc
// Pretty much rewrote this to be a lot simpler.  Now it works.
//
// Revision 1.1  1994/02/02  18:15:49  chrisc
// Initial revision
//
 *------------------------------------------------------------------------*/

#include "ModemFile.h"

#include <Vk/VkErrorDialog.h>

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

/*------------------------------------------------------------------------
 * importFile
 *------------------------------------------------------------------------*/

void importFile (const char *fileName, ModemFile *modemFile)
{
    enum STATE { START, REGNUM, REGVAL, FLAGNAM, FLAGVAL, COMMNT, IGNORE };

    ifstream		*ifile;
    char		c, line[128], reg[10], regval[10];
    char		*cptr, *rptr, *rvptr, *tptr;
    enum STATE		state;
    Registers		*Reg;
    int			n;

    ifile = new ifstream (fileName);
    if (ifile->fail ())
    {
	theErrorDialog->postAndWait ("Unable to open file.");
	return;
    }

    ifile->unsetf (ios::skipws);

    rptr = reg;
    rvptr = regval;
    for (state = START; !ifile->eof (); )
    {
	*ifile >> c;
	switch (state)
	{
	    /*----------------------------------------------------------
	     * START
	     *----*/

	  case START:
	    if (isspace (c))
		break;
	    switch (toupper (c))
	    {
	      case '#':			// Begin a comment
		cptr = line;
		state = COMMNT;
		break;

	      case 'S':			// Register definition
		state = REGNUM;
		break;

	      case '&':			// Two character flag name
		*rptr++ = c;
		state = FLAGNAM;
		break;

	      default:
		if (isalpha (c))
		{
		    *rptr++ = toupper (c);
		    *rptr = '\0';
		    state = FLAGVAL;
		}
		break;
	    }
	    break;

	    /*----------------------------------------------------------
	     * REGNUM
	     *----*/

	  case REGNUM:
	    if (c == '=')
	    {
		*rptr = '\0';
		state = REGVAL;
	    }
	    else if (isdigit (c))
	    {
		*rptr++ = c;
	    }
	    else
	    {
		rptr = reg;
		rvptr = regval;
		Reg = NULL;
		state = (isspace (c)) ? START : IGNORE;
	    }
	    break;

	    /*----------------------------------------------------------
	     * REGVAL
	     *----*/

	  case REGVAL:
	    if (isdigit (c))
	    {
		*rvptr++ = c;
	    }
	    else
	    {
		*rvptr = '\0';
		if (strlen (reg) < 3)
		{
		    n = 3 - strlen (reg);
		    strncpy (line, "000", n);
		    line[n] = '\0';
		    strcat (line, reg);
		    strcpy (reg, line);
		}

		Reg = modemFile->findRegister (reg);
		if (Reg)
		{
		    tptr = new char [strlen (regval) + 1];
		    strcpy (tptr, regval);
		    Reg->setValue (tptr);
		}
		else
		{
		    tptr = new char [strlen (reg) + 1];
		    strcpy (tptr, reg);
		    Reg = new Registers (tptr, Registers::REGISTER);
		    tptr = new char [strlen (regval) + 1];
		    strcpy (tptr, regval);
		    Reg->setValue (tptr);
		    tptr = new char [1];
		    tptr[0] = '\0';
		    Reg->setDocumentation (tptr);
		    modemFile->addRegister (Reg);
		}
		rptr = reg;
		rvptr = regval;
		state = START;
	    }
	    break;

	    /*----------------------------------------------------------
	     * FLAGNAM
	     *----*/

	  case FLAGNAM:
	    if (isalpha (c))
	    {
		*rptr++ = toupper (c);
		*rptr = '\0';
		state = FLAGVAL;
	    }
	    else
	    {
		rptr = reg;
		rvptr = regval;
		state = (isspace (c)) ? START : IGNORE;
	    }
	    break;

	    /*----------------------------------------------------------
	     * FLAGVAL
	     *----*/

	  case FLAGVAL:
	    if (isdigit (c))
	    {
		*rvptr++ = c;
	    }
	    else if (rvptr == regval)
	    {
		rptr = reg;
		rvptr = regval;
		state = (isspace (c)) ? START : IGNORE;
	    }
	    else
	    {
		*rvptr = '\0';
		Reg = modemFile->findRegister (reg);
		if (Reg)
		{
		    tptr = new char [strlen (regval) + 1];
		    strcpy (tptr, regval);
		    Reg->setValue (tptr);
		}
		else
		{
		    tptr = new char [strlen (reg) + 1];
		    strcpy (tptr, reg);
		    Reg = new Registers (tptr, Registers::FLAG);
		    tptr = new char [strlen (regval) + 1];
		    strcpy (tptr, regval);
		    Reg->setValue (tptr);
		    tptr = new char [1];
		    tptr[0] = '\0';
		    Reg->setDocumentation (tptr);
		    modemFile->addRegister (Reg);
		}
		rptr = reg;
		rvptr = regval;
		state = START;
	    }
	    break;

	    /*----------------------------------------------------------
	     * COMMNT
	     *----*/

	  case COMMNT:
	    if (c == '\n')
	    {
		if (Reg)
		{
		    tptr = Reg->getDocumentation ();
		    if (tptr)
			delete [] tptr;
		    *cptr = '\0';
		    tptr = new char [strlen (line) + 1];
		    strcpy (tptr, line);
		    Reg->setDocumentation (tptr);
		}
		cptr = line;
		Reg = NULL;
		state = START;
	    }
	    else
		*cptr++ = c;
	    break;

	    /*----------------------------------------------------------
	     * IGNORE
	     *----*/

	  case IGNORE:
	    if (isspace (c))
		state = START;
	    break;
	}
    }
}
