/*------------------------------------------------------------------------
 * $Id: linkList.c++,v 2.7 1994/01/18 04:38:10 carlson Exp $
 *
 * Simple linked list class.
 *
 * Revision History:
 *	$Log: linkList.c++,v $
// Revision 2.7  1994/01/18  04:38:10  carlson
// Need to verify that I have a value before returning it.
// Added methods for manipulating the LinkEntry flags.
//
// Revision 2.6  1994/01/17  20:43:55  carlson
// Had to change the name of the include file for Unix workstations.
//
// Revision 2.5  1994/01/17  20:37:26  carlson
// Changed all methods that return LinkEntry pointers to return LinkData
//   pointers.  Also, changed all methods that had LinkEntry parameters to
//   have LinkData parameters.
// Removed redundant method that took a 'char *' type as a parameter.
// Removed automatic variables that weren't needed.
// Added method to remove last found LinkData entry.
// Added methods to remove entries.
//
// Revision 2.4  1993/11/05  19:42:34  carlson
// Added the method 'remove' to remove an item.
//
// Revision 2.3  1993/10/19  22:46:50  carlson
// Added and modified code to allow LinkList class to be ordered either
//   ascending, descending, FIFO or FILO.
//
// Revision 2.2  1993/06/30  20:29:16  chris
// Have to use a method for getting the key value.
//
// Revision 2.1  1993/06/30  04:34:33  chris
// Updated to have new format.
//
// Revision 1.1  1993/05/05  17:57:29  chris
// Initial revision
//
 *------------------------------------------------------------------------*/

#include <malloc.h>
#include <string.h>

#include "linkList.h"
#ifdef unix
#include "linkEntry.h"
#else
#include "linkntry.h"
#endif

/*========================================================================
 * LinkData class
 *========================================================================*/

/*------------------------------------------------------------------------
 * LinkData::LinkData (const char *)
 *------------------------------------------------------------------------*/

LinkData::LinkData (const char *tag)
{
    key = new char [strlen (tag) + 1];
    strcpy (key, tag);
}

/*------------------------------------------------------------------------
 * LinkData::operator= (char *)
 *------------------------------------------------------------------------*/

LinkData& LinkData::operator= (char *tag)
{
    if (key)
	delete [] key;
    key = new char [strlen (tag) + 1];
    strcpy (key, tag);

    return *this;
}

/*------------------------------------------------------------------------
 * LinkData::setKey (char *)
 *------------------------------------------------------------------------*/

void LinkData::setKey (char *tag)
{
    if (key)
	delete [] key;
    key = new char [strlen (tag) + 1];
    strcpy (key, tag);
}

/*========================================================================
 * LinkEntry class
 *========================================================================*/

/*========================================================================
 * LinkList class
 *========================================================================*/

/*------------------------------------------------------------------------
 * LinkList::add (LinkData *)
 *------------------------------------------------------------------------*/

void LinkList::add (LinkData *pentry)
{
    LinkEntry		*newEntry;

    newEntry = new LinkEntry (pentry);

    /*----
     * If there is an entry in the list, then figure out where to
     * put the new entry based on the ordering.
     *----*/

    if (top)
    {
	switch (ordering)
	{
	  case ASCENDING:
	    for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
	    {
		if (strcmp (_last->key (), pentry->theKey ()) > 0)
		{
		    if (beforeLast)
			beforeLast->next = newEntry;
		    else
			top = newEntry;

		    newEntry->next = _last;
		    break;
		}
		beforeLast = _last;
	    }

	    if (_last == ENULL)
	    {
		beforeLast->next = newEntry;
		newEntry->next = ENULL;
	    }
	    break;

	  case DESCENDING:
	    for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
	    {
		if (strcmp (_last->key (), pentry->theKey ()) < 0)
		{
		    if (beforeLast)
			beforeLast->next = newEntry;
		    else
			top = newEntry;

		    newEntry->next = _last;
		    break;
		}
		beforeLast = _last;
	    }

	    if (_last == ENULL)
	    {
		beforeLast->next = newEntry;
		newEntry->next = ENULL;
	    }
	    break;

	  case FIFO:
	    for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
	    	beforeLast = _last;
	    beforeLast->next = newEntry;
	    break;

	  case FILO:
	    newEntry->next = top;
	    _last = top = newEntry;
	    beforeLast = ENULL;
	    break;
	}

    }
    else
    {
	_last = top = newEntry;
	newEntry->next = ENULL;
	beforeLast = ENULL;
    }
}

/*------------------------------------------------------------------------
 * LinkList::find (char *)
 *------------------------------------------------------------------------*/

LinkData *LinkList::find (char *tag)
{
    if (top)
    {
	for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
	{
	    if (strcmp (_last->key (), tag) == 0)
		break;
	    beforeLast = _last;
	}
    }

    if (_last)
    	return _last->Data ();
    else
	return NULL;
}

/*------------------------------------------------------------------------
 * LinkList::find (const char *)
 *------------------------------------------------------------------------*/

LinkData *LinkList::find (const char *tag)
{
    return find ((char *)tag);
}

/*------------------------------------------------------------------------
 * LinkList::find (LinkData *)
 *------------------------------------------------------------------------*/

LinkData *LinkList::find (LinkData *data)
{
    if (top)
    {
	for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
	{
	    if (data == _last->Data ())
		break;
	    beforeLast = _last;
	}
    }

    if (_last)
    	return _last->Data ();
    else
	return NULL;
}

/*------------------------------------------------------------------------
 * LinkList::first (void)
 *------------------------------------------------------------------------*/

LinkData* LinkList::first (void)
{
    beforeLast = NULL;
    _last = top;

    if (_last)
	return _last->Data ();

    return NULL;
}

/*------------------------------------------------------------------------
 * LinkList::last (void)
 *------------------------------------------------------------------------*/

LinkData* LinkList::last (void)
{
    return (_last) ? _last->Data () : NULL;
}

/*------------------------------------------------------------------------
 * LinkList::next (void)
 *------------------------------------------------------------------------*/

LinkData* LinkList::next (void)
{
    beforeLast = _last;

    if (_last)
	_last = _last->next;

    if (_last)
	return _last->Data ();

    return NULL;
}

/*------------------------------------------------------------------------
 * LinkList::~LinkList ()
 *------------------------------------------------------------------------*/

LinkList::~LinkList ()
{
    clean ();
}

/*------------------------------------------------------------------------
 * LinkList::clean (void)
 *------------------------------------------------------------------------*/

void LinkList::clean (void)
{
    LinkEntry	*current;

    while (top)
    {
	current = top;
	top = top->next;
	delete current;
    }

    beforeLast = _last = ENULL;
}

/*------------------------------------------------------------------------
 * LinkList::removeEntry (LinkData *)
 *------------------------------------------------------------------------*/

void LinkList::removeEntry (LinkData *d)
{
    for (_last = top, beforeLast = ENULL; _last; _last = _last->next)
    {
	if (_last->Data () == d)
	{
	    removeEntry ();
	    break;
	}
	beforeLast = _last;
    }
}

/*------------------------------------------------------------------------
 * LinkList::removeEntry (void)
 *------------------------------------------------------------------------*/

void LinkList::removeEntry (void)
{
    if (_last)
    {
	if (beforeLast)
	    beforeLast->next = _last->next;
	else
	    top = _last->next;
	delete _last;
    }
    _last = beforeLast = ENULL;
}

/*------------------------------------------------------------------------
 * LinkList::setOrdering (enum ORDERING)
 *------------------------------------------------------------------------*/

void LinkList::setOrdering (enum ORDERING newOrder)
{
    ordering = newOrder;
}

/*------------------------------------------------------------------------
 * LinkList::addBefore (LinkData *)
 *------------------------------------------------------------------------*/

void LinkList::addBefore (LinkData *pentry)
{
    if ((ordering != FIFO) && (ordering != FILO))
	return;

    LinkEntry		*newEntry = new LinkEntry (pentry);

    if (_last)
    {
	newEntry->next = _last;
	if (beforeLast)
	    beforeLast->next = newEntry;
	else
	    top = newEntry;
    }
}

/*------------------------------------------------------------------------
 * LinkList::addAfter (LinkData *)
 *------------------------------------------------------------------------*/

void LinkList::addAfter (LinkData *pentry)
{
    if ((ordering != FIFO) && (ordering != FILO))
	return;

    LinkEntry		*newEntry = new LinkEntry (pentry);

    if (_last)
    {
	newEntry->next = _last->next;
	_last->next = newEntry;
    }
}

/*------------------------------------------------------------------------
 * LinkList::lastFlag (void)
 *------------------------------------------------------------------------*/

int LinkList::lastFlag (void)
{
    if (_last)
	return _last->flag ();

    return 0;
}

/*------------------------------------------------------------------------
 * LinkList::setLastFlag (int)
 *------------------------------------------------------------------------*/

void LinkList::setLastFlag (int v)
{
    if (_last)
	_last->setFlag (v);
}
