/*---*- C++ -*------------------------------------------------------------
 * $Id: linkList.h,v 2.7 1994/01/18 03:36:47 carlson Exp $
 *
 * Header file for the linked list class.
 *
 * Revision History:
 *	$Log: linkList.h,v $
 * Revision 2.7  1994/01/18  03:36:47  carlson
 * Had to comment out 'const char *' conversion because it is used by
 *   VkComponent and causes a conflict.
 * Made many routines virtual so they can be redefined by subclasses.
 * Added prototypes for methods to manipulate LinkEntry flags.
 *
 * Revision 2.6  1994/01/17  20:32:27  carlson
 * Put back in const char * type conversion method.
 * Moved LinkEntry class to its own file.
 * Changed all methods that returned LinkEntry pointers to return LinkData
 *   pointers and all methods that took LinkEntry pointers as parameters to
 *   expect LinkData pointers.
 * Added a method 'last' which required changing the name of the variable
 *   'last' to '_last'.
 * Added methods to remove entries.
 * Removed the private method to delete an entry.
 *
 * Revision 2.5  1993/11/05  19:42:34  carlson
 * Added prototype for 'remove' method.
 * Changed some functions to be virtual.
 *
 * Revision 2.4  1993/10/19  22:46:50  carlson
 * Added and modified code to allow LinkList class to be ordered either
 *   ascending, descending, FIFO or FILO.
 * Also formatted things a little better.
 *
 * Revision 2.3  1993/10/06  00:17:45  chris
 * Added a flag to the class that allows us to mark LinkEntry's.
 *
 * Revision 2.2  1993/06/30  20:00:53  chris
 * Had to comment out the conversion methods because it conflicted with
 *   something in VkComponent up a ways.
 * Added a method which returns the key value.
 *
 * Revision 2.1  1993/06/30  04:34:33  chris
 * Updated to have new format.
 *
 * Revision 1.1  1993/05/05  15:48:38  chris
 * Initial revision
 *
 *------------------------------------------------------------------------*/

#ifndef _LINKLIST_H_
#define _LINKLIST_H_

class LinkEntry;

/*------------------------------------------------------------------------
 * class LinkData
 *------------------------------------------------------------------------*/

class LinkData
{
friend LinkEntry;
  public:
    LinkData () { key = 0; }
    LinkData (const char *tag);
    ~LinkData () { if (key) delete [] key; }

//    operator		const char *() { return key; }
    const char*		theKey (void) { return key; }
    virtual LinkData&	operator= (char *tag);

  protected:
    void		setKey (char *tag);

  private:
    char		*key;
};

#define ENULL	((LinkEntry *)0)

/*------------------------------------------------------------------------
 * class LinkList
 *------------------------------------------------------------------------*/

class LinkList
{
  public:
    enum ORDERING { ASCENDING, DESCENDING, FIFO, FILO };

    LinkList () { top = _last = ENULL; ordering = ASCENDING; }
    ~LinkList ();

    virtual void	add (LinkData *pentry);
    virtual LinkData*	find (char *key);
    virtual LinkData*	find (const char *key);
    virtual LinkData*	find (LinkData *data);
    virtual LinkData*	last (void);
    virtual LinkData*	next (void);
    virtual LinkData*	first (void);
    virtual void	clean (void);
    void		setOrdering (enum ORDERING);
    virtual void	addBefore (LinkData *pentry);
    virtual void	addAfter (LinkData *pentry);
    virtual void	removeEntry (void);
    virtual void	removeEntry (LinkData *d);
    int			lastFlag (void);
    void		setLastFlag (int);

  protected:
    LinkEntry		*top;
    LinkEntry		*beforeLast;		// Element in front of ...
    LinkEntry		*_last;			// Last found entry
    enum ORDERING	ordering;
};

#endif
