/*----------------------------------------------------------------------
 * $Id: mainWindow.c++,v 2.24 1994/02/17 21:06:06 chrisc Exp $
 *
 * Methods for creating main window.
 *
 * Revision History:
 *	$Log: mainWindow.c++,v $
// Revision 2.24  1994/02/17  21:06:06  chrisc
// Added help callback for the radio buttons.
//
// Revision 2.23  1994/02/17  04:47:25  chrisc
// Filled in save callback so it now works.
// Filled in delete callback so it now works.
// Added methods to support undo but couldn't figure out how to make it work.
//
// Revision 2.22  1994/02/16  23:57:37  chrisc
// Removed code for checking password file that isn't used anymore.
// Changed the name for the outer form to something different than the outer
//   shell widget.
// Changed the names of all the menus and menu options so their labelStrings
//   are now defined in the resource file.
// Added debug code (that is commented out) to allow for finding out what
//   the names of various widgets and their parents are.
//
// Revision 2.21  1994/02/16  22:26:41  chrisc
// Changed the reference to theApplication to use uucpApplication.
// Added callback methods to support the "Open file" button on the new
//   main window.
// Decided to put a form around the widgets on the left.  This will allow
//   widgets inside to change size (based on resource settings), without
//   having to change which widget the right stuff is supposed to line up
//   with.
// Added a label gadget to provide some startup help.
// Added callbacks for the new buttons.
// Changed the titles of the radio buttons so that they can be referenced
//   through resources.
//
// Revision 2.20  1994/02/15  05:30:37  chrisc
// Initial cut at making this a better looking window.  Probably cut out
//   instantiating PackageWindow all together.
//
// Revision 2.19  1994/02/15  04:28:59  chrisc
// Changed all references to myApplication to refer to uucpApplication.
// Support dialcodes from setup pulldown menu.
// Removed code to compute where the UUCP files should be and get the info
//   from uucpApplication resource.
//
// Revision 2.18  1994/02/15  01:16:02  chrisc
// Fixed Dialcodes setup pulldown menu item to work.
//
// Revision 2.17  1994/02/10  04:19:07  chrisc
// Fixed a bug that would have shown up later where I should use myApplication
//   and not theApplication.
//
// Revision 2.16  1994/02/08  20:55:13  chrisc
// Changed the prompt dialog used to get the name of a new package with the
//   selection dialog.  This allowed me to list the default package names
//   and allow the user to select one of them or something different.
// Added setup menu option for slip.  Also added methods to support it.
// Modified makeNewPackage to keep a list of all packages created.  This is
//   currently used by the "New package" menu to determine what default
//   names to list and to make sure a duplicate is not created.
//
// Revision 2.15  1994/01/21  01:45:48  carlson
// Now that I've learned how this works, I cleaned up a lot of code that
//   just looked messy or was confusing.
// Removed separators in pulldown menus.
//
// Revision 2.14  1994/01/20  20:17:37  carlson
// Modified to support new kind of help facility.
// Added support for help "On Window".
//
// Revision 2.13  1994/01/18  05:41:54  carlson
// Removed all references to LinkEntry since it no longer is required by
//   the LinkList class.
//
// Revision 2.12  1994/01/11  22:35:06  carlson
// No longer expect the modemsWindow to hang around.  When it is done,
//   it deletes itself.
//
// Revision 2.11  1994/01/05  02:37:08  carlson
// Added code to support modem setup.
//
// Revision 2.10  1993/12/30  19:42:14  carlson
// Removed some commented out print statements.
//
// Revision 2.9  1993/12/29  06:30:42  carlson
// Added help menu option to display a tutorial.
//
// Revision 2.8  1993/11/17  02:20:16  carlson
// Added callback methods for modifying the inittab file.
//
// Revision 2.7  1993/11/04  01:06:26  carlson
// Added help capabilities to buttons and labels.
// Deactivated menu items that aren't finished yet.
//
// Revision 2.6  1993/10/07  23:33:45  carlson
// Modified call to postAndWait to check the return status.  If it is "OK",
//   then go ahead and process the new package.
//
// Revision 2.5  1993/10/07  22:53:28  carlson
// Added code to determine the directory path to use for files.  It should
//   figure out what the path is from the password file.
// Need to pass this path down to the children.
//
// Revision 2.4  1993/10/07  20:01:00  chris
// Added functionality to "new" menu button.
// Added new menu button for "Setup".
// Added cascade menu for setting up dialcodes, permissions and modems.
// Modified shell widget to allow resize.
// Removed commented out stuff.
//
// Revision 2.3  1993/07/29  20:39:35  chris
// Added code to provide a button that would display some kind of debug
//   information.  This button only appears if the DEBUG symbol is defined
//   while compiling.  The concept is that if the function pointer some_func
//   is non-null, it calls it as a function when the button is selected.
//   After calling the function, it prints the contents of some_address
//   and some_int.
//
// Revision 2.2  1993/07/28  18:37:04  chris
// Pass defaults package when creating PackageWindow.
//
// Revision 2.1  1993/07/01  01:11:53  chris
// Modified to use new classes.
//
// Revision 1.5  1993/06/22  22:38:49  chris
// Added some documentation to callback routines/methods.
// Row/column is now changeable in size but only has two columns.
// We no longer want to pass the main window class to sysFWindows.
//
// Revision 1.4  1993/05/13  04:45:29  chris
// Added some print statements for testing.
// Added select function which activates menu items.
// Removed unused variable i.
//
// Revision 1.3  1993/05/12  17:37:50  chris
// Removed method for selecting the current subwindow.
// No longer create instances of sysFWindow class for each entry in the
//   Sysfiles file.  Now, just make one instance of sysFWindows.
//
// Revision 1.2  1993/05/10  18:58:20  chris
// Modified to build all the children, rather than have main program do it.
// Added callbacks for all the menu items.
// Improved menu items to be more accurate.
// Added select function to identify the selected systems files.
// Changed main form to a row-column and set up resources.
//
// Revision 1.1  1993/03/25  06:16:24  chris
// Initial revision
//
 *----------------------------------------------------------------------*/

#include "uucp.h"
#include "mainWindow.h"
#include "Sysfiles.h"
#include "PackageWindow.h"
#include "HelpDialog.h"
#include "Tutorial.h"
#include "ModemsWindow.h"
#include "VkSelectionDialog.h"
#include "DialcodesSetup.h"
#include "Dialcodes.h"

#include <string.h>
#include <iostream.h>
#include <pwd.h>

#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>
#include <Vk/VkHelpPane.h>
#include <Vk/VkMenuBar.h>
#include <Vk/VkRadioBox.h>

#include <Xm/Form.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/SeparatoG.h>

#ifdef DEBUG
void	(*some_func)() = 0;
void	*some_address = 0;
int	some_int = 0;
#endif

/*----------------------------------------------------------------------
 * saveCallback
 *
 * Static method called by X when the "Save" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::saveCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->save_cb ();
}

/*----------------------------------------------------------------------
 * save_cb
 *
 * Method called by MainWindow::saveCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::save_cb ()
{
    save ();
}

/*----------------------------------------------------------------------
 * saveAllCallback
 *
 * Static method called by X when the "SaveAll" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::saveAllCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->saveAll_cb ();
}

/*----------------------------------------------------------------------
 * saveAll_cb
 *
 * Method called by MainWindow::saveAllCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::saveAll_cb ()
{
    cout << "saveAll_cb callback" << endl;
}

/*----------------------------------------------------------------------
 * printCallback
 *
 * Static method called by X when the "Print" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::printCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->print_cb ();
}

/*----------------------------------------------------------------------
 * print_cb
 *
 * Method called by MainWindow::printCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::print_cb ()
{
    cout << "print_cb callback" << "\n";
}

/*----------------------------------------------------------------------
 * newCallback
 *
 * Static method called by X when the "New" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::newCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->new_cb ();
}

/*----------------------------------------------------------------------
 * new_cb
 *
 * Method called by MainWindow::newCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::new_cb ()
{
    const char		*pkgName;
    char		line[50], *pkgList[4];
    int			n;

    n = 0;
    if (!packages.find ("uucico"))
	pkgList[n++] = "uucico";
    if (!packages.find ("cu"))
	pkgList[n++] = "cu";
    if (!packages.find ("uux"))
	pkgList[n++] = "uux";
    pkgList[n] = NULL;

    theSelectionDialog->setTitle ("New package");
    theSelectionDialog->setList (pkgList);
    theSelectionDialog->mustMatch (False);

    if (theSelectionDialog->postAndWait ("Enter the package name:") ==
	VkDialogManager::OK)
    {
	pkgName = theSelectionDialog->getSelection ();
	if (pkgName == NULL || strlen (pkgName) == 0)
	    return;
	if (packages.find (pkgName))
	{
	    theErrorDialog->postAndWait ("There is already a package of "
					 "that name.");
	    return;
	}
	strcpy (line, "service=");
	strcat (line, pkgName);
	makeNewPackage (line);
    }
}

/*----------------------------------------------------------------------
 * deleteCallback
 *
 * Static method called by X when the "Delete" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::deleteCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->delete_cb ();
}

/*----------------------------------------------------------------------
 * delete_cb
 *
 * Method called by MainWindow::deleteCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::delete_cb ()
{
    int			*position_list, position_count;
    XmString		*selectedItems;
    char		*text = NULL, line[80];
    PackageWindow	*pw;

#ifdef DEBUG1
    cout << "delete_cb callback" << endl;
#endif

    if (XmListGetSelectedPos (pkg_list, &position_list, &position_count))
    {
	XtVaGetValues (pkg_list, XmNselectedItems, &selectedItems, NULL);
	XmStringGetLtoR (selectedItems[0], XmFONTLIST_DEFAULT_TAG, &text);
	XFree (position_list);
    }

    if (!text)
    {
	theErrorDialog->postAndWait ("You need to select a package to "
				     "delete.");
	return;
    }

#ifdef DEBUG1
    cout << "  Package selected = " << text << endl;
#endif

    pw = (PackageWindow *)packages.find (text);
    if (!pw)
    {
	theErrorDialog->postAndWait ("Can't find selected package.");
	return;
    }

    if (strcmp (text, "default") == 0)
    {
	theErrorDialog->postAndWait ("You can't delete the default package.");
	return;
    }

    strcpy (line, "Are you sure you want\nto delete package ");
    strcat (line, text);
    if (theQuestionDialog->postAndWait (line) == VkDialogManager::OK)
    {
	packages.removeEntry (pw);
	delete pw;
	XmListDeleteItem (pkg_list, selectedItems[0]);
    }
}

/*----------------------------------------------------------------------
 * undoDeleteCallback
 *
 * Static method called by X when the "UndoDelete" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::undoDeleteCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->undoDelete_cb ();
}

/*----------------------------------------------------------------------
 * undoDelete_cb
 *
 * Method called by MainWindow::undoDeleteCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::undoDelete_cb ()
{
    cout << "undoDelete_cb callback" << endl;
}

#ifdef DEBUG
/*----------------------------------------------------------------------
 * debugCallback
 *
 * Static method called by X when the "?" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::debugCallback (Widget w,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->debug_cb (XtName (w));
}

/*----------------------------------------------------------------------
 * debug_cb
 *
 * Method called by MainWindow::?Callback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::debug_cb (char *name)
{
    cout << "debug_cb callback from " << name << endl;

    cout << endl;
    if (some_func)
    {
	cout << "debug: Executing some_func" << endl;
	(*some_func)();
    }
    cout << "debug: Contents of some_address = " << some_address << endl;
    cout << "debug: Contents of some_int = " << some_int << endl;
}
#endif

/*----------------------------------------------------------------------
 * quitCallback
 *
 * Static method called by X when the "Quit" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::quitCallback (Widget,
			       XtPointer,
			       XtPointer)
{
    uucpApplication->quitYourself ();
}

/*----------------------------------------------------------------------
 * selectCallback
 *
 * Static method called by X when the "Select" menu button is selected.
 * "this" pointer must be computed from clientData.
 *----------------------------------------------------------------------*/

void MainWindow::selectCallback (Widget,
				 XtPointer clientData,
				 XtPointer)
{
    MainWindow *obj = (MainWindow *) clientData;
    obj->select_cb ();
}

/*----------------------------------------------------------------------
 * select_cb
 *
 * Method called by MainWindow::selectCallback.  "this" pointer
 * exists for the main window class.
 *----------------------------------------------------------------------*/

void MainWindow::select_cb ()
{
    VkRadioBox		*radioBox;
    int			choice;
    int			*position_list, position_count;
    XmString		*selectedItems;
    char		*text = NULL;
    PackageWindow	*pw;

#ifdef DEBUG1
    cout << "select_cb callback" << endl;
#endif

    radioBox = (VkRadioBox *)fileChoice;
    for (choice = 0; choice < 3; choice++)
	if (radioBox->getValue (choice))
	    break;

    if (choice >= 3)
    {
	theErrorDialog->postAndWait ("You need to select a file to open.");
	return;
    }

#ifdef DEBUG1
    cout << "  File selected = " << choice << endl;
#endif

    if (XmListGetSelectedPos (pkg_list, &position_list, &position_count))
    {
	XtVaGetValues (pkg_list, XmNselectedItems, &selectedItems, NULL);
	XmStringGetLtoR (selectedItems[0], XmFONTLIST_DEFAULT_TAG, &text);
	XFree (position_list);
    }

    if (!text)
    {
	theErrorDialog->postAndWait ("You need to select a package first.");
	return;
    }

#ifdef DEBUG1
    cout << "  Package selected = " << text << endl;
#endif

    pw = (PackageWindow *)packages.find (text);
    if (!pw)
    {
	theErrorDialog->postAndWait ("Can't find selected package.");
	return;
    }

    switch (choice)
    {
      case 0:
	pw->openDevices ();
	break;

      case 1:
	pw->openSystems ();
	break;

      case 2:
	pw->openDialers ();
	break;
    }
}

/*------------------------------------------------------------------------
 * MainWindow::dialcodesCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::dialcodesCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->dialcodes_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::dialcodes_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::dialcodes_cb ()
{
    dialcodes->show ();
    dialcodes->raise ();
}

/*------------------------------------------------------------------------
 * MainWindow::permissionsCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::permissionsCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->permissions_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::permissions_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::permissions_cb ()
{
    cout << "permissions_cb callback" << endl;
}

/*------------------------------------------------------------------------
 * MainWindow::modemsCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::modemsCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->modems_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::modems_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::modems_cb ()
{
    modemsWindow = new ModemsWindow ("Modems");
    modemsWindow->show ();
}

/*------------------------------------------------------------------------
 * MainWindow::inittabCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::inittabCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->inittab_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::inittab_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::inittab_cb ()
{
    cout << "inittab_cb callback" << endl;
}

/*------------------------------------------------------------------------
 * MainWindow::slipCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::slipCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->slip_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::slip_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::slip_cb ()
{
    cout << "slip_cb callback" << endl;
}

/*------------------------------------------------------------------------
 * MainWindow::onWindowCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::onWindowCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow		*mw;

    mw = (MainWindow *)clientData;
    mw->onWindow_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::onWindow_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::onWindow_cb ()
{
    theHelpDialog->displayHelp ("mainWindowOW");
}

/*------------------------------------------------------------------------
 * MainWindow::tutorialCallback (Widget, XtPointer, XtPointer)
 *------------------------------------------------------------------------*/

void MainWindow::tutorialCallback (Widget, XtPointer clientData, XtPointer)
{
    MainWindow *obj = (MainWindow *)clientData;
    obj->tutorial_cb ();
}

/*------------------------------------------------------------------------
 * MainWindow::tutorial_cb ()
 *------------------------------------------------------------------------*/

void MainWindow::tutorial_cb ()
{
    if (!tutorial)
	tutorial = new Tutorial ("Tutorial");

    tutorial->show ();
}

/*------------------------------------------------------------------------
 * MainWindow::makeNewPackage (const char *line)
 *------------------------------------------------------------------------*/

LinkData* MainWindow::makeNewPackage (const char *line)
{
    PackageWindow	*p;
    XmString		xmstr;

    p = new PackageWindow (_baseWidget, line, all_files, defaults, &dirPath);
    xmstr = XmStringCreateSimple ((char *)p->theKey ());
    XmListAddItemUnselected (pkg_list, xmstr, 0);
    XmStringFree (xmstr);
    packages.add (p);

    return p;
}

/*----------------------------------------------------------------------
 * setUpInterface
 *
 * This method is called when the interface for the class needs to
 * be set up.
 *----------------------------------------------------------------------*/

Widget MainWindow::setUpInterface (Widget parent)
{
    Arg			wargs[10];
    int			n;
    Widget		pd_button, form, label, temp, sep, form1;
    VkMenuBar		*menubar;
    VkHelpPane		*helpMenu;
    PathName		dialcodeName;
    VkRadioBox		*radioBox;

    tutorial = NULL;

    dirPath = uucpApplication->uucpDirectory ();

    dialcodeName = dirPath;
    dialcodeName += "Dialcodes";
    dialcodes = new DialcodesSetup (dialcodeName);

    uucpApplication->setDialcodes (dialcodes->dialcodes ());

    /*----
     * Create outer form to keep all the gadgets in.
     *----*/

    n = 0;
    form = XmCreateForm (parent, "selectPackage", wargs, n);

    /*----
     * Create a smaller form for the stuff on the left side
     * so we can have the items inside it change size or be
     * of different sizes but the stuff on the right won't
     * overlap anything.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    form1 = XmCreateForm (form, "packageForm", wargs, n);
    XtManageChild (form1);

    XtAddCallback (form1, XmNhelpCallback, aHelpCallback, "mmb_pkg_help");

    /*----
     * Create the label and package list.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form1, "packages", wargs, n);
    XtManageChild (label);

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, label); n++;
    XtSetArg (wargs[n], XmNlistSizePolicy, XmVARIABLE); n++;
    XtSetArg (wargs[n], XmNselectionPolicy, XmSINGLE_SELECT); n++;
    XtSetArg (wargs[n], XmNvisibleItemCount, 4); n++;
    pkg_list = XmCreateScrolledList (form1, "packageList", wargs, n);
    XtManageChild (pkg_list);

    /*----
     * Create buttons that make new packages or delete packages.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    pd_button = XmCreatePushButton (form1, "new", wargs, n);
    XtManageChild (pd_button);

    XtAddCallback (pd_button, XmNhelpCallback, aHelpCallback, "mmb_new");
    XtAddCallback (pd_button, XmNactivateCallback, newCallback, this);

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, pd_button); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    pd_button = XmCreatePushButton (form1, "delete", wargs, n);
    XtManageChild (pd_button);

    XtAddCallback (pd_button, XmNhelpCallback, aHelpCallback, "mmb_delete");
    XtAddCallback (pd_button, XmNactivateCallback, deleteCallback, this);

    /*----
     * Create a label gadget to tell the user what to do.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, label); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, pkg_list); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    label = XmCreateLabelGadget (form1, "explain", wargs, n);
    XtManageChild (label);

    /*----
     * Create a separator to separate the left side from
     * the right side stuff.
     *----*/

    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, form1); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNbottomAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNorientation, XmVERTICAL); n++;
    sep = XmCreateSeparatorGadget (form, "sep1", wargs, n);
    XtManageChild (sep);

    /*----
     * Create a radio box to select the file.
     *----*/

    radioBox = new VkRadioBox ("fileType", form);
    fileChoice = radioBox;
    temp = radioBox->baseWidget ();
    n = 0;
    XtSetArg (wargs[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNleftWidget, sep); n++;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_FORM); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    XtSetValues (temp, wargs, n);

    XtAddCallback (temp, XmNhelpCallback, aHelpCallback, "mmb_file_type");

    radioBox->addItem ("devices");
    radioBox->addItem ("systems");
    radioBox->addItem ("dialers");

    radioBox->manage ();

    n = 0;
    XtSetArg (wargs[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
    XtSetArg (wargs[n], XmNtopWidget, temp); n++;
    XtSetArg (wargs[n], XmNrightAttachment, XmATTACH_FORM); n++;
    pd_button = XmCreatePushButton (form, "select", wargs, n);
    XtManageChild (pd_button);

    XtAddCallback (pd_button, XmNhelpCallback, aHelpCallback, "mmb_select");
    XtAddCallback (pd_button, XmNactivateCallback, selectCallback, this);

    /*----
     * Add a menu pane
     *----*/

    fileMenuPane = addMenuPane ("files");
    pd_button = fileMenuPane->baseWidget ();
    XtAddCallback (pd_button, XmNhelpCallback, aHelpCallback, "mmb_file_help");

#ifdef DEBUG2
    cout << "Name of menu bar is " << (char *)XtName (pd_button) << endl;
    cout << "  parent is " << (char *)XtName (XtParent (pd_button)) << endl;
    cout << "  parent is " <<
	(char *)XtName (XtParent (XtParent (pd_button))) << endl;
#endif

    fileMenuPane->addAction ("save", saveCallback, this);
    fileMenuPane->addAction ("saveAll", saveAllCallback, this);
    fileMenuPane->addAction ("print", printCallback, this);
    fileMenuPane->addSeparator ();
    fileMenuPane->addAction ("new", newCallback, this);
    fileMenuPane->addAction ("delete", deleteCallback, this);
    fileMenuPane->addSeparator ();
#ifdef DEBUG
    fileMenuPane->addAction ("debug", debugCallback, this);
    fileMenuPane->addSeparator ();
#endif
    fileMenuPane->addAction ("quit", quitCallback, this);

#ifdef DEBUG2
    VkMenuItem *mi = fileMenuPane->findNamedItem ("Print");
    temp = mi->baseWidget ();
    cout << "MenuItem name is " << (char *)XtName (temp) << endl;
    cout << "   parent is " << (char *)XtName (XtParent (temp)) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (temp))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (temp)))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (XtParent (temp))))) << endl;
    cout << "   Files widget = " << pd_button << endl;
    cout << "   parent of mi = " << XtParent (temp) << endl;
    cout << "   parent of mi = " << XtParent (XtParent (temp)) << endl;
#endif

    // Add a menu second pane
    
    setupMenuPane = addMenuPane ("setup");
    pd_button = setupMenuPane->baseWidget ();
    XtAddCallback (pd_button, XmNhelpCallback, aHelpCallback,
		   "mmb_setup_help");

    setupMenuPane->addAction ("dialcodes", dialcodesCallback, this);
    setupMenuPane->addAction ("permissions", permissionsCallback, this);
    setupMenuPane->addAction ("modems", modemsCallback, this);
    setupMenuPane->addAction ("inittab", inittabCallback, this);
    setupMenuPane->addAction ("slip", slipCallback, this);

#ifdef DEBUG2
    VkMenuItem *mi = setupMenuPane->findNamedItem ("Slip");
    temp = mi->baseWidget ();
    cout << "MenuItem name is " << (char *)XtName (temp) << endl;
    cout << "   parent is " << (char *)XtName (XtParent (temp)) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (temp))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (temp)))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (XtParent (temp))))) << endl;
#endif

    // Add an item to the Help menu

    menubar = menu ();
    helpMenu = menubar->helpPane ();
    helpMenu->addSeparator ();
    helpMenu->addAction ("window", onWindowCallback, this);
    helpMenu->addAction ("tutorial", tutorialCallback, this);

#ifdef DEBUG2
    VkMenuItem *mi = helpMenu->findNamedItem ("Tutorial");
    temp = mi->baseWidget ();
    cout << "MenuItem name is " << (char *)XtName (temp) << endl;
    cout << "   parent is " << (char *)XtName (XtParent (temp)) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (temp))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (temp)))) << endl;
    cout << "   parent is " <<
	(char *)XtName (XtParent (XtParent (XtParent (XtParent (temp))))) << endl;
#endif

    PathName sysfile_name(dirPath);
    sysfile_name += "Sysfiles";
    create (sysfile_name);

    /*----
     * Deactivate those menu items that aren\'t working yet.
     *----*/

    fileMenuPane->deactivateItem ("saveAll");
    fileMenuPane->deactivateItem ("print");

    setupMenuPane->deactivateItem ("permissions");
    setupMenuPane->deactivateItem ("inittab");
    setupMenuPane->deactivateItem ("slip");

    return form;
}
