/*---*- C++ -*------------------------------------------------------------
 * $Id: mainWindow.h,v 2.16 1994/02/17 21:07:02 chrisc Exp $
 *
 * Header file for main window class.
 *
 * Revision History:
 *	$Log: mainWindow.h,v $
 * Revision 2.16  1994/02/17  21:07:02  chrisc
 * Just aligned things differently so that it matches my style.
 *
 * Revision 2.15  1994/02/17  04:48:47  chrisc
 * Added prototypes for undo callback methods.
 *
 * Revision 2.14  1994/02/16  22:30:00  chrisc
 * Added pointer to remember the component used for the radio box.
 * Added prototypes for the select callback.
 *
 * Revision 2.13  1994/02/15  05:31:34  chrisc
 * Initial cut at making this a better looking window.
 *
 * Revision 2.12  1994/02/15  01:16:35  chrisc
 * Added item to hold the current window showing dialcode setup.
 *
 * Revision 2.11  1994/02/08  20:58:02  chrisc
 * Added list for packages created to be kept in.
 * Added prototypes for callback methods to support slip setup.
 *
 * Revision 2.10  1994/01/19  20:53:10  carlson
 * Added callback methods for help "On Window".
 *
 * Revision 2.9  1994/01/17  23:57:57  carlson
 * Removed all references to LinkEntry since it no longer is required by
 *   the LinkList class.
 *
 * Revision 2.8  1994/01/05  02:31:30  carlson
 * Added a variable to hold modemswindow class.
 *
 * Revision 2.7  1993/12/29  06:30:42  carlson
 * Added callbacks for Tutorial help menu option.
 * Added location to keep tutorial class when it is created.
 *
 * Revision 2.6  1993/11/17  02:17:09  carlson
 * Added prototypes for methods to handle updating the inittab file.
 *
 * Revision 2.5  1993/11/12  00:32:46  carlson
 * Removed definition for dirPath, since it already exists in SysFile.
 *
 * Revision 2.4  1993/10/07  21:58:24  carlson
 * Added a variable to maintain the directory path.
 *
 * Revision 2.3  1993/10/07  20:03:37  chris
 * Added declarations for callbacks to new menus.
 *
 * Revision 2.2  1993/07/29  20:16:15  chris
 * Changed the sample callback functions to debug callback functions.
 *
 * Revision 2.1  1993/07/01  00:09:57  chris
 * Modified to use new classes.
 *
 * Revision 1.4  1993/05/13  04:35:49  chris
 * Made it so select method is not inline.
 *
 * Revision 1.3  1993/05/12  17:39:22  chris
 * Created a new class sysFWindows which covers all the sysFWindow
 *   instances.
 * Modified select to just set the current sysFWindow to the value passed.
 *
 * Revision 1.2  1993/05/10  18:58:20  chris
 * Fixed so it automatically comes up in c++-mode for Emacs.
 * Define separate callbacks for all menu items.
 * Define select function to select the current window.
 *
 * Revision 1.1  1993/03/25  06:16:23  chris
 * Initial revision
 *
 *----------------------------------------------------------------------*/

#include <Vk/VkWindow.h>
#include <Vk/VkMenu.h>
#include <Vk/VkSubMenu.h>

#include "Sysfiles.h"

class Tutorial;
class ModemsWindow;
class DialcodesSetup;

/*----------------------------------------------------------------------
 *  CCC   L       AAA    SSS    SSS   EEEEE   SSS
 * C   C  L      A   A  S   S  S   S  E      S   S
 * C      L      A   A  S      S      E      S
 * C      L      AAAAA   SSS    SSS   EEE     SSS
 * C      L      A   A      S      S  E          S
 * C   C  L      A   A  S   S  S   S  E      S   S
 *  CCC   LLLLL  A   A   SSS    SSS   EEEEE   SSS
 *----------------------------------------------------------------------*/

class MainWindow : public SysFile, public VkWindow
{
  public:
    MainWindow (const char *name) : SysFile (), VkWindow (name) {}

    virtual const char	*className () { return "MainWindow"; }

  protected:
    virtual Widget	setUpInterface (Widget parent);
    virtual LinkData*	makeNewPackage (const char *line);

  private:
    Widget		pkg_list;
    VkSubMenu		*fileMenuPane;
    VkSubMenu		*setupMenuPane;
    Tutorial		*tutorial;
    ModemsWindow	*modemsWindow;
    SmartList		packages;
    DialcodesSetup	*dialcodes;
    VkComponent		*fileChoice;

    static void		saveCallback (Widget, XtPointer, XtPointer);
    static void		saveAllCallback (Widget, XtPointer, XtPointer);
    static void		printCallback (Widget, XtPointer, XtPointer);
    static void		newCallback (Widget, XtPointer, XtPointer);
    static void		deleteCallback (Widget, XtPointer, XtPointer);
    static void		quitCallback (Widget, XtPointer, XtPointer);
    static void		selectCallback (Widget, XtPointer, XtPointer);
#ifdef DEBUG
    static void		debugCallback (Widget, XtPointer, XtPointer);
#endif
    static void		undoDeleteCallback (Widget, XtPointer, XtPointer);

    static void		dialcodesCallback (Widget, XtPointer, XtPointer);
    static void		permissionsCallback (Widget, XtPointer, XtPointer);
    static void		modemsCallback (Widget, XtPointer, XtPointer);
    static void		inittabCallback (Widget, XtPointer, XtPointer);
    static void		slipCallback (Widget, XtPointer, XtPointer);

    static void		tutorialCallback (Widget, XtPointer, XtPointer);
    static void		onWindowCallback (Widget, XtPointer, XtPointer);

    void		save_cb ();
    void		saveAll_cb ();
    void		print_cb ();
    void		new_cb ();
    void		delete_cb ();
    void		select_cb ();
#ifdef DEBUG
    void		debug_cb (char *);
#endif
    void		undoDelete_cb ();

    void		dialcodes_cb ();
    void		permissions_cb ();
    void		modems_cb ();
    void		inittab_cb ();
    void		slip_cb ();

    void		tutorial_cb ();
    void		onWindow_cb ();
};
