/*----------------------------------------------------------------------
 * $Id: uucp.c++,v 2.13 1994/02/28 23:24:11 carlson Exp $
 *
 * Main program for the UUCP visual admin program.
 *
 * Revision History:
 *	$Log: uucp.c++,v $
// Revision 2.13  1994/02/28  23:24:11  carlson
// Removed a variable I never use.
//
// Revision 2.12  1994/02/28  21:01:03  carlson
// Had to get login name from getuid instead of getlogin because getlogin says
//   we're root after we performed the INSTALL and had the toolchest restarted.
//
// Revision 2.11  1994/02/28  20:17:31  carlson
// Found out that fgetpwent uses the same address space as getpwuid and sets up
//   the same pointers and everything.  This caused lots of problems with having
//   two password entries to compare.  Finally just copied the root password to
//   a character array and used it to compare.
//
// Revision 2.10  1994/02/28  18:29:02  carlson
// Found that this program needs root privs to be able to save files.  It can't
//   do it with just uucp privs.  So, modified the code to make sure we can
//   take advantage of root access.
//
// Revision 2.9  1994/02/25  17:28:51  carlson
// Modified security stuff to check if the user has set up his knowledge of
//   the root password with the vadmin tools.  If yes, then it doesn't
//   require the user to enter the password.
//
// Revision 2.8  1994/02/25  03:04:00  carlson
// Added a bunch of comments.
// Created a password dialog from the prompt dialog so that I can change the
//   foreground color to be the same as the background color to avoid having
//   the password show.
// Added code to make sure this app was installed right.  Must be owned by
//   uucp and have set UID bit set.
// Added code to make sure user has root access or knows it.
//
// Revision 2.7  1994/02/15  04:29:36  chrisc
// Changed all references to myApplication to refer to uucpApplication.
// Added methods to support maintaining the dialcodes file information.
//
// Revision 2.6  1994/02/10  21:31:20  chrisc
// Added resource descriptions for the UUCP directory and the help file name.
//
// Revision 2.5  1994/01/05  00:49:19  carlson
// Added resources to identify what directory contains files used by this
//   program.
// Made application a global variable so that other methods can access stuff
//   retrieved from resources.
//
// Revision 2.4  1993/12/29  00:00:04  carlson
// Moved the class definition for MyApp to uucp.h.
// Include uucp.h instead of VkApp.h (uucp.h includes VkApp.h).
// Found out that there is no resource called 'defaultScheme'.  Changed it
//   to 'scheme' but that still doesn't seem to work.  Only if it is referenced
//   from the command line will you get the scheme defined.  The resource
//   'useSchemes' works, though.
// Use class MyApp instead of VkApp so that later I can add some features.
//
// Revision 2.3  1993/11/03  17:20:18  carlson
// I tried to create a help dialog using ViewKit but it didn't work as I
//   wanted.  So then I went back and used the help dialog that I lifted
//   from vcal but I wanted to have it instantiate 'theHelpDialog' at
//   startup.  It seems that since the help dialog is a VkSimpleWindow, it
//   is clashing with VkApp's idea of the initial/main window.  So, I had
//   to instantiate the help dialog class AFTER creating the main window
//   but before we start running.
//
// Revision 2.2  1993/08/17  00:22:58  chris
// No longer include uucp.h.  I haven't decided whether I want this include
//   file or not.
// Tried creating my own subclass for VkApp so that I can set up some default
//   resources.  I wasn't successful so they are commented out.
//
// Revision 2.1  1993/06/30  21:02:29  chris
// Instead of passing a name to the MainWindow class, pass the file name.
//
// Revision 1.3  1993/05/10  18:08:46  chris
// Removed creation of SysFiles and moved to mainWindow.
//
// Revision 1.2  1993/05/05  18:42:00  chris
// Added code to build and print system files.
//
// Revision 1.1  1993/03/25  06:16:24  chris
// Initial revision
//
 *----------------------------------------------------------------------*/

#include "uucp.h"
#include "mainWindow.h"
#include "HelpDialog.h"

#ifdef DEBUG1
#include <iostream.h>
#endif
#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>
#include <string.h>
#include <crypt.h>

#include <Vk/VkErrorDialog.h>
#include <Vk/VkPromptDialog.h>

#include <Xm/TextF.h>

/*------------------------------------------------------------------------
 * An attempt at setting up default resources for this application.
 *------------------------------------------------------------------------*/

String UUCPApp::defaultRes[] = {
    "*useSchemes: True",
    "*scheme: Rembrandt",
    NULL
};

/*------------------------------------------------------------------------
 * These are supposed to be the special resources recognized by this
 * application.
 *------------------------------------------------------------------------*/

XtResource UUCPApp::_resources[] = {
{
    "verbose", "Verbose", XmRBoolean, sizeof (Boolean),
    XtOffset (UUCPApp *, _verbose), XmRString, (XtPointer) "False",
},
{
    "directory", "Directory", XmRString, sizeof (char *),
    XtOffset (UUCPApp *, _directory), XmRString,
#ifdef DEBUG
    (XtPointer) ".",
#else
    (XtPointer) "/usr/lib/vadmin/uucp",
#endif
},
{
    "tutorial", "Tutorial", XmRString, sizeof (char *),
    XtOffset (UUCPApp *, _tutorial), XmRString, "TutorialText",
},
{
    "uucpDirectory", "Directory", XmRString, sizeof (char *),
    XtOffset (UUCPApp *, _uucpDirectory), XmRString,
#ifdef DEBUG
    (XtPointer) "./test",
#else
    (XtPointer) "/usr/lib/uucp",
#endif
},
{
    "helpFile", "HelpFile", XmRString, sizeof (char *),
    XtOffset (UUCPApp *, _helpFile), XmRString, "UUCPHelpText",
},
};

/*------------------------------------------------------------------------
 * Constructor for the UUCPApp class.
 *------------------------------------------------------------------------*/

UUCPApp::UUCPApp (char			*appClassName,
		  int			*arg_c,
		  char			**arg_v,
		  XrmOptionDescRec	*optionList,
		  int			sizeOfOptionList)
: VkApp (appClassName, arg_c, arg_v, optionList, sizeOfOptionList)
{
    setDefaultResources (_baseWidget, defaultRes);
    getResources (_resources, XtNumber (_resources));
    _dialcodes = NULL;
}

/*------------------------------------------------------------------------
 * Attempt at setting up options to be recognized on the command line.
 *------------------------------------------------------------------------*/

XrmOptionDescRec options [] = {
{ "-useSchemes",    "*useSchemes",        XrmoptionIsArg,  "True" },
{ "-scheme",        "*scheme",            XrmoptionSepArg, "Rembrandt" }
};
int numOptions = XtNumber (options);

/*========================================================================
 * Password Dialog
 *========================================================================*/

class PasswordDialog : public VkPromptDialog
{
  public:
    PasswordDialog (const char * name) : VkPromptDialog (name) { }

  protected:
    virtual Widget	prepost (const char	    *message,
				 XtCallbackProc	    okCB	= NULL,
				 XtCallbackProc	    cancelCB	= NULL,
				 XtCallbackProc	    applyCB	= NULL,
				 XtPointer	    clientData	= NULL,
				 const char	    *helpString	= NULL,
				 VkSimpleWindow	    *parent	= NULL);
};

/*------------------------------------------------------------------------
 * Prepost method for PasswordDialog.  This method sets the foreground
 * to the same value as the background so the characters won't print.
 *------------------------------------------------------------------------*/

Widget PasswordDialog::prepost (const char	    *message,
				XtCallbackProc	    okCB,
				XtCallbackProc	    cancelCB,
				XtCallbackProc	    applyCB,
				XtPointer	    clientData,
				const char	    *helpString,
				VkSimpleWindow	    *parent)
{
    Widget		pw;
    WidgetList		children;
    Cardinal		nchildren;
    Pixel		bgPixel;
    int			i;

    pw = VkPromptDialog::prepost (message, okCB, cancelCB, applyCB,
				  clientData, helpString, parent);

    XtVaGetValues (pw,
		   XmNchildren, &children,
		   XmNnumChildren, &nchildren, NULL);
    for (i = 0; i < nchildren; i++)
	if (XmIsTextField (children[i]))
	    break;
    if (i == nchildren)
	uucpApplication->quitYourself ();

    XtVaGetValues (children[i],
		   XmNbackground, &bgPixel, NULL);
    XtVaSetValues (children[i], XmNforeground, bgPixel, NULL);

    return pw;
}

static PasswordDialog *thePasswordDialog;

/*======================================================================
 * M   M   AAA    III   N   N
 * MM MM  A   A    I    NN  N
 * MM MM  A   A    I    NN  N
 * M M M  AAAAA    I    N N N
 * M   M  A   A    I    N  NN
 * M   M  A   A    I    N  NN
 * M   M  A   A   III   N   N
 *======================================================================*/

UUCPApp	*uucpApplication;

main (int argc, char *argv[])
{
    uid_t		euid;
    struct passwd	*pwdent, *vpwdent;
    int			i;
    char		*password;
    FILE		*vpasswd;
    Boolean		allowed_to_run = True;
    char		r_password[50], loginName[50];

    /*----
     * Create an instance of the application and the main window.
     *----*/

    uucpApplication  = new UUCPApp ("UUCP", &argc, argv, options, numOptions);
    MainWindow	*main = new MainWindow ("Sysfiles");

#ifdef DEBUG1
    /*----
     * Debug printout for testing the resources that are set.
     *----*/

    char	*cptr;
    cptr = app->verbosity () ? "True" : "False";
    cout << "Verbosity = " << cptr << endl;
    cout << "Directory = " << app->directory () << endl;
    cout << "Tutorial = " << app->tutorial () << endl;
#endif

    /*----
     * I think the better way of doing this would be to do this in the
     * help dialog method file.  But this worked, so I'm leaving it.
     *----*/

    theHelpDialog = new HelpDialog ("Help");

    /*----
     * If we don't have root privs, might as well quit now.  Sometime
     * in the future, I'll add code to allow a user to just look at
     * the entries without being able to edit them.
     *----*/

    euid = geteuid ();
#ifndef DEBUG
    if (euid != 0)
	allowed_to_run = False;
#endif

    /*----
     * If the user is not root, find out if they know the root password.
     * If they do, let them continue.
     *----*/

    pwdent = getpwuid (0);
    strcpy (r_password, pwdent->pw_passwd);
    if (allowed_to_run && (getuid () != 0))
    {
	pwdent = getpwuid (getuid ());
	strcpy (loginName, pwdent->pw_name);
	allowed_to_run = False;		// Assume they will fail

	/*----
	 * See if the user is privileged through vadmin.
	 *----*/

	if ((vpasswd = fopen ("/etc/vadmin.passwd", "r")))
	{
	    for (vpwdent = fgetpwent (vpasswd); vpwdent;
		 vpwdent = fgetpwent (vpasswd))
	    {
		if (strcmp (loginName, vpwdent->pw_name) == 0)
		{
		    if (strcmp (r_password, vpwdent->pw_passwd) == 0)
		    {
			allowed_to_run = True;
			break;
		    }
		}
	    }
	    fclose (vpasswd);
	}

	/*----
	 * If that didn't work, have them enter the root password
	 * and see if that succeeds.
	 *----*/

	if (!allowed_to_run)
	{
	    thePasswordDialog = new PasswordDialog ("Password");
	    for (i = 0; i < 3; i++)
	    {
		if (thePasswordDialog->postAndWait ("Enter root password:") !=
		    VkDialogManager::OK)
		    break;

		password = crypt (thePasswordDialog->text (), r_password);
		if (strcmp (password, r_password) == 0)
		{
		    allowed_to_run = True;
		    break;
		}
	    }
	    delete thePasswordDialog;
	}
    }

    /*----
     * If none of these things worked, tell the user and exit.
     *----*/

    if (!allowed_to_run)
    {
	theErrorDialog->postAndWait ("You need root privileges to\n"
				     "run this program.");
	uucpApplication->quitYourself ();
	return -1;
    }

    /*----
     * Build the main window (and all subwindows).
     *----*/

    main->show ();

    uucpApplication->run ();

    return 0;
}
