/*------------------------------- uGetGfxType ---------------------*/
/*
 * Copyright (c) 1993, 1994, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that (i) the above copyright notices and this
 * permission notice appear in all copies of the software and related
 * documentation, and (ii) the name of Silicon Graphics may not be
 * used in any advertising or publicity relating to the software
 * without the specific, prior written permission of Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include "getgfxtype.h"

long _GfxType = -1;
/* only works for local graphics */
long uGetGfxType(void)
{
    long gfxhw = 0;

    if (_GfxType >= 0)
	return _GfxType;
    else
    {
	inventory_t *next = (inventory_t *)-1;
	unsigned long mem;
	/***static long envGfxType = -1;***/
	long foundGfx = 0;
	_GfxType = 0;

#ifdef not_used
	if (envGfxType == -1)
	{
	    char *tmp;
	    tmp = getenv("PFGFXTYPE");
	    if (tmp)
	    {
		envGfxType = strtol(tmp, NULL, 0);
#ifdef NOTIFIER
		pfNotify(PFNFY_NOTICE, PFNFY_PRINT, 
			 "Environment variable PFGFXTYPE overriding to 0x%x\n",
			 envGfxType);
#endif
	    }
	    else
		envGfxType = -2;
	}
#endif /* not_used */
	
	while (next)
	{
	    next = getinvent();
	    if (!next)
		break;
	    
	    switch(next->INV_CLASS)
	    {
	    case INV_MEMORY:
		switch (next->INV_TYPE)
		{
		case INV_MAIN:
		    mem = next->INV_STATE/(1024*1024);
		    _GfxType |= (mem<<PFGFX_MEM_SHIFT);
		    break;
		default:
		    break;
		}
		break;
	    case INV_GRAPHICS:
		gfxhw = next->INV_TYPE;
		switch (next->INV_TYPE)
		{
		case INV_RE:
		case INV_VTX:
		    _GfxType |= (((getgdesc(GD_MULTISAMPLE) == TRUE) ?
				    PFGFX_MSAMPLE : 0) | 
				   PFGFX_SPLINE_FOG |
				   PFGFX_USE_MM_TRILINEAR |
#ifndef IRIX4
				   PFGFX_IS_TEX_LOADED |
				   PFGFX_MS_LPOINT |
#endif
				   PFGFX_DETAIL_TEX |
				   PFGFX_AFUNC_ALL |
				   PFGFX_VSYNC | 
				   PFGFX_FAST_TEX |
			   /* XXX is displacepolygon in pre-405H VENICE? */
				   PFGFX_DISPLACE |
				   PFGFX_USE_DISPLACE |
				   PFGFX_GANGDRAW |
				   PFGFX_PIPE_TIMES);
		    foundGfx = 1;
		    break;
		case INV_VGX:
		    _GfxType |= (PFGFX_VSYNC | 
				   PFGFX_ZWRAP_BUG |
				   PFGFX_FAST_TEX);
		    foundGfx = 1;
		    break;
		case INV_VGXT:
		    _GfxType |= (PFGFX_VSYNC | 
				   PFGFX_ZWRAP_BUG |
				   PFGFX_AFUNC_ALL |
				   PFGFX_FAST_TEX);
		    if (getgdesc(GD_MUXPIPES) > 0)
			_GfxType |= PFGFX_SKYWRITER;
		    foundGfx = 1;
		    break;
		case INV_GR1BOARD:
		    _GfxType |= PFGFX_HPC_33NS;
		    foundGfx = 1;
		    break;
		case INV_GRODEV:
		case INV_GMDEV:
		    _GfxType = 0;
		    foundGfx = 1;
		    break;
		case INV_LIGHT: /* Inidgo starter */
		    _GfxType |= PFGFX_HPC_30NS;
		    foundGfx = 1;
		    break;
		case INV_GR2:   /* Express */
		    switch (next->INV_STATE)
		    {
		    case INV_GU1_EXTREME:
			_GfxType |= (PFGFX_HPC_40NS |
				       /* XXX does it? PFGFX_DISPLACE | */
				       PFGFX_VSYNC);
			foundGfx = 1;
			break;
		    case INV_GR2_XS:
		    case INV_GR2_XSZ:
		    case INV_GR2_XS24:
		    case INV_GR2_XSM:
		    case INV_GR2_XS24Z:
		    case INV_GR2_ELAN:
		    case INV_GR3_XSM:
		    case INV_GR3_ELAN:
		    default:
			_GfxType |= (PFGFX_HPC_40NS |
				       /* XXX does it? PFGFX_DISPLACE | */
				       /* VSYNC in IRIX 5.0 and 4.0.5H */
#ifdef IRIX4			
				       0);
#else
				       PFGFX_VSYNC);
#endif
			foundGfx = 1;

			break;
		    }
		    break;
		case INV_NEWPORT: /* Indy */
		    foundGfx = 1;
		    switch (next->INV_STATE)
		    {
			case INV_NEWPORT_XL: /* Indigo2 XL model */
			case INV_NEWPORT_24: /* board has 24 bitplanes */
			default:
			    break;
		    }
		    break;
		default:
		    break;
		}
	    }
	}
	if (!foundGfx)
	{
	    _GfxType |= (PFGFX_VSYNC | 
			   PFGFX_ZWRAP_BUG |
			   PFGFX_FAST_TEX);
#ifdef NOTIFIER
	    pfNotify(PFNFY_WARN, PFNFY_RESOURCE,
		     "unable to determine graphics type %d.  "
		     "Default: VGXT using gfx type mode = 0x%x", 
			 gfxhw, _GfxType);
#endif
	}
    }
    return _GfxType;
}

