/*
 * Copyright (c) 1993, 1994, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that (i) the above copyright notices and this
 * permission notice appear in all copies of the software and related
 * documentation, and (ii) the name of Silicon Graphics may not be
 * used in any advertising or publicity relating to the software
 * without the specific, prior written permission of Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include "uLinMath.h"  /* nested uSmoke.h */
#include "getgfxtype.h"

 long     InitRand = 0;
 uVec3    RandVec3[MAX_RAND], smokeDir;

 long     smokeCount;
 uSmoke   **smokeList;

 uTexture *smokeTex, *fireTex;
 uTexEnv  *smokeTEnv;


/* [ 1 ] */
void uInitSmokes(void)
{
    smokeList = (uSmoke**)calloc(MAXSMOKES, sizeof(uSmoke*));

    smokeDir[0] = 0.0f;
    smokeDir[1] = 1.0f;
    smokeDir[2] = 0.0f;

    uNormalizeVec3(smokeDir);

}


/* [ 2 ] */
uSmoke* uNewSmoke(void)
{
	static uVec2 tc[4][4] = {
                {{0.0f, 0.0f},
                {1.0f, 0.0f},
                {1.0f, 1.0f},
                {0.0f, 1.0f}},

                {{1.0f, 0.0f},
                {0.0f, 0.0f},
                {0.0f, 1.0f},
                {1.0f, 1.0f}},

                {{0.0f, 1.0f},
                {1.0f, 1.0f},
                {1.0f, 0.0f},
                {0.0f, 0.0f}},

                {{0.0f, 0.0f},
                {1.0f, 0.0f},
                {1.0f, 1.0f},
                {0.0f, 1.0f}}
	}; 
    long	i;
    uSmoke 	*smoke;

    smoke = (uSmoke*)calloc(1,sizeof(uSmoke));

    smoke->radius = 1.0f;
    uSetVec3(smoke->origin, 0.0f, 0.0f, 0.0f);
    uCopyVec3(smoke->direction, smokeDir);
    smoke->speed = .1f;	
    smoke->turbulence = 1.0f;
    smoke->mode = USMOKE_STOP;
    smoke->type = USMOKE_SMOKE;
    smoke->startPuff = 0; 
    smoke->numPuffs = 0;
    smoke->prevTime = -1.0f;
    smoke->startTime = -1.0f;
    smoke->tex = NULL;
    smoke->duration = -1.0f;
    smoke->lastPuffTime = -1.;

    uSmokeType(smoke, smoke->type);

    for(i=0; i<MAX_PUFFS; i++)
    {
	long	j, r;
        smoke->puffs[i] = (uPuff*)calloc(1,sizeof(uPuff));
	if(smoke->puffs[i])
	   initPuff(smoke->puffs[i], smoke); /* 13A-1*/

	r = random() & 0x3;

	for(j=0; j<4; j++)
	{
	    uCopyVec2(smoke->puffs[i]->tcoords[j], tc[r][j]);
	    smoke->puffs[i]->tcoords[j][2] = 0.0f;
	}
    }
    if(smokeCount >= MAXSMOKES)
    {
	fprintf(stderr, "Too many smokes !!!!!!!!!!!!!! \n");
	return smokeList[smokeCount-1];
    }
    else
    {
	smokeList[smokeCount] = smoke;
	smokeCount++;
    }

    return smoke;
}

/* [ 5 ] */
void uSmokeOrigin(uSmoke *smoke, uVec3 origin, float radius)
{
    uCopyVec3(smoke->origin, origin);
    smoke->radius = radius;
}

/* [ 6 ] */
void uSmokeMode(uSmoke *smoke, long mode)
{
    smoke->mode = mode;
    if (smoke->mode == USMOKE_START)
    {
	smoke->startTime = -1.;
	smoke->lastPuffTime = -1.;
    }
}

/* [ 7 ] */
void uSmokeDuration(uSmoke *smoke, float duration)
{
    smoke->duration = duration;
}    

/* [ 8 ] */
void uSmokeDir(uSmoke *smoke, uVec3 dir) 
{
    uCopyVec3(smoke->direction, dir);
}
    
/* [ 9 ] */
void uSmokeVelocity(uSmoke *smoke, float turbulence, float speed)
{
    smoke->speed = speed;
    smoke->turbulence = turbulence;
}

/* [ 10 ] */
void uSmokeColor(uSmoke *smoke, uVec3 bgn, uVec3 end)
{
    uCopyVec3(smoke->bgnColor, bgn);
    uSubVec3(smoke->deltaColor, end, smoke->bgnColor);
}

/* [ 11 ] */
void uSmokeTex(uSmoke *smoke, uTexture *tex)
{
    smoke->tex = tex;
}

/* [ 12 ] */
void uSmokeDensity(uSmoke *smoke, float dens, float diss, float expansion)
{
    if (dens <= 0.0f || diss <= 0.0f || expansion <= 0.0f)
	return;
    smoke->density = dens;
    smoke->dissipation = diss;
    smoke->puffInterval = diss / (float)MAX_PUFFS / dens; 
    smoke->expansion = expansion;
}    

/* [ 9.1 ] */
void uGetSmokeVelocity(uSmoke *smoke, float *turbulence, float *speed)
{
    *speed = smoke->speed;
    *turbulence = smoke->turbulence;
}

/* [ 12.1 ] */
void uGetSmokeDensity(uSmoke *smoke, float *dens, float *diss, float *expansion)
{
    *dens = smoke->density;
    *diss = smoke->dissipation;
    *expansion = smoke->expansion;
}    

/* [ 4 ] */
void uSmokeType(uSmoke *smoke, long type)
{
    static uVec3	wht = {1.0f, 1.0f, 1.0f};
    static uVec3	blk = {.01f, .01f, .01f};
    static uVec3	brn = {.2f, .1f, .05f};

    smoke->type = type;
    switch(type) {
        case USMOKE_BILLOW:
	    smoke->speed = 1.73f;
            uSmokeDensity(smoke, 0.33f, 2.42f, 1.83f);
            uSmokeColor(smoke, wht, blk);
            uSmokeDuration(smoke,.5f);
            /* no texture map use this as 1st test*/
	case USMOKE_MISSLE:
	    smoke->speed = 32.3f;
    	    uSmokeDensity(smoke, 0.52f, .05f, .17f);
    	    uSmokeColor(smoke, wht, blk);
    	    uSmokeTex(smoke, fireTex);
	    break;
	case USMOKE_EXPLOSION:
	    smoke->speed = 0.0f;
    	    uSmokeDensity(smoke, 1.0f, .2f, 4.0f);
    	    uSmokeDuration(smoke, .2f);
    	    uSmokeColor(smoke, wht, wht);
    	    uSmokeTex(smoke, fireTex);
	    break;
	case USMOKE_FIRE:
            { 
	    static uVec3 yellow={1, 1, 0};
	    static uVec3 yellow2={.8, .6, 0};
/*-spe 2.5 -exp .03 -den .11 -dis .09*/
	    smoke->speed = 2.5f;
	    uSmokeDensity(smoke, 0.11f, .09f, .03f);
    	    /*uSmokeDensity(smoke,  8.f, 5.f, .5f);*/
    	    uSmokeColor(smoke, yellow, yellow2);
    	    uSmokeTex(smoke, fireTex);
            }
	    break;
	case USMOKE_SMOKE:
	    smoke->speed = 1.73f;
    	    uSmokeDensity(smoke, 0.33f, 2.42f, 1.83f);
    	    uSmokeColor(smoke, wht, blk);
    	    uSmokeTex(smoke, smokeTex);
	    break;
	case USMOKE_DUST:
	    smoke->speed = .8f;
    	    uSmokeDensity(smoke, 0.1f, 2.0f, 2.0f);
    	    uSmokeColor(smoke, brn, brn);
    	    uSmokeTex(smoke, smokeTex);
	    break;
	default:
	    smoke->speed = 1.73f;
    	    uSmokeDensity(smoke, 0.33f, 2.42f, 1.83f);
    	    uSmokeColor(smoke, wht, blk);
    	    uSmokeTex(smoke, smokeTex);
	    break;
    }
}


/* [ 5-12 || 13  ] */
void uDrawSmokes(uVec3 eye)
{
    long	i, n;


    zwritemask(0x00000000);
if (UGET_GFX_TYPE() & PFGFX_HPC_40NS)	/*express gfx*/
    afunction(0,AF_NOTEQUAL);
else
    afunction(40,AF_GREATER);

    blendfunction(BF_SA, BF_MSA);

    n = smokeCount;
    for(i=0; i<n; i++) {
        if(smokeList[i]->mode != USMOKE_STOP) {
	    drawSmoke(smokeList[i], eye);
        }
    }

    blendfunction(BF_ONE, BF_ZERO);
    zwritemask(0xffffffff);

}

