/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
#include "uSmoke.h"
#include "uLinMath.h"
#include "getgfxtype.h"

#ifdef try_limagedata
long *longimagedata(char *file);
#endif

extern float params[4];
extern float params2[4];

static tex_index_cntr=1;
static tev_index_cntr=1;
uTexture* uNewTex(void)
{
    uTexture *tex;
    
    tex = (uTexture *) calloc(1, sizeof(uTexture));

    tex->image = NULL;
    tex->name = NULL;
    tex->index = tex_index_cntr++;
    tex->dirty = 1;
    tex->comp = -1;
#if 0
    if (UGET_GFX_TYPE() & PFGFX_USE_MM_TRILINEAR)
	tex->minfilter = TX_MIPMAP_TRILINEAR;
    else
	tex->minfilter = TX_MIPMAP_BILINEAR;
    
    tex->wrap = TX_REPEAT;
    tex->magfilter = TX_BILINEAR;
    tex->alphamag = -1;
    tex->colormag = -1;
    tex->rwrap = TX_REPEAT;
    tex->swrap = TX_REPEAT;
    tex->twrap = TX_REPEAT;
    tex->freeimg = 0;
    tex->intformat = -1;
    tex->extformat = TX_PACK_8;
    tex->fastdefine = 0;
    tex->detail = NULL;
    tex->dprops[0] = 4;
    tex->dprops[1] = 4;
    tex->dprops[2] = 4;
    tex->dprops[3] = 4;
    tex->dprops[4] = 0;
    
    uSetVec2(tex->detailspline[0], 0.0f, 0.0f);
    uSetVec2(tex->detailspline[1], 0.5f, 0.466f);
    uSetVec2(tex->detailspline[2], 1.0f, 0.875f);
    uSetVec2(tex->detailspline[3], 2.0f, 1.5f);
    tex->detailclamp = -1.0f;		/* -1 indicates use default */
    
    uSetVec2(tex->sharpspline[0], 0.0f, 0.0f);
    uSetVec2(tex->sharpspline[1], 0.55f, 1.0f);
    uSetVec2(tex->sharpspline[2], 1.0f, 1.55f);
    uSetVec2(tex->sharpspline[3], 2.0f, 2.0f);
    tex->sharpclamp = -1.0f;		/* -1 indicates use default */
#endif
    
    return tex;
}

/*
 * PROCEDURE:
 *	findFilePath
 *
 * ACTION:
 *	Test named file for accessibility in indicated mode.
 *	first look for file exactly as named, then prepend
 *	each of the PFPATH environment variable strings and
 *	pfFilePath strings one at a time and try those.
 *
 *	If a matching file is found, the path to that file
 *	is returned in "path". if no file is found, then
 *	the path variable is reset to the empty string.
 *
 * INPUTS:
 *	const char *name -- suggested file name
 *	int mode -- desired access mode
 *
 * OUTPUTS:
 *	char path[_MAXSTRING] -- actual file name
 *
 * RETURNS:
 *	TRUE -- accessible file found in path returned in "path"
 *	FALSE -- accessible file not found
 */
/*char *FilePaths[10] = {NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};*/
char **FilePaths = NULL;
static long findFilePath(const char *name, char path[_MAXSTRING], int mode)
{
    int	i;
    char	*environment;
    
    /* check filename argument */
    if (name == NULL || path == NULL || *name == '\0' )
	return FALSE;
    
#ifdef	VERBOSE
    /* identify directories searched */
    printf("findFilePath() -> looking for %s\n", name);
#endif
    
    /* does file exist exactly indicated */
    if (access(name, mode) == 0)
    {
	/* return path path to file in path */
	strcpy(path, name);
	return TRUE;
    }
    
    /* try directories indicated in UPATH environment variable */
    if ((environment = getenv("UPATH")) != NULL)
    {
	char	*places = calloc(strlen(environment) + 1,sizeof(char));
	char	*place;
	
	/* isolate first colon-delimited directory name */
	strcpy(places, environment);
	place = strtok(places, ":");
	
	/* try each combination */
	while (place != NULL)
	{
	    if (strlen(name) + strlen(place) >= _MAXSTRING)
		return FALSE;
	    
	    /* build complete path */
	    strcpy(path, place);
	    if (place[strlen(place) - 1] != '/')
		strcat(path, "/");
	    strcat(path, name);
	    
#ifdef	VERBOSE
	    /* identify directories searched */
	    printf("findFilePath() -> looking for %s\n", path);
#endif
	    
	    /* test complete path */
	    if (access(path, mode) == 0)
		return TRUE;
	    
	    /* isolate next colon-delimited directory name */
	    place = strtok(NULL, ":");
	}
    }
    
    /* try directories indicated in pfFilePath array */
    if (FilePaths != NULL)
    {
	for (i = 1; FilePaths[i] != NULL; i++)
	{
	    if (strlen(name) + strlen(FilePaths[i]) >= _MAXSTRING)
		return FALSE;
	    
	    /* build complete path */
	    strcpy(path, FilePaths[i]);
	    if (path[strlen(FilePaths[i]) - 1] != '/')
		strcat(path, "/");
	    strcat(path, name);
	    
#ifdef	VERBOSE
	    /* identify directories searched */
	    printf("findFilePath() -> looking for %s\n", path);
#endif
	    
	    /* test complete path */
	    if (access(path, mode) == 0)
		return TRUE;
	}
    }
    
    /* failure -- file not found or not accessible via MODE */
    return FALSE;
}


/* from "libpr/rtfile.c" */
/*
 * PROCEDURE:
 *      pfFindFile
 *
 * ACTION:
 *      Test named file for accessibility in indicated mode. The search
 *      order is:
 *       a) look for file exactly as named
 *       b) prepend prefixes from PFPATH in order
 *       c) prepend prefixes from pfFilePath() in order
 *
 *      If a matching file is found, the path to that file is returned
 *      in "path". If no file is found, then the path variable is
 *      reset to the empty string.
 *
 * INPUTS:
 *      const char *name -- suggested file name
 *      int mode -- desired access mode
 *
 * OUTPUTS:
 *      char path[_MAXSTRING] -- actual file name
 *
 * RETURNS:
 *        TRUE -- accessible file found in path returned in "path"
 *        FALSE -- accessible file not found
 */

long uFindFile(const char *name, char path[_MAXSTRING], int mode)
{
    int status;
   
    /* reset path string */
    path[0] = '\0';
   
    /* check pointer arguments */
    if (name == NULL || path == NULL || *name == '\0')
        status = FALSE;
    /* look for file by prefixing directory names to given name */
    else
    if (findFilePath(name, path, mode))
        status = TRUE;
    /* look for file by prefixing directory names to base name */
    else
    if (findFilePath(strrchr(name, '/'), path, mode))
        status = TRUE;
    /* failure -- file not found or not accessible via mode */
    else
        status = FALSE;

    /* return file-accessibility status for file named in path */
    return status;
}

void uTexName(uTexture *tex, char *name)
{
    if(tex == NULL)
    {
#ifdef NOTIFIER
        pfNotify(PFNFY_WARN, PFNFY_USAGE, "uTexName() Null uTexture.");
#else
        fprintf(stderr,"uTexName() Null uTexture.");
#endif
        return;
    }

    /* Copy file name */
    if (tex->name)
        /***prUnrefDelete(tex->name);***/
        free(tex->name);

    tex->name = (char*)  calloc((strlen(name) + 1),sizeof(char));
    /***prRef(tex->name);***/

    strcpy(tex->name, name);
}


long uBuildTex(uTexture *tex, char path[_MAXSTRING], IMAGE *image_in)
{
    unsigned long   row;	/* unsigned for getrow */
    register long   i, sx, remain;
    register unsigned long	*image;
    register unsigned char	*cimage;
    
    
    /* Free old image and name */
    tex->sx = image_in->xsize;
    tex->sy = image_in->ysize;
    tex->sz = 1;
    tex->comp = image_in->zsize;
    
    /* 
     * This is required because some MultiGen image files have bogus dim's
     * that are == image->zsize and img_seek barfs.
     */
    image_in->dim = (image_in->xsize > 1) + (image_in->ysize > 1) + 
	(image_in->zsize > 1);
    
    /* Round up so rows are long-word aligned. */
    sx = (tex->sx * tex->comp + 3) >> 2;
    
    /***prUnref(tex->image); ***/

#ifdef try_limagedata
    tex->image = (unsigned long *) longimagedata(path);
#else
    image = tex->image = (unsigned long *) 
	calloc(sx * tex->sy, sizeof(unsigned long));

    /*** prRef(tex->image); ***/

    switch(tex->comp) {
    case 1:
	sx = tex->sx >> 2;
	remain = tex->sx - (sx<<2);
	break;
    case 2:
	sx = tex->sx >> 1;
	remain = tex->sx - (sx<<1);
	break;
    case 3:
	cimage = (unsigned char*)tex->image;
	sx = tex->sx;
	remain = (sx * 3) & 0x3;
	if(remain)
	    remain = 4 - remain;
	break;
    case 4:
	sx = tex->sx;
	remain = 0;
	break;
    }
  
    for (row = 0; row < tex->sy; row++)
    {
	unsigned short redbuff[4096], greenbuff[4096]; 
	unsigned short bluebuff[4096], alphabuff[4096]; 
	register unsigned short	*rbuff = redbuff, *gbuff = greenbuff;
	register unsigned short	*bbuff = bluebuff, *abuff = alphabuff;
	
	switch(tex->comp) {
	case 1:
	    getrow(image_in, redbuff, row, 0);
	    for (i = 0; i < sx; i++)
	    {
		*image++ = (*rbuff << 24) | ((*(rbuff + 1)) << 16) | 
		    ((*(rbuff + 2)) << 8) | (*(rbuff + 3));
		rbuff += 4;
		
	    }
	    /* Finish remainder of row */
	    if(remain)
	    {
		cimage = (unsigned char*)image;
		for (i = 0; i < remain; i++)
		    *cimage++ = *rbuff++;
		image++;
	    }
	    break;
	case 2:
	    getrow(image_in, redbuff, row, 0);
	    getrow(image_in, alphabuff, row, 1);
	    for (i = 0; i < sx; i++)
	    {
		*image++ = (*abuff << 24) | (*rbuff << 16) | 
		    ((*(abuff+1)) << 8) | (*(rbuff+1));
		
		rbuff += 2;
		abuff += 2;
	    }
	    /* Finish remainder of row */
	    if(remain)
	    {
		cimage = (unsigned char*)image;
		for (i = 0; i < remain; i++)
		{
		    *cimage++ = *abuff++;
		    *cimage++ = *rbuff++;
		}
		image++;
	    }
	    break;
	case 3:
	    getrow(image_in, redbuff, row, 0);
	    getrow(image_in, greenbuff, row, 1);
	    getrow(image_in, bluebuff, row, 2);
	    
	    for (i = 0; i < sx; i++)
	    {
		*cimage++ = *bbuff++;
		*cimage++ = *gbuff++;
		*cimage++ = *rbuff++;
	    }
	    /* Skip to next long-word boundary */
	    cimage += remain;
	    break;
	case 4:
	    getrow(image_in, redbuff, row, 0);
	    getrow(image_in, greenbuff, row, 1);
	    getrow(image_in, bluebuff, row, 2);
	    getrow(image_in, alphabuff, row, 3);
	    
	    for (i = 0; i < sx; i++)
	    {
		*image++ = (*abuff++ << 24) | (*bbuff++ << 16) | 
		    (*gbuff++ << 8) | (*rbuff++);
		
	    }
	    break;
	}
    }
#endif /*try_limagedata*/
    
    uTexName(tex, path);
    tex->dirty++;

    return TRUE;
}


/* load an image file 
 * XXX - any changes to this routine should be reflected in
 * _LoadENCTexFile in fltencrypt.c in the flt converter.
 */
long uLoadTexFile(uTexture *tex, char *name)
{
    IMAGE          *image_in;
    char            path[_MAXSTRING];
    long            ret;
    
    if(tex == NULL)
    {
        fprintf(stderr,"uLoadTexFile() Null uTexture.\n");
        return -1;
    }
    
    if(!uFindFile(name, path, R_OK))
    {
        fprintf(stderr,"uLoadTexFile: could not find image file %s\n", name);
        return FALSE;
    }
    
    if ((image_in = iopen(path, "r")) == NULL)
    {
        fprintf(stderr,"uLoadTexFile: could not load image %s\n", name);
        return FALSE;
    }
    
    ret = uBuildTex(tex, name, image_in);
    
    iclose(image_in);
    return ret;
}

void uTexRepeat(uTexture *tex, long wrap, long type)
{
    if(tex == NULL)
    {
#ifdef NOTIFIER
        pfNotify(PFNFY_WARN, PFNFY_USAGE, "pfTexRepeat() Null pfTexture.");
#else
        fprintf(stderr,"uTexRepeat() Null uTexture.\n");
#endif
        return;
    }

    switch (wrap)
    {
    case TX_WRAP:
        tex->wrap = type;
        tex->rwrap = type;
        tex->swrap = type;
        tex->twrap = type;
        break;
    case TX_WRAP_S:
        tex->swrap = type;
        break;
    case TX_WRAP_T:
        tex->twrap = type;
        break;
    case TX_WRAP_R:
        tex->rwrap = type;
        break;
    default:
#ifdef NOTIFIER
        pfNotify(PFNFY_WARN, PFNFY_USAGE,
                 "pfTexRepeat() Bad wrap %ld\n", wrap);
#else
        fprintf(stderr,"uTexRepeat() Bad wrap %ld\n", wrap);
#endif
        return;
    }
    tex->dirty++;
}

void uApplyTex(uTexture *tex)
{

    if(tex == NULL)
    {
        fprintf(stderr,"uApplyTex() Null uTexture.\n");
        return;
    }

    texbind(TX_TEXTURE_0, tex->index);

}

/* make a texture environment control structure */
uTexEnv* uNewTEnv(void)
{
    uTexEnv    *tev;

    tev = (uTexEnv *) calloc(1, sizeof(uTexEnv));
    /***_prInitObject(tev, PFTYPE_TEXENV);***/

    tev->dirty = 1;
    tev->index = tev_index_cntr++;
    tev->mode = TV_MODULATE;
    uSetVec4(tev->blend, 1.0f, 1.0f, 1.0f, 1.0f);
    return tev;
}


void  uApplyTEnv(uTexEnv * tev)
{
    if(tev == NULL)
    {
#ifdef NOTIFIER
	pfNotify(PFNFY_WARN, PFNFY_USAGE, "pfApplyTEnv() Null pfTexEnv.");
#else
        fprintf(stderr,"uApplyTEnv() Null uTexEnv.\n");
#endif
	return;
    }

    if(tev->dirty)
    {
	float           props[MAXPROPS];
	long             i = 0;

	props[i++] = tev->mode;
	if(tev->mode == TV_BLEND) /**********start here ***********/
	{
	    props[i++] = TV_COLOR;
	    uCopyVec4(props + i, tev->blend);
	    i += 4;
	}
	if(tev->compsel > 0)
	{
	    props[i++] = TV_COMPONENT_SELECT;
	    props[i++] = tev->compsel;
	}
	
	props[i++] = TV_NULL;

	tevdef(tev->index, i, props);

        tevbind(TV_ENV0, tev->index);
        
        tev->dirty=0;
    }
/* geostate stuff */
#ifdef PERFORMER_GEO_STATE_STUFF
    if (pfCurGState->tev != tev)
    {
	if (pfCurGState->entex)
	{
	    tevbind(0, tev->index);
	    if (pfCurStateStats)
		pfCurStateStats->modeChanges.texEnvs++;
	}
	pfCurGState->tev = tev;
    }
    pfCurGState->modvec |= MVTEV;
#endif
}

