/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// Attributes.c++ 

#include "Attributes.h"
#include "CardView.h"

#include "OkStr.h"
#include "OkArgs.h"

#include <Xm/RowColumn.h>
#include <Xm/BulletinB.h>
#include <assert.h>

AttributeToggle::AttributeToggle( const char* name, Widget parent, 
					Attributes* parentView ) :
 OkToggleButton( name, parent )
{
  _parent = parentView;
}


void 
AttributeToggle::valueChanged( Boolean )
{ 
  _parent->_parentView->setDirty(); 
}

Attributes::Attributes( const char* name, Widget parent ) :
  OkComponent( name )
{
  OkArgs args( 1 );
  args.set( XmNresizePolicy, XmRESIZE_GROW );
  _baseWidget = XmCreateBulletinBoard( parent, "bulletinboard",
			args.list(), args.count() );
  installDestroyHandler();

  args.reset();
  args.set( XmNorientation, XmHORIZONTAL );
  _rowColumn = XmCreateRowColumn( _baseWidget, _name, 
			args.list(), args.count() );
  _radioRowColumn = XmCreateRadioBox( _baseWidget, _name, 
			args.list(), args.count() );

  for ( int i=0; i<ATTR_COUNT; i++ ) 
    _attributeToggles[ i ] = NULL;
  
  _radioStyle = TRUE;
  setRadioStyle( FALSE );
}

Attributes::~Attributes()
{
  for ( int i=0; i<ATTR_COUNT; i++ ) 
    delete _attributeToggles[ i ];
}


void 
Attributes::setAttrLabel( int i, const char* label )
{ 
  _attributeToggles[ i ]->setLabelString( label ); 
//  XmUpdateDisplay( *( _attributeToggles[ i ] ) );
}

void 
Attributes::hideAttr( int i ) 
{
  XtUnmanageChild( *(_attributeToggles[ i ]) ); 
//  XmUpdateDisplay( *( _attributeToggles[ i ] ) );
}

void 
Attributes::showAttr( int i ) 
{ 
  XtManageChild( *(_attributeToggles[ i ]) ); 
//  XmUpdateDisplay( *( _attributeToggles[ i ] ) );
}


void
Attributes::setRadioStyle( Boolean b )
{
  if ( b && _radioStyle )
    return;

  _radioStyle = b;

  if ( _radioStyle ) {
    XtManageChild( _radioRowColumn );
    XtUnmanageChild( _rowColumn );
  } else {
    XtManageChild( _rowColumn );
    XtUnmanageChild( _radioRowColumn );
  }

  for ( int i=0; i<ATTR_COUNT; i++ ) {
    Boolean value = ( _attributeToggles[ i ] ? 
				_attributeToggles[ i ]->getValue() :
				FALSE );

    delete _attributeToggles[ i ];
    _attributeToggles[ i ] = new AttributeToggle( OkStr( i, "attribute%d" ), 	
 			( _radioStyle ? _radioRowColumn : _rowColumn ), this ); 
    assert( _attributeToggles[ i ] );

    _attributeToggles[ i ]->setValue( value );
// Don't show it here, ow. it will cause unnecessary flashing.
//    _attributeToggles[ i ]->show();
  }

}
