/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// Buttons.c++

#include "Buttons.h"
#include "ListView.h"
#include "CardList.h"
#include "CardView.h"
#include "SrchDialog.h"

#include <Vk/VkApp.h>
#include <Vk/VkInfoDialog.h>
#include <Vk/VkQuestionDialog.h>


void
ShowBtn::activate()
{
  theListView->showCard( theListView->cardList()->getSelectedItem() );
}


void
DeleteBtn::activate()
{
  theListView->deleteCard( theListView->cardList()->getSelectedItem() );
}

void
NewBtn::activate()
{
  theListView->addCard();
}

////////////////////////////////////////////////////////////////////

FirstBtn::FirstBtn( Widget parent, CardView* parentView ) : 
  OkPushButton( "firstBtn", parent )
{ _parentView = parentView; }

PrevBtn::PrevBtn( Widget parent, CardView* parentView ) :
  OkPushButton( "prevBtn", parent )
{ _parentView = parentView; }

NextBtn::NextBtn( Widget parent, CardView* parentView ) :
  OkPushButton( "nextBtn", parent )
{ _parentView = parentView; }

LastBtn::LastBtn( Widget parent, CardView* parentView ) :
  OkPushButton( "lastBtn", parent )
{ _parentView = parentView; }

CloseBtn::CloseBtn( Widget parent, CardView* parentView ) :
  OkPushButton( "closeBtn", parent )
{ _parentView = parentView; }

void CloseBtn::activate() { _parentView->hide(); }

SrchBtn::SrchBtn( CardView* parentView ) :
  OkPushButton( "srchBtn", parentView->viewWidget() )
{ _parentView = parentView; }



void
FirstBtn::activate()
{
  theListView->cardList()->firstVisibleItem( 1 );
  theListView->showCard( 1 );
}


void
PrevBtn::activate()
{
  if ( theListView->currCardNo() == 1 )
    theInfoDialog->postBlocked( "This is the first card.", _parentView );
  else {
    theListView->cardList()->firstVisibleItem( theListView->currCardNo()-1 );
    theListView->showCard( theListView->currCardNo()-1 );
  }
}


void
NextBtn::activate()
{
  if ( theListView->currCardNo() == theListView->totalCards() )
    theInfoDialog->postBlocked( "This is the last card.", _parentView );
  else {
    theListView->cardList()->lastVisibleItem( theListView->currCardNo()+1 );
    theListView->showCard( theListView->currCardNo()+1 );
  }
}


void
LastBtn::activate()
{
  theListView->cardList()->lastVisibleItem( theListView->totalCards() );
  theListView->showCard( theListView->totalCards() );
}


void
SrchBtn::activate()
{
  // Make the card that owns this btn the active viewer before searching.
  theListView->setActiveCardViewer( _parentView );
  theSrchDialog->post( NULL, _parentView );
}

