/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// CardInfo.c++

#include "CardInfo.h"

#include <stdio.h>
#include <string.h>

CardInfo::CardInfo( ObjAddr obj ) :
  Persistent( *theDatabase, CARD_INFO )
{
  init( "", obj );
}


CardInfo::CardInfo( const char* name, ObjAddr obj ) :
  Persistent( *theDatabase, CARD_INFO )
{
  init( (name?name:""), obj );
}

void
CardInfo::init( const char* name, ObjAddr obj )
{
  // Initialize all fields.
  _name = name;
  _phone1 = "";
  _phone2 = "";
  _fax = "";
  _mobilePhone = "";
  _email = "";

  _addressLine1 = "";
  _addressLine2 = "";
  _city = "";
  _state = "";
  _zip = "";
  _country = "";

  for ( int i=0; i<ATTR_COUNT; i++ )
    _attribute[ i ] = FALSE;

  // Get from db.
  LoadObject( obj );

  if ( ! ObjectExists() )
    // Need to add the object right away after creation to
    // get the persistent object address.
    AddObject();
}

CardInfo::~CardInfo()
{
  SaveObject();
}


void
CardInfo::save()
{
  ChangeObject();
  SaveObject();

  // Save the database.
  theDatabase->save();
}



void
CardInfo::Read()
{
  pString s;

  ReadObject( s );
  _name = s;
  ReadObject( s );
  _phone1 = s;
  ReadObject( s );
  _phone2 = s;
  ReadObject( s );
  _fax = s;
  ReadObject( s );
  _mobilePhone = s;
  ReadObject( s );
  _email = s;

  ReadObject( s );
  _addressLine1 = s;
  ReadObject( s );
  _addressLine2 = s;
  ReadObject( s );
  _city = s;
  ReadObject( s );
  _state = s;
  ReadObject( s );
  _zip = s;
  ReadObject( s );
  _country = s;

  for ( int i=0; i<ATTR_COUNT; i++ )
    ReadObject( _attribute[ i ] );

  ReadObject( s );
  _notes = s;

}


void
CardInfo::Write()
{
  WriteObject( (const char *) _name );
  WriteObject( (const char *) _phone1 );
  WriteObject( (const char *) _phone2 );
  WriteObject( (const char *) _fax );
  WriteObject( (const char *) _mobilePhone );
  WriteObject( (const char *) _email );

  WriteObject( (const char *) _addressLine1 );
  WriteObject( (const char *) _addressLine2 );
  WriteObject( (const char *) _city );
  WriteObject( (const char *) _state );
  WriteObject( (const char *) _zip );
  WriteObject( (const char *) _country );

  for ( int i=0; i<ATTR_COUNT; i++ )
    WriteObject( _attribute[ i ] );

  WriteObject( (const char *) _notes );
}


Boolean
CardInfo::containString( OkStr s, Boolean caseSensitive, Boolean srchNotesToo )
{
  if ( ContainString( s, _name, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _phone1, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _phone2, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _fax, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _mobilePhone, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _email, caseSensitive ) )
    return TRUE;


  if ( ContainString( s, _addressLine1, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _addressLine2, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _city, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _state, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _zip, caseSensitive ) )
    return TRUE;

  if ( ContainString( s, _country, caseSensitive ) )
    return TRUE;

  if ( srchNotesToo && ContainString( s, _notes, caseSensitive ) )
    return TRUE;

  return FALSE;
}


Boolean
CardInfo::ContainString( OkStr s, OkStr target, Boolean caseSensitive )
{
  // If s is longer than target, then it's always FALSE.
  if ( s.length() > target.length() )
    return FALSE;

  char* targetPtr = target;
  if ( caseSensitive ) {

    for ( int i=0; i<=( target.length() - s.length() ); i++ )
      if ( strncmp( s, targetPtr+i, s.length() ) == 0 )
      	return TRUE;

  } else {
    // Case-insensitive

    for ( int i=0; i<=( target.length() - s.length() ); i++ )
      if ( strncasecmp( s, targetPtr+i, s.length() ) == 0 )
      	return TRUE;

  }

  return FALSE;
}
