/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// CardList.c++

#include "CardList.h"
#include "ListView.h"
#include "Database.h"

#include <Vk/VkWarningDialog.h>

#include <iostream.h>
#include <string.h>
#include <assert.h>



CardList::CardList( Widget parent, OkStrArray& items, Widget srchText ) :
  OkScrolledList( "cardList", parent, items, srchText )
{
  assert( theListView );

  listSearch()->setListSelectPos( &CardList::selectPosProc );
  listSearch()->setListDeselectAllItems( &CardList::deselectAllProc );
}


CardList::~CardList()
{
}


void
CardList::singleSelection()
{
  theListView->setCardSensitivity();
  theListView->updateIndicatorLabel( getSelectedItem() );
}


void
CardList::defaultAction()
{
 int i = theListView->cardList()->getSelectedItem();
 theListView->showCard( i );

 // If I do the following, I get segmentation fault...  WHY???
 // theListView->showCard( theListView->cardList()->getSelectedItem() );
 //cerr << "No: " << i << endl;
}


void
CardList::deselectAllProc( Widget w, caddr_t obj )
{
  VkListSearch::xmListDeselectAllItems( w, obj );

  theListView->setCardSensitivity();
  theListView->updateIndicatorLabel();
}

void
CardList::selectPosProc( Widget w, int position, Boolean notify, caddr_t data )
{
  VkListSearch::xmListSelectPos( w, position, notify, data );

  theListView->setCardSensitivity();
  theListView->updateIndicatorLabel( position );
}


unsigned
CardList::add( OkStr s )
{
  unsigned pos = 1;

  if ( itemCount() )
    pos = add( s, 1, itemCount() );

  if ( pos>itemCount() )
    // Add at the end.
    addItem( s );
  else
    addItem( s, pos );

  return pos;

}


unsigned
CardList::add( OkStr& s, unsigned bottom, unsigned top )
{
  // Found the spot.
  if ( (top-bottom)<=1 )
    if ( theDatabase->sortOrder()==ASCENDING ) {
      if ( s<item( bottom ) )
	return bottom;
      else if ( s<item( top ) )
	return top;
      else 
        return top+1;
    } else {
      if ( s>item( bottom ) ) 
	return bottom;
      else if ( s>item( top ) )
	return top;
      else 
        return top+1;
    }
  

  unsigned middle = (bottom+top)/2;

  if ( theDatabase->sortOrder()==ASCENDING ) {

    if ( s<item( middle ) )
      return add( s, bottom, middle );
    else
      return add( s, middle, top );

  } else {

    if ( s>item( middle ) )
      return add( s, bottom, middle );
    else
      return add( s, middle, top );

  }

}
