/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// CardView.c++

#include "CardView.h"
#include "CardInfo.h"
#include "ListView.h"
#include "Buttons.h"
#include "Attributes.h"
#include "PrefFile.h"
#include "Print.h"
#include "SetupFields.h"

#include "OkArgs.h"
#include "OkFile.h"

#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkErrorDialog.h>
#include <Vk/VkQuestionDialog.h>

#include <Xm/Form.h>
#include <Xm/Separator.h>
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>

#include <assert.h>
#include <iostream.h>
#include <stdlib.h>


void FieldText::valueChanged() { _parentView->setDirty(); }
void Notes::valueChanged() { _parentView->setDirty(); }



VkMenuDesc
CardView::notesMenuPane[] = {
  { ACTION, "insertFromFile", &CardView::insertFromFileCB },
  { ACTION, "saveToFile", &CardView::saveToFileCB },
  { SEPARATOR },
  { ACTION, "clearNotes", &CardView::clearNotesCB },
  { END },
};


VkMenuDesc
CardView::cardMenuPane[] = {
  { ACTION, "search", &CardView::searchCB },
  { SEPARATOR },
  { ACTION, "save", &CardView::saveCB },
  { ACTION, "delete", &CardView::deleteCB },
  { ACTION, "new", &CardView::newCB },
  { SEPARATOR },
  { ACTION, "print", &CardView::printCB },
  { SEPARATOR },
  { ACTION, "close", &CardView::closeCB },
  { END },
};

VkMenuDesc
CardView::mainMenuPane[] = {
  { SUBMENU, "card", NULL, CardView::cardMenuPane },
  { SUBMENU, "notes", NULL, CardView::notesMenuPane },
  { END },
};


CardView::CardView( int viewerNo ) : 
  VkWindow( "cardView" ), _cardInfo( NULL ), _cardPos( 0 )
{
  setMenuBar( mainMenuPane );

  OkArgs args( 7 );
  if ( thePrefFile->activeCardViewWidth() && 
	thePrefFile->activeCardViewHeight() ) {
    args.set( XmNwidth, thePrefFile->activeCardViewWidth() );
    args.set( XmNheight, thePrefFile->activeCardViewHeight() );
  }
  Widget form = XmCreateForm( mainWindowWidget(), "form", 
                                        args.list(), args.count() );
  addView( form );

  _faceImg = NULL;

  _nameField = new OkLabeledComponent<FieldText>( "name", form );
  assert( _nameField );
  XtVaSetValues( *_nameField,
    XmNtopAttachment, XmATTACH_FORM,
    XmNleftAttachment, XmATTACH_FORM,
    NULL );
  _nameField->show();
  _nameField->t()->setParentView( this );

  _phone1Field = new OkLabeledComponent<FieldText>( "phone1", form );
  assert( _phone1Field );
  XtVaSetValues( *_phone1Field,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _nameField->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    NULL );
  _phone1Field->show();
  _phone1Field->t()->setParentView( this );

  _phone2Field = new OkLabeledComponent<FieldText>( "phone2", form );
  assert( _phone2Field );
  XtVaSetValues( *_phone2Field,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _nameField->baseWidget(),
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _phone2Field->show();
  _phone2Field->t()->setParentView( this );

  _faxField = new OkLabeledComponent<FieldText>( "fax", form );
  assert( _faxField );
  XtVaSetValues( *_faxField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _phone1Field->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    NULL );
  _faxField->show();
  _faxField->t()->setParentView( this );

  _mobilePhoneField = new OkLabeledComponent<FieldText>( "mobilePhone", form );
  assert( _mobilePhoneField );
  XtVaSetValues( *_mobilePhoneField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _phone2Field->baseWidget(),
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _mobilePhoneField->show();
  _mobilePhoneField->t()->setParentView( this );

  _emailField = new OkLabeledComponent<FieldText>( "email", form );
  assert( _emailField );
  XtVaSetValues( *_emailField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _faxField->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _emailField->show();
  _emailField->t()->setParentView( this );
   
  ///////////////////////////////////////////////////////////////////////

  args.reset();
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _emailField->baseWidget() );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  Widget separator = XmCreateSeparator( form, "separator",
				args.list(), args.count() );
  XtManageChild( separator );

  ///////////////////////////////////////////////////////////////////////

  _addressLine1Field = new OkLabeledComponent<FieldText>( "addressLine1", form );
  assert( _addressLine1Field );
  XtVaSetValues( *_addressLine1Field,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, separator,
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _addressLine1Field->show();
  _addressLine1Field->t()->setParentView( this );

  _addressLine2Field = new OkLabeledComponent<FieldText>( "addressLine2", form );
  assert( _addressLine2Field );
  XtVaSetValues( *_addressLine2Field,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _addressLine1Field->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _addressLine2Field->show();
  _addressLine2Field->t()->setParentView( this );

  _cityField = new OkLabeledComponent<FieldText>( "city", form );
  assert( _cityField );
  XtVaSetValues( *_cityField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _addressLine2Field->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    NULL );
  _cityField->show();
  _cityField->t()->setParentView( this );

  _stateField = new OkLabeledComponent<FieldText>( "state", form );
  assert( _stateField );
  XtVaSetValues( *_stateField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _addressLine2Field->baseWidget(),
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _stateField->show();
  _stateField->t()->setParentView( this );

  _zipField = new OkLabeledComponent<FieldText>( "zip", form );
  assert( _zipField );
  XtVaSetValues( *_zipField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _stateField->baseWidget(),
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _zipField->show();
  _zipField->t()->setParentView( this );

  _countryField = new OkLabeledComponent<FieldText>( "country", form );
  assert( _countryField );
  XtVaSetValues( *_countryField,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _cityField->baseWidget(),
    XmNleftAttachment, XmATTACH_FORM,
    NULL );
  _countryField->show();
  _countryField->t()->setParentView( this );

  ///////////////////////////////////////////////////////////////

  args.reset();
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _countryField->baseWidget() );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  separator = XmCreateSeparator( form, "separator",
				args.list(), args.count() );
  XtManageChild( separator );

  ///////////////////////////////////////////////////////////////

  _attributes = new OkLabeledComponent<Attributes>( "attributes",
		form, OkLabeledComponent<Attributes>::HORIZONTAL );
  assert( _attributes );
  XtVaSetValues( *_attributes,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, separator,
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );

  _attributes->t()->_parentView = this;
  _attributes->show();

  ////////////////////////////////////////////////////////////////
  args.reset();
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _attributes->baseWidget() );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  separator = XmCreateSeparator( form, "separator",
				args.list(), args.count() );
  XtManageChild( separator );

  _notes = new OkLabeledComponent<Notes>( "notes", form, 
				OkLabeledComponent<OkText>::VERTICAL );
  XtVaSetValues( *_notes,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, separator,
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );

  XtVaSetValues( _notes->component()->baseWidget(),
    XmNeditMode, XmMULTI_LINE_EDIT,
    NULL );

  _notes->component()->setParentView( this );

  args.reset();
  args.set( XmNbottomAttachment, XmATTACH_FORM );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  separator = XmCreateSeparator( form, "separator",
				args.list(), args.count() );
  XtManageChild( separator );

  ///////////////////////////////////////////////////////////////

  // Attach bottom of notes, so that notes is vertically stretchable.
  XtVaSetValues( *_notes,
    XmNbottomAttachment, XmATTACH_WIDGET,
    XmNbottomWidget, separator,
    NULL );
  _notes->show();

  
  //////////////////////////////////////////////////////////////
  // Controls.
  //////////////////////////////////////////////////////////////
  _activeToggle = new ActiveToggle( viewerNo, form );
  assert( _activeToggle );
  XtVaSetValues( *_activeToggle,
    XmNleftAttachment, XmATTACH_FORM,
    XmNbottomAttachment, XmATTACH_FORM,
    NULL );
  _activeToggle->show();

  _srchBtn = new SrchBtn( this );
  assert( _srchBtn );
  XtVaSetValues( *_srchBtn,
    XmNleftAttachment, XmATTACH_WIDGET,
    XmNleftWidget, _activeToggle->baseWidget(),
    XmNbottomAttachment, XmATTACH_FORM,
    NULL );
  _srchBtn->show();

  args.reset();
  args.set( XmNorientation, XmHORIZONTAL );
  args.set( XmNpacking, XmPACK_COLUMN );
  args.set( XmNentryAlignment, XmALIGNMENT_CENTER );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  args.set( XmNbottomAttachment, XmATTACH_FORM );
  Widget rowColumn = XmCreateRowColumn( form, "rowColumn", 
					args.list(), args.count() );
  XtManageChild( rowColumn );

  _firstBtn = new FirstBtn( rowColumn, this );
  assert( _firstBtn );
  _firstBtn->show();

  _prevBtn = new PrevBtn( rowColumn, this );
  assert( _prevBtn );
  _prevBtn->show();

  _nextBtn = new NextBtn( rowColumn, this );
  assert( _nextBtn );
  _nextBtn->show();

  _lastBtn = new LastBtn( rowColumn, this );
  assert( _lastBtn );
  _lastBtn->show();

  _closeBtn = new CloseBtn( rowColumn, this );
  assert( _closeBtn );
  _closeBtn->show();

  if ( thePrefFile->showFaceImg() )
    showFaceImg();
  else
    hideFaceImg();

  setFieldLabels();

  clearDirty();
}

CardView::~CardView()
{
  // Save current card info.
  saveAndDeleteCardInfo();


  // Clean up.
  delete _nameField;
  delete _phone1Field;
  delete _phone2Field;
  delete _faxField;
  delete _mobilePhoneField;
  delete _emailField;

  delete _addressLine1Field;
  delete _addressLine2Field;
  delete _cityField;
  delete _stateField;
  delete _zipField;
  delete _countryField;

  delete _notes;

  delete _activeToggle;

  delete _firstBtn;
  delete _prevBtn;
  delete _nextBtn;
  delete _lastBtn;
  delete _srchBtn;

  delete _faceImg;

}


void
CardView::setFieldLabels()
{
  theApplication->busy();

  // Field Labels:
  XtVaSetValues( _nameField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::NAME ) ), 
    NULL );
  XtVaSetValues( _phone1Field->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::PHONE1 ) ), 
    NULL );
  XtVaSetValues( _phone2Field->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::PHONE2 ) ), 
    NULL );
  XtVaSetValues( _faxField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::FAX ) ), 
    NULL );
  XtVaSetValues( _mobilePhoneField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel(SetupFields::MOBILE_PHONE) ), 
    NULL );
  XtVaSetValues( _emailField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::EMAIL ) ), 
    NULL );

  XtVaSetValues( _addressLine1Field->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel(SetupFields::ADDRESS_LINE1) ), 
    NULL );
  XtVaSetValues( _addressLine2Field->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel(SetupFields::ADDRESS_LINE2) ), 
    NULL );
  XtVaSetValues( _cityField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::CITY ) ), 
    NULL );
  XtVaSetValues( _stateField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::STATE ) ), 
    NULL );
  XtVaSetValues( _zipField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::ZIP ) ), 
    NULL );
  XtVaSetValues( _countryField->label(), XmNlabelString, 
    OkCompoundString( theSetupFields->fieldLabel( SetupFields::COUNTRY ) ), 
    NULL );

  // To line up the left labels evenly, we need to find the widest label.(>85)
  Dimension width, widest=85;
  XtVaGetValues( _nameField->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _phone1Field->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _faxField->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _emailField->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _addressLine1Field->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _addressLine2Field->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _cityField->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);
  XtVaGetValues( _countryField->label(), XmNwidth, &width, NULL );
  widest = (widest>width?widest:width);

  // Found the widest label.  Set the width of other left labels to widest.
  XtVaSetValues( _nameField->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _phone1Field->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _faxField->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _emailField->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _addressLine1Field->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _addressLine2Field->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _cityField->label(), XmNwidth, widest, NULL );
  XtVaSetValues( _countryField->label(), XmNwidth, widest, NULL );
 
  theApplication->notBusy();
}


void
CardView::hide()
{
  saveAndDeleteCardInfo();

  if ( theListView->activeCardViewer() == viewerNo() ) {
    // If we are active viewer, just hide ourselves to be reused later.
    // Invalidate card position.
    _cardPos = 0;
    VkWindow::hide();
  } else {
    // We're no longer needed.  Self destruct.
    theListView->resetCardViewTableEntry( viewerNo() );
    delete this;
  }
}


void
CardView::makeActive( Boolean value )
{
  if ( value ) {
    enableNavigationalBtns();
    _activeToggle->setValue( TRUE );
  } else {
    disableNavigationalBtns();
    _activeToggle->setValue( FALSE );
  }
}


void
CardView::enableNavigationalBtns()
{
  XtSetSensitive( *_firstBtn, TRUE );
  XtSetSensitive( *_prevBtn, TRUE );
  XtSetSensitive( *_nextBtn, TRUE );
  XtSetSensitive( *_lastBtn, TRUE );
}

void
CardView::disableNavigationalBtns()
{
  XtSetSensitive( *_firstBtn, FALSE );
  XtSetSensitive( *_prevBtn, FALSE );
  XtSetSensitive( *_nextBtn, FALSE );
  XtSetSensitive( *_lastBtn, FALSE );
}

/////////////////////////////////////
// Control classes definition
/////////////////////////////////////
ActiveToggle::ActiveToggle( int viewerNo, Widget parent ) :
  OkToggleButton( "activeToggle", parent )
{
  _viewerNo = viewerNo;
}

void
ActiveToggle::valueChanged(Boolean newValue)
{
  assert( theListView );

  if ( newValue ) {
    if ( theListView->activeCardViewer() == _viewerNo )
      return;  // We're already the active one.

    // Make this viewer the active one.
    theListView->setActiveCardViewer( _viewerNo );

  } else if ( theListView->activeCardViewer() == _viewerNo ) {
    // We are currently the active viewer.  Turn it off.
    theListView->setNoActiveCardViewer();
  }

  // No need to do anything if user turn it off, and we're not
  // the active viewer anyway.  (should be impossible to happen).
  return;
}



Boolean
CardView::deleteCard()
{
  if ( theListView->deleteCard( _cardPos, _cardInfo, this ) ) {
    // The card was deleted successfully, shutdown this view.
    // By pass our hide routine.
    delete _cardInfo;
    _cardInfo = NULL;
    _cardPos = 0;
    VkWindow::hide();
    return TRUE;
  }

  return FALSE;
}


void
CardView::setAttributeValue( unsigned n, Boolean value )
{
  _attributes->t()->_attributeToggles[ n ]->setValue( value );
}

Boolean
CardView::attributeValue( unsigned n ) const
{
  return _attributes->t()->_attributeToggles[ n ]->getValue();
}


Boolean
CardView::okToQuit()
{
  Dimension w,h;
  XtVaGetValues( viewWidget(),
    XmNwidth, &w,
    XmNheight, &h,
    NULL );

  thePrefFile->setActiveCardViewWidth( w );
  thePrefFile->setActiveCardViewHeight( h );

  return TRUE;
}


void
CardView::printCB( Widget, XtPointer obj, XtPointer )
{
  thePrintCardDialog->printCardView( (CardView*) obj );
}


void
CardView::insertFromFile()
{
  theFileSelectionDialog->setTitle( "Insert File to Notes" );
  if ( theFileSelectionDialog->postAndWait( "", this ) == 
						VkDialogManager::CANCEL )
	return;

  OkFile file( theFileSelectionDialog->fileName() );
  if ( ! file.file() ) {
    OkStr s = "Cannot open file: " | OkStr(theFileSelectionDialog->fileName());
    theErrorDialog->postBlocked( s, this );
    return;
  }

  OkStr s = "";
  char buf[ BUFSIZ ];
  while ( 1 ) {
    file.file().getline( buf, BUFSIZ );
    if ( file.file().eof() )
      break;
    s.append( buf );
    s.append( "\n" );
  }

  _notes->t()->insert( s );

}


void
CardView::saveToFile()
{
  theFileSelectionDialog->setTitle( "Save Notes to File" );
  if ( theFileSelectionDialog->postAndWait( "", this ) == 
						VkDialogManager::CANCEL )
	return;

  OkFile file( theFileSelectionDialog->fileName(), ios::out );
  if ( ! file.file() ) {
    OkStr s = "Cannot open file: " | OkStr(theFileSelectionDialog->fileName());
    theErrorDialog->postBlocked( s, this );
    return;
  }

  file.file() << notes();
}


void
CardView::clearNotes()
{
  theQuestionDialog->setButtonLabels( "Yes", "No" );
  if ( theQuestionDialog->
	postAndWait( "Are you sure you want to clear notes ?", this ) ==
	VkDialogManager::OK ) {
    theApplication->busy();
    setNotes( "" );
    theApplication->notBusy();
  }



#if 0
 // This used to be launching editor...  too much trouble!!
  char *wineditor = getenv( "WINEDITOR" );
  OkStr editor = ( wineditor ? wineditor : "/usr/sbin/jot" );

Some people alias WINEDITOR to non executable...
  if ( ! OkFile::FileExecutable( editor ) ) {
    OkStr s = "Cannot execute: " | editor;
    theWarningDialog->postBlocked( s, this );
  }

  editor.append( " " );
  editor.append( _notes->t()->fileName() );
  editor.append( "&" );

  // Flush all changes so far.
  _notes->t()->saveToFile();

  if ( system( editor ) == -1 ) {
    OkStr s = "Cannot execute: " | editor;
    theErrorDialog->postBlocked( s, this );
  }
#endif
}

void
CardView::showFaceImg()
{
  if ( ! _faceImg ) {
    OkArgs args( 2 );
    args.set( XmNtopAttachment, XmATTACH_FORM );
    args.set( XmNrightAttachment, XmATTACH_FORM );
    Widget frame = XmCreateFrame( viewWidget(), "faceImg", 
					args.list(), args.count() );
    XtManageChild( frame );
    _faceImg = new OkImageView( "view", frame );
    assert( _faceImg );
  }


  OkStr tmps = email();
  u_int pos=0;
  tmps = tmps.token( pos, "@" );

  OkStr fileName = ( tmps=="" ? email() : tmps );

  // If email field is left empty, put guest as file name. (see **)
  if ( fileName=="" )
    fileName = "guest";

  Boolean noFile=TRUE;
  while ( noFile ) {
    for ( int i=0;;i++ ) {
      OkStr path = thePrefFile->faceImgLoc( i );
      if ( path=="" )
        break;

      if ( OkFile::Exists( OkStr( path|"/"|fileName ) ) ) {
        fileName.insert( OkStr( path|"/" ) );
	noFile=FALSE; // Found image file.
        break;
      }
    }
    if ( noFile && fileName!="guest" ) {
      // (**: Using guest filename for record that doesn't have face file)
      // Still no file.  Just for fun let's try guest.
      // SGI Mountain View campus maintain variety of guest faces.
      fileName = "guest";
    } else {
      // Even guest file doesn't exist.  Give up.
      noFile=FALSE;
    }
  }


  if ( _faceImg->imgFile() &&
       _faceImg->imgFile()->getFileName() &&
       _faceImg->imgFile()->getFileName()==fileName )
    return;  // Same one, done.

  _faceImg->open( fileName );
  _faceImg->show();

  XtVaSetValues( *_nameField,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, XtParent( _faceImg->baseWidget() ),
      NULL );
  XtVaSetValues( *_phone2Field,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, XtParent( _faceImg->baseWidget() ),
      NULL );
  XtVaSetValues( *_mobilePhoneField,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, XtParent( _faceImg->baseWidget() ),
      NULL );
  XtVaSetValues( *_emailField,
      XmNrightAttachment, XmATTACH_WIDGET,
      XmNrightWidget, XtParent( _faceImg->baseWidget() ),
      NULL );
  
}

void
CardView::hideFaceImg()
{
  if ( _faceImg )
    _faceImg->hide();

  XtVaSetValues( *_nameField,
      XmNrightAttachment, XmATTACH_FORM,
      NULL );
  XtVaSetValues( *_phone2Field,
      XmNrightAttachment, XmATTACH_FORM,
      NULL );
  XtVaSetValues( *_mobilePhoneField,
      XmNrightAttachment, XmATTACH_FORM,
      NULL );
  XtVaSetValues( *_emailField,
      XmNrightAttachment, XmATTACH_FORM,
      NULL );
}



void
CardView::setCardInfo( CardInfo* c )
{
  assert( c );

  // Save previous card info.
  saveAndDeleteCardInfo();

  // Populate view widgets with new info.
  _cardInfo = c;

  _nameField->t()->setValue( _cardInfo->name() );
  _phone1Field->t()->setValue( _cardInfo->phone1() );
  _phone2Field->t()->setValue( _cardInfo->phone2() );
  _faxField->t()->setValue( _cardInfo->fax() );
  _mobilePhoneField->t()->setValue( _cardInfo->mobilePhone() );
  _emailField->t()->setValue( _cardInfo->email() );

  _addressLine1Field->t()->setValue( _cardInfo->addressLine1() );
  _addressLine2Field->t()->setValue( _cardInfo->addressLine2() );
  _cityField->t()->setValue( _cardInfo->city() );
  _stateField->t()->setValue( _cardInfo->state() );
  _zipField->t()->setValue( _cardInfo->zip() );
  _countryField->t()->setValue( _cardInfo->country() );

  for ( int i=0; i<ATTR_COUNT; i++ )
    attributes()->_attributeToggles[ i ]->setValue( _cardInfo->attribute( i ) );
    
  _notes->t()->setValue( _cardInfo->notes() );

  if ( thePrefFile->showFaceImg() )
    showFaceImg();
  else
    hideFaceImg();

  clearDirty();

  // Bring it up.
  setIconName( _cardInfo->name() );
  open();
  show();
}


void
CardView::save( Boolean redrawFace )
{
  if ( ! _cardInfo )
    return;  // Nothing to save.

  if ( dirty() ) {
    // Save changes made to the view to info.
    _cardInfo->setName( _nameField->t()->getValue() );
    _cardInfo->setPhone1( _phone1Field->t()->getValue() );
    _cardInfo->setPhone2( _phone2Field->t()->getValue() ); 
    _cardInfo->setFax( _faxField->t()->getValue() );
    _cardInfo->setMobilePhone( _mobilePhoneField->t()->getValue() );
    _cardInfo->setEmail( _emailField->t()->getValue() );

    _cardInfo->setAddressLine1( _addressLine1Field->t()->getValue() );
    _cardInfo->setAddressLine2( _addressLine2Field->t()->getValue() );
    _cardInfo->setCity( _cityField->t()->getValue() );
    _cardInfo->setState( _stateField->t()->getValue() );
    _cardInfo->setZip( _zipField->t()->getValue() );
    _cardInfo->setCountry( _countryField->t()->getValue() );

    for ( int i=0; i<ATTR_COUNT; i++ )
      _cardInfo->setAttribute( i, attributes()->
				  _attributeToggles[ i ]->getValue() );

    _cardInfo->setNotes( _notes->t()->getValue() );

    _cardInfo->save();

    if ( redrawFace && thePrefFile->showFaceImg() )
      showFaceImg();

    // Synch the info on list view.
    theListView->updateCardListFromCardInfo( cardPos(), _cardInfo );
  }
}

void
CardView::saveAndDeleteCardInfo()
{
  save();
  delete _cardInfo;
  _cardInfo = NULL;
}
