/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// Database.c++

#include "Database.h"
#include "PrefFile.h"
#include "CardInfo.h"
#include "SetupAttributes.h"
#include "SetupFields.h"
#include "SrchDialog.h"
#include "ListView.h"

#include <Vk/VkInterruptDialog.h>
#include <Vk/VkApp.h>

#include <iostream.h>
#include <stdio.h>

Database *theDatabase = NULL;

Database::Database( const char* dbName ) :
  Parody( (char *) dbName ), _cardDirectory( NULL )
{
  // Make sure we shut down previously opened db.
  delete theDatabase;
  theDatabase = this;

  // Instantiate SetupFields and SetupAttributes.
  new SetupFields;
  assert( theSetupFields );
  new SetupAttributes;
  assert( theSetupAttributes );

  _cardDirectory = new CardDirectory();
  assert( _cardDirectory );

  thePrefFile->setLastRolodex( dbName );  
}

Database::~Database()
{
  delete _cardDirectory;
  delete theSetupFields;
  delete theSetupAttributes;
}

void
Database::save()
{
  _cardDirectory->SaveObject();

  Flush();
}


CardInfo*
Database::cardInfo( unsigned pos )
{
  assert( pos );

  CardInfo* cInfo = new CardInfo( _cardDirectory->objAddr( pos ) );
  assert( cInfo && cInfo->ObjectExists() );

  return cInfo;
}

CardInfo*
Database::addCard( const char* name, unsigned pos )
{
  assert( name );

  CardInfo* cInfo = new CardInfo( name );
  return addCard( cInfo, pos );
}

CardInfo*
Database::addCard( CardInfo* cInfo, unsigned pos )
{
  assert( cInfo );

  _cardDirectory->add( cInfo->ObjectAddress(), pos );

  return cInfo;
}

void
Database::deleteCard( unsigned pos, CardInfo* cInfo )
{
  if ( cInfo ) {
    assert( cInfo->ObjectExists() );
    cInfo->DeleteObject(); 
  } else {
    CardInfo cInfo( _cardDirectory->objAddr( pos ) );
    assert( cInfo.ObjectExists() );
    cInfo.DeleteObject();
  }

  _cardDirectory->remove( pos );
}


void
Database::deleteAllCards()
{
  for ( int i=1; i<=totalCards(); i++ ) {
    CardInfo cInfo( _cardDirectory->objAddr( i ) );
    assert( cInfo.ObjectExists() );
    cInfo.DeleteObject();
  }
  _cardDirectory->removeAll();  
}




void
Database::load( OkStr& listHeader, OkStrArray& listItems )
{
  // Load the list header.
  char buf[ BUFSIZ ];  // This buffer could overflow, and crash the program !
  for ( register int i=0; i<BUFSIZ; i++ ) buf[ i ] = ' ';

  char* bufPtr = buf;
  for ( i=SetupFields::NAME; i<SetupFields::TOTAL_FIELDS; i++ ) 
    if ( theSetupFields->showInList( (SetupFields::Field) i ) ) {
      strncpy( bufPtr, theSetupFields->fieldLabel( (SetupFields::Field) i ),
	       strlen( theSetupFields->fieldLabel( (SetupFields::Field) i ) ) );
      bufPtr += theSetupFields->fieldLen( (SetupFields::Field) i );
      *bufPtr = ' ';
      bufPtr++;
    }
  *bufPtr = NULL;
  listHeader = buf;

  // Load the list items.
  listItems.resize( 0 );

  for ( int j=0; j<totalCards(); j++ ) {
    CardInfo cInfo( _cardDirectory->objAddr( j+1 ) );
    OkStr item;
    Load( item, &cInfo );
    listItems.append( item );
  } // for.
}


void
Database::Load( OkStr& item, const CardInfo* cInfo )
{
    assert( cInfo );

    char buf[ BUFSIZ ];  // This buffer could overflow, and crash the program !
    for ( register int i=0; i<BUFSIZ; i++ ) buf[ i ] = ' ';
    char* bufPtr = buf;

    if ( theSetupFields->showInList( SetupFields::NAME ) ) {
      size_t slen = strlen( cInfo->name() );
      size_t flen = theSetupFields->fieldLen( SetupFields::NAME );
      strncpy( bufPtr, cInfo->name(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::NAME );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::PHONE1 ) ) {
      size_t slen = strlen( cInfo->phone1() );
      size_t flen = theSetupFields->fieldLen( SetupFields::PHONE1 );
      strncpy( bufPtr, cInfo->phone1(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::PHONE1 );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::PHONE2 ) ) {
      size_t slen = strlen( cInfo->phone2() );
      size_t flen = theSetupFields->fieldLen( SetupFields::PHONE2 );
      strncpy( bufPtr, cInfo->phone2(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::PHONE2 );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::FAX ) ) {
      size_t slen = strlen( cInfo->fax() );
      size_t flen = theSetupFields->fieldLen( SetupFields::FAX );
      strncpy( bufPtr, cInfo->fax(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::FAX );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::MOBILE_PHONE ) ) {
      size_t slen = strlen( cInfo->mobilePhone() );
      size_t flen = theSetupFields->fieldLen( SetupFields::MOBILE_PHONE );
      strncpy( bufPtr, cInfo->mobilePhone(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::MOBILE_PHONE );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::EMAIL ) ) {
      size_t slen = strlen( cInfo->email() );
      size_t flen = theSetupFields->fieldLen( SetupFields::EMAIL );
      strncpy( bufPtr, cInfo->email(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::EMAIL );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::ADDRESS_LINE1 ) ) {
      size_t slen = strlen( cInfo->addressLine1() );
      size_t flen = theSetupFields->fieldLen( SetupFields::ADDRESS_LINE1 );
      strncpy( bufPtr, cInfo->addressLine1(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::ADDRESS_LINE1 );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::ADDRESS_LINE2 ) ) {
      size_t slen = strlen( cInfo->addressLine2() );
      size_t flen = theSetupFields->fieldLen( SetupFields::ADDRESS_LINE2 );
      strncpy( bufPtr, cInfo->addressLine2(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::ADDRESS_LINE2 );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::CITY ) ) {
      size_t slen = strlen( cInfo->city() );
      size_t flen = theSetupFields->fieldLen( SetupFields::CITY );
      strncpy( bufPtr, cInfo->city(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::CITY );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::STATE ) ) {
      size_t slen = strlen( cInfo->state() );
      size_t flen = theSetupFields->fieldLen( SetupFields::STATE );
      strncpy( bufPtr, cInfo->state(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::STATE );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::ZIP ) ) {
      size_t slen = strlen( cInfo->zip() );
      size_t flen = theSetupFields->fieldLen( SetupFields::ZIP );
      strncpy( bufPtr, cInfo->zip(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::ZIP );
      *bufPtr = ' ';
      bufPtr++;
    }

    if ( theSetupFields->showInList( SetupFields::COUNTRY ) ) {
      size_t slen = strlen( cInfo->country() );
      size_t flen = theSetupFields->fieldLen( SetupFields::COUNTRY );
      strncpy( bufPtr, cInfo->country(), (slen<flen?slen:flen) );
      bufPtr += theSetupFields->fieldLen( SetupFields::COUNTRY );
      *bufPtr = ' ';
      bufPtr++;
    }

    *bufPtr = NULL;

    item = buf;
}


//////////////////////////////////////////////////////////
// CardDirectory implementation

CardDirectory::CardDirectory() :
  Persistent( *theDatabase, CARD_DIRECTORY ), 
  _key( CARD_DIRECTORY ),
  _sortOrder( ASCENDING )
{
  LoadObject();

  if ( ! ObjectExists() )
    AddObject();
}

CardDirectory::~CardDirectory()
{
  SaveObject();
}


void
CardDirectory::Read()
{
  register int i;
  ReadObject( i );
  _sortOrder = SortOrder( i );

  unsigned len = 0;
  ReadObject( len );

  ObjAddr objAddr;
  for ( i=0; i<len; i++ ) {
    ReadObject( objAddr );
    _objAddrArray.append( objAddr );
  }

}

void
CardDirectory::Write()
{

  WriteObject( int( _sortOrder ) );
  WriteObject( _objAddrArray.length() );
  for ( int i=0; i<_objAddrArray.length(); i++ ) {
    WriteObject( _objAddrArray[ i ] );
  }
}


void
CardDirectory::add( ObjAddr objAddr, unsigned pos )
{
  if ( pos )
    _objAddrArray.insert( objAddr, pos-1 );
  else
    _objAddrArray.append( objAddr );

  ChangeObject();
}

void
CardDirectory::removeAll()
{
  _objAddrArray.resize( 0 );
  ChangeObject();
}

void
CardDirectory::remove( unsigned pos )
{
  _objAddrArray.remove( pos-1 );

  ChangeObject();
}


void
CardDirectory::sort( SortOrder sortOrder )
{
  // New sort order.
  _sortOrder = sortOrder;

  ObjAddrArray tmpObjAddrArray = _objAddrArray;

  // The corresponding list widget has been cleared in ListView::sort.
  _objAddrArray.resize( 0 );

  // I need to minimize disk accesses, so use the viewer to re-sort.
  // (actually deleting everything and readding to the empty viewer)
  for ( int i=0; i<tmpObjAddrArray.length(); i++ ) {
    CardInfo cInfo( tmpObjAddrArray[ i ] );
    assert( cInfo.ObjectExists() );
    CardInfo* cInfoPtr = &cInfo;
    OkStr item;
    Database::Load( item, cInfoPtr );
    theListView->addCard( item, &cInfoPtr );

    if ( (i%50)==0 )
      theApplication->progressing( OkStr( i, "Re-sorting %d..." ) );
  }

  ChangeObject();
}


int
Database::searchCard( OkStr s, int from, Direction direction,
                        Boolean caseSensitive, Boolean srchNotesToo )
{
  // Card number starts from 1.
  // Treat everything less than 1 to mean search from the beginning.
  if ( from<1 )
    from=1;

  if ( direction == FORWARD ) {

    from++; // Start with the next one.
    while( from<=totalCards() ) {

      // Check if user interrupt the search.
      if ( theSrchDialog->allowInterruption() &&
           theInterruptDialog->wasInterrupted() )
        return -1;

      CardInfo cInfo( _cardDirectory->objAddr( from ) );
      assert( cInfo.ObjectExists() );

      if ( cInfo.containString( s, caseSensitive, srchNotesToo ) ) {
        return from;
      }

      from++;
    }

  } else {

    from--; // Start with the one before this one.
    while( from>0 ) {

      // Check if user interrupt the search.
      if ( theSrchDialog->allowInterruption() &&
           theInterruptDialog->wasInterrupted() )
        return -1;

      CardInfo cInfo( _cardDirectory->objAddr( from ) );
      assert( cInfo.ObjectExists() );

      if ( cInfo.containString( s, caseSensitive, srchNotesToo ) ) {
        return from;
      }

      from--;
    }
  }

  // Not found.
  return 0;
}



