/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// GeneralPrefs.c++

#include "GeneralPrefs.h"
#include "PrefFile.h"
#include "OkFile.h"
#include "OkArgs.h"
#include "OkStr.h"

#include <Vk/VkErrorDialog.h>


GeneralPrefDialog::GeneralPrefDialog() :
  VkPrefDialog( "generalPrefDialog" ) 
{
  _faceImgLoc = new VkPrefText( "faceImgLoc" );
  _defRolodexLoc = new VkPrefText( "defRolodexLoc" );

  VkPrefGroup* textGroup = new VkPrefGroup( "textGroup", FALSE, TRUE );
  textGroup->addItem( _defRolodexLoc );
  textGroup->addItem( _faceImgLoc );

  _showFaceImg = new VkPrefToggle( "showFaceImg" );
  _alwaysUseIntrDialog = new VkPrefToggle( "alwaysUseIntrDialog" );

  VkPrefGroup* toggleGroup = new VkPrefGroup( "toggleGroup", FALSE, TRUE );
  toggleGroup->addItem( _showFaceImg );
  toggleGroup->addItem( _alwaysUseIntrDialog );

  VkPrefList *prefList = new VkPrefList( "prefList" );
  prefList->addItem( textGroup );
  prefList->addItem( new VkPrefSeparator( "separator" ) );
  prefList->addItem( toggleGroup );

  setItem( prefList );


  // Get setting from PrefFile.
  _faceImgLoc->setValue( thePrefFile->faceImgLoc() );
  _defRolodexLoc->setValue( thePrefFile->dataFilePath() );

  _showFaceImg->setValue( thePrefFile->showFaceImg() );
  _alwaysUseIntrDialog->setValue( thePrefFile->alwaysUseIntrDialog() );
}

const char* 
GeneralPrefDialog::faceImgLoc() const
{
  static char* tmps = NULL;
  delete tmps;
  tmps = OkFile::PathExpandTilde( _faceImgLoc->getValue() );

  return tmps;
}

const char* 
GeneralPrefDialog::defRolodexLoc() const
{
  static char* tmps = NULL;
  delete tmps;
  tmps = OkFile::PathExpandTilde( _defRolodexLoc->getValue() );

  return tmps;
}

Widget
GeneralPrefDialog::createDialog( Widget parent )
{
  return VkPrefDialog::createDialog( parent );
}
