/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// PrefFile.c++

#include "PrefFile.h"
#include "OkFile.h"

#include <stdio.h>
#include <assert.h>

PrefFile* thePrefFile = new PrefFile;


PrefFile::PrefFile()
{
  setFactoryDefault();

  char *tmps;
  tmps = OkFile::PathExpandTilde( "~/.xcontactPrefs" );
  _fileName = tmps;
  delete tmps;

  _file = new OkFile( _fileName, ios::in );
  assert( _file );

  if ( ! _file->file() ) {
    // File not found.
    _notFound = TRUE;
  } else {
    _notFound = FALSE;

    // Load the contents.
    _file->read( this );
    _file->close();
  }

}

PrefFile::~PrefFile()
{
}


void
PrefFile::setFactoryDefault()
{
  _lastRolodex = "";
  char *tmps;
  tmps = OkFile::PathExpandTilde( "~/.XContact/" );
  _dataFilePath = tmps;
  delete tmps;

  _faceImgLoc = "/usr/lib/faces/;/usr/local/lib/faces/";
  _showFaceImg = TRUE;

  setAscendingSortOrder();

  _listViewWidth = _listViewHeight = 0;
  _activeCardViewWidth = _activeCardViewHeight = 0;
}


Boolean
PrefFile::save()
{
  // Dump prefs back to file.

  _file->open( _fileName, ios::out );
  if ( ! _file->file() )
    return FALSE;  // Cannot.

  _file->write( this );
  _file->close();

  return TRUE;
}

void
PrefFile::read( fstream& infile )
{
  char buf[ BUFSIZ ];
  OkStr line;

  while ( ! infile.eof() ) {
    infile.getline( buf, BUFSIZ );
    line = buf;

    u_int pos=0;
    OkStr token = line.token( pos, '=' );
    line.remove( 0, pos );

    if ( token == "DATAFILEPATH" )
      _dataFilePath = line;
    else if ( token == "FACEIMGLOC" ) {
      _faceImgLoc = line;
      setFaceImgLoc( _faceImgLoc );
    } else if ( token == "SHOWFACEIMG" )
      _showFaceImg = int( line );
    else if ( token == "LASTROLODEX" )
      _lastRolodex = line;

    else if ( token == "SORTORDER" )
      _sortOrder = line;

    else if ( token == "LISTVIEWWIDTH" )
      _listViewWidth = int( line );
    else if ( token == "LISTVIEWHEIGHT" )
      _listViewHeight = int( line );
    else if ( token == "ACTIVECARDVIEWWIDTH" )
      _activeCardViewWidth = int( line );
    else if ( token == "ACTIVECARDVIEWHEIGHT" )
      _activeCardViewHeight = int( line );

    else 
      cerr << "Warning: Unknown preference " << (char*) token << " ignored.\n";
  }

}

void
PrefFile::write( fstream& outfile )
{
  putline( outfile, "DATAFILEPATH", _dataFilePath );
  putline( outfile, "FACEIMGLOC", _faceImgLoc );
  putline( outfile, "SHOWFACEIMG", _showFaceImg );
  putline( outfile, "LASTROLODEX", _lastRolodex );

  putline( outfile, "SORTORDER", _sortOrder );

  putline( outfile, "LISTVIEWWIDTH", _listViewWidth );
  putline( outfile, "LISTVIEWHEIGHT", _listViewHeight );
  putline( outfile, "ACTIVECARDVIEWWIDTH", _activeCardViewWidth );
  putline( outfile, "ACTIVECARDVIEWHEIGHT", _activeCardViewHeight );
}


void
PrefFile::putline( fstream& outfile, const char* pref, const char* prefValue )
{
  outfile << pref << "=" << prefValue << endl;
}

void
PrefFile::putline( fstream& outfile, const char* pref, int prefValue )
{
  outfile << pref << "=" << prefValue << endl;
}

void
PrefFile::setFaceImgLoc( const char* path )
{
  _faceImgLoc = path;
  _faceImgLocList.resize( 0 );
  u_int pos=0;
  for (;;) {
    OkStr loc = _faceImgLoc.ptoken( pos, ';' );

    // Remove leading spaces.
    while( loc.length() && loc[ 0 ] == ' ' )
      loc.cut( 0, 1 );
    // Remove trailing spaces.
    while( loc.length() && loc[ loc.length()-1 ] == ' ' )
      loc.cut( loc.length()-1, 1 );

    _faceImgLocList.append( loc );

    if ( pos>=_faceImgLoc.length() ) 
      break;  // Done parsing.
  } 
}

const char*
PrefFile::faceImgLoc( int i )
{
  if ( i>=_faceImgLocList.length() )
    return "";
  else
    return _faceImgLocList[ i ];
}
