/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// Print.c++

#include "Print.h"
#include "CardView.h"
#include "ListView.h"
#include "SetupAttributes.h"
#include "CardList.h"

#include "OkArgs.h"

#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>

PrintCardDialog* thePrintCardDialog = new PrintCardDialog;


void
RolodexInfoToggle::valueChanged( Boolean )
{
  thePrintCardDialog->toggleValuesChanged();
}

void
NotesToggle::valueChanged( Boolean )
{
  thePrintCardDialog->toggleValuesChanged();
}

PrintCardDialog::PrintCardDialog() :
  VkPrintDialog( "printCardDialog" )
{
  _showFileName = FALSE;
  _showHelp = FALSE;

  _jobType = VkPrintDialog::BUFFER;
}


Widget
PrintCardDialog::createWorkArea( Widget parent )
{
  if ( _view == NULL )
    return NULL;	// work area not needed.

  OkArgs args( 4 );
  Widget form = XmCreateForm( parent, "form", 
				args.list(), args.count() );

  args.reset();
  args.set( XmNtopAttachment, XmATTACH_FORM );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  _label = new OkLabel( "label", form, args.list(), args.count() );
  _label->setLabelString( _printLabel );
  _label->show();

  args.reset();
  args.set( XmNorientation, XmHORIZONTAL );
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _label->baseWidget() );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  Widget rowColumn = XmCreateRowColumn( form, "rowColumn",
				args.list(), args.count() );
  XtManageChild( rowColumn );

  args.reset();
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, rowColumn );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  Widget separator = XmCreateSeparator( form, "separator",
				args.list(), args.count() );
  XtManageChild( separator );


  _rolodexInfoToggle = new RolodexInfoToggle( rowColumn );
  _rolodexInfoToggle->setValue( TRUE );
  _rolodexInfoToggle->show();

  _notesToggle = new NotesToggle( rowColumn );
  _notesToggle->show();


  return form;
}


void
PrintCardDialog::printCardView( const CardView* view )
{
  _view = view;

  if ( _view ) {

    // Print Card.
    setTitle( "Print Card" );

    if ( _workArea )
      XtManageChild( _workArea );

    _printLabel = "Print " | view->name() | ":";

    if ( _label )
      _label->setLabelString( _printLabel );
    if ( _rolodexInfoToggle )
      _rolodexInfoToggle->setValue( TRUE );
    if ( _notesToggle )
      _notesToggle->setValue( FALSE );

    _printBuf = "";
    composeRolodexInfo();
    setBuffer( _printBuf, _printBuf.length() );

  } else {

    // Print List.
    setTitle( "Print List" );

    if ( _workArea )
      XtUnmanageChild( _workArea );

    _printBuf = "";
    for ( int i=1; i<=theListView->totalCards(); i++ ) {
      _printBuf.append( theListView->cardList()->item( i ) );
      _printBuf.append( "\n" );
    }

    setBuffer( _printBuf, _printBuf.length() );
  }

  postBlocked( NULL,NULL,NULL,NULL,NULL,NULL, (VkSimpleWindow*) view );
}

void
PrintCardDialog::composeRolodexInfo()
{
  _printBuf.append( "        Name: " );
  _printBuf.append( _view->name() );
  _printBuf.append( "\n" );
  _printBuf.append( "      Phone1: " );
  _printBuf.append( _view->phone1() );
  _printBuf.append( "\n" );
  _printBuf.append( "      Phone2: " );
  _printBuf.append( _view->phone2() );
  _printBuf.append( "\n" );
  _printBuf.append( "         Fax: " );
  _printBuf.append( _view->fax() );
  _printBuf.append( "\n" );
  _printBuf.append( "Mobile Phone: " );
  _printBuf.append( _view->mobilePhone() );
  _printBuf.append( "\n" );
  _printBuf.append( "      E-Mail: " );
  _printBuf.append( _view->email() );
  _printBuf.append( "\n\n" );
  _printBuf.append( "     Address: " );
  _printBuf.append( _view->addressLine1() );
  _printBuf.append( "\n" );
  _printBuf.append( "              " );
  _printBuf.append( _view->addressLine2() );
  _printBuf.append( "\n" );
  _printBuf.append( "        City: " );
  _printBuf.append( _view->city() );
  _printBuf.append( "\n" );
  _printBuf.append( "       State: " );
  _printBuf.append( _view->state() );
  _printBuf.append( "\n" );
  _printBuf.append( "         Zip: " );
  _printBuf.append( _view->zip() );
  _printBuf.append( "\n" );
  _printBuf.append( "     Country: " );
  _printBuf.append( _view->country() );
  _printBuf.append( "\n\n" );
  _printBuf.append( "  Attributes: " );
  for ( int i=0; i<ATTR_COUNT; i++ ) 
    if ( _view->attributeValue( i ) ) {
      _printBuf.append( theSetupAttributes->attrLabel( i ) );
      _printBuf.append( "\n              " );
    }
}


void
PrintCardDialog::composeNotes()
{
  _printBuf.append( "\nNotes:\n\n" );
  _printBuf.append( _view->notes() );
}


void
PrintCardDialog::toggleValuesChanged()
{
  _printBuf = "";

  if ( _rolodexInfoToggle->getValue() )
    composeRolodexInfo();
  if ( _notesToggle->getValue() )
    composeNotes();

  setBuffer( _printBuf, _printBuf.length() );
}
