/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// SetupAttributes.c++

#include "SetupAttributes.h"

#include "OkFile.h"
#include "OkArgs.h"
#include "OkStr.h"

#include <Vk/VkErrorDialog.h>

#include <assert.h>

SetupAttributes* theSetupAttributes = NULL;

SetupAttributes::SetupAttributes() :
  VkPrefDialog( "setAttrPrefDialog" ),
  Persistent( *theDatabase, SETUP_ATTRIBUTES ),
  _key( SETUP_ATTRIBUTES )
{
  assert( ! theSetupAttributes );
  theSetupAttributes = this;

  _someOfMany = new VkPrefToggle( "someOfMany" );
  _oneOfMany = new VkPrefToggle( "oneOfMany" );
  VkPrefGroup *attrStyleGroup = new VkPrefRadio( "attrStyleGroup", TRUE );
  attrStyleGroup->addItem( _someOfMany );
  attrStyleGroup->addItem( _oneOfMany );


  VkPrefGroup *showAttrGroup = new VkPrefGroup( "showAttrGroup", TRUE, TRUE );
  showAttrGroup->addItem( new VkPrefLabel( "showAttrLabel" ) );
  for ( int i=0; i<ATTR_COUNT; i++ ) {
    _showAttr[ i ] = new VkPrefToggle( OkStr( i, "%d" ) );
    showAttrGroup->addItem( _showAttr[ i ] );
  }


  VkPrefGroup *attrLabelGroup = new VkPrefGroup( "attrLabelGroup" );
  for ( i=0; i<ATTR_COUNT; i++ ) {
    _attrLabel[ i ] = new VkPrefText( OkStr( i, "attribute%d" ) );
    attrLabelGroup->addItem( _attrLabel[ i ] );
  }


  VkPrefList *prefList = new VkPrefList( "prefList" );
  prefList->addItem( attrStyleGroup );
  prefList->addItem( new VkPrefSeparator( "separator" ) );
  prefList->addItem( showAttrGroup );
  prefList->addItem( new VkPrefSeparator( "separator" ) );
  prefList->addItem( attrLabelGroup );

  // Attach it to dialog.
  setItem( prefList );

  setFactoryDefault();

  LoadObject();
}


void
SetupAttributes::setFactoryDefault()
{
  // Factory default setting.
  _someOfMany->setValue( TRUE );
  _oneOfMany->setValue( FALSE );

  for ( int i=0; i<ATTR_COUNT; i++ ) {
    _showAttr[ i ]->setValue( TRUE );
    _attrLabel[ i ]->setValue( OkStr( i, "Attribute %d" ) );
  }
}

SetupAttributes::~SetupAttributes()
{
  theSetupAttributes = NULL;
}

Widget
SetupAttributes::createDialog( Widget parent )
{
  return VkPrefDialog::createDialog( parent );
}

void
SetupAttributes::Read()
{
  Boolean b;
  ReadObject( b );
  _oneOfMany->setValue( b );
  _someOfMany->setValue( ! b );

  char buf[ BUFSIZ ];
  for ( int i=0; i<ATTR_COUNT; i++ ) {

    ReadObject( b );
    _showAttr[ i ]->setValue( b );

    ReadObject( buf );
    _attrLabel[ i ]->setValue( buf );
  }
}

void
SetupAttributes::Write()
{
  WriteObject( radioStyle() );

  for ( int i=0; i<ATTR_COUNT; i++ ) {
    WriteObject( showAttr( i ) );
    WriteObject( attrLabel( i ) );
  }
}


void
SetupAttributes::save()
{
  if ( ObjectExists() )
    ChangeObject();
  else
    AddObject();

  SaveObject();
}
