/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// SetupFields.c++

#include "SetupFields.h"

#include "OkArgs.h"

// Motif includes
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/SelectioB.h>

// Vk includes.
#include <Vk/VkApp.h>
#include <Vk/VkQuestionDialog.h>
#include <Vk/VkErrorDialog.h>

#include <iostream.h>
#include <assert.h>

SetupFields* theSetupFields = NULL;

VkMenuDesc 
SetupFields::_fieldsDesc[] = {
  { ACTION, "Name", &SetupFields::setupNameCB },
  { ACTION, "Phone 1", &SetupFields::setupPhone1CB },
  { ACTION, "Phone 2", &SetupFields::setupPhone2CB },
  { ACTION, "Fax", &SetupFields::setupFaxCB },
  { ACTION, "Mobile Phone", &SetupFields::setupMobilePhoneCB },
  { ACTION, "E-Mail", &SetupFields::setupEmailCB },
  { ACTION, "Address Line 1", &SetupFields::setupAddressLine1CB },
  { ACTION, "Address Line 2", &SetupFields::setupAddressLine2CB },
  { ACTION, "City", &SetupFields::setupCityCB },
  { ACTION, "State", &SetupFields::setupStateCB },
  { ACTION, "Zip", &SetupFields::setupZipCB },
  { ACTION, "Country", &SetupFields::setupCountryCB },
  { END }
};


SetupFields::SetupFields() : 
  VkGenericDialog( "SetupFields" ), 
  Persistent( *theDatabase, SETUP_FIELDS ),
  _key( SETUP_FIELDS )
{
  // Init global.
  assert( ! theSetupFields );
  theSetupFields = this;

  _showApply = TRUE;

  _fieldsOptionMenu = NULL;
  _fieldLabelWidget = NULL;
  _fieldLenWidget = NULL;
  _showInListToggle = NULL;

  setFactoryDefault();

  LoadObject();

  loadSavedValues();
}


SetupFields::~SetupFields()
{
  delete _fieldsOptionMenu;
  delete _fieldLabelWidget;
  delete _fieldLenWidget;
  delete _showInListToggle;

  theSetupFields = NULL;
}

void
SetupFields::save()
{
  if ( ObjectExists() )
    ChangeObject();
  else
    AddObject();

  SaveObject();
}


void
SetupFields::setFactoryDefault()
{
  theApplication->busy();

  _showInList[ NAME ] = TRUE;
  _fieldLabel[ NAME ] = "Name:";
  _fieldLen[ NAME ] = 20;

  _showInList[ PHONE1 ] = TRUE;
  _fieldLabel[ PHONE1 ] = "Phone 1:";
  _fieldLen[ PHONE1 ] = 20;

  _fieldLabel[ PHONE2 ] = "Phone 2:";
  _fieldLabel[ FAX ] = "Fax:";
  _fieldLabel[ MOBILE_PHONE ] = "Mobile Phone:";
  _fieldLabel[ EMAIL ] = "E-Mail:";
  _fieldLabel[ ADDRESS_LINE1 ] = "Address:";
  _fieldLabel[ ADDRESS_LINE2 ] = "";
  _fieldLabel[ CITY ] = "City:";
  _fieldLabel[ STATE ] = "State:";
  _fieldLabel[ ZIP ] = "Zip:";
  _fieldLabel[ COUNTRY ] = "Country:";

  for ( int i=PHONE2; i<TOTAL_FIELDS; i++ ) {
    _showInList[ i ] = FALSE;
    _fieldLen[ i ] = 20;
  }

  _prevField = NONE;

  theApplication->notBusy();
}

void
SetupFields::Read()
{
  pString s;
  for ( int i=0; i<TOTAL_FIELDS; i++ ) {
    ReadObject( _showInList[ i ] );
    ReadObject( s );
    _fieldLabel[ i ] = s;
    ReadObject( _fieldLen[ i ] );
  }
}

void
SetupFields::Write()
{
  for ( int i=0; i<TOTAL_FIELDS; i++ ) {
    WriteObject( _showInList[ i ] );
    WriteObject( pString( (char *)_fieldLabel[ i ] ) );
    WriteObject( _fieldLen[ i ] );
  }
}


Widget
SetupFields::createDialog( Widget parent )
{
  OkArgs args( 4 );
  args.set( XmNautoUnmanage, FALSE );
  Widget dialog = XmCreatePromptDialog( parent, _name, 
					args.list(), args.count() );

  // Remove unneeded children.
  XtUnmanageChild(XmSelectionBoxGetChild(dialog, XmDIALOG_TEXT));
  XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_SELECTION_LABEL));

  Widget form = XmCreateForm( dialog, "form", NULL, 0 );
  XtManageChild( form );

  args.reset();
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNtopAttachment, XmATTACH_FORM );
  Widget setupFieldLabel = XmCreateLabel( form, "setupFieldLabel", 
				args.list(), args.count() );
  XtManageChild( setupFieldLabel );

  _fieldsOptionMenu = new VkOptionMenu( form, _fieldsDesc, XtPointer( this ) );
  XtVaSetValues( *_fieldsOptionMenu,
    XmNleftAttachment, XmATTACH_WIDGET,
    XmNleftWidget, setupFieldLabel,
    XmNtopAttachment, XmATTACH_FORM,
    NULL );
  _fieldsOptionMenu->show();

  args.reset();
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNrightAttachment, XmATTACH_FORM );
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _fieldsOptionMenu->baseWidget() );
  Widget frame = XmCreateFrame( form, "frame", args.list(), args.count() );
  XtManageChild( frame );

  Widget frameBase = XmCreateForm( frame, "frameBase", NULL, 0 );
  XtManageChild( frameBase );

  _fieldLabelWidget = new OkLabeledComponent<OkText>( "fieldLabelWidget",
							frameBase );
  XtVaSetValues( *_fieldLabelWidget,
    XmNleftAttachment, XmATTACH_FORM,
    XmNtopAttachment, XmATTACH_FORM,
    NULL );
  _fieldLabelWidget->show();

  _fieldLenWidget = new OkLabeledComponent<OkText>( "fieldLenWidget",
							frameBase );
  XtVaSetValues( *_fieldLenWidget,
    XmNleftAttachment, XmATTACH_FORM,
    XmNtopAttachment, XmATTACH_WIDGET,
    XmNtopWidget, _fieldLabelWidget->baseWidget(),
    NULL );
  _fieldLenWidget->show();

  args.reset();
  args.set( XmNrightAttachment, XmATTACH_FORM );
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _fieldLenWidget->baseWidget() );
  _showInListToggle = new OkToggleButton( "showInListToggle", frameBase,
						args.list(), args.count() );
  _showInListToggle->show();

  setup();

  return dialog;
}


void
SetupFields::loadSavedValues()
{
  for ( int i=0; i<TOTAL_FIELDS; i++ ) {
    _newShowInList[ i ] = _showInList[ i ];
    _newFieldLabel[ i ] = _fieldLabel[ i ];
    _newFieldLen[ i ] = _fieldLen[ i ];
  }
}

void
SetupFields::saveCurrentValues()
{
  for ( int i=0; i<TOTAL_FIELDS; i++ ) {
    _showInList[ i ] = _newShowInList[ i ];
    _fieldLabel[ i ] = _newFieldLabel[ i ];
    _fieldLen[ i ] = _newFieldLen[ i ];
  }
}


void
SetupFields::populateWidgets( Field f )
{
  if ( f<NAME && f>COUNTRY )
    return;

  _fieldLabelWidget->component()->setValue( _newFieldLabel[ f ] );
  _fieldLenWidget->component()->setValue( OkStr( _newFieldLen[ f ], "%d" ) );
  _showInListToggle->setValue( _newShowInList[ f ] );
}

Boolean
SetupFields::getCurrentUserInput( Field f )
{
  if ( f<NAME && f>COUNTRY )
    return FALSE;

  OkStr s = _fieldLenWidget->component()->getValue();
  int len = int( s );
  if ( len<1 ) {
    s = "Bad value for Field Length: " | s;
    theErrorDialog->postBlocked( s );
    return FALSE;
  }

  _newFieldLen[ f ] = len;
  _newFieldLabel[ f ] = _fieldLabelWidget->component()->getValue();
  _newShowInList[ f ] = _showInListToggle->getValue();
  
  return TRUE;
}


void
SetupFields::setup( Field f )
{
  if ( _prevField==f )
    return; // No change.

  if ( _prevField==NONE )
    _fieldsOptionMenu->set( f );
  else  
    if ( ! getCurrentUserInput( _prevField ) ) {
      // Illegal input found.  Let user re-enter.
      _fieldsOptionMenu->set( _prevField );
      return;
    }

  populateWidgets( f );

  if ( f==NAME )
    XtUnmanageChild( *_showInListToggle ); // Name field always shown in list.
  else
    XtManageChild( *_showInListToggle );
  
  _prevField = f;

}


void
SetupFields::ok(Widget, XtPointer)
{
  // Get the last changes, and save the new values permanently.
  if ( ! getCurrentUserInput( _prevField ) )
    return;

  unpost();
  saveCurrentValues();

  // Persistent save.
  save();
}

void
SetupFields::cancel(Widget, XtPointer)
{
  // No need to show user the works needed to cancel.
  unpost();

  theApplication->busy();

  // Revert all new changes.
  loadSavedValues();
  populateWidgets( _prevField );

  theApplication->notBusy();
}

void
SetupFields::apply(Widget, XtPointer)
{
  if ( theQuestionDialog->postAndWait( "Delete your customization ?" ) == 
							CANCEL ) 
    return;

  setFactoryDefault();  // Reset all saved values to default.
  loadSavedValues();	// Load the values to working arrays.

  // Reset and reload widgets.
  setup();
}
