/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// SrchDialog.c++

#include "SrchDialog.h"
#include "ListView.h"
#include "CardList.h"
#include "Database.h"
#include "PrefFile.h"

#include "OkArgs.h"

#include <Vk/VkSimpleWindow.h>
#include <Vk/VkInfoDialog.h>
#include <Vk/VkInterruptDialog.h>
#include <Vk/VkApp.h>

#include <Xm/SelectioB.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <assert.h>

SrchDialog* theSrchDialog = new SrchDialog;

SrchDialog::SrchDialog() : VkGenericDialog( "srchDialog" )
{
  theSrchDialog = this;
}

Widget 
SrchDialog::createDialog( Widget parent )
{
  _showApply = TRUE;
//  _allowMultipleDialogs = TRUE;
  _allowInterruption = FALSE;

  OkArgs args( 5 );
  args.set( XmNautoUnmanage, FALSE );
  Widget dialog = XmCreatePromptDialog( parent, _name, 
                                        args.list(), args.count() );
  // Remove unneeded children.
  XtUnmanageChild(XmSelectionBoxGetChild(dialog, XmDIALOG_TEXT));
  XtUnmanageChild(XmSelectionBoxGetChild(dialog,XmDIALOG_SELECTION_LABEL));

  Widget form = XmCreateForm( dialog, "form", NULL, 0 );
  XtManageChild( form );


  _srchString = new OkLabeledComponent<OkText>( "srchString", form, 
				OkLabeledComponent<OkText>::VERTICAL );
  assert( _srchString );
  XtVaSetValues( *_srchString,
    XmNtopAttachment, XmATTACH_FORM,
    XmNleftAttachment, XmATTACH_FORM,
    XmNrightAttachment, XmATTACH_FORM,
    NULL );
  _srchString->show();

  XtVaSetValues( _srchString->label(),
    XmNlabelString, 
    XmString( OkCompoundString( "Search for the card containing:" ) ),
    NULL );

  args.reset();
  args.set( XmNtopAttachment, XmATTACH_WIDGET );
  args.set( XmNtopWidget, _srchString->baseWidget() );
  args.set( XmNleftAttachment, XmATTACH_FORM );
  args.set( XmNorientation, XmHORIZONTAL );
  Widget rc = XmCreateRowColumn( form, "rc", args.list(), args.count() );
  XtManageChild( rc );

  _caseSensitive = new OkToggleButton( "caseSensitive", rc, NULL, 0 );
  _caseSensitive->show();
  _srchNotes = new OkToggleButton( "srchNotes", rc, NULL, 0 );
  _srchNotes->show();

  return dialog;

}

Widget 
SrchDialog::prepost ( const char      *message,
                             XtCallbackProc   okCB,
                             XtCallbackProc   cancelCB,
                             XtCallbackProc   applyCB,
                             XtPointer        clientData,
                             const char      *helpString,
                             VkSimpleWindow  *parentWindow )
{
  assert( parentWindow );
  _currentParentWindow = parentWindow;

  // Bring up window and dialog to top.
  _currentParentWindow->raise(); 
  unpost(); 

  setButtonLabels( "Forward", "Done", "Backward" );
  return VkDialogManager::prepost( message,
				okCB,
				cancelCB,
				applyCB,
				clientData,
				helpString,
				_currentParentWindow );
}



void
SrchDialog::srchForward()
{
  if ( OkStr( srchString() ) == "" ) {
    // Nothing to search.
    theInfoDialog->postBlocked( "Nothing to search.",
				_currentParentWindow );
    return;
  }

  if ( thePrefFile->alwaysUseIntrDialog() || srchNotes() ) {
    _allowInterruption = TRUE;
    theApplication->setBusyDialog( theInterruptDialog );
    OkStr s = "Searching \"";
    s.append( srchString() );
    s.append( "\" ..." );
    theApplication->busy( s, _currentParentWindow );
  } else {
    _allowInterruption = FALSE;
    theApplication->busy();
  }

  int cardNo = theDatabase->searchCard( srchString(),
			   theListView->currCardNo(),
			   Database::FORWARD,
			   caseSensitive(),
			   srchNotes() );

  if ( cardNo>0 ) {
    theListView->cardList()->firstVisibleItem( cardNo );
    theListView->showCard( cardNo );
  }

  theApplication->notBusy();
  theApplication->setBusyDialog( NULL );

  if ( cardNo==0 )
    theInfoDialog->postBlocked( "No card contains that string.", 
				_currentParentWindow );
}

void
SrchDialog::srchBackward()
{
  if ( OkStr( srchString() ) == "" ) {
    // Nothing to search.
    theInfoDialog->postBlocked( "Nothing to search.",
				_currentParentWindow );
    return;
  }

  if ( thePrefFile->alwaysUseIntrDialog() || srchNotes() ) {
    _allowInterruption = TRUE;
    theApplication->setBusyDialog( theInterruptDialog );
    OkStr s = "Searching \"";
    s.append( srchString() );
    s.append( "\" ..." );
    theApplication->busy( s, _currentParentWindow );
  } else {
    _allowInterruption = FALSE;
    theApplication->busy();
  }

  int cardNo = theDatabase->searchCard( srchString(),
			   theListView->currCardNo(),
			   Database::BACKWARD,
			   caseSensitive(),
			   srchNotes() );

  if ( cardNo>0 ) {
    theListView->cardList()->lastVisibleItem( cardNo );
    theListView->showCard( cardNo );
  }

  theApplication->notBusy();
  theApplication->setBusyDialog( NULL );

  if ( cardNo==0 )
    theInfoDialog->postBlocked( "No card contains that string.", 
				_currentParentWindow );
}
