/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
// XContact.c++

#include "Version.h"
#include "XContact.h"
#include "ListView.h"
#include "Database.h"
#include "PrefFile.h"

#include "OkFile.h"

#include <Vk/VkErrorDialog.h>
#include <Vk/VkInfoDialog.h>
#include <Vk/VkPromptDialog.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>

float 
XContact::Version() { return XCONTACT_VERSION; }


XContact::XContact( int* argc, char** argv, const char* bookName ) :
  VkApp( "XContact", argc, argv )
{
#if 0
  Using IL, catching signals seems to leave process orphaned...
  Bug in ImgVision Lib ??
  signal( SIGHUP, &XContact::QuitYourself );
  signal( SIGQUIT, &XContact::QuitYourself );
  signal( SIGINT, &XContact::QuitYourself );
#endif

  OkStr s = "XContact - Address Book and Contact Manager\n";
  s.append( "Version " ); 
  s.append( OkStr( Version(), "%.2f, " ) ); s.append( BUILD_DATE );
  s.append( "\n\nhajadi@sgi.com\n" );
  s.append( "Copyright (c) 1993,1994  Ivan M. Hajadi" );

  setVersionString( s );

  // Setup main window.
  new ListView;
  assert( theListView );
  theListView->show();

  // If data file directory doesn't exist yet, create it.
  if ( access( thePrefFile->dataFilePath(), F_OK ) ) 
      if ( mkdir( thePrefFile->dataFilePath(), 0755 ) ) {
	OkStr s = "Cannot create data directory: ";
	s.append( thePrefFile->dataFilePath() );
	theErrorDialog->postBlocked( s );
      }


  if ( bookName ) {
    // User supplied book name on the command line.
    // If exists, open that book, but preserve the old last open book.
    OkStr oldLastOpenRolodex = thePrefFile->lastRolodex();

    // Pass ListView::openFile expects bookName w.o. .DAT extension.
    OkStr dataFileName = OkStr( bookName )|".DAT";

    if ( OkFile::Exists( dataFileName ) )
      // Rolodex found in current directory.
      theListView->openFile( bookName );
    else if ( OkFile::Exists( OkStr(thePrefFile->dataFilePath())|dataFileName) )
      // Rolodex found in the data file directory.
      theListView->openFile( OkStr(thePrefFile->dataFilePath())|bookName );
    else 
      theErrorDialog->postBlocked( "Cannot open contact book: "|dataFileName );

    // Restore last book info.
    thePrefFile->setLastRolodex( oldLastOpenRolodex );

  } else if ( ! thePrefFile->noLastRolodex() ) {
    // Open the last book opened when vbook quit, if any.

    theListView->openFile( thePrefFile->lastRolodex() );
  }

  if ( ! theDatabase ) {
    // No database loaded.
    OkStr s = "Use File->New to create new contact book,\n";
    s.append( "or File->Open to open existing contact book." );
    theInfoDialog->postBlocked( s );
  }
}

XContact::~XContact() 
{
  if ( ! thePrefFile->save() ) {
    // Can't save.
    OkStr s = "Cannot save preference file: " ;
    s.append( thePrefFile->fileName() );
    theErrorDialog->postBlocked( s );
  }

#if 0
  signal( SIGHUP, SIG_IGN );
  signal( SIGINT, SIG_IGN );
  signal( SIGQUIT, SIG_IGN );
#endif

  // Make sure book closed properly.
  delete theDatabase;
  terminate();
}


void
XContact::quitYourself()
{
  if ( ! theListView->okToQuit() ) {
    // Change mind.
    return;
  }

  // Suicide.
  delete this;
}

